s/@@NASM_MAJOR@@/2/g
s/@@NASM_MINOR@@/10/g
s/@@NASM_SUBMINOR@@/9/g
s/@@NASM_PATCHLEVEL@@/0/g
s/@@NASM_SNAPSHOT@@/0/g
s/@@NASM_VERSION_ID@@/34212096/g
s/@@NASM_VERSION_XID@@/0x020a0900/g
s/@@NASM_VER@@/2.10.09/g
s/@@NASM_MANGLED_VER@@/2.10.09/g
