/* This file auto-generated from insns.dat by insns.pl - don't edit it */

#include "nasm.h"
#include "insns.h"

static const struct itemplate instrux[] = {
    /*    0 */ {I_RESB, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+21504, IF_8086},
    /*    1 */ {I_AAA, 0, {0,0,0,0,0}, nasm_bytecodes+22354, IF_8086|IF_NOLONG},
    /*    2 */ {I_AAD, 0, {0,0,0,0,0}, nasm_bytecodes+21362, IF_8086|IF_NOLONG},
    /*    3 */ {I_AAD, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+21366, IF_8086|IF_SB|IF_NOLONG},
    /*    4 */ {I_AAM, 0, {0,0,0,0,0}, nasm_bytecodes+21370, IF_8086|IF_NOLONG},
    /*    5 */ {I_AAM, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+21374, IF_8086|IF_SB|IF_NOLONG},
    /*    6 */ {I_AAS, 0, {0,0,0,0,0}, nasm_bytecodes+22357, IF_8086|IF_NOLONG},
    /*    7 */ {I_ADC, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+19617, IF_8086|IF_SM|IF_LOCK},
    /*    8 */ {I_ADC, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+19618, IF_8086},
    /*    9 */ {I_ADC, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16077, IF_8086|IF_SM|IF_LOCK},
    /*   10 */ {I_ADC, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16078, IF_8086},
    /*   11 */ {I_ADC, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16083, IF_386|IF_SM|IF_LOCK},
    /*   12 */ {I_ADC, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16084, IF_386},
    /*   13 */ {I_ADC, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16089, IF_X64|IF_SM|IF_LOCK},
    /*   14 */ {I_ADC, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16090, IF_X64},
    /*   15 */ {I_ADC, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+10788, IF_8086|IF_SM},
    /*   16 */ {I_ADC, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+10788, IF_8086},
    /*   17 */ {I_ADC, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+19622, IF_8086|IF_SM},
    /*   18 */ {I_ADC, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+19622, IF_8086},
    /*   19 */ {I_ADC, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+19627, IF_386|IF_SM},
    /*   20 */ {I_ADC, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+19627, IF_386},
    /*   21 */ {I_ADC, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+19632, IF_X64|IF_SM},
    /*   22 */ {I_ADC, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+19632, IF_X64},
    /*   23 */ {I_ADC, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7404, IF_8086|IF_LOCK},
    /*   24 */ {I_ADC, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7411, IF_386|IF_LOCK},
    /*   25 */ {I_ADC, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7418, IF_X64|IF_LOCK},
    /*   26 */ {I_ADC, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+21378, IF_8086|IF_SM},
    /*   27 */ {I_ADC, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+19637, IF_8086|IF_SM},
    /*   28 */ {I_ADC, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19642, IF_386|IF_SM},
    /*   29 */ {I_ADC, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19647, IF_X64|IF_SM},
    /*   30 */ {I_ADC, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+16095, IF_8086|IF_SM|IF_LOCK},
    /*   31 */ {I_ADC, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+7425, IF_8086|IF_SM|IF_LOCK},
    /*   32 */ {I_ADC, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+7432, IF_386|IF_SM|IF_LOCK},
    /*   33 */ {I_ADC, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+7439, IF_X64|IF_SM|IF_LOCK},
    /*   34 */ {I_ADC, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+7425, IF_8086|IF_SM|IF_LOCK},
    /*   35 */ {I_ADC, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+7432, IF_386|IF_SM|IF_LOCK},
    /*   36 */ {I_ADD, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+19652, IF_8086|IF_SM|IF_LOCK},
    /*   37 */ {I_ADD, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+19653, IF_8086},
    /*   38 */ {I_ADD, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16107, IF_8086|IF_SM|IF_LOCK},
    /*   39 */ {I_ADD, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16108, IF_8086},
    /*   40 */ {I_ADD, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16113, IF_386|IF_SM|IF_LOCK},
    /*   41 */ {I_ADD, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16114, IF_386},
    /*   42 */ {I_ADD, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16119, IF_X64|IF_SM|IF_LOCK},
    /*   43 */ {I_ADD, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16120, IF_X64},
    /*   44 */ {I_ADD, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+14533, IF_8086|IF_SM},
    /*   45 */ {I_ADD, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+14533, IF_8086},
    /*   46 */ {I_ADD, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+19657, IF_8086|IF_SM},
    /*   47 */ {I_ADD, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+19657, IF_8086},
    /*   48 */ {I_ADD, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+19662, IF_386|IF_SM},
    /*   49 */ {I_ADD, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+19662, IF_386},
    /*   50 */ {I_ADD, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+19667, IF_X64|IF_SM},
    /*   51 */ {I_ADD, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+19667, IF_X64},
    /*   52 */ {I_ADD, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7446, IF_8086|IF_LOCK},
    /*   53 */ {I_ADD, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7453, IF_386|IF_LOCK},
    /*   54 */ {I_ADD, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7460, IF_X64|IF_LOCK},
    /*   55 */ {I_ADD, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+21382, IF_8086|IF_SM},
    /*   56 */ {I_ADD, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+19672, IF_8086|IF_SM},
    /*   57 */ {I_ADD, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19677, IF_386|IF_SM},
    /*   58 */ {I_ADD, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19682, IF_X64|IF_SM},
    /*   59 */ {I_ADD, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+16125, IF_8086|IF_SM|IF_LOCK},
    /*   60 */ {I_ADD, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+7467, IF_8086|IF_SM|IF_LOCK},
    /*   61 */ {I_ADD, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+7474, IF_386|IF_SM|IF_LOCK},
    /*   62 */ {I_ADD, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+7481, IF_X64|IF_SM|IF_LOCK},
    /*   63 */ {I_ADD, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16125, IF_8086|IF_SM|IF_LOCK},
    /*   64 */ {I_ADD, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+7467, IF_8086|IF_SM|IF_LOCK},
    /*   65 */ {I_ADD, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+7474, IF_386|IF_SM|IF_LOCK},
    /*   66 */ {I_AND, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+19687, IF_8086|IF_SM|IF_LOCK},
    /*   67 */ {I_AND, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+19688, IF_8086},
    /*   68 */ {I_AND, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16137, IF_8086|IF_SM|IF_LOCK},
    /*   69 */ {I_AND, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16138, IF_8086},
    /*   70 */ {I_AND, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16143, IF_386|IF_SM|IF_LOCK},
    /*   71 */ {I_AND, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16144, IF_386},
    /*   72 */ {I_AND, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16149, IF_X64|IF_SM|IF_LOCK},
    /*   73 */ {I_AND, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16150, IF_X64},
    /*   74 */ {I_AND, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+14813, IF_8086|IF_SM},
    /*   75 */ {I_AND, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+14813, IF_8086},
    /*   76 */ {I_AND, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+19692, IF_8086|IF_SM},
    /*   77 */ {I_AND, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+19692, IF_8086},
    /*   78 */ {I_AND, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+19697, IF_386|IF_SM},
    /*   79 */ {I_AND, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+19697, IF_386},
    /*   80 */ {I_AND, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+19702, IF_X64|IF_SM},
    /*   81 */ {I_AND, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+19702, IF_X64},
    /*   82 */ {I_AND, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7488, IF_8086|IF_LOCK},
    /*   83 */ {I_AND, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7495, IF_386|IF_LOCK},
    /*   84 */ {I_AND, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7502, IF_X64|IF_LOCK},
    /*   85 */ {I_AND, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+21386, IF_8086|IF_SM},
    /*   86 */ {I_AND, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+19707, IF_8086|IF_SM},
    /*   87 */ {I_AND, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19712, IF_386|IF_SM},
    /*   88 */ {I_AND, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19717, IF_X64|IF_SM},
    /*   89 */ {I_AND, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+16155, IF_8086|IF_SM|IF_LOCK},
    /*   90 */ {I_AND, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+7509, IF_8086|IF_SM|IF_LOCK},
    /*   91 */ {I_AND, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+7516, IF_386|IF_SM|IF_LOCK},
    /*   92 */ {I_AND, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+7523, IF_X64|IF_SM|IF_LOCK},
    /*   93 */ {I_AND, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16155, IF_8086|IF_SM|IF_LOCK},
    /*   94 */ {I_AND, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+7509, IF_8086|IF_SM|IF_LOCK},
    /*   95 */ {I_AND, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+7516, IF_386|IF_SM|IF_LOCK},
    /*   96 */ {I_ARPL, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+21390, IF_286|IF_PROT|IF_SM|IF_NOLONG},
    /*   97 */ {I_ARPL, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+21390, IF_286|IF_PROT|IF_NOLONG},
    /*   98 */ {I_BOUND, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+19722, IF_186|IF_NOLONG},
    /*   99 */ {I_BOUND, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+19727, IF_386|IF_NOLONG},
    /*  100 */ {I_BSF, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+7530, IF_386|IF_SM},
    /*  101 */ {I_BSF, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7530, IF_386},
    /*  102 */ {I_BSF, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+7537, IF_386|IF_SM},
    /*  103 */ {I_BSF, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7537, IF_386},
    /*  104 */ {I_BSF, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+7544, IF_X64|IF_SM},
    /*  105 */ {I_BSF, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7544, IF_X64},
    /*  106 */ {I_BSR, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+7551, IF_386|IF_SM},
    /*  107 */ {I_BSR, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7551, IF_386},
    /*  108 */ {I_BSR, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+7558, IF_386|IF_SM},
    /*  109 */ {I_BSR, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7558, IF_386},
    /*  110 */ {I_BSR, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+7565, IF_X64|IF_SM},
    /*  111 */ {I_BSR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7565, IF_X64},
    /*  112 */ {I_BSWAP, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16167, IF_486},
    /*  113 */ {I_BSWAP, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+16173, IF_X64},
    /*  114 */ {I_BT, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16179, IF_386|IF_SM},
    /*  115 */ {I_BT, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16179, IF_386},
    /*  116 */ {I_BT, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16185, IF_386|IF_SM},
    /*  117 */ {I_BT, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16185, IF_386},
    /*  118 */ {I_BT, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16191, IF_X64|IF_SM},
    /*  119 */ {I_BT, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16191, IF_X64},
    /*  120 */ {I_BT, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+7572, IF_386|IF_SB},
    /*  121 */ {I_BT, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+7579, IF_386|IF_SB},
    /*  122 */ {I_BT, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+7586, IF_X64|IF_SB},
    /*  123 */ {I_BTC, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7593, IF_386|IF_SM|IF_LOCK},
    /*  124 */ {I_BTC, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7594, IF_386},
    /*  125 */ {I_BTC, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7600, IF_386|IF_SM|IF_LOCK},
    /*  126 */ {I_BTC, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7601, IF_386},
    /*  127 */ {I_BTC, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7607, IF_X64|IF_SM|IF_LOCK},
    /*  128 */ {I_BTC, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7608, IF_X64},
    /*  129 */ {I_BTC, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+3636, IF_386|IF_SB|IF_LOCK},
    /*  130 */ {I_BTC, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+3644, IF_386|IF_SB|IF_LOCK},
    /*  131 */ {I_BTC, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+3652, IF_X64|IF_SB|IF_LOCK},
    /*  132 */ {I_BTR, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7614, IF_386|IF_SM|IF_LOCK},
    /*  133 */ {I_BTR, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7615, IF_386},
    /*  134 */ {I_BTR, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7621, IF_386|IF_SM|IF_LOCK},
    /*  135 */ {I_BTR, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7622, IF_386},
    /*  136 */ {I_BTR, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7628, IF_X64|IF_SM|IF_LOCK},
    /*  137 */ {I_BTR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7629, IF_X64},
    /*  138 */ {I_BTR, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+3660, IF_386|IF_SB|IF_LOCK},
    /*  139 */ {I_BTR, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+3668, IF_386|IF_SB|IF_LOCK},
    /*  140 */ {I_BTR, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+3676, IF_X64|IF_SB|IF_LOCK},
    /*  141 */ {I_BTS, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7635, IF_386|IF_SM|IF_LOCK},
    /*  142 */ {I_BTS, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7636, IF_386},
    /*  143 */ {I_BTS, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7642, IF_386|IF_SM|IF_LOCK},
    /*  144 */ {I_BTS, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7643, IF_386},
    /*  145 */ {I_BTS, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7649, IF_X64|IF_SM|IF_LOCK},
    /*  146 */ {I_BTS, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7650, IF_X64},
    /*  147 */ {I_BTS, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+3684, IF_386|IF_SB|IF_LOCK},
    /*  148 */ {I_BTS, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+3692, IF_386|IF_SB|IF_LOCK},
    /*  149 */ {I_BTS, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+3700, IF_X64|IF_SB|IF_LOCK},
    /*  150 */ {I_CALL, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+19732, IF_8086},
    /*  151 */ {I_CALL, 1, {IMMEDIATE|BITS16,0,0,0,0}, nasm_bytecodes+19737, IF_8086|IF_NOLONG},
    /*  152 */ {I_CALL, 1, {IMMEDIATE|BITS32,0,0,0,0}, nasm_bytecodes+19742, IF_386|IF_NOLONG},
    /*  153 */ {I_CALL, 1, {IMMEDIATE|BITS64,0,0,0,0}, nasm_bytecodes+19747, IF_X64},
    /*  154 */ {I_CALL, 2, {IMMEDIATE|COLON,IMMEDIATE,0,0,0}, nasm_bytecodes+16215, IF_8086|IF_NOLONG},
    /*  155 */ {I_CALL, 2, {IMMEDIATE|BITS16|COLON,IMMEDIATE,0,0,0}, nasm_bytecodes+16221, IF_8086|IF_NOLONG},
    /*  156 */ {I_CALL, 2, {IMMEDIATE|COLON,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+16221, IF_8086|IF_NOLONG},
    /*  157 */ {I_CALL, 2, {IMMEDIATE|BITS32|COLON,IMMEDIATE,0,0,0}, nasm_bytecodes+16227, IF_386|IF_NOLONG},
    /*  158 */ {I_CALL, 2, {IMMEDIATE|COLON,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+16227, IF_386|IF_NOLONG},
    /*  159 */ {I_CALL, 1, {MEMORY|FAR,0,0,0,0}, nasm_bytecodes+19752, IF_8086|IF_NOLONG},
    /*  160 */ {I_CALL, 1, {MEMORY|FAR,0,0,0,0}, nasm_bytecodes+19757, IF_X64},
    /*  161 */ {I_CALL, 1, {MEMORY|BITS16|FAR,0,0,0,0}, nasm_bytecodes+19762, IF_8086},
    /*  162 */ {I_CALL, 1, {MEMORY|BITS32|FAR,0,0,0,0}, nasm_bytecodes+19767, IF_386},
    /*  163 */ {I_CALL, 1, {MEMORY|BITS64|FAR,0,0,0,0}, nasm_bytecodes+19757, IF_X64},
    /*  164 */ {I_CALL, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+19772, IF_8086},
    /*  165 */ {I_CALL, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19777, IF_8086|IF_NOLONG},
    /*  166 */ {I_CALL, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19782, IF_386|IF_NOLONG},
    /*  167 */ {I_CALL, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19787, IF_X64},
    /*  168 */ {I_CBW, 0, {0,0,0,0,0}, nasm_bytecodes+21402, IF_8086},
    /*  169 */ {I_CDQ, 0, {0,0,0,0,0}, nasm_bytecodes+21406, IF_386},
    /*  170 */ {I_CDQE, 0, {0,0,0,0,0}, nasm_bytecodes+21410, IF_X64},
    /*  171 */ {I_CLC, 0, {0,0,0,0,0}, nasm_bytecodes+21154, IF_8086},
    /*  172 */ {I_CLD, 0, {0,0,0,0,0}, nasm_bytecodes+22360, IF_8086},
    /*  173 */ {I_CLI, 0, {0,0,0,0,0}, nasm_bytecodes+22363, IF_8086},
    /*  174 */ {I_CLTS, 0, {0,0,0,0,0}, nasm_bytecodes+21414, IF_286|IF_PRIV},
    /*  175 */ {I_CMC, 0, {0,0,0,0,0}, nasm_bytecodes+22366, IF_8086},
    /*  176 */ {I_CMP, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21418, IF_8086|IF_SM},
    /*  177 */ {I_CMP, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21418, IF_8086},
    /*  178 */ {I_CMP, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+19792, IF_8086|IF_SM},
    /*  179 */ {I_CMP, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+19792, IF_8086},
    /*  180 */ {I_CMP, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+19797, IF_386|IF_SM},
    /*  181 */ {I_CMP, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+19797, IF_386},
    /*  182 */ {I_CMP, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+19802, IF_X64|IF_SM},
    /*  183 */ {I_CMP, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+19802, IF_X64},
    /*  184 */ {I_CMP, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+14771, IF_8086|IF_SM},
    /*  185 */ {I_CMP, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+14771, IF_8086},
    /*  186 */ {I_CMP, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+19807, IF_8086|IF_SM},
    /*  187 */ {I_CMP, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+19807, IF_8086},
    /*  188 */ {I_CMP, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+19812, IF_386|IF_SM},
    /*  189 */ {I_CMP, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+19812, IF_386},
    /*  190 */ {I_CMP, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+19817, IF_X64|IF_SM},
    /*  191 */ {I_CMP, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+19817, IF_X64},
    /*  192 */ {I_CMP, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16233, IF_8086},
    /*  193 */ {I_CMP, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16239, IF_386},
    /*  194 */ {I_CMP, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16245, IF_X64},
    /*  195 */ {I_CMP, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+21422, IF_8086|IF_SM},
    /*  196 */ {I_CMP, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+19822, IF_8086|IF_SM},
    /*  197 */ {I_CMP, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19827, IF_386|IF_SM},
    /*  198 */ {I_CMP, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+19832, IF_X64|IF_SM},
    /*  199 */ {I_CMP, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+19837, IF_8086|IF_SM},
    /*  200 */ {I_CMP, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+16251, IF_8086|IF_SM},
    /*  201 */ {I_CMP, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+16257, IF_386|IF_SM},
    /*  202 */ {I_CMP, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+16263, IF_X64|IF_SM},
    /*  203 */ {I_CMP, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+19837, IF_8086|IF_SM},
    /*  204 */ {I_CMP, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+16251, IF_8086|IF_SM},
    /*  205 */ {I_CMP, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+16257, IF_386|IF_SM},
    /*  206 */ {I_CMPSB, 0, {0,0,0,0,0}, nasm_bytecodes+21426, IF_8086},
    /*  207 */ {I_CMPSD, 0, {0,0,0,0,0}, nasm_bytecodes+19847, IF_386},
    /*  208 */ {I_CMPSQ, 0, {0,0,0,0,0}, nasm_bytecodes+19852, IF_X64},
    /*  209 */ {I_CMPSW, 0, {0,0,0,0,0}, nasm_bytecodes+19857, IF_8086},
    /*  210 */ {I_CMPXCHG, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+16269, IF_PENT|IF_SM|IF_LOCK},
    /*  211 */ {I_CMPXCHG, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+16270, IF_PENT},
    /*  212 */ {I_CMPXCHG, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7656, IF_PENT|IF_SM|IF_LOCK},
    /*  213 */ {I_CMPXCHG, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+7657, IF_PENT},
    /*  214 */ {I_CMPXCHG, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7663, IF_PENT|IF_SM|IF_LOCK},
    /*  215 */ {I_CMPXCHG, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+7664, IF_PENT},
    /*  216 */ {I_CMPXCHG, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7670, IF_X64|IF_SM|IF_LOCK},
    /*  217 */ {I_CMPXCHG, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+7671, IF_X64},
    /*  218 */ {I_CMPXCHG8B, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+7677, IF_PENT|IF_LOCK},
    /*  219 */ {I_CMPXCHG16B, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+16287, IF_X64|IF_LOCK},
    /*  220 */ {I_CPUID, 0, {0,0,0,0,0}, nasm_bytecodes+21430, IF_PENT},
    /*  221 */ {I_CPU_READ, 0, {0,0,0,0,0}, nasm_bytecodes+21434, IF_PENT|IF_CYRIX},
    /*  222 */ {I_CPU_WRITE, 0, {0,0,0,0,0}, nasm_bytecodes+21438, IF_PENT|IF_CYRIX},
    /*  223 */ {I_CQO, 0, {0,0,0,0,0}, nasm_bytecodes+21442, IF_X64},
    /*  224 */ {I_CWD, 0, {0,0,0,0,0}, nasm_bytecodes+21446, IF_8086},
    /*  225 */ {I_CWDE, 0, {0,0,0,0,0}, nasm_bytecodes+21450, IF_386},
    /*  226 */ {I_DAA, 0, {0,0,0,0,0}, nasm_bytecodes+22369, IF_8086|IF_NOLONG},
    /*  227 */ {I_DAS, 0, {0,0,0,0,0}, nasm_bytecodes+22372, IF_8086|IF_NOLONG},
    /*  228 */ {I_DEC, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+21454, IF_8086|IF_NOLONG},
    /*  229 */ {I_DEC, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+21458, IF_386|IF_NOLONG},
    /*  230 */ {I_DEC, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+19867, IF_8086|IF_LOCK},
    /*  231 */ {I_DEC, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+16293, IF_8086|IF_LOCK},
    /*  232 */ {I_DEC, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16299, IF_386|IF_LOCK},
    /*  233 */ {I_DEC, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+16305, IF_X64|IF_LOCK},
    /*  234 */ {I_DIV, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+21462, IF_8086},
    /*  235 */ {I_DIV, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19872, IF_8086},
    /*  236 */ {I_DIV, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19877, IF_386},
    /*  237 */ {I_DIV, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19882, IF_X64},
    /*  238 */ {I_DMINT, 0, {0,0,0,0,0}, nasm_bytecodes+21466, IF_P6|IF_CYRIX},
    /*  239 */ {I_EMMS, 0, {0,0,0,0,0}, nasm_bytecodes+21470, IF_PENT|IF_MMX},
    /*  240 */ {I_ENTER, 2, {IMMEDIATE,IMMEDIATE,0,0,0}, nasm_bytecodes+19887, IF_186},
    /*  241 */ {I_EQU, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+22413, IF_8086},
    /*  242 */ {I_EQU, 2, {IMMEDIATE|COLON,IMMEDIATE,0,0,0}, nasm_bytecodes+22413, IF_8086},
    /*  243 */ {I_F2XM1, 0, {0,0,0,0,0}, nasm_bytecodes+21474, IF_8086|IF_FPU},
    /*  244 */ {I_FABS, 0, {0,0,0,0,0}, nasm_bytecodes+21478, IF_8086|IF_FPU},
    /*  245 */ {I_FADD, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21482, IF_8086|IF_FPU},
    /*  246 */ {I_FADD, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21486, IF_8086|IF_FPU},
    /*  247 */ {I_FADD, 1, {FPUREG|TO,0,0,0,0}, nasm_bytecodes+19892, IF_8086|IF_FPU},
    /*  248 */ {I_FADD, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19897, IF_8086|IF_FPU},
    /*  249 */ {I_FADD, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+19892, IF_8086|IF_FPU},
    /*  250 */ {I_FADD, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19902, IF_8086|IF_FPU},
    /*  251 */ {I_FADDP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19907, IF_8086|IF_FPU},
    /*  252 */ {I_FADDP, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+19907, IF_8086|IF_FPU},
    /*  253 */ {I_FBLD, 1, {MEMORY|BITS80,0,0,0,0}, nasm_bytecodes+21494, IF_8086|IF_FPU},
    /*  254 */ {I_FBLD, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21494, IF_8086|IF_FPU},
    /*  255 */ {I_FBSTP, 1, {MEMORY|BITS80,0,0,0,0}, nasm_bytecodes+21498, IF_8086|IF_FPU},
    /*  256 */ {I_FBSTP, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21498, IF_8086|IF_FPU},
    /*  257 */ {I_FCHS, 0, {0,0,0,0,0}, nasm_bytecodes+21502, IF_8086|IF_FPU},
    /*  258 */ {I_FCLEX, 0, {0,0,0,0,0}, nasm_bytecodes+19912, IF_8086|IF_FPU},
    /*  259 */ {I_FCMOVB, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19917, IF_P6|IF_FPU},
    /*  260 */ {I_FCMOVB, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19922, IF_P6|IF_FPU},
    /*  261 */ {I_FCMOVBE, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19927, IF_P6|IF_FPU},
    /*  262 */ {I_FCMOVBE, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19932, IF_P6|IF_FPU},
    /*  263 */ {I_FCMOVE, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19937, IF_P6|IF_FPU},
    /*  264 */ {I_FCMOVE, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19942, IF_P6|IF_FPU},
    /*  265 */ {I_FCMOVNB, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19947, IF_P6|IF_FPU},
    /*  266 */ {I_FCMOVNB, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19952, IF_P6|IF_FPU},
    /*  267 */ {I_FCMOVNBE, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19957, IF_P6|IF_FPU},
    /*  268 */ {I_FCMOVNBE, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19962, IF_P6|IF_FPU},
    /*  269 */ {I_FCMOVNE, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19967, IF_P6|IF_FPU},
    /*  270 */ {I_FCMOVNE, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19972, IF_P6|IF_FPU},
    /*  271 */ {I_FCMOVNU, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19977, IF_P6|IF_FPU},
    /*  272 */ {I_FCMOVNU, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19982, IF_P6|IF_FPU},
    /*  273 */ {I_FCMOVU, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19987, IF_P6|IF_FPU},
    /*  274 */ {I_FCMOVU, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+19992, IF_P6|IF_FPU},
    /*  275 */ {I_FCOM, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21538, IF_8086|IF_FPU},
    /*  276 */ {I_FCOM, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21542, IF_8086|IF_FPU},
    /*  277 */ {I_FCOM, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+19997, IF_8086|IF_FPU},
    /*  278 */ {I_FCOM, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20002, IF_8086|IF_FPU},
    /*  279 */ {I_FCOMI, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20007, IF_P6|IF_FPU},
    /*  280 */ {I_FCOMI, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20012, IF_P6|IF_FPU},
    /*  281 */ {I_FCOMIP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20017, IF_P6|IF_FPU},
    /*  282 */ {I_FCOMIP, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20022, IF_P6|IF_FPU},
    /*  283 */ {I_FCOMP, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21558, IF_8086|IF_FPU},
    /*  284 */ {I_FCOMP, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21562, IF_8086|IF_FPU},
    /*  285 */ {I_FCOMP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20027, IF_8086|IF_FPU},
    /*  286 */ {I_FCOMP, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20032, IF_8086|IF_FPU},
    /*  287 */ {I_FCOMPP, 0, {0,0,0,0,0}, nasm_bytecodes+21570, IF_8086|IF_FPU},
    /*  288 */ {I_FCOS, 0, {0,0,0,0,0}, nasm_bytecodes+21574, IF_386|IF_FPU},
    /*  289 */ {I_FDECSTP, 0, {0,0,0,0,0}, nasm_bytecodes+21578, IF_8086|IF_FPU},
    /*  290 */ {I_FDISI, 0, {0,0,0,0,0}, nasm_bytecodes+20037, IF_8086|IF_FPU},
    /*  291 */ {I_FDIV, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21582, IF_8086|IF_FPU},
    /*  292 */ {I_FDIV, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21586, IF_8086|IF_FPU},
    /*  293 */ {I_FDIV, 1, {FPUREG|TO,0,0,0,0}, nasm_bytecodes+20042, IF_8086|IF_FPU},
    /*  294 */ {I_FDIV, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20047, IF_8086|IF_FPU},
    /*  295 */ {I_FDIV, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20042, IF_8086|IF_FPU},
    /*  296 */ {I_FDIV, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20052, IF_8086|IF_FPU},
    /*  297 */ {I_FDIVP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20057, IF_8086|IF_FPU},
    /*  298 */ {I_FDIVP, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20057, IF_8086|IF_FPU},
    /*  299 */ {I_FDIVR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21594, IF_8086|IF_FPU},
    /*  300 */ {I_FDIVR, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21598, IF_8086|IF_FPU},
    /*  301 */ {I_FDIVR, 1, {FPUREG|TO,0,0,0,0}, nasm_bytecodes+20062, IF_8086|IF_FPU},
    /*  302 */ {I_FDIVR, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20062, IF_8086|IF_FPU},
    /*  303 */ {I_FDIVR, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20067, IF_8086|IF_FPU},
    /*  304 */ {I_FDIVR, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20072, IF_8086|IF_FPU},
    /*  305 */ {I_FDIVRP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20077, IF_8086|IF_FPU},
    /*  306 */ {I_FDIVRP, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20077, IF_8086|IF_FPU},
    /*  307 */ {I_FEMMS, 0, {0,0,0,0,0}, nasm_bytecodes+21606, IF_PENT|IF_3DNOW},
    /*  308 */ {I_FENI, 0, {0,0,0,0,0}, nasm_bytecodes+20082, IF_8086|IF_FPU},
    /*  309 */ {I_FFREE, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20087, IF_8086|IF_FPU},
    /*  310 */ {I_FFREE, 0, {0,0,0,0,0}, nasm_bytecodes+21610, IF_8086|IF_FPU},
    /*  311 */ {I_FFREEP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20092, IF_286|IF_FPU|IF_UNDOC},
    /*  312 */ {I_FFREEP, 0, {0,0,0,0,0}, nasm_bytecodes+21614, IF_286|IF_FPU|IF_UNDOC},
    /*  313 */ {I_FIADD, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21618, IF_8086|IF_FPU},
    /*  314 */ {I_FIADD, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21622, IF_8086|IF_FPU},
    /*  315 */ {I_FICOM, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21626, IF_8086|IF_FPU},
    /*  316 */ {I_FICOM, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21630, IF_8086|IF_FPU},
    /*  317 */ {I_FICOMP, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21634, IF_8086|IF_FPU},
    /*  318 */ {I_FICOMP, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21638, IF_8086|IF_FPU},
    /*  319 */ {I_FIDIV, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21642, IF_8086|IF_FPU},
    /*  320 */ {I_FIDIV, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21646, IF_8086|IF_FPU},
    /*  321 */ {I_FIDIVR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21650, IF_8086|IF_FPU},
    /*  322 */ {I_FIDIVR, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21654, IF_8086|IF_FPU},
    /*  323 */ {I_FILD, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21658, IF_8086|IF_FPU},
    /*  324 */ {I_FILD, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21662, IF_8086|IF_FPU},
    /*  325 */ {I_FILD, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21666, IF_8086|IF_FPU},
    /*  326 */ {I_FIMUL, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21670, IF_8086|IF_FPU},
    /*  327 */ {I_FIMUL, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21674, IF_8086|IF_FPU},
    /*  328 */ {I_FINCSTP, 0, {0,0,0,0,0}, nasm_bytecodes+21678, IF_8086|IF_FPU},
    /*  329 */ {I_FINIT, 0, {0,0,0,0,0}, nasm_bytecodes+20097, IF_8086|IF_FPU},
    /*  330 */ {I_FIST, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21682, IF_8086|IF_FPU},
    /*  331 */ {I_FIST, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21686, IF_8086|IF_FPU},
    /*  332 */ {I_FISTP, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21690, IF_8086|IF_FPU},
    /*  333 */ {I_FISTP, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21694, IF_8086|IF_FPU},
    /*  334 */ {I_FISTP, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21698, IF_8086|IF_FPU},
    /*  335 */ {I_FISTTP, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21702, IF_PRESCOTT|IF_FPU},
    /*  336 */ {I_FISTTP, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21706, IF_PRESCOTT|IF_FPU},
    /*  337 */ {I_FISTTP, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21710, IF_PRESCOTT|IF_FPU},
    /*  338 */ {I_FISUB, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21714, IF_8086|IF_FPU},
    /*  339 */ {I_FISUB, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21718, IF_8086|IF_FPU},
    /*  340 */ {I_FISUBR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21722, IF_8086|IF_FPU},
    /*  341 */ {I_FISUBR, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21726, IF_8086|IF_FPU},
    /*  342 */ {I_FLD, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21730, IF_8086|IF_FPU},
    /*  343 */ {I_FLD, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21734, IF_8086|IF_FPU},
    /*  344 */ {I_FLD, 1, {MEMORY|BITS80,0,0,0,0}, nasm_bytecodes+21738, IF_8086|IF_FPU},
    /*  345 */ {I_FLD, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20102, IF_8086|IF_FPU},
    /*  346 */ {I_FLD1, 0, {0,0,0,0,0}, nasm_bytecodes+21746, IF_8086|IF_FPU},
    /*  347 */ {I_FLDCW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21750, IF_8086|IF_FPU|IF_SW},
    /*  348 */ {I_FLDENV, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21754, IF_8086|IF_FPU},
    /*  349 */ {I_FLDL2E, 0, {0,0,0,0,0}, nasm_bytecodes+21758, IF_8086|IF_FPU},
    /*  350 */ {I_FLDL2T, 0, {0,0,0,0,0}, nasm_bytecodes+21762, IF_8086|IF_FPU},
    /*  351 */ {I_FLDLG2, 0, {0,0,0,0,0}, nasm_bytecodes+21766, IF_8086|IF_FPU},
    /*  352 */ {I_FLDLN2, 0, {0,0,0,0,0}, nasm_bytecodes+21770, IF_8086|IF_FPU},
    /*  353 */ {I_FLDPI, 0, {0,0,0,0,0}, nasm_bytecodes+21774, IF_8086|IF_FPU},
    /*  354 */ {I_FLDZ, 0, {0,0,0,0,0}, nasm_bytecodes+21778, IF_8086|IF_FPU},
    /*  355 */ {I_FMUL, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21782, IF_8086|IF_FPU},
    /*  356 */ {I_FMUL, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21786, IF_8086|IF_FPU},
    /*  357 */ {I_FMUL, 1, {FPUREG|TO,0,0,0,0}, nasm_bytecodes+20107, IF_8086|IF_FPU},
    /*  358 */ {I_FMUL, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20107, IF_8086|IF_FPU},
    /*  359 */ {I_FMUL, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20112, IF_8086|IF_FPU},
    /*  360 */ {I_FMUL, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20117, IF_8086|IF_FPU},
    /*  361 */ {I_FMULP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20122, IF_8086|IF_FPU},
    /*  362 */ {I_FMULP, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20122, IF_8086|IF_FPU},
    /*  363 */ {I_FNCLEX, 0, {0,0,0,0,0}, nasm_bytecodes+19913, IF_8086|IF_FPU},
    /*  364 */ {I_FNDISI, 0, {0,0,0,0,0}, nasm_bytecodes+20038, IF_8086|IF_FPU},
    /*  365 */ {I_FNENI, 0, {0,0,0,0,0}, nasm_bytecodes+20083, IF_8086|IF_FPU},
    /*  366 */ {I_FNINIT, 0, {0,0,0,0,0}, nasm_bytecodes+20098, IF_8086|IF_FPU},
    /*  367 */ {I_FNOP, 0, {0,0,0,0,0}, nasm_bytecodes+21794, IF_8086|IF_FPU},
    /*  368 */ {I_FNSAVE, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20128, IF_8086|IF_FPU},
    /*  369 */ {I_FNSTCW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20138, IF_8086|IF_FPU|IF_SW},
    /*  370 */ {I_FNSTENV, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20143, IF_8086|IF_FPU},
    /*  371 */ {I_FNSTSW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20153, IF_8086|IF_FPU|IF_SW},
    /*  372 */ {I_FNSTSW, 1, {REG_AX,0,0,0,0}, nasm_bytecodes+20158, IF_286|IF_FPU},
    /*  373 */ {I_FPATAN, 0, {0,0,0,0,0}, nasm_bytecodes+21798, IF_8086|IF_FPU},
    /*  374 */ {I_FPREM, 0, {0,0,0,0,0}, nasm_bytecodes+21802, IF_8086|IF_FPU},
    /*  375 */ {I_FPREM1, 0, {0,0,0,0,0}, nasm_bytecodes+21806, IF_386|IF_FPU},
    /*  376 */ {I_FPTAN, 0, {0,0,0,0,0}, nasm_bytecodes+21810, IF_8086|IF_FPU},
    /*  377 */ {I_FRNDINT, 0, {0,0,0,0,0}, nasm_bytecodes+21814, IF_8086|IF_FPU},
    /*  378 */ {I_FRSTOR, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21818, IF_8086|IF_FPU},
    /*  379 */ {I_FSAVE, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20127, IF_8086|IF_FPU},
    /*  380 */ {I_FSCALE, 0, {0,0,0,0,0}, nasm_bytecodes+21822, IF_8086|IF_FPU},
    /*  381 */ {I_FSETPM, 0, {0,0,0,0,0}, nasm_bytecodes+21826, IF_286|IF_FPU},
    /*  382 */ {I_FSIN, 0, {0,0,0,0,0}, nasm_bytecodes+21830, IF_386|IF_FPU},
    /*  383 */ {I_FSINCOS, 0, {0,0,0,0,0}, nasm_bytecodes+21834, IF_386|IF_FPU},
    /*  384 */ {I_FSQRT, 0, {0,0,0,0,0}, nasm_bytecodes+21838, IF_8086|IF_FPU},
    /*  385 */ {I_FST, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21842, IF_8086|IF_FPU},
    /*  386 */ {I_FST, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21846, IF_8086|IF_FPU},
    /*  387 */ {I_FST, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20132, IF_8086|IF_FPU},
    /*  388 */ {I_FSTCW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20137, IF_8086|IF_FPU|IF_SW},
    /*  389 */ {I_FSTENV, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20142, IF_8086|IF_FPU},
    /*  390 */ {I_FSTP, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21854, IF_8086|IF_FPU},
    /*  391 */ {I_FSTP, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21858, IF_8086|IF_FPU},
    /*  392 */ {I_FSTP, 1, {MEMORY|BITS80,0,0,0,0}, nasm_bytecodes+21862, IF_8086|IF_FPU},
    /*  393 */ {I_FSTP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20147, IF_8086|IF_FPU},
    /*  394 */ {I_FSTSW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20152, IF_8086|IF_FPU|IF_SW},
    /*  395 */ {I_FSTSW, 1, {REG_AX,0,0,0,0}, nasm_bytecodes+20157, IF_286|IF_FPU},
    /*  396 */ {I_FSUB, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21870, IF_8086|IF_FPU},
    /*  397 */ {I_FSUB, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21874, IF_8086|IF_FPU},
    /*  398 */ {I_FSUB, 1, {FPUREG|TO,0,0,0,0}, nasm_bytecodes+20162, IF_8086|IF_FPU},
    /*  399 */ {I_FSUB, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20162, IF_8086|IF_FPU},
    /*  400 */ {I_FSUB, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20167, IF_8086|IF_FPU},
    /*  401 */ {I_FSUB, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20172, IF_8086|IF_FPU},
    /*  402 */ {I_FSUBP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20177, IF_8086|IF_FPU},
    /*  403 */ {I_FSUBP, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20177, IF_8086|IF_FPU},
    /*  404 */ {I_FSUBR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+21882, IF_8086|IF_FPU},
    /*  405 */ {I_FSUBR, 1, {MEMORY|BITS64,0,0,0,0}, nasm_bytecodes+21886, IF_8086|IF_FPU},
    /*  406 */ {I_FSUBR, 1, {FPUREG|TO,0,0,0,0}, nasm_bytecodes+20182, IF_8086|IF_FPU},
    /*  407 */ {I_FSUBR, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20182, IF_8086|IF_FPU},
    /*  408 */ {I_FSUBR, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20187, IF_8086|IF_FPU},
    /*  409 */ {I_FSUBR, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20192, IF_8086|IF_FPU},
    /*  410 */ {I_FSUBRP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20197, IF_8086|IF_FPU},
    /*  411 */ {I_FSUBRP, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20197, IF_8086|IF_FPU},
    /*  412 */ {I_FTST, 0, {0,0,0,0,0}, nasm_bytecodes+21894, IF_8086|IF_FPU},
    /*  413 */ {I_FUCOM, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20202, IF_386|IF_FPU},
    /*  414 */ {I_FUCOM, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20207, IF_386|IF_FPU},
    /*  415 */ {I_FUCOMI, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20212, IF_P6|IF_FPU},
    /*  416 */ {I_FUCOMI, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20217, IF_P6|IF_FPU},
    /*  417 */ {I_FUCOMIP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20222, IF_P6|IF_FPU},
    /*  418 */ {I_FUCOMIP, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20227, IF_P6|IF_FPU},
    /*  419 */ {I_FUCOMP, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20232, IF_386|IF_FPU},
    /*  420 */ {I_FUCOMP, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20237, IF_386|IF_FPU},
    /*  421 */ {I_FUCOMPP, 0, {0,0,0,0,0}, nasm_bytecodes+21914, IF_386|IF_FPU},
    /*  422 */ {I_FXAM, 0, {0,0,0,0,0}, nasm_bytecodes+21918, IF_8086|IF_FPU},
    /*  423 */ {I_FXCH, 1, {FPUREG,0,0,0,0}, nasm_bytecodes+20242, IF_8086|IF_FPU},
    /*  424 */ {I_FXCH, 2, {FPUREG,FPU0,0,0,0}, nasm_bytecodes+20242, IF_8086|IF_FPU},
    /*  425 */ {I_FXCH, 2, {FPU0,FPUREG,0,0,0}, nasm_bytecodes+20247, IF_8086|IF_FPU},
    /*  426 */ {I_FXTRACT, 0, {0,0,0,0,0}, nasm_bytecodes+21926, IF_8086|IF_FPU},
    /*  427 */ {I_FYL2X, 0, {0,0,0,0,0}, nasm_bytecodes+21930, IF_8086|IF_FPU},
    /*  428 */ {I_FYL2XP1, 0, {0,0,0,0,0}, nasm_bytecodes+21934, IF_8086|IF_FPU},
    /*  429 */ {I_HLT, 0, {0,0,0,0,0}, nasm_bytecodes+22375, IF_8086|IF_PRIV},
    /*  430 */ {I_IDIV, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+21938, IF_8086},
    /*  431 */ {I_IDIV, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20252, IF_8086},
    /*  432 */ {I_IDIV, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+20257, IF_386},
    /*  433 */ {I_IDIV, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+20262, IF_X64},
    /*  434 */ {I_IMUL, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+21942, IF_8086},
    /*  435 */ {I_IMUL, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20267, IF_8086},
    /*  436 */ {I_IMUL, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+20272, IF_386},
    /*  437 */ {I_IMUL, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+20277, IF_X64},
    /*  438 */ {I_IMUL, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16311, IF_386|IF_SM},
    /*  439 */ {I_IMUL, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16311, IF_386},
    /*  440 */ {I_IMUL, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+16317, IF_386|IF_SM},
    /*  441 */ {I_IMUL, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16317, IF_386},
    /*  442 */ {I_IMUL, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+16323, IF_X64|IF_SM},
    /*  443 */ {I_IMUL, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16323, IF_X64},
    /*  444 */ {I_IMUL, 3, {REG_GPR|BITS16,MEMORY,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+16329, IF_186|IF_SM},
    /*  445 */ {I_IMUL, 3, {REG_GPR|BITS16,MEMORY,IMMEDIATE|BITS16,0,0}, nasm_bytecodes+16335, IF_186|IF_SM},
    /*  446 */ {I_IMUL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+16329, IF_186},
    /*  447 */ {I_IMUL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,IMMEDIATE|BITS16,0,0}, nasm_bytecodes+16335, IF_186},
    /*  448 */ {I_IMUL, 3, {REG_GPR|BITS32,MEMORY,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+16341, IF_386|IF_SM},
    /*  449 */ {I_IMUL, 3, {REG_GPR|BITS32,MEMORY,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+16347, IF_386|IF_SM},
    /*  450 */ {I_IMUL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+16341, IF_386},
    /*  451 */ {I_IMUL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+16347, IF_386},
    /*  452 */ {I_IMUL, 3, {REG_GPR|BITS64,MEMORY,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+16353, IF_X64|IF_SM},
    /*  453 */ {I_IMUL, 3, {REG_GPR|BITS64,MEMORY,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+16359, IF_X64|IF_SM},
    /*  454 */ {I_IMUL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+16353, IF_X64},
    /*  455 */ {I_IMUL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+16359, IF_X64},
    /*  456 */ {I_IMUL, 2, {REG_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16371, IF_186},
    /*  457 */ {I_IMUL, 2, {REG_GPR|BITS16,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+16377, IF_186},
    /*  458 */ {I_IMUL, 2, {REG_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16383, IF_386},
    /*  459 */ {I_IMUL, 2, {REG_GPR|BITS32,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+16389, IF_386},
    /*  460 */ {I_IMUL, 2, {REG_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16395, IF_X64},
    /*  461 */ {I_IMUL, 2, {REG_GPR|BITS64,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+16401, IF_X64},
    /*  462 */ {I_IN, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+21946, IF_8086|IF_SB},
    /*  463 */ {I_IN, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+20282, IF_8086|IF_SB},
    /*  464 */ {I_IN, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+20287, IF_386|IF_SB},
    /*  465 */ {I_IN, 2, {REG_AL,REG_DX,0,0,0}, nasm_bytecodes+22381, IF_8086},
    /*  466 */ {I_IN, 2, {REG_AX,REG_DX,0,0,0}, nasm_bytecodes+21950, IF_8086},
    /*  467 */ {I_IN, 2, {REG_EAX,REG_DX,0,0,0}, nasm_bytecodes+21954, IF_386},
    /*  468 */ {I_INC, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+21958, IF_8086|IF_NOLONG},
    /*  469 */ {I_INC, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+21962, IF_386|IF_NOLONG},
    /*  470 */ {I_INC, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+20292, IF_8086|IF_LOCK},
    /*  471 */ {I_INC, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+16407, IF_8086|IF_LOCK},
    /*  472 */ {I_INC, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16413, IF_386|IF_LOCK},
    /*  473 */ {I_INC, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+16419, IF_X64|IF_LOCK},
    /*  474 */ {I_INSB, 0, {0,0,0,0,0}, nasm_bytecodes+22384, IF_186},
    /*  475 */ {I_INSD, 0, {0,0,0,0,0}, nasm_bytecodes+21966, IF_386},
    /*  476 */ {I_INSW, 0, {0,0,0,0,0}, nasm_bytecodes+21970, IF_186},
    /*  477 */ {I_INT, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+21974, IF_8086|IF_SB},
    /*  478 */ {I_INT1, 0, {0,0,0,0,0}, nasm_bytecodes+22378, IF_386},
    /*  479 */ {I_INT3, 0, {0,0,0,0,0}, nasm_bytecodes+22387, IF_8086},
    /*  480 */ {I_INTO, 0, {0,0,0,0,0}, nasm_bytecodes+22390, IF_8086|IF_NOLONG},
    /*  481 */ {I_INVD, 0, {0,0,0,0,0}, nasm_bytecodes+21978, IF_486|IF_PRIV},
    /*  482 */ {I_INVPCID, 2, {REG_GPR|BITS32,MEMORY|BITS128,0,0,0}, nasm_bytecodes+7684, IF_FUTURE|IF_INVPCID|IF_PRIV|IF_NOLONG},
    /*  483 */ {I_INVPCID, 2, {REG_GPR|BITS64,MEMORY|BITS128,0,0,0}, nasm_bytecodes+7684, IF_FUTURE|IF_INVPCID|IF_PRIV|IF_LONG},
    /*  484 */ {I_INVLPG, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20297, IF_486|IF_PRIV},
    /*  485 */ {I_INVLPGA, 2, {REG_AX,REG_ECX,0,0,0}, nasm_bytecodes+16425, IF_X86_64|IF_AMD|IF_NOLONG},
    /*  486 */ {I_INVLPGA, 2, {REG_EAX,REG_ECX,0,0,0}, nasm_bytecodes+16431, IF_X86_64|IF_AMD},
    /*  487 */ {I_INVLPGA, 2, {REG_RAX,REG_ECX,0,0,0}, nasm_bytecodes+7691, IF_X64|IF_AMD},
    /*  488 */ {I_INVLPGA, 0, {0,0,0,0,0}, nasm_bytecodes+16432, IF_X86_64|IF_AMD},
    /*  489 */ {I_IRET, 0, {0,0,0,0,0}, nasm_bytecodes+21982, IF_8086},
    /*  490 */ {I_IRETD, 0, {0,0,0,0,0}, nasm_bytecodes+21986, IF_386},
    /*  491 */ {I_IRETQ, 0, {0,0,0,0,0}, nasm_bytecodes+21990, IF_X64},
    /*  492 */ {I_IRETW, 0, {0,0,0,0,0}, nasm_bytecodes+21994, IF_8086},
    /*  493 */ {I_JCXZ, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20302, IF_8086|IF_NOLONG},
    /*  494 */ {I_JECXZ, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20307, IF_386},
    /*  495 */ {I_JRCXZ, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20312, IF_X64},
    /*  496 */ {I_JMP, 1, {IMMEDIATE|SHORT,0,0,0,0}, nasm_bytecodes+20318, IF_8086},
    /*  497 */ {I_JMP, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20322, IF_8086},
    /*  498 */ {I_JMP, 1, {IMMEDIATE|BITS16,0,0,0,0}, nasm_bytecodes+20327, IF_8086|IF_NOLONG},
    /*  499 */ {I_JMP, 1, {IMMEDIATE|BITS32,0,0,0,0}, nasm_bytecodes+20332, IF_386|IF_NOLONG},
    /*  500 */ {I_JMP, 1, {IMMEDIATE|BITS64,0,0,0,0}, nasm_bytecodes+20337, IF_X64},
    /*  501 */ {I_JMP, 2, {IMMEDIATE|COLON,IMMEDIATE,0,0,0}, nasm_bytecodes+16455, IF_8086|IF_NOLONG},
    /*  502 */ {I_JMP, 2, {IMMEDIATE|BITS16|COLON,IMMEDIATE,0,0,0}, nasm_bytecodes+16461, IF_8086|IF_NOLONG},
    /*  503 */ {I_JMP, 2, {IMMEDIATE|COLON,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+16461, IF_8086|IF_NOLONG},
    /*  504 */ {I_JMP, 2, {IMMEDIATE|BITS32|COLON,IMMEDIATE,0,0,0}, nasm_bytecodes+16467, IF_386|IF_NOLONG},
    /*  505 */ {I_JMP, 2, {IMMEDIATE|COLON,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+16467, IF_386|IF_NOLONG},
    /*  506 */ {I_JMP, 1, {MEMORY|FAR,0,0,0,0}, nasm_bytecodes+20342, IF_8086|IF_NOLONG},
    /*  507 */ {I_JMP, 1, {MEMORY|FAR,0,0,0,0}, nasm_bytecodes+20347, IF_X64},
    /*  508 */ {I_JMP, 1, {MEMORY|BITS16|FAR,0,0,0,0}, nasm_bytecodes+20352, IF_8086},
    /*  509 */ {I_JMP, 1, {MEMORY|BITS32|FAR,0,0,0,0}, nasm_bytecodes+20357, IF_386},
    /*  510 */ {I_JMP, 1, {MEMORY|BITS64|FAR,0,0,0,0}, nasm_bytecodes+20347, IF_X64},
    /*  511 */ {I_JMP, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20362, IF_8086},
    /*  512 */ {I_JMP, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20367, IF_8086|IF_NOLONG},
    /*  513 */ {I_JMP, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+20372, IF_386|IF_NOLONG},
    /*  514 */ {I_JMP, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+20377, IF_X64},
    /*  515 */ {I_JMPE, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+16473, IF_IA64},
    /*  516 */ {I_JMPE, 1, {IMMEDIATE|BITS16,0,0,0,0}, nasm_bytecodes+16479, IF_IA64},
    /*  517 */ {I_JMPE, 1, {IMMEDIATE|BITS32,0,0,0,0}, nasm_bytecodes+16485, IF_IA64},
    /*  518 */ {I_JMPE, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+16491, IF_IA64},
    /*  519 */ {I_JMPE, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16497, IF_IA64},
    /*  520 */ {I_LAHF, 0, {0,0,0,0,0}, nasm_bytecodes+22393, IF_8086},
    /*  521 */ {I_LAR, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16503, IF_286|IF_PROT|IF_SW},
    /*  522 */ {I_LAR, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16503, IF_286|IF_PROT},
    /*  523 */ {I_LAR, 2, {REG_GPR|BITS16,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16503, IF_386|IF_PROT},
    /*  524 */ {I_LAR, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+16509, IF_386|IF_PROT|IF_SW},
    /*  525 */ {I_LAR, 2, {REG_GPR|BITS32,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16509, IF_386|IF_PROT},
    /*  526 */ {I_LAR, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16509, IF_386|IF_PROT},
    /*  527 */ {I_LAR, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+16515, IF_X64|IF_PROT|IF_SW},
    /*  528 */ {I_LAR, 2, {REG_GPR|BITS64,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16515, IF_X64|IF_PROT},
    /*  529 */ {I_LAR, 2, {REG_GPR|BITS64,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16515, IF_X64|IF_PROT},
    /*  530 */ {I_LAR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16515, IF_X64|IF_PROT},
    /*  531 */ {I_LDS, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+20382, IF_8086|IF_NOLONG},
    /*  532 */ {I_LDS, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+20387, IF_386|IF_NOLONG},
    /*  533 */ {I_LEA, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+20392, IF_8086},
    /*  534 */ {I_LEA, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+20397, IF_386},
    /*  535 */ {I_LEA, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+20402, IF_X64},
    /*  536 */ {I_LEAVE, 0, {0,0,0,0,0}, nasm_bytecodes+20649, IF_186},
    /*  537 */ {I_LES, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+20407, IF_8086|IF_NOLONG},
    /*  538 */ {I_LES, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+20412, IF_386|IF_NOLONG},
    /*  539 */ {I_LFENCE, 0, {0,0,0,0,0}, nasm_bytecodes+16521, IF_X64|IF_AMD},
    /*  540 */ {I_LFS, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16527, IF_386},
    /*  541 */ {I_LFS, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+16533, IF_386},
    /*  542 */ {I_LFS, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+16539, IF_X64},
    /*  543 */ {I_LGDT, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20417, IF_286|IF_PRIV},
    /*  544 */ {I_LGS, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16545, IF_386},
    /*  545 */ {I_LGS, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+16551, IF_386},
    /*  546 */ {I_LGS, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+16557, IF_X64},
    /*  547 */ {I_LIDT, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20422, IF_286|IF_PRIV},
    /*  548 */ {I_LLDT, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20427, IF_286|IF_PROT|IF_PRIV},
    /*  549 */ {I_LLDT, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+20427, IF_286|IF_PROT|IF_PRIV},
    /*  550 */ {I_LLDT, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20427, IF_286|IF_PROT|IF_PRIV},
    /*  551 */ {I_LMSW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20432, IF_286|IF_PRIV},
    /*  552 */ {I_LMSW, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+20432, IF_286|IF_PRIV},
    /*  553 */ {I_LMSW, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20432, IF_286|IF_PRIV},
    /*  554 */ {I_LOADALL, 0, {0,0,0,0,0}, nasm_bytecodes+21998, IF_386|IF_UNDOC},
    /*  555 */ {I_LOADALL286, 0, {0,0,0,0,0}, nasm_bytecodes+22002, IF_286|IF_UNDOC},
    /*  556 */ {I_LODSB, 0, {0,0,0,0,0}, nasm_bytecodes+22396, IF_8086},
    /*  557 */ {I_LODSD, 0, {0,0,0,0,0}, nasm_bytecodes+22006, IF_386},
    /*  558 */ {I_LODSQ, 0, {0,0,0,0,0}, nasm_bytecodes+22010, IF_X64},
    /*  559 */ {I_LODSW, 0, {0,0,0,0,0}, nasm_bytecodes+22014, IF_8086},
    /*  560 */ {I_LOOP, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20437, IF_8086},
    /*  561 */ {I_LOOP, 2, {IMMEDIATE,REG_CX,0,0,0}, nasm_bytecodes+20442, IF_8086|IF_NOLONG},
    /*  562 */ {I_LOOP, 2, {IMMEDIATE,REG_ECX,0,0,0}, nasm_bytecodes+20447, IF_386},
    /*  563 */ {I_LOOP, 2, {IMMEDIATE,REG_RCX,0,0,0}, nasm_bytecodes+20452, IF_X64},
    /*  564 */ {I_LOOPE, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20457, IF_8086},
    /*  565 */ {I_LOOPE, 2, {IMMEDIATE,REG_CX,0,0,0}, nasm_bytecodes+20462, IF_8086|IF_NOLONG},
    /*  566 */ {I_LOOPE, 2, {IMMEDIATE,REG_ECX,0,0,0}, nasm_bytecodes+20467, IF_386},
    /*  567 */ {I_LOOPE, 2, {IMMEDIATE,REG_RCX,0,0,0}, nasm_bytecodes+20472, IF_X64},
    /*  568 */ {I_LOOPNE, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20477, IF_8086},
    /*  569 */ {I_LOOPNE, 2, {IMMEDIATE,REG_CX,0,0,0}, nasm_bytecodes+20482, IF_8086|IF_NOLONG},
    /*  570 */ {I_LOOPNE, 2, {IMMEDIATE,REG_ECX,0,0,0}, nasm_bytecodes+20487, IF_386},
    /*  571 */ {I_LOOPNE, 2, {IMMEDIATE,REG_RCX,0,0,0}, nasm_bytecodes+20492, IF_X64},
    /*  572 */ {I_LOOPNZ, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20477, IF_8086},
    /*  573 */ {I_LOOPNZ, 2, {IMMEDIATE,REG_CX,0,0,0}, nasm_bytecodes+20482, IF_8086|IF_NOLONG},
    /*  574 */ {I_LOOPNZ, 2, {IMMEDIATE,REG_ECX,0,0,0}, nasm_bytecodes+20487, IF_386},
    /*  575 */ {I_LOOPNZ, 2, {IMMEDIATE,REG_RCX,0,0,0}, nasm_bytecodes+20492, IF_X64},
    /*  576 */ {I_LOOPZ, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+20457, IF_8086},
    /*  577 */ {I_LOOPZ, 2, {IMMEDIATE,REG_CX,0,0,0}, nasm_bytecodes+20462, IF_8086|IF_NOLONG},
    /*  578 */ {I_LOOPZ, 2, {IMMEDIATE,REG_ECX,0,0,0}, nasm_bytecodes+20467, IF_386},
    /*  579 */ {I_LOOPZ, 2, {IMMEDIATE,REG_RCX,0,0,0}, nasm_bytecodes+20472, IF_X64},
    /*  580 */ {I_LSL, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16563, IF_286|IF_PROT|IF_SW},
    /*  581 */ {I_LSL, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16563, IF_286|IF_PROT},
    /*  582 */ {I_LSL, 2, {REG_GPR|BITS16,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16563, IF_386|IF_PROT},
    /*  583 */ {I_LSL, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+16569, IF_386|IF_PROT|IF_SW},
    /*  584 */ {I_LSL, 2, {REG_GPR|BITS32,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16569, IF_386|IF_PROT},
    /*  585 */ {I_LSL, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16569, IF_386|IF_PROT},
    /*  586 */ {I_LSL, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+16575, IF_X64|IF_PROT|IF_SW},
    /*  587 */ {I_LSL, 2, {REG_GPR|BITS64,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16575, IF_X64|IF_PROT},
    /*  588 */ {I_LSL, 2, {REG_GPR|BITS64,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16575, IF_X64|IF_PROT},
    /*  589 */ {I_LSL, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16575, IF_X64|IF_PROT},
    /*  590 */ {I_LSS, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16581, IF_386},
    /*  591 */ {I_LSS, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+16587, IF_386},
    /*  592 */ {I_LSS, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+16593, IF_X64},
    /*  593 */ {I_LTR, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20497, IF_286|IF_PROT|IF_PRIV},
    /*  594 */ {I_LTR, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+20497, IF_286|IF_PROT|IF_PRIV},
    /*  595 */ {I_LTR, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20497, IF_286|IF_PROT|IF_PRIV},
    /*  596 */ {I_MFENCE, 0, {0,0,0,0,0}, nasm_bytecodes+16599, IF_X64|IF_AMD},
    /*  597 */ {I_MONITOR, 0, {0,0,0,0,0}, nasm_bytecodes+20502, IF_PRESCOTT},
    /*  598 */ {I_MOV, 2, {MEMORY,REG_SREG,0,0,0}, nasm_bytecodes+20523, IF_8086|IF_SW},
    /*  599 */ {I_MOV, 2, {REG_GPR|BITS16,REG_SREG,0,0,0}, nasm_bytecodes+20507, IF_8086},
    /*  600 */ {I_MOV, 2, {REG_GPR|BITS32,REG_SREG,0,0,0}, nasm_bytecodes+20512, IF_386},
    /*  601 */ {I_MOV, 2, {RM_GPR|BITS64,REG_SREG,0,0,0}, nasm_bytecodes+20522, IF_X64},
    /*  602 */ {I_MOV, 2, {REG_SREG,MEMORY,0,0,0}, nasm_bytecodes+20543, IF_8086|IF_SW},
    /*  603 */ {I_MOV, 2, {REG_SREG,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+20532, IF_8086},
    /*  604 */ {I_MOV, 2, {REG_SREG,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+20537, IF_386},
    /*  605 */ {I_MOV, 2, {REG_SREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+20542, IF_X64},
    /*  606 */ {I_MOV, 2, {REG_AL,MEM_OFFS,0,0,0}, nasm_bytecodes+22018, IF_8086|IF_SM},
    /*  607 */ {I_MOV, 2, {REG_AX,MEM_OFFS,0,0,0}, nasm_bytecodes+20547, IF_8086|IF_SM},
    /*  608 */ {I_MOV, 2, {REG_EAX,MEM_OFFS,0,0,0}, nasm_bytecodes+20552, IF_386|IF_SM},
    /*  609 */ {I_MOV, 2, {REG_RAX,MEM_OFFS,0,0,0}, nasm_bytecodes+20557, IF_X64|IF_SM},
    /*  610 */ {I_MOV, 2, {MEM_OFFS,REG_AL,0,0,0}, nasm_bytecodes+22022, IF_8086|IF_SM|IF_NOHLE},
    /*  611 */ {I_MOV, 2, {MEM_OFFS,REG_AX,0,0,0}, nasm_bytecodes+20562, IF_8086|IF_SM|IF_NOHLE},
    /*  612 */ {I_MOV, 2, {MEM_OFFS,REG_EAX,0,0,0}, nasm_bytecodes+20567, IF_386|IF_SM|IF_NOHLE},
    /*  613 */ {I_MOV, 2, {MEM_OFFS,REG_RAX,0,0,0}, nasm_bytecodes+20572, IF_X64|IF_SM|IF_NOHLE},
    /*  614 */ {I_MOV, 2, {REG_GPR|BITS32,REG_CREG,0,0,0}, nasm_bytecodes+16605, IF_386|IF_PRIV|IF_NOLONG},
    /*  615 */ {I_MOV, 2, {REG_GPR|BITS64,REG_CREG,0,0,0}, nasm_bytecodes+16611, IF_X64|IF_PRIV},
    /*  616 */ {I_MOV, 2, {REG_CREG,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16617, IF_386|IF_PRIV|IF_NOLONG},
    /*  617 */ {I_MOV, 2, {REG_CREG,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16623, IF_X64|IF_PRIV},
    /*  618 */ {I_MOV, 2, {REG_GPR|BITS32,REG_DREG,0,0,0}, nasm_bytecodes+16630, IF_386|IF_PRIV|IF_NOLONG},
    /*  619 */ {I_MOV, 2, {REG_GPR|BITS64,REG_DREG,0,0,0}, nasm_bytecodes+16629, IF_X64|IF_PRIV},
    /*  620 */ {I_MOV, 2, {REG_DREG,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16636, IF_386|IF_PRIV|IF_NOLONG},
    /*  621 */ {I_MOV, 2, {REG_DREG,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16635, IF_X64|IF_PRIV},
    /*  622 */ {I_MOV, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+20587, IF_8086|IF_SM},
    /*  623 */ {I_MOV, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+20588, IF_8086},
    /*  624 */ {I_MOV, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16641, IF_8086|IF_SM},
    /*  625 */ {I_MOV, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16642, IF_8086},
    /*  626 */ {I_MOV, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16647, IF_386|IF_SM},
    /*  627 */ {I_MOV, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16648, IF_386},
    /*  628 */ {I_MOV, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16653, IF_X64|IF_SM},
    /*  629 */ {I_MOV, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16654, IF_X64},
    /*  630 */ {I_MOV, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+22026, IF_8086|IF_SM},
    /*  631 */ {I_MOV, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+22026, IF_8086},
    /*  632 */ {I_MOV, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+20592, IF_8086|IF_SM},
    /*  633 */ {I_MOV, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+20592, IF_8086},
    /*  634 */ {I_MOV, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+20597, IF_386|IF_SM},
    /*  635 */ {I_MOV, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+20597, IF_386},
    /*  636 */ {I_MOV, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+20602, IF_X64|IF_SM},
    /*  637 */ {I_MOV, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+20602, IF_X64},
    /*  638 */ {I_MOV, 2, {REG_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+22030, IF_8086|IF_SM},
    /*  639 */ {I_MOV, 2, {REG_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+20607, IF_8086|IF_SM},
    /*  640 */ {I_MOV, 2, {REG_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+20612, IF_386|IF_SM},
    /*  641 */ {I_MOV, 2, {REG_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+20622, IF_X64|IF_SM},
    /*  642 */ {I_MOV, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+16659, IF_8086|IF_SM},
    /*  643 */ {I_MOV, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+7726, IF_8086|IF_SM},
    /*  644 */ {I_MOV, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+7733, IF_386|IF_SM},
    /*  645 */ {I_MOV, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+7740, IF_X64|IF_SM},
    /*  646 */ {I_MOV, 2, {RM_GPR|BITS64,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+7740, IF_X64},
    /*  647 */ {I_MOV, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16659, IF_8086|IF_SM},
    /*  648 */ {I_MOV, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+7726, IF_8086|IF_SM},
    /*  649 */ {I_MOV, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+7733, IF_386|IF_SM},
    /*  650 */ {I_MOVD, 2, {MMXREG,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+16665, IF_PENT|IF_MMX|IF_SD},
    /*  651 */ {I_MOVD, 2, {RM_GPR|BITS32,MMXREG,0,0,0}, nasm_bytecodes+16671, IF_PENT|IF_MMX|IF_SD},
    /*  652 */ {I_MOVQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+16677, IF_PENT|IF_MMX|IF_SQ},
    /*  653 */ {I_MOVQ, 2, {RM_MMX,MMXREG,0,0,0}, nasm_bytecodes+16683, IF_PENT|IF_MMX|IF_SQ},
    /*  654 */ {I_MOVQ, 2, {MMXREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+7747, IF_X64|IF_MMX},
    /*  655 */ {I_MOVQ, 2, {RM_GPR|BITS64,MMXREG,0,0,0}, nasm_bytecodes+7754, IF_X64|IF_MMX},
    /*  656 */ {I_MOVSB, 0, {0,0,0,0,0}, nasm_bytecodes+3769, IF_8086},
    /*  657 */ {I_MOVSD, 0, {0,0,0,0,0}, nasm_bytecodes+22034, IF_386},
    /*  658 */ {I_MOVSQ, 0, {0,0,0,0,0}, nasm_bytecodes+22038, IF_X64},
    /*  659 */ {I_MOVSW, 0, {0,0,0,0,0}, nasm_bytecodes+22042, IF_8086},
    /*  660 */ {I_MOVSX, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16689, IF_386|IF_SB},
    /*  661 */ {I_MOVSX, 2, {REG_GPR|BITS16,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+16689, IF_386},
    /*  662 */ {I_MOVSX, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, nasm_bytecodes+16695, IF_386},
    /*  663 */ {I_MOVSX, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+16701, IF_386},
    /*  664 */ {I_MOVSX, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, nasm_bytecodes+16707, IF_X64},
    /*  665 */ {I_MOVSX, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+16713, IF_X64},
    /*  666 */ {I_MOVSXD, 2, {REG_GPR|BITS64,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+20627, IF_X64},
    /*  667 */ {I_MOVZX, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+16719, IF_386|IF_SB},
    /*  668 */ {I_MOVZX, 2, {REG_GPR|BITS16,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+16719, IF_386},
    /*  669 */ {I_MOVZX, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, nasm_bytecodes+16725, IF_386},
    /*  670 */ {I_MOVZX, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+16731, IF_386},
    /*  671 */ {I_MOVZX, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, nasm_bytecodes+16737, IF_X64},
    /*  672 */ {I_MOVZX, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+16743, IF_X64},
    /*  673 */ {I_MUL, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+22046, IF_8086},
    /*  674 */ {I_MUL, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20632, IF_8086},
    /*  675 */ {I_MUL, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+20637, IF_386},
    /*  676 */ {I_MUL, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+20642, IF_X64},
    /*  677 */ {I_MWAIT, 0, {0,0,0,0,0}, nasm_bytecodes+20647, IF_PRESCOTT},
    /*  678 */ {I_NEG, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+20652, IF_8086|IF_LOCK},
    /*  679 */ {I_NEG, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+16749, IF_8086|IF_LOCK},
    /*  680 */ {I_NEG, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16755, IF_386|IF_LOCK},
    /*  681 */ {I_NEG, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+16761, IF_X64|IF_LOCK},
    /*  682 */ {I_NOP, 0, {0,0,0,0,0}, nasm_bytecodes+20657, IF_8086},
    /*  683 */ {I_NOP, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+16767, IF_P6},
    /*  684 */ {I_NOP, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16773, IF_P6},
    /*  685 */ {I_NOP, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+16779, IF_X64},
    /*  686 */ {I_NOT, 1, {RM_GPR|BITS8,0,0,0,0}, nasm_bytecodes+20662, IF_8086|IF_LOCK},
    /*  687 */ {I_NOT, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+16785, IF_8086|IF_LOCK},
    /*  688 */ {I_NOT, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16791, IF_386|IF_LOCK},
    /*  689 */ {I_NOT, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+16797, IF_X64|IF_LOCK},
    /*  690 */ {I_OR, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+20667, IF_8086|IF_SM|IF_LOCK},
    /*  691 */ {I_OR, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+20668, IF_8086},
    /*  692 */ {I_OR, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16803, IF_8086|IF_SM|IF_LOCK},
    /*  693 */ {I_OR, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16804, IF_8086},
    /*  694 */ {I_OR, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16809, IF_386|IF_SM|IF_LOCK},
    /*  695 */ {I_OR, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16810, IF_386},
    /*  696 */ {I_OR, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16815, IF_X64|IF_SM|IF_LOCK},
    /*  697 */ {I_OR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16816, IF_X64},
    /*  698 */ {I_OR, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+15058, IF_8086|IF_SM},
    /*  699 */ {I_OR, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+15058, IF_8086},
    /*  700 */ {I_OR, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+20672, IF_8086|IF_SM},
    /*  701 */ {I_OR, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+20672, IF_8086},
    /*  702 */ {I_OR, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+20677, IF_386|IF_SM},
    /*  703 */ {I_OR, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+20677, IF_386},
    /*  704 */ {I_OR, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+20682, IF_X64|IF_SM},
    /*  705 */ {I_OR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+20682, IF_X64},
    /*  706 */ {I_OR, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7761, IF_8086|IF_LOCK},
    /*  707 */ {I_OR, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7768, IF_386|IF_LOCK},
    /*  708 */ {I_OR, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7775, IF_X64|IF_LOCK},
    /*  709 */ {I_OR, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+22050, IF_8086|IF_SM},
    /*  710 */ {I_OR, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+20687, IF_8086|IF_SM},
    /*  711 */ {I_OR, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+20692, IF_386|IF_SM},
    /*  712 */ {I_OR, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+20697, IF_X64|IF_SM},
    /*  713 */ {I_OR, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+16821, IF_8086|IF_SM|IF_LOCK},
    /*  714 */ {I_OR, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+7782, IF_8086|IF_SM|IF_LOCK},
    /*  715 */ {I_OR, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+7789, IF_386|IF_SM|IF_LOCK},
    /*  716 */ {I_OR, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+7796, IF_X64|IF_SM|IF_LOCK},
    /*  717 */ {I_OR, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16821, IF_8086|IF_SM|IF_LOCK},
    /*  718 */ {I_OR, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+7782, IF_8086|IF_SM|IF_LOCK},
    /*  719 */ {I_OR, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+7789, IF_386|IF_SM|IF_LOCK},
    /*  720 */ {I_OUT, 2, {IMMEDIATE,REG_AL,0,0,0}, nasm_bytecodes+22054, IF_8086|IF_SB},
    /*  721 */ {I_OUT, 2, {IMMEDIATE,REG_AX,0,0,0}, nasm_bytecodes+20702, IF_8086|IF_SB},
    /*  722 */ {I_OUT, 2, {IMMEDIATE,REG_EAX,0,0,0}, nasm_bytecodes+20707, IF_386|IF_SB},
    /*  723 */ {I_OUT, 2, {REG_DX,REG_AL,0,0,0}, nasm_bytecodes+22399, IF_8086},
    /*  724 */ {I_OUT, 2, {REG_DX,REG_AX,0,0,0}, nasm_bytecodes+22058, IF_8086},
    /*  725 */ {I_OUT, 2, {REG_DX,REG_EAX,0,0,0}, nasm_bytecodes+22062, IF_386},
    /*  726 */ {I_OUTSB, 0, {0,0,0,0,0}, nasm_bytecodes+22402, IF_186},
    /*  727 */ {I_OUTSD, 0, {0,0,0,0,0}, nasm_bytecodes+22066, IF_386},
    /*  728 */ {I_OUTSW, 0, {0,0,0,0,0}, nasm_bytecodes+22070, IF_186},
    /*  729 */ {I_PACKSSDW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7803, IF_PENT|IF_MMX|IF_SQ},
    /*  730 */ {I_PACKSSWB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7810, IF_PENT|IF_MMX|IF_SQ},
    /*  731 */ {I_PACKUSWB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7817, IF_PENT|IF_MMX|IF_SQ},
    /*  732 */ {I_PADDB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7824, IF_PENT|IF_MMX|IF_SQ},
    /*  733 */ {I_PADDD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7831, IF_PENT|IF_MMX|IF_SQ},
    /*  734 */ {I_PADDSB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7838, IF_PENT|IF_MMX|IF_SQ},
    /*  735 */ {I_PADDSIW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+16833, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  736 */ {I_PADDSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7845, IF_PENT|IF_MMX|IF_SQ},
    /*  737 */ {I_PADDUSB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7852, IF_PENT|IF_MMX|IF_SQ},
    /*  738 */ {I_PADDUSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7859, IF_PENT|IF_MMX|IF_SQ},
    /*  739 */ {I_PADDW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7866, IF_PENT|IF_MMX|IF_SQ},
    /*  740 */ {I_PAND, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7873, IF_PENT|IF_MMX|IF_SQ},
    /*  741 */ {I_PANDN, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7880, IF_PENT|IF_MMX|IF_SQ},
    /*  742 */ {I_PAUSE, 0, {0,0,0,0,0}, nasm_bytecodes+22074, IF_8086},
    /*  743 */ {I_PAVEB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+16839, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  744 */ {I_PAVGUSB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3708, IF_PENT|IF_3DNOW|IF_SQ},
    /*  745 */ {I_PCMPEQB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7887, IF_PENT|IF_MMX|IF_SQ},
    /*  746 */ {I_PCMPEQD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7894, IF_PENT|IF_MMX|IF_SQ},
    /*  747 */ {I_PCMPEQW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7901, IF_PENT|IF_MMX|IF_SQ},
    /*  748 */ {I_PCMPGTB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7908, IF_PENT|IF_MMX|IF_SQ},
    /*  749 */ {I_PCMPGTD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7915, IF_PENT|IF_MMX|IF_SQ},
    /*  750 */ {I_PCMPGTW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7922, IF_PENT|IF_MMX|IF_SQ},
    /*  751 */ {I_PDISTIB, 2, {MMXREG,MEMORY,0,0,0}, nasm_bytecodes+17998, IF_PENT|IF_MMX|IF_SM|IF_CYRIX},
    /*  752 */ {I_PF2ID, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3716, IF_PENT|IF_3DNOW|IF_SQ},
    /*  753 */ {I_PFACC, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3724, IF_PENT|IF_3DNOW|IF_SQ},
    /*  754 */ {I_PFADD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3732, IF_PENT|IF_3DNOW|IF_SQ},
    /*  755 */ {I_PFCMPEQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3740, IF_PENT|IF_3DNOW|IF_SQ},
    /*  756 */ {I_PFCMPGE, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3748, IF_PENT|IF_3DNOW|IF_SQ},
    /*  757 */ {I_PFCMPGT, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3756, IF_PENT|IF_3DNOW|IF_SQ},
    /*  758 */ {I_PFMAX, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3764, IF_PENT|IF_3DNOW|IF_SQ},
    /*  759 */ {I_PFMIN, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3772, IF_PENT|IF_3DNOW|IF_SQ},
    /*  760 */ {I_PFMUL, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3780, IF_PENT|IF_3DNOW|IF_SQ},
    /*  761 */ {I_PFRCP, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3788, IF_PENT|IF_3DNOW|IF_SQ},
    /*  762 */ {I_PFRCPIT1, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3796, IF_PENT|IF_3DNOW|IF_SQ},
    /*  763 */ {I_PFRCPIT2, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3804, IF_PENT|IF_3DNOW|IF_SQ},
    /*  764 */ {I_PFRSQIT1, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3812, IF_PENT|IF_3DNOW|IF_SQ},
    /*  765 */ {I_PFRSQRT, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3820, IF_PENT|IF_3DNOW|IF_SQ},
    /*  766 */ {I_PFSUB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3828, IF_PENT|IF_3DNOW|IF_SQ},
    /*  767 */ {I_PFSUBR, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3836, IF_PENT|IF_3DNOW|IF_SQ},
    /*  768 */ {I_PI2FD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3844, IF_PENT|IF_3DNOW|IF_SQ},
    /*  769 */ {I_PMACHRIW, 2, {MMXREG,MEMORY,0,0,0}, nasm_bytecodes+18094, IF_PENT|IF_MMX|IF_SM|IF_CYRIX},
    /*  770 */ {I_PMADDWD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7929, IF_PENT|IF_MMX|IF_SQ},
    /*  771 */ {I_PMAGW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+16845, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  772 */ {I_PMULHRIW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+16851, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  773 */ {I_PMULHRWA, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3852, IF_PENT|IF_3DNOW|IF_SQ},
    /*  774 */ {I_PMULHRWC, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+16857, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  775 */ {I_PMULHW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7936, IF_PENT|IF_MMX|IF_SQ},
    /*  776 */ {I_PMULLW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7943, IF_PENT|IF_MMX|IF_SQ},
    /*  777 */ {I_PMVGEZB, 2, {MMXREG,MEMORY,0,0,0}, nasm_bytecodes+18226, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  778 */ {I_PMVLZB, 2, {MMXREG,MEMORY,0,0,0}, nasm_bytecodes+18082, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  779 */ {I_PMVNZB, 2, {MMXREG,MEMORY,0,0,0}, nasm_bytecodes+18064, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  780 */ {I_PMVZB, 2, {MMXREG,MEMORY,0,0,0}, nasm_bytecodes+17986, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  781 */ {I_POP, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+22078, IF_8086},
    /*  782 */ {I_POP, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+22082, IF_386|IF_NOLONG},
    /*  783 */ {I_POP, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+22086, IF_X64},
    /*  784 */ {I_POP, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20712, IF_8086},
    /*  785 */ {I_POP, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+20717, IF_386|IF_NOLONG},
    /*  786 */ {I_POP, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+20722, IF_X64},
    /*  787 */ {I_POP, 1, {REG_ES,0,0,0,0}, nasm_bytecodes+4145, IF_8086|IF_NOLONG},
    /*  788 */ {I_POP, 1, {REG_SS,0,0,0,0}, nasm_bytecodes+3381, IF_8086|IF_NOLONG},
    /*  789 */ {I_POP, 1, {REG_DS,0,0,0,0}, nasm_bytecodes+3525, IF_8086|IF_NOLONG},
    /*  790 */ {I_POP, 1, {REG_FS,0,0,0,0}, nasm_bytecodes+22090, IF_386},
    /*  791 */ {I_POP, 1, {REG_GS,0,0,0,0}, nasm_bytecodes+22094, IF_386},
    /*  792 */ {I_POPA, 0, {0,0,0,0,0}, nasm_bytecodes+22098, IF_186|IF_NOLONG},
    /*  793 */ {I_POPAD, 0, {0,0,0,0,0}, nasm_bytecodes+22102, IF_386|IF_NOLONG},
    /*  794 */ {I_POPAW, 0, {0,0,0,0,0}, nasm_bytecodes+22106, IF_186|IF_NOLONG},
    /*  795 */ {I_POPF, 0, {0,0,0,0,0}, nasm_bytecodes+22110, IF_8086},
    /*  796 */ {I_POPFD, 0, {0,0,0,0,0}, nasm_bytecodes+22114, IF_386|IF_NOLONG},
    /*  797 */ {I_POPFQ, 0, {0,0,0,0,0}, nasm_bytecodes+22114, IF_X64},
    /*  798 */ {I_POPFW, 0, {0,0,0,0,0}, nasm_bytecodes+22118, IF_8086},
    /*  799 */ {I_POR, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7950, IF_PENT|IF_MMX|IF_SQ},
    /*  800 */ {I_PREFETCH, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20727, IF_PENT|IF_3DNOW|IF_SQ},
    /*  801 */ {I_PREFETCHW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+20732, IF_PENT|IF_3DNOW|IF_SQ},
    /*  802 */ {I_PSLLD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7957, IF_PENT|IF_MMX|IF_SQ},
    /*  803 */ {I_PSLLD, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+7964, IF_PENT|IF_MMX},
    /*  804 */ {I_PSLLQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7971, IF_PENT|IF_MMX|IF_SQ},
    /*  805 */ {I_PSLLQ, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+7978, IF_PENT|IF_MMX},
    /*  806 */ {I_PSLLW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7985, IF_PENT|IF_MMX|IF_SQ},
    /*  807 */ {I_PSLLW, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+7992, IF_PENT|IF_MMX},
    /*  808 */ {I_PSRAD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+7999, IF_PENT|IF_MMX|IF_SQ},
    /*  809 */ {I_PSRAD, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8006, IF_PENT|IF_MMX},
    /*  810 */ {I_PSRAW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8013, IF_PENT|IF_MMX|IF_SQ},
    /*  811 */ {I_PSRAW, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8020, IF_PENT|IF_MMX},
    /*  812 */ {I_PSRLD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8027, IF_PENT|IF_MMX|IF_SQ},
    /*  813 */ {I_PSRLD, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8034, IF_PENT|IF_MMX},
    /*  814 */ {I_PSRLQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8041, IF_PENT|IF_MMX|IF_SQ},
    /*  815 */ {I_PSRLQ, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8048, IF_PENT|IF_MMX},
    /*  816 */ {I_PSRLW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8055, IF_PENT|IF_MMX|IF_SQ},
    /*  817 */ {I_PSRLW, 2, {MMXREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8062, IF_PENT|IF_MMX},
    /*  818 */ {I_PSUBB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8069, IF_PENT|IF_MMX|IF_SQ},
    /*  819 */ {I_PSUBD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8076, IF_PENT|IF_MMX|IF_SQ},
    /*  820 */ {I_PSUBSB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8083, IF_PENT|IF_MMX|IF_SQ},
    /*  821 */ {I_PSUBSIW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+16863, IF_PENT|IF_MMX|IF_SQ|IF_CYRIX},
    /*  822 */ {I_PSUBSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8090, IF_PENT|IF_MMX|IF_SQ},
    /*  823 */ {I_PSUBUSB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8097, IF_PENT|IF_MMX|IF_SQ},
    /*  824 */ {I_PSUBUSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8104, IF_PENT|IF_MMX|IF_SQ},
    /*  825 */ {I_PSUBW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8111, IF_PENT|IF_MMX|IF_SQ},
    /*  826 */ {I_PUNPCKHBW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8118, IF_PENT|IF_MMX|IF_SQ},
    /*  827 */ {I_PUNPCKHDQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8125, IF_PENT|IF_MMX|IF_SQ},
    /*  828 */ {I_PUNPCKHWD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8132, IF_PENT|IF_MMX|IF_SQ},
    /*  829 */ {I_PUNPCKLBW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8139, IF_PENT|IF_MMX|IF_SQ},
    /*  830 */ {I_PUNPCKLDQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8146, IF_PENT|IF_MMX|IF_SQ},
    /*  831 */ {I_PUNPCKLWD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8153, IF_PENT|IF_MMX|IF_SQ},
    /*  832 */ {I_PUSH, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+22122, IF_8086},
    /*  833 */ {I_PUSH, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+22126, IF_386|IF_NOLONG},
    /*  834 */ {I_PUSH, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+22130, IF_X64},
    /*  835 */ {I_PUSH, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+20737, IF_8086},
    /*  836 */ {I_PUSH, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+20742, IF_386|IF_NOLONG},
    /*  837 */ {I_PUSH, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+20747, IF_X64},
    /*  838 */ {I_PUSH, 1, {REG_ES,0,0,0,0}, nasm_bytecodes+4113, IF_8086|IF_NOLONG},
    /*  839 */ {I_PUSH, 1, {REG_CS,0,0,0,0}, nasm_bytecodes+3237, IF_8086|IF_NOLONG},
    /*  840 */ {I_PUSH, 1, {REG_SS,0,0,0,0}, nasm_bytecodes+3363, IF_8086|IF_NOLONG},
    /*  841 */ {I_PUSH, 1, {REG_DS,0,0,0,0}, nasm_bytecodes+3507, IF_8086|IF_NOLONG},
    /*  842 */ {I_PUSH, 1, {REG_FS,0,0,0,0}, nasm_bytecodes+22134, IF_386},
    /*  843 */ {I_PUSH, 1, {REG_GS,0,0,0,0}, nasm_bytecodes+22138, IF_386},
    /*  844 */ {I_PUSH, 1, {IMMEDIATE|BITS8,0,0,0,0}, nasm_bytecodes+20773, IF_186},
    /*  845 */ {I_PUSH, 1, {IMMEDIATE|BITS16,0,0,0,0}, nasm_bytecodes+20757, IF_186|IF_AR0|IF_SZ},
    /*  846 */ {I_PUSH, 1, {IMMEDIATE|BITS32,0,0,0,0}, nasm_bytecodes+20767, IF_386|IF_NOLONG|IF_AR0|IF_SZ},
    /*  847 */ {I_PUSH, 1, {IMMEDIATE|BITS32,0,0,0,0}, nasm_bytecodes+20767, IF_386|IF_NOLONG|IF_SD},
    /*  848 */ {I_PUSH, 1, {IMMEDIATE|BITS64,0,0,0,0}, nasm_bytecodes+20777, IF_X64|IF_AR0|IF_SZ},
    /*  849 */ {I_PUSH, 1, {IMMEDIATE|BITS32,0,0,0,0}, nasm_bytecodes+20777, IF_X64|IF_AR0|IF_SZ},
    /*  850 */ {I_PUSHA, 0, {0,0,0,0,0}, nasm_bytecodes+22142, IF_186|IF_NOLONG},
    /*  851 */ {I_PUSHAD, 0, {0,0,0,0,0}, nasm_bytecodes+22146, IF_386|IF_NOLONG},
    /*  852 */ {I_PUSHAW, 0, {0,0,0,0,0}, nasm_bytecodes+22150, IF_186|IF_NOLONG},
    /*  853 */ {I_PUSHF, 0, {0,0,0,0,0}, nasm_bytecodes+22154, IF_8086},
    /*  854 */ {I_PUSHFD, 0, {0,0,0,0,0}, nasm_bytecodes+22158, IF_386|IF_NOLONG},
    /*  855 */ {I_PUSHFQ, 0, {0,0,0,0,0}, nasm_bytecodes+22158, IF_X64},
    /*  856 */ {I_PUSHFW, 0, {0,0,0,0,0}, nasm_bytecodes+22162, IF_8086},
    /*  857 */ {I_PXOR, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8160, IF_PENT|IF_MMX|IF_SQ},
    /*  858 */ {I_RCL, 2, {RM_GPR|BITS8,UNITY,0,0,0}, nasm_bytecodes+22166, IF_8086},
    /*  859 */ {I_RCL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, nasm_bytecodes+22170, IF_8086},
    /*  860 */ {I_RCL, 2, {RM_GPR|BITS8,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+20782, IF_186},
    /*  861 */ {I_RCL, 2, {RM_GPR|BITS16,UNITY,0,0,0}, nasm_bytecodes+20787, IF_8086},
    /*  862 */ {I_RCL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, nasm_bytecodes+20792, IF_8086},
    /*  863 */ {I_RCL, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16869, IF_186},
    /*  864 */ {I_RCL, 2, {RM_GPR|BITS32,UNITY,0,0,0}, nasm_bytecodes+20797, IF_386},
    /*  865 */ {I_RCL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, nasm_bytecodes+20802, IF_386},
    /*  866 */ {I_RCL, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16875, IF_386},
    /*  867 */ {I_RCL, 2, {RM_GPR|BITS64,UNITY,0,0,0}, nasm_bytecodes+20807, IF_X64},
    /*  868 */ {I_RCL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, nasm_bytecodes+20812, IF_X64},
    /*  869 */ {I_RCL, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16881, IF_X64},
    /*  870 */ {I_RCR, 2, {RM_GPR|BITS8,UNITY,0,0,0}, nasm_bytecodes+22174, IF_8086},
    /*  871 */ {I_RCR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, nasm_bytecodes+22178, IF_8086},
    /*  872 */ {I_RCR, 2, {RM_GPR|BITS8,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+20817, IF_186},
    /*  873 */ {I_RCR, 2, {RM_GPR|BITS16,UNITY,0,0,0}, nasm_bytecodes+20822, IF_8086},
    /*  874 */ {I_RCR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, nasm_bytecodes+20827, IF_8086},
    /*  875 */ {I_RCR, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16887, IF_186},
    /*  876 */ {I_RCR, 2, {RM_GPR|BITS32,UNITY,0,0,0}, nasm_bytecodes+20832, IF_386},
    /*  877 */ {I_RCR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, nasm_bytecodes+20837, IF_386},
    /*  878 */ {I_RCR, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16893, IF_386},
    /*  879 */ {I_RCR, 2, {RM_GPR|BITS64,UNITY,0,0,0}, nasm_bytecodes+20842, IF_X64},
    /*  880 */ {I_RCR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, nasm_bytecodes+20847, IF_X64},
    /*  881 */ {I_RCR, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16899, IF_X64},
    /*  882 */ {I_RDSHR, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16905, IF_P6|IF_CYRIX|IF_SMM},
    /*  883 */ {I_RDMSR, 0, {0,0,0,0,0}, nasm_bytecodes+22182, IF_PENT|IF_PRIV},
    /*  884 */ {I_RDPMC, 0, {0,0,0,0,0}, nasm_bytecodes+22186, IF_P6},
    /*  885 */ {I_RDTSC, 0, {0,0,0,0,0}, nasm_bytecodes+22190, IF_PENT},
    /*  886 */ {I_RDTSCP, 0, {0,0,0,0,0}, nasm_bytecodes+20852, IF_X86_64},
    /*  887 */ {I_RET, 0, {0,0,0,0,0}, nasm_bytecodes+21314, IF_8086},
    /*  888 */ {I_RET, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+22194, IF_8086|IF_SW},
    /*  889 */ {I_RETF, 0, {0,0,0,0,0}, nasm_bytecodes+21339, IF_8086},
    /*  890 */ {I_RETF, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+22198, IF_8086|IF_SW},
    /*  891 */ {I_RETN, 0, {0,0,0,0,0}, nasm_bytecodes+21314, IF_8086},
    /*  892 */ {I_RETN, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+22194, IF_8086|IF_SW},
    /*  893 */ {I_ROL, 2, {RM_GPR|BITS8,UNITY,0,0,0}, nasm_bytecodes+22202, IF_8086},
    /*  894 */ {I_ROL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, nasm_bytecodes+22206, IF_8086},
    /*  895 */ {I_ROL, 2, {RM_GPR|BITS8,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+20857, IF_186},
    /*  896 */ {I_ROL, 2, {RM_GPR|BITS16,UNITY,0,0,0}, nasm_bytecodes+20862, IF_8086},
    /*  897 */ {I_ROL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, nasm_bytecodes+20867, IF_8086},
    /*  898 */ {I_ROL, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16911, IF_186},
    /*  899 */ {I_ROL, 2, {RM_GPR|BITS32,UNITY,0,0,0}, nasm_bytecodes+20872, IF_386},
    /*  900 */ {I_ROL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, nasm_bytecodes+20877, IF_386},
    /*  901 */ {I_ROL, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16917, IF_386},
    /*  902 */ {I_ROL, 2, {RM_GPR|BITS64,UNITY,0,0,0}, nasm_bytecodes+20882, IF_X64},
    /*  903 */ {I_ROL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, nasm_bytecodes+20887, IF_X64},
    /*  904 */ {I_ROL, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16923, IF_X64},
    /*  905 */ {I_ROR, 2, {RM_GPR|BITS8,UNITY,0,0,0}, nasm_bytecodes+22210, IF_8086},
    /*  906 */ {I_ROR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, nasm_bytecodes+22214, IF_8086},
    /*  907 */ {I_ROR, 2, {RM_GPR|BITS8,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+20892, IF_186},
    /*  908 */ {I_ROR, 2, {RM_GPR|BITS16,UNITY,0,0,0}, nasm_bytecodes+20897, IF_8086},
    /*  909 */ {I_ROR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, nasm_bytecodes+20902, IF_8086},
    /*  910 */ {I_ROR, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16929, IF_186},
    /*  911 */ {I_ROR, 2, {RM_GPR|BITS32,UNITY,0,0,0}, nasm_bytecodes+20907, IF_386},
    /*  912 */ {I_ROR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, nasm_bytecodes+20912, IF_386},
    /*  913 */ {I_ROR, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16935, IF_386},
    /*  914 */ {I_ROR, 2, {RM_GPR|BITS64,UNITY,0,0,0}, nasm_bytecodes+20917, IF_X64},
    /*  915 */ {I_ROR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, nasm_bytecodes+20922, IF_X64},
    /*  916 */ {I_ROR, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16941, IF_X64},
    /*  917 */ {I_RSDC, 2, {REG_SREG,MEMORY|BITS80,0,0,0}, nasm_bytecodes+18346, IF_486|IF_CYRIX|IF_SMM},
    /*  918 */ {I_RSLDT, 1, {MEMORY|BITS80,0,0,0,0}, nasm_bytecodes+20927, IF_486|IF_CYRIX|IF_SMM},
    /*  919 */ {I_RSM, 0, {0,0,0,0,0}, nasm_bytecodes+22218, IF_PENT|IF_SMM},
    /*  920 */ {I_RSTS, 1, {MEMORY|BITS80,0,0,0,0}, nasm_bytecodes+20932, IF_486|IF_CYRIX|IF_SMM},
    /*  921 */ {I_SAHF, 0, {0,0,0,0,0}, nasm_bytecodes+3737, IF_8086},
    /*  922 */ {I_SALC, 0, {0,0,0,0,0}, nasm_bytecodes+21359, IF_8086|IF_UNDOC},
    /*  923 */ {I_SAR, 2, {RM_GPR|BITS8,UNITY,0,0,0}, nasm_bytecodes+22230, IF_8086},
    /*  924 */ {I_SAR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, nasm_bytecodes+22234, IF_8086},
    /*  925 */ {I_SAR, 2, {RM_GPR|BITS8,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+20972, IF_186},
    /*  926 */ {I_SAR, 2, {RM_GPR|BITS16,UNITY,0,0,0}, nasm_bytecodes+20977, IF_8086},
    /*  927 */ {I_SAR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, nasm_bytecodes+20982, IF_8086},
    /*  928 */ {I_SAR, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16965, IF_186},
    /*  929 */ {I_SAR, 2, {RM_GPR|BITS32,UNITY,0,0,0}, nasm_bytecodes+20987, IF_386},
    /*  930 */ {I_SAR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, nasm_bytecodes+20992, IF_386},
    /*  931 */ {I_SAR, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16971, IF_386},
    /*  932 */ {I_SAR, 2, {RM_GPR|BITS64,UNITY,0,0,0}, nasm_bytecodes+20997, IF_X64},
    /*  933 */ {I_SAR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, nasm_bytecodes+21002, IF_X64},
    /*  934 */ {I_SAR, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16977, IF_X64},
    /*  935 */ {I_SBB, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21007, IF_8086|IF_SM|IF_LOCK},
    /*  936 */ {I_SBB, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21008, IF_8086},
    /*  937 */ {I_SBB, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16983, IF_8086|IF_SM|IF_LOCK},
    /*  938 */ {I_SBB, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+16984, IF_8086},
    /*  939 */ {I_SBB, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16989, IF_386|IF_SM|IF_LOCK},
    /*  940 */ {I_SBB, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+16990, IF_386},
    /*  941 */ {I_SBB, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16995, IF_X64|IF_SM|IF_LOCK},
    /*  942 */ {I_SBB, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+16996, IF_X64},
    /*  943 */ {I_SBB, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+9675, IF_8086|IF_SM},
    /*  944 */ {I_SBB, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+9675, IF_8086},
    /*  945 */ {I_SBB, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+21012, IF_8086|IF_SM},
    /*  946 */ {I_SBB, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+21012, IF_8086},
    /*  947 */ {I_SBB, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+21017, IF_386|IF_SM},
    /*  948 */ {I_SBB, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+21017, IF_386},
    /*  949 */ {I_SBB, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+21022, IF_X64|IF_SM},
    /*  950 */ {I_SBB, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+21022, IF_X64},
    /*  951 */ {I_SBB, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8167, IF_8086|IF_LOCK},
    /*  952 */ {I_SBB, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8174, IF_386|IF_LOCK},
    /*  953 */ {I_SBB, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8181, IF_X64|IF_LOCK},
    /*  954 */ {I_SBB, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+22238, IF_8086|IF_SM},
    /*  955 */ {I_SBB, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+21027, IF_8086|IF_SM},
    /*  956 */ {I_SBB, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21032, IF_386|IF_SM},
    /*  957 */ {I_SBB, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21037, IF_X64|IF_SM},
    /*  958 */ {I_SBB, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+17001, IF_8086|IF_SM|IF_LOCK},
    /*  959 */ {I_SBB, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+8188, IF_8086|IF_SM|IF_LOCK},
    /*  960 */ {I_SBB, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+8195, IF_386|IF_SM|IF_LOCK},
    /*  961 */ {I_SBB, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+8202, IF_X64|IF_SM|IF_LOCK},
    /*  962 */ {I_SBB, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+17001, IF_8086|IF_SM|IF_LOCK},
    /*  963 */ {I_SBB, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+8188, IF_8086|IF_SM|IF_LOCK},
    /*  964 */ {I_SBB, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+8195, IF_386|IF_SM|IF_LOCK},
    /*  965 */ {I_SCASB, 0, {0,0,0,0,0}, nasm_bytecodes+22242, IF_8086},
    /*  966 */ {I_SCASD, 0, {0,0,0,0,0}, nasm_bytecodes+21042, IF_386},
    /*  967 */ {I_SCASQ, 0, {0,0,0,0,0}, nasm_bytecodes+21047, IF_X64},
    /*  968 */ {I_SCASW, 0, {0,0,0,0,0}, nasm_bytecodes+21052, IF_8086},
    /*  969 */ {I_SFENCE, 0, {0,0,0,0,0}, nasm_bytecodes+17013, IF_X64|IF_AMD},
    /*  970 */ {I_SGDT, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21057, IF_286},
    /*  971 */ {I_SHL, 2, {RM_GPR|BITS8,UNITY,0,0,0}, nasm_bytecodes+22222, IF_8086},
    /*  972 */ {I_SHL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, nasm_bytecodes+22226, IF_8086},
    /*  973 */ {I_SHL, 2, {RM_GPR|BITS8,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+20937, IF_186},
    /*  974 */ {I_SHL, 2, {RM_GPR|BITS16,UNITY,0,0,0}, nasm_bytecodes+20942, IF_8086},
    /*  975 */ {I_SHL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, nasm_bytecodes+20947, IF_8086},
    /*  976 */ {I_SHL, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16947, IF_186},
    /*  977 */ {I_SHL, 2, {RM_GPR|BITS32,UNITY,0,0,0}, nasm_bytecodes+20952, IF_386},
    /*  978 */ {I_SHL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, nasm_bytecodes+20957, IF_386},
    /*  979 */ {I_SHL, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16953, IF_386},
    /*  980 */ {I_SHL, 2, {RM_GPR|BITS64,UNITY,0,0,0}, nasm_bytecodes+20962, IF_X64},
    /*  981 */ {I_SHL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, nasm_bytecodes+20967, IF_X64},
    /*  982 */ {I_SHL, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+16959, IF_X64},
    /*  983 */ {I_SHLD, 3, {MEMORY,REG_GPR|BITS16,IMMEDIATE,0,0}, nasm_bytecodes+8209, IF_386|IF_SM2|IF_SB|IF_AR2},
    /*  984 */ {I_SHLD, 3, {REG_GPR|BITS16,REG_GPR|BITS16,IMMEDIATE,0,0}, nasm_bytecodes+8209, IF_386|IF_SM2|IF_SB|IF_AR2},
    /*  985 */ {I_SHLD, 3, {MEMORY,REG_GPR|BITS32,IMMEDIATE,0,0}, nasm_bytecodes+8216, IF_386|IF_SM2|IF_SB|IF_AR2},
    /*  986 */ {I_SHLD, 3, {REG_GPR|BITS32,REG_GPR|BITS32,IMMEDIATE,0,0}, nasm_bytecodes+8216, IF_386|IF_SM2|IF_SB|IF_AR2},
    /*  987 */ {I_SHLD, 3, {MEMORY,REG_GPR|BITS64,IMMEDIATE,0,0}, nasm_bytecodes+8223, IF_X64|IF_SM2|IF_SB|IF_AR2},
    /*  988 */ {I_SHLD, 3, {REG_GPR|BITS64,REG_GPR|BITS64,IMMEDIATE,0,0}, nasm_bytecodes+8223, IF_X64|IF_SM2|IF_SB|IF_AR2},
    /*  989 */ {I_SHLD, 3, {MEMORY,REG_GPR|BITS16,REG_CL,0,0}, nasm_bytecodes+17019, IF_386|IF_SM},
    /*  990 */ {I_SHLD, 3, {REG_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, nasm_bytecodes+17019, IF_386},
    /*  991 */ {I_SHLD, 3, {MEMORY,REG_GPR|BITS32,REG_CL,0,0}, nasm_bytecodes+17025, IF_386|IF_SM},
    /*  992 */ {I_SHLD, 3, {REG_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, nasm_bytecodes+17025, IF_386},
    /*  993 */ {I_SHLD, 3, {MEMORY,REG_GPR|BITS64,REG_CL,0,0}, nasm_bytecodes+17031, IF_X64|IF_SM},
    /*  994 */ {I_SHLD, 3, {REG_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, nasm_bytecodes+17031, IF_X64},
    /*  995 */ {I_SHR, 2, {RM_GPR|BITS8,UNITY,0,0,0}, nasm_bytecodes+22246, IF_8086},
    /*  996 */ {I_SHR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, nasm_bytecodes+22250, IF_8086},
    /*  997 */ {I_SHR, 2, {RM_GPR|BITS8,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+21062, IF_186},
    /*  998 */ {I_SHR, 2, {RM_GPR|BITS16,UNITY,0,0,0}, nasm_bytecodes+21067, IF_8086},
    /*  999 */ {I_SHR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, nasm_bytecodes+21072, IF_8086},
    /* 1000 */ {I_SHR, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+17037, IF_186},
    /* 1001 */ {I_SHR, 2, {RM_GPR|BITS32,UNITY,0,0,0}, nasm_bytecodes+21077, IF_386},
    /* 1002 */ {I_SHR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, nasm_bytecodes+21082, IF_386},
    /* 1003 */ {I_SHR, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+17043, IF_386},
    /* 1004 */ {I_SHR, 2, {RM_GPR|BITS64,UNITY,0,0,0}, nasm_bytecodes+21087, IF_X64},
    /* 1005 */ {I_SHR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, nasm_bytecodes+21092, IF_X64},
    /* 1006 */ {I_SHR, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+17049, IF_X64},
    /* 1007 */ {I_SHRD, 3, {MEMORY,REG_GPR|BITS16,IMMEDIATE,0,0}, nasm_bytecodes+8230, IF_386|IF_SM2|IF_SB|IF_AR2},
    /* 1008 */ {I_SHRD, 3, {REG_GPR|BITS16,REG_GPR|BITS16,IMMEDIATE,0,0}, nasm_bytecodes+8230, IF_386|IF_SM2|IF_SB|IF_AR2},
    /* 1009 */ {I_SHRD, 3, {MEMORY,REG_GPR|BITS32,IMMEDIATE,0,0}, nasm_bytecodes+8237, IF_386|IF_SM2|IF_SB|IF_AR2},
    /* 1010 */ {I_SHRD, 3, {REG_GPR|BITS32,REG_GPR|BITS32,IMMEDIATE,0,0}, nasm_bytecodes+8237, IF_386|IF_SM2|IF_SB|IF_AR2},
    /* 1011 */ {I_SHRD, 3, {MEMORY,REG_GPR|BITS64,IMMEDIATE,0,0}, nasm_bytecodes+8244, IF_X64|IF_SM2|IF_SB|IF_AR2},
    /* 1012 */ {I_SHRD, 3, {REG_GPR|BITS64,REG_GPR|BITS64,IMMEDIATE,0,0}, nasm_bytecodes+8244, IF_X64|IF_SM2|IF_SB|IF_AR2},
    /* 1013 */ {I_SHRD, 3, {MEMORY,REG_GPR|BITS16,REG_CL,0,0}, nasm_bytecodes+17055, IF_386|IF_SM},
    /* 1014 */ {I_SHRD, 3, {REG_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, nasm_bytecodes+17055, IF_386},
    /* 1015 */ {I_SHRD, 3, {MEMORY,REG_GPR|BITS32,REG_CL,0,0}, nasm_bytecodes+17061, IF_386|IF_SM},
    /* 1016 */ {I_SHRD, 3, {REG_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, nasm_bytecodes+17061, IF_386},
    /* 1017 */ {I_SHRD, 3, {MEMORY,REG_GPR|BITS64,REG_CL,0,0}, nasm_bytecodes+17067, IF_X64|IF_SM},
    /* 1018 */ {I_SHRD, 3, {REG_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, nasm_bytecodes+17067, IF_X64},
    /* 1019 */ {I_SIDT, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21097, IF_286},
    /* 1020 */ {I_SLDT, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+17092, IF_286},
    /* 1021 */ {I_SLDT, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+17092, IF_286},
    /* 1022 */ {I_SLDT, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+17073, IF_286},
    /* 1023 */ {I_SLDT, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+17079, IF_386},
    /* 1024 */ {I_SLDT, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+17091, IF_X64},
    /* 1025 */ {I_SKINIT, 0, {0,0,0,0,0}, nasm_bytecodes+21102, IF_X64},
    /* 1026 */ {I_SMI, 0, {0,0,0,0,0}, nasm_bytecodes+22378, IF_386|IF_UNDOC},
    /* 1027 */ {I_SMSW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+17104, IF_286},
    /* 1028 */ {I_SMSW, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+17104, IF_286},
    /* 1029 */ {I_SMSW, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+17097, IF_286},
    /* 1030 */ {I_SMSW, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+17103, IF_386},
    /* 1031 */ {I_STC, 0, {0,0,0,0,0}, nasm_bytecodes+20854, IF_8086},
    /* 1032 */ {I_STD, 0, {0,0,0,0,0}, nasm_bytecodes+22405, IF_8086},
    /* 1033 */ {I_STI, 0, {0,0,0,0,0}, nasm_bytecodes+22408, IF_8086},
    /* 1034 */ {I_STOSB, 0, {0,0,0,0,0}, nasm_bytecodes+3841, IF_8086},
    /* 1035 */ {I_STOSD, 0, {0,0,0,0,0}, nasm_bytecodes+22262, IF_386},
    /* 1036 */ {I_STOSQ, 0, {0,0,0,0,0}, nasm_bytecodes+22266, IF_X64},
    /* 1037 */ {I_STOSW, 0, {0,0,0,0,0}, nasm_bytecodes+22270, IF_8086},
    /* 1038 */ {I_STR, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+17122, IF_286|IF_PROT},
    /* 1039 */ {I_STR, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+17122, IF_286|IF_PROT},
    /* 1040 */ {I_STR, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+17109, IF_286|IF_PROT},
    /* 1041 */ {I_STR, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+17115, IF_386|IF_PROT},
    /* 1042 */ {I_STR, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+17121, IF_X64},
    /* 1043 */ {I_SUB, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21107, IF_8086|IF_SM|IF_LOCK},
    /* 1044 */ {I_SUB, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21108, IF_8086},
    /* 1045 */ {I_SUB, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+17127, IF_8086|IF_SM|IF_LOCK},
    /* 1046 */ {I_SUB, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+17128, IF_8086},
    /* 1047 */ {I_SUB, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+17133, IF_386|IF_SM|IF_LOCK},
    /* 1048 */ {I_SUB, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+17134, IF_386},
    /* 1049 */ {I_SUB, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+17139, IF_X64|IF_SM|IF_LOCK},
    /* 1050 */ {I_SUB, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+17140, IF_X64},
    /* 1051 */ {I_SUB, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+15415, IF_8086|IF_SM},
    /* 1052 */ {I_SUB, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+15415, IF_8086},
    /* 1053 */ {I_SUB, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+21112, IF_8086|IF_SM},
    /* 1054 */ {I_SUB, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+21112, IF_8086},
    /* 1055 */ {I_SUB, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+21117, IF_386|IF_SM},
    /* 1056 */ {I_SUB, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+21117, IF_386},
    /* 1057 */ {I_SUB, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+21122, IF_X64|IF_SM},
    /* 1058 */ {I_SUB, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+21122, IF_X64},
    /* 1059 */ {I_SUB, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8251, IF_8086|IF_LOCK},
    /* 1060 */ {I_SUB, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8258, IF_386|IF_LOCK},
    /* 1061 */ {I_SUB, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8265, IF_X64|IF_LOCK},
    /* 1062 */ {I_SUB, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+22274, IF_8086|IF_SM},
    /* 1063 */ {I_SUB, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+21127, IF_8086|IF_SM},
    /* 1064 */ {I_SUB, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21132, IF_386|IF_SM},
    /* 1065 */ {I_SUB, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21137, IF_X64|IF_SM},
    /* 1066 */ {I_SUB, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+17145, IF_8086|IF_SM|IF_LOCK},
    /* 1067 */ {I_SUB, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+8272, IF_8086|IF_SM|IF_LOCK},
    /* 1068 */ {I_SUB, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+8279, IF_386|IF_SM|IF_LOCK},
    /* 1069 */ {I_SUB, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+8286, IF_X64|IF_SM|IF_LOCK},
    /* 1070 */ {I_SUB, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+17145, IF_8086|IF_SM|IF_LOCK},
    /* 1071 */ {I_SUB, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+8272, IF_8086|IF_SM|IF_LOCK},
    /* 1072 */ {I_SUB, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+8279, IF_386|IF_SM|IF_LOCK},
    /* 1073 */ {I_SVDC, 2, {MEMORY|BITS80,REG_SREG,0,0,0}, nasm_bytecodes+8820, IF_486|IF_CYRIX|IF_SMM},
    /* 1074 */ {I_SVTS, 1, {MEMORY|BITS80,0,0,0,0}, nasm_bytecodes+21147, IF_486|IF_CYRIX|IF_SMM},
    /* 1075 */ {I_SWAPGS, 0, {0,0,0,0,0}, nasm_bytecodes+21152, IF_X64},
    /* 1076 */ {I_SYSCALL, 0, {0,0,0,0,0}, nasm_bytecodes+22002, IF_P6|IF_AMD},
    /* 1077 */ {I_SYSENTER, 0, {0,0,0,0,0}, nasm_bytecodes+22278, IF_P6},
    /* 1078 */ {I_SYSEXIT, 0, {0,0,0,0,0}, nasm_bytecodes+22282, IF_P6|IF_PRIV},
    /* 1079 */ {I_SYSRET, 0, {0,0,0,0,0}, nasm_bytecodes+21998, IF_P6|IF_PRIV|IF_AMD},
    /* 1080 */ {I_TEST, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+22286, IF_8086|IF_SM},
    /* 1081 */ {I_TEST, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+22286, IF_8086},
    /* 1082 */ {I_TEST, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+21157, IF_8086|IF_SM},
    /* 1083 */ {I_TEST, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+21157, IF_8086},
    /* 1084 */ {I_TEST, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+21162, IF_386|IF_SM},
    /* 1085 */ {I_TEST, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+21162, IF_386},
    /* 1086 */ {I_TEST, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+21167, IF_X64|IF_SM},
    /* 1087 */ {I_TEST, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+21167, IF_X64},
    /* 1088 */ {I_TEST, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+22290, IF_8086|IF_SM},
    /* 1089 */ {I_TEST, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+21172, IF_8086|IF_SM},
    /* 1090 */ {I_TEST, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+21177, IF_386|IF_SM},
    /* 1091 */ {I_TEST, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+21182, IF_X64|IF_SM},
    /* 1092 */ {I_TEST, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+22294, IF_8086|IF_SM},
    /* 1093 */ {I_TEST, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+21187, IF_8086|IF_SM},
    /* 1094 */ {I_TEST, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21192, IF_386|IF_SM},
    /* 1095 */ {I_TEST, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21197, IF_X64|IF_SM},
    /* 1096 */ {I_TEST, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+21202, IF_8086|IF_SM},
    /* 1097 */ {I_TEST, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+17157, IF_8086|IF_SM},
    /* 1098 */ {I_TEST, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+17163, IF_386|IF_SM},
    /* 1099 */ {I_TEST, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+17169, IF_X64|IF_SM},
    /* 1100 */ {I_TEST, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+21202, IF_8086|IF_SM},
    /* 1101 */ {I_TEST, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+17157, IF_8086|IF_SM},
    /* 1102 */ {I_TEST, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+17163, IF_386|IF_SM},
    /* 1103 */ {I_UD0, 0, {0,0,0,0,0}, nasm_bytecodes+22298, IF_186|IF_UNDOC},
    /* 1104 */ {I_UD1, 0, {0,0,0,0,0}, nasm_bytecodes+22302, IF_186|IF_UNDOC},
    /* 1105 */ {I_UD2, 0, {0,0,0,0,0}, nasm_bytecodes+22306, IF_186},
    /* 1106 */ {I_VERR, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21207, IF_286|IF_PROT},
    /* 1107 */ {I_VERR, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21207, IF_286|IF_PROT},
    /* 1108 */ {I_VERR, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+21207, IF_286|IF_PROT},
    /* 1109 */ {I_VERW, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+21212, IF_286|IF_PROT},
    /* 1110 */ {I_VERW, 1, {MEMORY|BITS16,0,0,0,0}, nasm_bytecodes+21212, IF_286|IF_PROT},
    /* 1111 */ {I_VERW, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+21212, IF_286|IF_PROT},
    /* 1112 */ {I_FWAIT, 0, {0,0,0,0,0}, nasm_bytecodes+21900, IF_8086},
    /* 1113 */ {I_WBINVD, 0, {0,0,0,0,0}, nasm_bytecodes+22310, IF_486|IF_PRIV},
    /* 1114 */ {I_WRSHR, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+17187, IF_P6|IF_CYRIX|IF_SMM},
    /* 1115 */ {I_WRMSR, 0, {0,0,0,0,0}, nasm_bytecodes+22314, IF_PENT|IF_PRIV},
    /* 1116 */ {I_XADD, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+17193, IF_486|IF_SM|IF_LOCK},
    /* 1117 */ {I_XADD, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+17194, IF_486},
    /* 1118 */ {I_XADD, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+8321, IF_486|IF_SM|IF_LOCK},
    /* 1119 */ {I_XADD, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+8322, IF_486},
    /* 1120 */ {I_XADD, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+8328, IF_486|IF_SM|IF_LOCK},
    /* 1121 */ {I_XADD, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+8329, IF_486},
    /* 1122 */ {I_XADD, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+8335, IF_X64|IF_SM|IF_LOCK},
    /* 1123 */ {I_XADD, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+8336, IF_X64},
    /* 1124 */ {I_XCHG, 2, {REG_AX,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+22318, IF_8086},
    /* 1125 */ {I_XCHG, 2, {REG_EAX,REG32NA,0,0,0}, nasm_bytecodes+22322, IF_386},
    /* 1126 */ {I_XCHG, 2, {REG_RAX,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+22326, IF_X64},
    /* 1127 */ {I_XCHG, 2, {REG_GPR|BITS16,REG_AX,0,0,0}, nasm_bytecodes+22330, IF_8086},
    /* 1128 */ {I_XCHG, 2, {REG32NA,REG_EAX,0,0,0}, nasm_bytecodes+22334, IF_386},
    /* 1129 */ {I_XCHG, 2, {REG_GPR|BITS64,REG_RAX,0,0,0}, nasm_bytecodes+22338, IF_X64},
    /* 1130 */ {I_XCHG, 2, {REG_EAX,REG_EAX,0,0,0}, nasm_bytecodes+22342, IF_386|IF_NOLONG},
    /* 1131 */ {I_XCHG, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+21217, IF_8086|IF_SM|IF_LOCK},
    /* 1132 */ {I_XCHG, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21218, IF_8086},
    /* 1133 */ {I_XCHG, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+17211, IF_8086|IF_SM|IF_LOCK},
    /* 1134 */ {I_XCHG, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+17212, IF_8086},
    /* 1135 */ {I_XCHG, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+17217, IF_386|IF_SM|IF_LOCK},
    /* 1136 */ {I_XCHG, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+17218, IF_386},
    /* 1137 */ {I_XCHG, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+17223, IF_X64|IF_SM|IF_LOCK},
    /* 1138 */ {I_XCHG, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+17224, IF_X64},
    /* 1139 */ {I_XCHG, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21222, IF_8086|IF_SM|IF_LOCK},
    /* 1140 */ {I_XCHG, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21223, IF_8086},
    /* 1141 */ {I_XCHG, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+17229, IF_8086|IF_SM|IF_LOCK},
    /* 1142 */ {I_XCHG, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+17230, IF_8086},
    /* 1143 */ {I_XCHG, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+17235, IF_386|IF_SM|IF_LOCK},
    /* 1144 */ {I_XCHG, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+17236, IF_386},
    /* 1145 */ {I_XCHG, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+17241, IF_X64|IF_SM|IF_LOCK},
    /* 1146 */ {I_XCHG, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+17242, IF_X64},
    /* 1147 */ {I_XLATB, 0, {0,0,0,0,0}, nasm_bytecodes+22411, IF_8086},
    /* 1148 */ {I_XLAT, 0, {0,0,0,0,0}, nasm_bytecodes+22411, IF_8086},
    /* 1149 */ {I_XOR, 2, {MEMORY,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21227, IF_8086|IF_SM|IF_LOCK},
    /* 1150 */ {I_XOR, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+21228, IF_8086},
    /* 1151 */ {I_XOR, 2, {MEMORY,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+17247, IF_8086|IF_SM|IF_LOCK},
    /* 1152 */ {I_XOR, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+17248, IF_8086},
    /* 1153 */ {I_XOR, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+17253, IF_386|IF_SM|IF_LOCK},
    /* 1154 */ {I_XOR, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+17254, IF_386},
    /* 1155 */ {I_XOR, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+17259, IF_X64|IF_SM|IF_LOCK},
    /* 1156 */ {I_XOR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+17260, IF_X64},
    /* 1157 */ {I_XOR, 2, {REG_GPR|BITS8,MEMORY,0,0,0}, nasm_bytecodes+14855, IF_8086|IF_SM},
    /* 1158 */ {I_XOR, 2, {REG_GPR|BITS8,REG_GPR|BITS8,0,0,0}, nasm_bytecodes+14855, IF_8086},
    /* 1159 */ {I_XOR, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+21232, IF_8086|IF_SM},
    /* 1160 */ {I_XOR, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+21232, IF_8086},
    /* 1161 */ {I_XOR, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+21237, IF_386|IF_SM},
    /* 1162 */ {I_XOR, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+21237, IF_386},
    /* 1163 */ {I_XOR, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+21242, IF_X64|IF_SM},
    /* 1164 */ {I_XOR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+21242, IF_X64},
    /* 1165 */ {I_XOR, 2, {RM_GPR|BITS16,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8342, IF_8086|IF_LOCK},
    /* 1166 */ {I_XOR, 2, {RM_GPR|BITS32,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8349, IF_386|IF_LOCK},
    /* 1167 */ {I_XOR, 2, {RM_GPR|BITS64,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+8356, IF_X64|IF_LOCK},
    /* 1168 */ {I_XOR, 2, {REG_AL,IMMEDIATE,0,0,0}, nasm_bytecodes+22346, IF_8086|IF_SM},
    /* 1169 */ {I_XOR, 2, {REG_AX,IMMEDIATE,0,0,0}, nasm_bytecodes+21247, IF_8086|IF_SM},
    /* 1170 */ {I_XOR, 2, {REG_EAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21252, IF_386|IF_SM},
    /* 1171 */ {I_XOR, 2, {REG_RAX,IMMEDIATE,0,0,0}, nasm_bytecodes+21257, IF_X64|IF_SM},
    /* 1172 */ {I_XOR, 2, {RM_GPR|BITS8,IMMEDIATE,0,0,0}, nasm_bytecodes+17265, IF_8086|IF_SM|IF_LOCK},
    /* 1173 */ {I_XOR, 2, {RM_GPR|BITS16,IMMEDIATE,0,0,0}, nasm_bytecodes+8363, IF_8086|IF_SM|IF_LOCK},
    /* 1174 */ {I_XOR, 2, {RM_GPR|BITS32,IMMEDIATE,0,0,0}, nasm_bytecodes+8370, IF_386|IF_SM|IF_LOCK},
    /* 1175 */ {I_XOR, 2, {RM_GPR|BITS64,IMMEDIATE,0,0,0}, nasm_bytecodes+8377, IF_X64|IF_SM|IF_LOCK},
    /* 1176 */ {I_XOR, 2, {MEMORY,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+17265, IF_8086|IF_SM|IF_LOCK},
    /* 1177 */ {I_XOR, 2, {MEMORY,IMMEDIATE|BITS16,0,0,0}, nasm_bytecodes+8363, IF_8086|IF_SM|IF_LOCK},
    /* 1178 */ {I_XOR, 2, {MEMORY,IMMEDIATE|BITS32,0,0,0}, nasm_bytecodes+8370, IF_386|IF_SM|IF_LOCK},
    /* 1179 */ {I_CMOVcc, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, nasm_bytecodes+8384, IF_P6|IF_SM},
    /* 1180 */ {I_CMOVcc, 2, {REG_GPR|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+8384, IF_P6},
    /* 1181 */ {I_CMOVcc, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+8391, IF_P6|IF_SM},
    /* 1182 */ {I_CMOVcc, 2, {REG_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+8391, IF_P6},
    /* 1183 */ {I_CMOVcc, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+8398, IF_X64|IF_SM},
    /* 1184 */ {I_CMOVcc, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+8398, IF_X64},
    /* 1185 */ {I_Jcc, 1, {IMMEDIATE|NEAR,0,0,0,0}, nasm_bytecodes+8405, IF_386},
    /* 1186 */ {I_Jcc, 1, {IMMEDIATE|BITS16|NEAR,0,0,0,0}, nasm_bytecodes+8412, IF_386|IF_NOLONG},
    /* 1187 */ {I_Jcc, 1, {IMMEDIATE|BITS32|NEAR,0,0,0,0}, nasm_bytecodes+8419, IF_386|IF_NOLONG},
    /* 1188 */ {I_Jcc, 1, {IMMEDIATE|BITS64|NEAR,0,0,0,0}, nasm_bytecodes+8426, IF_X64},
    /* 1189 */ {I_Jcc, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+21263, IF_8086},
    /* 1190 */ {I_SETcc, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+17277, IF_386|IF_SB},
    /* 1191 */ {I_SETcc, 1, {REG_GPR|BITS8,0,0,0,0}, nasm_bytecodes+17277, IF_386},
    /* 1192 */ {I_ADDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17283, IF_KATMAI|IF_SSE},
    /* 1193 */ {I_ADDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17289, IF_KATMAI|IF_SSE},
    /* 1194 */ {I_ANDNPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17295, IF_KATMAI|IF_SSE},
    /* 1195 */ {I_ANDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17301, IF_KATMAI|IF_SSE},
    /* 1196 */ {I_CMPEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3860, IF_KATMAI|IF_SSE},
    /* 1197 */ {I_CMPEQSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3868, IF_KATMAI|IF_SSE},
    /* 1198 */ {I_CMPLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3876, IF_KATMAI|IF_SSE},
    /* 1199 */ {I_CMPLESS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3884, IF_KATMAI|IF_SSE},
    /* 1200 */ {I_CMPLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3892, IF_KATMAI|IF_SSE},
    /* 1201 */ {I_CMPLTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3900, IF_KATMAI|IF_SSE},
    /* 1202 */ {I_CMPNEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3908, IF_KATMAI|IF_SSE},
    /* 1203 */ {I_CMPNEQSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3916, IF_KATMAI|IF_SSE},
    /* 1204 */ {I_CMPNLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3924, IF_KATMAI|IF_SSE},
    /* 1205 */ {I_CMPNLESS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3932, IF_KATMAI|IF_SSE},
    /* 1206 */ {I_CMPNLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3940, IF_KATMAI|IF_SSE},
    /* 1207 */ {I_CMPNLTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3948, IF_KATMAI|IF_SSE},
    /* 1208 */ {I_CMPORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3956, IF_KATMAI|IF_SSE},
    /* 1209 */ {I_CMPORDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3964, IF_KATMAI|IF_SSE},
    /* 1210 */ {I_CMPUNORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3972, IF_KATMAI|IF_SSE},
    /* 1211 */ {I_CMPUNORDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+3980, IF_KATMAI|IF_SSE},
    /* 1212 */ {I_CMPPS, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8440, IF_KATMAI|IF_SSE|IF_SB|IF_AR2},
    /* 1213 */ {I_CMPPS, 3, {XMMREG,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+8440, IF_KATMAI|IF_SSE|IF_SB|IF_AR2},
    /* 1214 */ {I_CMPSS, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8447, IF_KATMAI|IF_SSE|IF_SB|IF_AR2},
    /* 1215 */ {I_CMPSS, 3, {XMMREG,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+8447, IF_KATMAI|IF_SSE|IF_SB|IF_AR2},
    /* 1216 */ {I_COMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17307, IF_KATMAI|IF_SSE},
    /* 1217 */ {I_CVTPI2PS, 2, {XMMREG,RM_MMX|BITS64,0,0,0}, nasm_bytecodes+17313, IF_KATMAI|IF_SSE|IF_MMX},
    /* 1218 */ {I_CVTPS2PI, 2, {MMXREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+17319, IF_KATMAI|IF_SSE|IF_MMX},
    /* 1219 */ {I_CVTSI2SS, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+8455, IF_KATMAI|IF_SSE|IF_SD|IF_AR1},
    /* 1220 */ {I_CVTSI2SS, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+8454, IF_X64|IF_SSE|IF_SQ|IF_AR1},
    /* 1221 */ {I_CVTSS2SI, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+8462, IF_KATMAI|IF_SSE|IF_SD|IF_AR1},
    /* 1222 */ {I_CVTSS2SI, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+8462, IF_KATMAI|IF_SSE|IF_SD|IF_AR1},
    /* 1223 */ {I_CVTSS2SI, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+8461, IF_X64|IF_SSE|IF_SD|IF_AR1},
    /* 1224 */ {I_CVTSS2SI, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+8461, IF_X64|IF_SSE|IF_SD|IF_AR1},
    /* 1225 */ {I_CVTTPS2PI, 2, {MMXREG,RM_XMM,0,0,0}, nasm_bytecodes+17325, IF_KATMAI|IF_SSE|IF_MMX|IF_SQ},
    /* 1226 */ {I_CVTTSS2SI, 2, {REG_GPR|BITS32,RM_XMM,0,0,0}, nasm_bytecodes+8469, IF_KATMAI|IF_SSE|IF_SD|IF_AR1},
    /* 1227 */ {I_CVTTSS2SI, 2, {REG_GPR|BITS64,RM_XMM,0,0,0}, nasm_bytecodes+8468, IF_X64|IF_SSE|IF_SD|IF_AR1},
    /* 1228 */ {I_DIVPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17331, IF_KATMAI|IF_SSE},
    /* 1229 */ {I_DIVSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17337, IF_KATMAI|IF_SSE},
    /* 1230 */ {I_LDMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+17343, IF_KATMAI|IF_SSE},
    /* 1231 */ {I_MAXPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17349, IF_KATMAI|IF_SSE},
    /* 1232 */ {I_MAXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17355, IF_KATMAI|IF_SSE},
    /* 1233 */ {I_MINPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17361, IF_KATMAI|IF_SSE},
    /* 1234 */ {I_MINSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17367, IF_KATMAI|IF_SSE},
    /* 1235 */ {I_MOVAPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17373, IF_KATMAI|IF_SSE},
    /* 1236 */ {I_MOVAPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+17379, IF_KATMAI|IF_SSE},
    /* 1237 */ {I_MOVHPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+17385, IF_KATMAI|IF_SSE},
    /* 1238 */ {I_MOVHPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+17391, IF_KATMAI|IF_SSE},
    /* 1239 */ {I_MOVLHPS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17385, IF_KATMAI|IF_SSE},
    /* 1240 */ {I_MOVLPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+17181, IF_KATMAI|IF_SSE},
    /* 1241 */ {I_MOVLPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+17397, IF_KATMAI|IF_SSE},
    /* 1242 */ {I_MOVHLPS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17181, IF_KATMAI|IF_SSE},
    /* 1243 */ {I_MOVMSKPS, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+17403, IF_KATMAI|IF_SSE},
    /* 1244 */ {I_MOVMSKPS, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+8475, IF_X64|IF_SSE},
    /* 1245 */ {I_MOVNTPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, nasm_bytecodes+17409, IF_KATMAI|IF_SSE},
    /* 1246 */ {I_MOVSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17415, IF_KATMAI|IF_SSE},
    /* 1247 */ {I_MOVSS, 2, {MEMORY|BITS32,XMMREG,0,0,0}, nasm_bytecodes+17421, IF_KATMAI|IF_SSE},
    /* 1248 */ {I_MOVSS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17415, IF_KATMAI|IF_SSE},
    /* 1249 */ {I_MOVUPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17427, IF_KATMAI|IF_SSE},
    /* 1250 */ {I_MOVUPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+17433, IF_KATMAI|IF_SSE},
    /* 1251 */ {I_MULPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17439, IF_KATMAI|IF_SSE},
    /* 1252 */ {I_MULSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17445, IF_KATMAI|IF_SSE},
    /* 1253 */ {I_ORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17451, IF_KATMAI|IF_SSE},
    /* 1254 */ {I_RCPPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17457, IF_KATMAI|IF_SSE},
    /* 1255 */ {I_RCPSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17463, IF_KATMAI|IF_SSE},
    /* 1256 */ {I_RSQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17469, IF_KATMAI|IF_SSE},
    /* 1257 */ {I_RSQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17475, IF_KATMAI|IF_SSE},
    /* 1258 */ {I_SHUFPS, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+8482, IF_KATMAI|IF_SSE},
    /* 1259 */ {I_SQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17481, IF_KATMAI|IF_SSE},
    /* 1260 */ {I_SQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17487, IF_KATMAI|IF_SSE},
    /* 1261 */ {I_STMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+17493, IF_KATMAI|IF_SSE},
    /* 1262 */ {I_SUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17499, IF_KATMAI|IF_SSE},
    /* 1263 */ {I_SUBSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17505, IF_KATMAI|IF_SSE},
    /* 1264 */ {I_UCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+17511, IF_KATMAI|IF_SSE},
    /* 1265 */ {I_UNPCKHPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17517, IF_KATMAI|IF_SSE},
    /* 1266 */ {I_UNPCKLPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17523, IF_KATMAI|IF_SSE},
    /* 1267 */ {I_XORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+17529, IF_KATMAI|IF_SSE},
    /* 1268 */ {I_FXRSTOR, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8490, IF_P6|IF_SSE|IF_FPU},
    /* 1269 */ {I_FXRSTOR64, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8489, IF_X64|IF_SSE|IF_FPU},
    /* 1270 */ {I_FXSAVE, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8497, IF_P6|IF_SSE|IF_FPU},
    /* 1271 */ {I_FXSAVE64, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8496, IF_X64|IF_SSE|IF_FPU},
    /* 1272 */ {I_XGETBV, 0, {0,0,0,0,0}, nasm_bytecodes+21267, IF_NEHALEM},
    /* 1273 */ {I_XSETBV, 0, {0,0,0,0,0}, nasm_bytecodes+21272, IF_NEHALEM|IF_PRIV},
    /* 1274 */ {I_XSAVE, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8504, IF_NEHALEM},
    /* 1275 */ {I_XSAVE64, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8503, IF_LONG|IF_NEHALEM},
    /* 1276 */ {I_XSAVEOPT, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8511, IF_FUTURE},
    /* 1277 */ {I_XSAVEOPT64, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8510, IF_LONG|IF_FUTURE},
    /* 1278 */ {I_XRSTOR, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8518, IF_NEHALEM},
    /* 1279 */ {I_XRSTOR64, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+8517, IF_LONG|IF_NEHALEM},
    /* 1280 */ {I_PREFETCHNTA, 1, {MEMORY|BITS8,0,0,0,0}, nasm_bytecodes+18496, IF_KATMAI},
    /* 1281 */ {I_PREFETCHT0, 1, {MEMORY|BITS8,0,0,0,0}, nasm_bytecodes+18514, IF_KATMAI},
    /* 1282 */ {I_PREFETCHT1, 1, {MEMORY|BITS8,0,0,0,0}, nasm_bytecodes+18532, IF_KATMAI},
    /* 1283 */ {I_PREFETCHT2, 1, {MEMORY|BITS8,0,0,0,0}, nasm_bytecodes+18550, IF_KATMAI},
    /* 1284 */ {I_SFENCE, 0, {0,0,0,0,0}, nasm_bytecodes+17013, IF_KATMAI},
    /* 1285 */ {I_MASKMOVQ, 2, {MMXREG,MMXREG,0,0,0}, nasm_bytecodes+17535, IF_KATMAI|IF_MMX},
    /* 1286 */ {I_MOVNTQ, 2, {MEMORY,MMXREG,0,0,0}, nasm_bytecodes+17541, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1287 */ {I_PAVGB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8524, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1288 */ {I_PAVGW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8531, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1289 */ {I_PEXTRW, 3, {REG_GPR|BITS32,MMXREG,IMMEDIATE,0,0}, nasm_bytecodes+8538, IF_KATMAI|IF_MMX|IF_SB|IF_AR2},
    /* 1290 */ {I_PINSRW, 3, {MMXREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8545, IF_KATMAI|IF_MMX|IF_SB|IF_AR2},
    /* 1291 */ {I_PINSRW, 3, {MMXREG,RM_GPR|BITS16,IMMEDIATE,0,0}, nasm_bytecodes+8545, IF_KATMAI|IF_MMX|IF_SB|IF_AR2},
    /* 1292 */ {I_PINSRW, 3, {MMXREG,REG_GPR|BITS32,IMMEDIATE,0,0}, nasm_bytecodes+8545, IF_KATMAI|IF_MMX|IF_SB|IF_AR2},
    /* 1293 */ {I_PMAXSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8552, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1294 */ {I_PMAXUB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8559, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1295 */ {I_PMINSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8566, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1296 */ {I_PMINUB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8573, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1297 */ {I_PMOVMSKB, 2, {REG_GPR|BITS32,MMXREG,0,0,0}, nasm_bytecodes+17547, IF_KATMAI|IF_MMX},
    /* 1298 */ {I_PMULHUW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8580, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1299 */ {I_PSADBW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8587, IF_KATMAI|IF_MMX|IF_SQ},
    /* 1300 */ {I_PSHUFW, 3, {MMXREG,RM_MMX,IMMEDIATE,0,0}, nasm_bytecodes+3988, IF_KATMAI|IF_MMX|IF_SM2|IF_SB|IF_AR2},
    /* 1301 */ {I_PF2IW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+3996, IF_PENT|IF_3DNOW|IF_SQ},
    /* 1302 */ {I_PFNACC, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+4004, IF_PENT|IF_3DNOW|IF_SQ},
    /* 1303 */ {I_PFPNACC, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+4012, IF_PENT|IF_3DNOW|IF_SQ},
    /* 1304 */ {I_PI2FW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+4020, IF_PENT|IF_3DNOW|IF_SQ},
    /* 1305 */ {I_PSWAPD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+4028, IF_PENT|IF_3DNOW|IF_SQ},
    /* 1306 */ {I_MASKMOVDQU, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17553, IF_WILLAMETTE|IF_SSE2},
    /* 1307 */ {I_CLFLUSH, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+17559, IF_WILLAMETTE|IF_SSE2},
    /* 1308 */ {I_MOVNTDQ, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+17565, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1309 */ {I_MOVNTI, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+8595, IF_WILLAMETTE|IF_SD},
    /* 1310 */ {I_MOVNTI, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+8594, IF_X64|IF_SQ},
    /* 1311 */ {I_MOVNTPD, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+17571, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1312 */ {I_LFENCE, 0, {0,0,0,0,0}, nasm_bytecodes+16521, IF_WILLAMETTE|IF_SSE2},
    /* 1313 */ {I_MFENCE, 0, {0,0,0,0,0}, nasm_bytecodes+16599, IF_WILLAMETTE|IF_SSE2},
    /* 1314 */ {I_MOVD, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+8601, IF_WILLAMETTE|IF_SSE2|IF_SD},
    /* 1315 */ {I_MOVD, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+8608, IF_WILLAMETTE|IF_SSE2|IF_SD},
    /* 1316 */ {I_MOVD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+8608, IF_WILLAMETTE|IF_SSE2},
    /* 1317 */ {I_MOVD, 2, {RM_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+8601, IF_WILLAMETTE|IF_SSE2},
    /* 1318 */ {I_MOVDQA, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17577, IF_WILLAMETTE|IF_SSE2},
    /* 1319 */ {I_MOVDQA, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+17583, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1320 */ {I_MOVDQA, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+17577, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1321 */ {I_MOVDQA, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17583, IF_WILLAMETTE|IF_SSE2},
    /* 1322 */ {I_MOVDQU, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17589, IF_WILLAMETTE|IF_SSE2},
    /* 1323 */ {I_MOVDQU, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+17595, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1324 */ {I_MOVDQU, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+17589, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1325 */ {I_MOVDQU, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17595, IF_WILLAMETTE|IF_SSE2},
    /* 1326 */ {I_MOVDQ2Q, 2, {MMXREG,XMMREG,0,0,0}, nasm_bytecodes+17601, IF_WILLAMETTE|IF_SSE2},
    /* 1327 */ {I_MOVQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17607, IF_WILLAMETTE|IF_SSE2},
    /* 1328 */ {I_MOVQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+17613, IF_WILLAMETTE|IF_SSE2},
    /* 1329 */ {I_MOVQ, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+17613, IF_WILLAMETTE|IF_SSE2|IF_SQ},
    /* 1330 */ {I_MOVQ, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+17607, IF_WILLAMETTE|IF_SSE2|IF_SQ},
    /* 1331 */ {I_MOVQ, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+8615, IF_X64|IF_SSE2},
    /* 1332 */ {I_MOVQ, 2, {RM_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+8622, IF_X64|IF_SSE2},
    /* 1333 */ {I_MOVQ2DQ, 2, {XMMREG,MMXREG,0,0,0}, nasm_bytecodes+17619, IF_WILLAMETTE|IF_SSE2},
    /* 1334 */ {I_PACKSSWB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17625, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1335 */ {I_PACKSSDW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17631, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1336 */ {I_PACKUSWB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17637, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1337 */ {I_PADDB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17643, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1338 */ {I_PADDW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17649, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1339 */ {I_PADDD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17655, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1340 */ {I_PADDQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+17661, IF_WILLAMETTE|IF_MMX|IF_SQ},
    /* 1341 */ {I_PADDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17667, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1342 */ {I_PADDSB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17673, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1343 */ {I_PADDSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17679, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1344 */ {I_PADDUSB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17685, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1345 */ {I_PADDUSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17691, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1346 */ {I_PAND, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17697, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1347 */ {I_PANDN, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17703, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1348 */ {I_PAVGB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17709, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1349 */ {I_PAVGW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17715, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1350 */ {I_PCMPEQB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17721, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1351 */ {I_PCMPEQW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17727, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1352 */ {I_PCMPEQD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17733, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1353 */ {I_PCMPGTB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17739, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1354 */ {I_PCMPGTW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17745, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1355 */ {I_PCMPGTD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17751, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1356 */ {I_PEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+8629, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1357 */ {I_PINSRW, 3, {XMMREG,REG_GPR|BITS16,IMMEDIATE,0,0}, nasm_bytecodes+8636, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1358 */ {I_PINSRW, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8636, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1359 */ {I_PINSRW, 3, {XMMREG,MEMORY|BITS16,IMMEDIATE,0,0}, nasm_bytecodes+8636, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1360 */ {I_PMADDWD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17757, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1361 */ {I_PMAXSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17763, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1362 */ {I_PMAXUB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17769, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1363 */ {I_PMINSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17775, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1364 */ {I_PMINUB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17781, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1365 */ {I_PMOVMSKB, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+17787, IF_WILLAMETTE|IF_SSE2},
    /* 1366 */ {I_PMULHUW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17793, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1367 */ {I_PMULHW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17799, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1368 */ {I_PMULLW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17805, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1369 */ {I_PMULUDQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8643, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1370 */ {I_PMULUDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17811, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1371 */ {I_POR, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17817, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1372 */ {I_PSADBW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17823, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1373 */ {I_PSHUFD, 3, {XMMREG,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+8650, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1374 */ {I_PSHUFD, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8650, IF_WILLAMETTE|IF_SSE2|IF_SM2|IF_SB|IF_AR2},
    /* 1375 */ {I_PSHUFHW, 3, {XMMREG,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+8657, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1376 */ {I_PSHUFHW, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8657, IF_WILLAMETTE|IF_SSE2|IF_SM2|IF_SB|IF_AR2},
    /* 1377 */ {I_PSHUFLW, 3, {XMMREG,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+8664, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1378 */ {I_PSHUFLW, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8664, IF_WILLAMETTE|IF_SSE2|IF_SM2|IF_SB|IF_AR2},
    /* 1379 */ {I_PSLLDQ, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8671, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1380 */ {I_PSLLW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17829, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1381 */ {I_PSLLW, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8678, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1382 */ {I_PSLLD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17835, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1383 */ {I_PSLLD, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8685, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1384 */ {I_PSLLQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17841, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1385 */ {I_PSLLQ, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8692, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1386 */ {I_PSRAW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17847, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1387 */ {I_PSRAW, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8699, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1388 */ {I_PSRAD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17853, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1389 */ {I_PSRAD, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8706, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1390 */ {I_PSRLDQ, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8713, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1391 */ {I_PSRLW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17859, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1392 */ {I_PSRLW, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8720, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1393 */ {I_PSRLD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17865, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1394 */ {I_PSRLD, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8727, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1395 */ {I_PSRLQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17871, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1396 */ {I_PSRLQ, 2, {XMMREG,IMMEDIATE,0,0,0}, nasm_bytecodes+8734, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR1},
    /* 1397 */ {I_PSUBB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17877, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1398 */ {I_PSUBW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17883, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1399 */ {I_PSUBD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17889, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1400 */ {I_PSUBQ, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8741, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1401 */ {I_PSUBQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17895, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1402 */ {I_PSUBSB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17901, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1403 */ {I_PSUBSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17907, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1404 */ {I_PSUBUSB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17913, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1405 */ {I_PSUBUSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17919, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1406 */ {I_PUNPCKHBW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17925, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1407 */ {I_PUNPCKHWD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17931, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1408 */ {I_PUNPCKHDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17937, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1409 */ {I_PUNPCKHQDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17943, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1410 */ {I_PUNPCKLBW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17949, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1411 */ {I_PUNPCKLWD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17955, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1412 */ {I_PUNPCKLDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17961, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1413 */ {I_PUNPCKLQDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17967, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1414 */ {I_PXOR, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17973, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1415 */ {I_ADDPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17979, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1416 */ {I_ADDSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17985, IF_WILLAMETTE|IF_SSE2|IF_SQ},
    /* 1417 */ {I_ANDNPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17991, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1418 */ {I_ANDPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+17997, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1419 */ {I_CMPEQPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4036, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1420 */ {I_CMPEQSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4044, IF_WILLAMETTE|IF_SSE2},
    /* 1421 */ {I_CMPLEPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4052, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1422 */ {I_CMPLESD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4060, IF_WILLAMETTE|IF_SSE2},
    /* 1423 */ {I_CMPLTPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4068, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1424 */ {I_CMPLTSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4076, IF_WILLAMETTE|IF_SSE2},
    /* 1425 */ {I_CMPNEQPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4084, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1426 */ {I_CMPNEQSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4092, IF_WILLAMETTE|IF_SSE2},
    /* 1427 */ {I_CMPNLEPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4100, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1428 */ {I_CMPNLESD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4108, IF_WILLAMETTE|IF_SSE2},
    /* 1429 */ {I_CMPNLTPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4116, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1430 */ {I_CMPNLTSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4124, IF_WILLAMETTE|IF_SSE2},
    /* 1431 */ {I_CMPORDPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4132, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1432 */ {I_CMPORDSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4140, IF_WILLAMETTE|IF_SSE2},
    /* 1433 */ {I_CMPUNORDPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4148, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1434 */ {I_CMPUNORDSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+4156, IF_WILLAMETTE|IF_SSE2},
    /* 1435 */ {I_CMPPD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+8748, IF_WILLAMETTE|IF_SSE2},
    /* 1436 */ {I_CMPSD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+8755, IF_WILLAMETTE|IF_SSE2},
    /* 1437 */ {I_COMISD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18003, IF_WILLAMETTE|IF_SSE2},
    /* 1438 */ {I_CVTDQ2PD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18009, IF_WILLAMETTE|IF_SSE2|IF_SQ},
    /* 1439 */ {I_CVTDQ2PS, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18015, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1440 */ {I_CVTPD2DQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18021, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1441 */ {I_CVTPD2PI, 2, {MMXREG,RM_XMM,0,0,0}, nasm_bytecodes+18027, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1442 */ {I_CVTPD2PS, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18033, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1443 */ {I_CVTPI2PD, 2, {XMMREG,RM_MMX,0,0,0}, nasm_bytecodes+18039, IF_WILLAMETTE|IF_SSE2|IF_SQ},
    /* 1444 */ {I_CVTPS2DQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18045, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1445 */ {I_CVTPS2PD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18051, IF_WILLAMETTE|IF_SSE2|IF_SQ},
    /* 1446 */ {I_CVTSD2SI, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+8762, IF_WILLAMETTE|IF_SSE2|IF_SQ|IF_AR1},
    /* 1447 */ {I_CVTSD2SI, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+8762, IF_WILLAMETTE|IF_SSE2|IF_SQ|IF_AR1},
    /* 1448 */ {I_CVTSD2SI, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+8769, IF_X64|IF_SSE2|IF_SQ|IF_AR1},
    /* 1449 */ {I_CVTSD2SI, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+8769, IF_X64|IF_SSE2|IF_SQ|IF_AR1},
    /* 1450 */ {I_CVTSD2SS, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18057, IF_WILLAMETTE|IF_SSE2|IF_SQ},
    /* 1451 */ {I_CVTSI2SD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+8776, IF_WILLAMETTE|IF_SSE2|IF_SD|IF_AR1},
    /* 1452 */ {I_CVTSI2SD, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+8783, IF_X64|IF_SSE2|IF_SQ|IF_AR1},
    /* 1453 */ {I_CVTSS2SD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18063, IF_WILLAMETTE|IF_SSE2|IF_SD},
    /* 1454 */ {I_CVTTPD2PI, 2, {MMXREG,RM_XMM,0,0,0}, nasm_bytecodes+18069, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1455 */ {I_CVTTPD2DQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18075, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1456 */ {I_CVTTPS2DQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18081, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1457 */ {I_CVTTSD2SI, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+8790, IF_WILLAMETTE|IF_SSE2|IF_SQ|IF_AR1},
    /* 1458 */ {I_CVTTSD2SI, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+8790, IF_WILLAMETTE|IF_SSE2|IF_SQ|IF_AR1},
    /* 1459 */ {I_CVTTSD2SI, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+8797, IF_X64|IF_SSE2|IF_SQ|IF_AR1},
    /* 1460 */ {I_CVTTSD2SI, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+8797, IF_X64|IF_SSE2|IF_SQ|IF_AR1},
    /* 1461 */ {I_DIVPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18087, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1462 */ {I_DIVSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18093, IF_WILLAMETTE|IF_SSE2},
    /* 1463 */ {I_MAXPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18099, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1464 */ {I_MAXSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18105, IF_WILLAMETTE|IF_SSE2},
    /* 1465 */ {I_MINPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18111, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1466 */ {I_MINSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18117, IF_WILLAMETTE|IF_SSE2},
    /* 1467 */ {I_MOVAPD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18123, IF_WILLAMETTE|IF_SSE2},
    /* 1468 */ {I_MOVAPD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18129, IF_WILLAMETTE|IF_SSE2},
    /* 1469 */ {I_MOVAPD, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+18129, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1470 */ {I_MOVAPD, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+18123, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1471 */ {I_MOVHPD, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+18135, IF_WILLAMETTE|IF_SSE2},
    /* 1472 */ {I_MOVHPD, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+18141, IF_WILLAMETTE|IF_SSE2},
    /* 1473 */ {I_MOVLPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+18147, IF_WILLAMETTE|IF_SSE2},
    /* 1474 */ {I_MOVLPD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+18153, IF_WILLAMETTE|IF_SSE2},
    /* 1475 */ {I_MOVMSKPD, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+18159, IF_WILLAMETTE|IF_SSE2},
    /* 1476 */ {I_MOVMSKPD, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+8804, IF_X64|IF_SSE2},
    /* 1477 */ {I_MOVSD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18165, IF_WILLAMETTE|IF_SSE2},
    /* 1478 */ {I_MOVSD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18171, IF_WILLAMETTE|IF_SSE2},
    /* 1479 */ {I_MOVSD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+18171, IF_WILLAMETTE|IF_SSE2},
    /* 1480 */ {I_MOVSD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+18165, IF_WILLAMETTE|IF_SSE2},
    /* 1481 */ {I_MOVUPD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18177, IF_WILLAMETTE|IF_SSE2},
    /* 1482 */ {I_MOVUPD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18183, IF_WILLAMETTE|IF_SSE2},
    /* 1483 */ {I_MOVUPD, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+18183, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1484 */ {I_MOVUPD, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+18177, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1485 */ {I_MULPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18189, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1486 */ {I_MULSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18195, IF_WILLAMETTE|IF_SSE2},
    /* 1487 */ {I_ORPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18201, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1488 */ {I_SHUFPD, 3, {XMMREG,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+8811, IF_WILLAMETTE|IF_SSE2|IF_SB|IF_AR2},
    /* 1489 */ {I_SHUFPD, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+8811, IF_WILLAMETTE|IF_SSE2|IF_SM|IF_SB|IF_AR2},
    /* 1490 */ {I_SQRTPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18207, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1491 */ {I_SQRTSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18213, IF_WILLAMETTE|IF_SSE2},
    /* 1492 */ {I_SUBPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18219, IF_WILLAMETTE|IF_SSE2|IF_SO},
    /* 1493 */ {I_SUBSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18225, IF_WILLAMETTE|IF_SSE2},
    /* 1494 */ {I_UCOMISD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18231, IF_WILLAMETTE|IF_SSE2},
    /* 1495 */ {I_UNPCKHPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+18237, IF_WILLAMETTE|IF_SSE2},
    /* 1496 */ {I_UNPCKLPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+18243, IF_WILLAMETTE|IF_SSE2},
    /* 1497 */ {I_XORPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+18249, IF_WILLAMETTE|IF_SSE2},
    /* 1498 */ {I_ADDSUBPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18255, IF_PRESCOTT|IF_SSE3|IF_SO},
    /* 1499 */ {I_ADDSUBPS, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18261, IF_PRESCOTT|IF_SSE3|IF_SO},
    /* 1500 */ {I_HADDPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18267, IF_PRESCOTT|IF_SSE3|IF_SO},
    /* 1501 */ {I_HADDPS, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18273, IF_PRESCOTT|IF_SSE3|IF_SO},
    /* 1502 */ {I_HSUBPD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18279, IF_PRESCOTT|IF_SSE3|IF_SO},
    /* 1503 */ {I_HSUBPS, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18285, IF_PRESCOTT|IF_SSE3|IF_SO},
    /* 1504 */ {I_LDDQU, 2, {XMMREG,MEMORY,0,0,0}, nasm_bytecodes+18291, IF_PRESCOTT|IF_SSE3|IF_SO},
    /* 1505 */ {I_MOVDDUP, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18297, IF_PRESCOTT|IF_SSE3},
    /* 1506 */ {I_MOVSHDUP, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18303, IF_PRESCOTT|IF_SSE3},
    /* 1507 */ {I_MOVSLDUP, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+18309, IF_PRESCOTT|IF_SSE3},
    /* 1508 */ {I_CLGI, 0, {0,0,0,0,0}, nasm_bytecodes+21277, IF_VMX|IF_AMD},
    /* 1509 */ {I_STGI, 0, {0,0,0,0,0}, nasm_bytecodes+21282, IF_VMX|IF_AMD},
    /* 1510 */ {I_VMCALL, 0, {0,0,0,0,0}, nasm_bytecodes+21287, IF_VMX},
    /* 1511 */ {I_VMCLEAR, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+18315, IF_VMX},
    /* 1512 */ {I_VMFUNC, 0, {0,0,0,0,0}, nasm_bytecodes+21292, IF_VMX},
    /* 1513 */ {I_VMLAUNCH, 0, {0,0,0,0,0}, nasm_bytecodes+21297, IF_VMX},
    /* 1514 */ {I_VMLOAD, 0, {0,0,0,0,0}, nasm_bytecodes+21302, IF_VMX|IF_AMD},
    /* 1515 */ {I_VMMCALL, 0, {0,0,0,0,0}, nasm_bytecodes+21307, IF_VMX|IF_AMD},
    /* 1516 */ {I_VMPTRLD, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+18321, IF_VMX},
    /* 1517 */ {I_VMPTRST, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+18327, IF_VMX},
    /* 1518 */ {I_VMREAD, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+8819, IF_VMX|IF_NOLONG|IF_SD},
    /* 1519 */ {I_VMREAD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+8818, IF_X64|IF_VMX|IF_SQ},
    /* 1520 */ {I_VMRESUME, 0, {0,0,0,0,0}, nasm_bytecodes+21312, IF_VMX},
    /* 1521 */ {I_VMRUN, 0, {0,0,0,0,0}, nasm_bytecodes+21317, IF_VMX|IF_AMD},
    /* 1522 */ {I_VMSAVE, 0, {0,0,0,0,0}, nasm_bytecodes+21322, IF_VMX|IF_AMD},
    /* 1523 */ {I_VMWRITE, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+8826, IF_VMX|IF_NOLONG|IF_SD},
    /* 1524 */ {I_VMWRITE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+8825, IF_X64|IF_VMX|IF_SQ},
    /* 1525 */ {I_VMXOFF, 0, {0,0,0,0,0}, nasm_bytecodes+21327, IF_VMX},
    /* 1526 */ {I_VMXON, 1, {MEMORY,0,0,0,0}, nasm_bytecodes+18333, IF_VMX},
    /* 1527 */ {I_INVEPT, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+4165, IF_VMX|IF_SO|IF_NOLONG},
    /* 1528 */ {I_INVEPT, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+4164, IF_VMX|IF_SO|IF_LONG},
    /* 1529 */ {I_INVVPID, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, nasm_bytecodes+4173, IF_VMX|IF_SO|IF_NOLONG},
    /* 1530 */ {I_INVVPID, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, nasm_bytecodes+4172, IF_VMX|IF_SO|IF_LONG},
    /* 1531 */ {I_PABSB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8832, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1532 */ {I_PABSB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8839, IF_SSSE3},
    /* 1533 */ {I_PABSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8846, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1534 */ {I_PABSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8853, IF_SSSE3},
    /* 1535 */ {I_PABSD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8860, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1536 */ {I_PABSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8867, IF_SSSE3},
    /* 1537 */ {I_PALIGNR, 3, {MMXREG,RM_MMX,IMMEDIATE,0,0}, nasm_bytecodes+4180, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1538 */ {I_PALIGNR, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4188, IF_SSSE3},
    /* 1539 */ {I_PHADDW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8874, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1540 */ {I_PHADDW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8881, IF_SSSE3},
    /* 1541 */ {I_PHADDD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8888, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1542 */ {I_PHADDD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8895, IF_SSSE3},
    /* 1543 */ {I_PHADDSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8902, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1544 */ {I_PHADDSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8909, IF_SSSE3},
    /* 1545 */ {I_PHSUBW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8916, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1546 */ {I_PHSUBW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8923, IF_SSSE3},
    /* 1547 */ {I_PHSUBD, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8930, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1548 */ {I_PHSUBD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8937, IF_SSSE3},
    /* 1549 */ {I_PHSUBSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8944, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1550 */ {I_PHSUBSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8951, IF_SSSE3},
    /* 1551 */ {I_PMADDUBSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8958, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1552 */ {I_PMADDUBSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8965, IF_SSSE3},
    /* 1553 */ {I_PMULHRSW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8972, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1554 */ {I_PMULHRSW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8979, IF_SSSE3},
    /* 1555 */ {I_PSHUFB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+8986, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1556 */ {I_PSHUFB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+8993, IF_SSSE3},
    /* 1557 */ {I_PSIGNB, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+9000, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1558 */ {I_PSIGNB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9007, IF_SSSE3},
    /* 1559 */ {I_PSIGNW, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+9014, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1560 */ {I_PSIGNW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9021, IF_SSSE3},
    /* 1561 */ {I_PSIGND, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+9028, IF_SSSE3|IF_MMX|IF_SQ},
    /* 1562 */ {I_PSIGND, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9035, IF_SSSE3},
    /* 1563 */ {I_EXTRQ, 3, {XMMREG,IMMEDIATE,IMMEDIATE,0,0}, nasm_bytecodes+4196, IF_SSE4A|IF_AMD},
    /* 1564 */ {I_EXTRQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18339, IF_SSE4A|IF_AMD},
    /* 1565 */ {I_INSERTQ, 4, {XMMREG,XMMREG,IMMEDIATE,IMMEDIATE,0}, nasm_bytecodes+4204, IF_SSE4A|IF_AMD},
    /* 1566 */ {I_INSERTQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+18345, IF_SSE4A|IF_AMD},
    /* 1567 */ {I_MOVNTSD, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+18351, IF_SSE4A|IF_AMD|IF_SQ},
    /* 1568 */ {I_MOVNTSS, 2, {MEMORY,XMMREG,0,0,0}, nasm_bytecodes+18357, IF_SSE4A|IF_AMD|IF_SD},
    /* 1569 */ {I_LZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+9042, IF_P6|IF_AMD},
    /* 1570 */ {I_LZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+9049, IF_P6|IF_AMD},
    /* 1571 */ {I_LZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+9056, IF_X64|IF_AMD},
    /* 1572 */ {I_BLENDPD, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4212, IF_SSE41},
    /* 1573 */ {I_BLENDPS, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4220, IF_SSE41},
    /* 1574 */ {I_BLENDVPD, 3, {XMMREG,RM_XMM,XMM0,0,0}, nasm_bytecodes+9063, IF_SSE41},
    /* 1575 */ {I_BLENDVPS, 3, {XMMREG,RM_XMM,XMM0,0,0}, nasm_bytecodes+9070, IF_SSE41},
    /* 1576 */ {I_DPPD, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4228, IF_SSE41},
    /* 1577 */ {I_DPPS, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4236, IF_SSE41},
    /* 1578 */ {I_EXTRACTPS, 3, {RM_GPR|BITS32,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+1, IF_SSE41},
    /* 1579 */ {I_EXTRACTPS, 3, {REG_GPR|BITS64,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+0, IF_SSE41|IF_X64},
    /* 1580 */ {I_INSERTPS, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4244, IF_SSE41|IF_SD},
    /* 1581 */ {I_MOVNTDQA, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+9077, IF_SSE41},
    /* 1582 */ {I_MPSADBW, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4252, IF_SSE41},
    /* 1583 */ {I_PACKUSDW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9084, IF_SSE41},
    /* 1584 */ {I_PBLENDVB, 3, {XMMREG,RM_XMM,XMM0,0,0}, nasm_bytecodes+9091, IF_SSE41},
    /* 1585 */ {I_PBLENDW, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4260, IF_SSE41},
    /* 1586 */ {I_PCMPEQQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9098, IF_SSE41},
    /* 1587 */ {I_PEXTRB, 3, {REG_GPR|BITS32,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+10, IF_SSE41},
    /* 1588 */ {I_PEXTRB, 3, {MEMORY|BITS8,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+10, IF_SSE41},
    /* 1589 */ {I_PEXTRB, 3, {REG_GPR|BITS64,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+9, IF_SSE41|IF_X64},
    /* 1590 */ {I_PEXTRD, 3, {RM_GPR|BITS32,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+18, IF_SSE41},
    /* 1591 */ {I_PEXTRQ, 3, {RM_GPR|BITS64,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+27, IF_SSE41|IF_X64},
    /* 1592 */ {I_PEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+37, IF_SSE41},
    /* 1593 */ {I_PEXTRW, 3, {MEMORY|BITS16,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+37, IF_SSE41},
    /* 1594 */ {I_PEXTRW, 3, {REG_GPR|BITS64,XMMREG,IMMEDIATE,0,0}, nasm_bytecodes+36, IF_SSE41|IF_X64},
    /* 1595 */ {I_PHMINPOSUW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9105, IF_SSE41},
    /* 1596 */ {I_PINSRB, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+46, IF_SSE41|IF_SB|IF_AR2},
    /* 1597 */ {I_PINSRB, 3, {XMMREG,RM_GPR|BITS8,IMMEDIATE,0,0}, nasm_bytecodes+45, IF_SSE41|IF_SB|IF_AR2},
    /* 1598 */ {I_PINSRB, 3, {XMMREG,REG_GPR|BITS32,IMMEDIATE,0,0}, nasm_bytecodes+46, IF_SSE41|IF_SB|IF_AR2},
    /* 1599 */ {I_PINSRD, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+54, IF_SSE41|IF_SB|IF_AR2},
    /* 1600 */ {I_PINSRD, 3, {XMMREG,RM_GPR|BITS32,IMMEDIATE,0,0}, nasm_bytecodes+54, IF_SSE41|IF_SB|IF_AR2},
    /* 1601 */ {I_PINSRQ, 3, {XMMREG,MEMORY,IMMEDIATE,0,0}, nasm_bytecodes+63, IF_SSE41|IF_X64|IF_SB|IF_AR2},
    /* 1602 */ {I_PINSRQ, 3, {XMMREG,RM_GPR|BITS64,IMMEDIATE,0,0}, nasm_bytecodes+63, IF_SSE41|IF_X64|IF_SB|IF_AR2},
    /* 1603 */ {I_PMAXSB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9112, IF_SSE41},
    /* 1604 */ {I_PMAXSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9119, IF_SSE41},
    /* 1605 */ {I_PMAXUD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9126, IF_SSE41},
    /* 1606 */ {I_PMAXUW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9133, IF_SSE41},
    /* 1607 */ {I_PMINSB, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9140, IF_SSE41},
    /* 1608 */ {I_PMINSD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9147, IF_SSE41},
    /* 1609 */ {I_PMINUD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9154, IF_SSE41},
    /* 1610 */ {I_PMINUW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9161, IF_SSE41},
    /* 1611 */ {I_PMOVSXBW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9168, IF_SSE41|IF_SQ},
    /* 1612 */ {I_PMOVSXBD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9175, IF_SSE41|IF_SD},
    /* 1613 */ {I_PMOVSXBQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9182, IF_SSE41|IF_SW},
    /* 1614 */ {I_PMOVSXWD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9189, IF_SSE41|IF_SQ},
    /* 1615 */ {I_PMOVSXWQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9196, IF_SSE41|IF_SD},
    /* 1616 */ {I_PMOVSXDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9203, IF_SSE41|IF_SQ},
    /* 1617 */ {I_PMOVZXBW, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9210, IF_SSE41|IF_SQ},
    /* 1618 */ {I_PMOVZXBD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9217, IF_SSE41|IF_SD},
    /* 1619 */ {I_PMOVZXBQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9224, IF_SSE41|IF_SW},
    /* 1620 */ {I_PMOVZXWD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9231, IF_SSE41|IF_SQ},
    /* 1621 */ {I_PMOVZXWQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9238, IF_SSE41|IF_SD},
    /* 1622 */ {I_PMOVZXDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9245, IF_SSE41|IF_SQ},
    /* 1623 */ {I_PMULDQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9252, IF_SSE41},
    /* 1624 */ {I_PMULLD, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9259, IF_SSE41},
    /* 1625 */ {I_PTEST, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9266, IF_SSE41},
    /* 1626 */ {I_ROUNDPD, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4268, IF_SSE41},
    /* 1627 */ {I_ROUNDPS, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4276, IF_SSE41},
    /* 1628 */ {I_ROUNDSD, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4284, IF_SSE41},
    /* 1629 */ {I_ROUNDSS, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4292, IF_SSE41},
    /* 1630 */ {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, nasm_bytecodes+4317, IF_SSE42},
    /* 1631 */ {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+4300, IF_SSE42},
    /* 1632 */ {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+4308, IF_SSE42},
    /* 1633 */ {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, nasm_bytecodes+4316, IF_SSE42|IF_X64},
    /* 1634 */ {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+4324, IF_SSE42|IF_X64},
    /* 1635 */ {I_PCMPESTRI, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4332, IF_SSE42},
    /* 1636 */ {I_PCMPESTRM, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4340, IF_SSE42},
    /* 1637 */ {I_PCMPISTRI, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4348, IF_SSE42},
    /* 1638 */ {I_PCMPISTRM, 3, {XMMREG,RM_XMM,IMMEDIATE,0,0}, nasm_bytecodes+4356, IF_SSE42},
    /* 1639 */ {I_PCMPGTQ, 2, {XMMREG,RM_XMM,0,0,0}, nasm_bytecodes+9273, IF_SSE42},
    /* 1640 */ {I_POPCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+9280, IF_NEHALEM|IF_SW},
    /* 1641 */ {I_POPCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+9287, IF_NEHALEM|IF_SD},
    /* 1642 */ {I_POPCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+9294, IF_NEHALEM|IF_SQ|IF_X64},
    /* 1643 */ {I_GETSEC, 0, {0,0,0,0,0}, nasm_bytecodes+22350, IF_KATMAI},
    /* 1644 */ {I_PFRCPV, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+4364, IF_PENT|IF_3DNOW|IF_SQ|IF_CYRIX},
    /* 1645 */ {I_PFRSQRTV, 2, {MMXREG,RM_MMX,0,0,0}, nasm_bytecodes+4372, IF_PENT|IF_3DNOW|IF_SQ|IF_CYRIX},
    /* 1646 */ {I_MOVBE, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, nasm_bytecodes+4380, IF_NEHALEM|IF_SM},
    /* 1647 */ {I_MOVBE, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, nasm_bytecodes+4388, IF_NEHALEM|IF_SM},
    /* 1648 */ {I_MOVBE, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, nasm_bytecodes+4396, IF_NEHALEM|IF_SM},
    /* 1649 */ {I_MOVBE, 2, {MEMORY|BITS16,REG_GPR|BITS16,0,0,0}, nasm_bytecodes+4404, IF_NEHALEM|IF_SM},
    /* 1650 */ {I_MOVBE, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, nasm_bytecodes+4412, IF_NEHALEM|IF_SM},
    /* 1651 */ {I_MOVBE, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, nasm_bytecodes+4420, IF_NEHALEM|IF_SM},
    /* 1652 */ {I_AESENC, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9301, IF_SSE|IF_WESTMERE},
    /* 1653 */ {I_AESENCLAST, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9308, IF_SSE|IF_WESTMERE},
    /* 1654 */ {I_AESDEC, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9315, IF_SSE|IF_WESTMERE},
    /* 1655 */ {I_AESDECLAST, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9322, IF_SSE|IF_WESTMERE},
    /* 1656 */ {I_AESIMC, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9329, IF_SSE|IF_WESTMERE},
    /* 1657 */ {I_AESKEYGENASSIST, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4428, IF_SSE|IF_WESTMERE},
    /* 1658 */ {I_VAESENC, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9336, IF_AVX|IF_SANDYBRIDGE},
    /* 1659 */ {I_VAESENC, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9343, IF_AVX|IF_SANDYBRIDGE},
    /* 1660 */ {I_VAESENCLAST, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9350, IF_AVX|IF_SANDYBRIDGE},
    /* 1661 */ {I_VAESENCLAST, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9357, IF_AVX|IF_SANDYBRIDGE},
    /* 1662 */ {I_VAESDEC, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9364, IF_AVX|IF_SANDYBRIDGE},
    /* 1663 */ {I_VAESDEC, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9371, IF_AVX|IF_SANDYBRIDGE},
    /* 1664 */ {I_VAESDECLAST, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9378, IF_AVX|IF_SANDYBRIDGE},
    /* 1665 */ {I_VAESDECLAST, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9385, IF_AVX|IF_SANDYBRIDGE},
    /* 1666 */ {I_VAESIMC, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9392, IF_AVX|IF_SANDYBRIDGE},
    /* 1667 */ {I_VAESKEYGENASSIST, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4436, IF_AVX|IF_SANDYBRIDGE},
    /* 1668 */ {I_VADDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9399, IF_AVX|IF_SANDYBRIDGE},
    /* 1669 */ {I_VADDPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9406, IF_AVX|IF_SANDYBRIDGE},
    /* 1670 */ {I_VADDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9413, IF_AVX|IF_SANDYBRIDGE},
    /* 1671 */ {I_VADDPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9420, IF_AVX|IF_SANDYBRIDGE},
    /* 1672 */ {I_VADDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9427, IF_AVX|IF_SANDYBRIDGE},
    /* 1673 */ {I_VADDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9434, IF_AVX|IF_SANDYBRIDGE},
    /* 1674 */ {I_VADDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9441, IF_AVX|IF_SANDYBRIDGE},
    /* 1675 */ {I_VADDPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9448, IF_AVX|IF_SANDYBRIDGE},
    /* 1676 */ {I_VADDSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+9455, IF_AVX|IF_SANDYBRIDGE},
    /* 1677 */ {I_VADDSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9462, IF_AVX|IF_SANDYBRIDGE},
    /* 1678 */ {I_VADDSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+9469, IF_AVX|IF_SANDYBRIDGE},
    /* 1679 */ {I_VADDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+9476, IF_AVX|IF_SANDYBRIDGE},
    /* 1680 */ {I_VADDSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9483, IF_AVX|IF_SANDYBRIDGE},
    /* 1681 */ {I_VADDSUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9490, IF_AVX|IF_SANDYBRIDGE},
    /* 1682 */ {I_VADDSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9497, IF_AVX|IF_SANDYBRIDGE},
    /* 1683 */ {I_VADDSUBPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9504, IF_AVX|IF_SANDYBRIDGE},
    /* 1684 */ {I_VADDSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9511, IF_AVX|IF_SANDYBRIDGE},
    /* 1685 */ {I_VADDSUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9518, IF_AVX|IF_SANDYBRIDGE},
    /* 1686 */ {I_VADDSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9525, IF_AVX|IF_SANDYBRIDGE},
    /* 1687 */ {I_VADDSUBPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9532, IF_AVX|IF_SANDYBRIDGE},
    /* 1688 */ {I_VANDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9539, IF_AVX|IF_SANDYBRIDGE},
    /* 1689 */ {I_VANDPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9546, IF_AVX|IF_SANDYBRIDGE},
    /* 1690 */ {I_VANDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9553, IF_AVX|IF_SANDYBRIDGE},
    /* 1691 */ {I_VANDPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9560, IF_AVX|IF_SANDYBRIDGE},
    /* 1692 */ {I_VANDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9567, IF_AVX|IF_SANDYBRIDGE},
    /* 1693 */ {I_VANDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9574, IF_AVX|IF_SANDYBRIDGE},
    /* 1694 */ {I_VANDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9581, IF_AVX|IF_SANDYBRIDGE},
    /* 1695 */ {I_VANDPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9588, IF_AVX|IF_SANDYBRIDGE},
    /* 1696 */ {I_VANDNPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9595, IF_AVX|IF_SANDYBRIDGE},
    /* 1697 */ {I_VANDNPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9602, IF_AVX|IF_SANDYBRIDGE},
    /* 1698 */ {I_VANDNPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9609, IF_AVX|IF_SANDYBRIDGE},
    /* 1699 */ {I_VANDNPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9616, IF_AVX|IF_SANDYBRIDGE},
    /* 1700 */ {I_VANDNPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9623, IF_AVX|IF_SANDYBRIDGE},
    /* 1701 */ {I_VANDNPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9630, IF_AVX|IF_SANDYBRIDGE},
    /* 1702 */ {I_VANDNPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9637, IF_AVX|IF_SANDYBRIDGE},
    /* 1703 */ {I_VANDNPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9644, IF_AVX|IF_SANDYBRIDGE},
    /* 1704 */ {I_VBLENDPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4444, IF_AVX|IF_SANDYBRIDGE},
    /* 1705 */ {I_VBLENDPD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4452, IF_AVX|IF_SANDYBRIDGE},
    /* 1706 */ {I_VBLENDPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+4460, IF_AVX|IF_SANDYBRIDGE},
    /* 1707 */ {I_VBLENDPD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4468, IF_AVX|IF_SANDYBRIDGE},
    /* 1708 */ {I_VBLENDPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4476, IF_AVX|IF_SANDYBRIDGE},
    /* 1709 */ {I_VBLENDPS, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4484, IF_AVX|IF_SANDYBRIDGE},
    /* 1710 */ {I_VBLENDPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+4492, IF_AVX|IF_SANDYBRIDGE},
    /* 1711 */ {I_VBLENDPS, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4500, IF_AVX|IF_SANDYBRIDGE},
    /* 1712 */ {I_VBLENDVPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+4508, IF_AVX|IF_SANDYBRIDGE},
    /* 1713 */ {I_VBLENDVPD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+4516, IF_AVX|IF_SANDYBRIDGE},
    /* 1714 */ {I_VBLENDVPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+4524, IF_AVX|IF_SANDYBRIDGE},
    /* 1715 */ {I_VBLENDVPD, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+4532, IF_AVX|IF_SANDYBRIDGE},
    /* 1716 */ {I_VBLENDVPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+4540, IF_AVX|IF_SANDYBRIDGE},
    /* 1717 */ {I_VBLENDVPS, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+4548, IF_AVX|IF_SANDYBRIDGE},
    /* 1718 */ {I_VBLENDVPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+4556, IF_AVX|IF_SANDYBRIDGE},
    /* 1719 */ {I_VBLENDVPS, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+4564, IF_AVX|IF_SANDYBRIDGE},
    /* 1720 */ {I_VBROADCASTSS, 2, {XMMREG,MEMORY|BITS32,0,0,0}, nasm_bytecodes+9651, IF_AVX|IF_SANDYBRIDGE},
    /* 1721 */ {I_VBROADCASTSS, 2, {YMMREG,MEMORY|BITS32,0,0,0}, nasm_bytecodes+9658, IF_AVX|IF_SANDYBRIDGE},
    /* 1722 */ {I_VBROADCASTSD, 2, {YMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+9665, IF_AVX|IF_SANDYBRIDGE},
    /* 1723 */ {I_VBROADCASTF128, 2, {YMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+9672, IF_AVX|IF_SANDYBRIDGE},
    /* 1724 */ {I_VCMPEQ_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+72, IF_AVX|IF_SANDYBRIDGE},
    /* 1725 */ {I_VCMPEQ_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+81, IF_AVX|IF_SANDYBRIDGE},
    /* 1726 */ {I_VCMPEQ_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+90, IF_AVX|IF_SANDYBRIDGE},
    /* 1727 */ {I_VCMPEQ_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+99, IF_AVX|IF_SANDYBRIDGE},
    /* 1728 */ {I_VCMPEQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+108, IF_AVX|IF_SANDYBRIDGE},
    /* 1729 */ {I_VCMPEQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+117, IF_AVX|IF_SANDYBRIDGE},
    /* 1730 */ {I_VCMPEQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+126, IF_AVX|IF_SANDYBRIDGE},
    /* 1731 */ {I_VCMPEQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+135, IF_AVX|IF_SANDYBRIDGE},
    /* 1732 */ {I_VCMPLT_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+144, IF_AVX|IF_SANDYBRIDGE},
    /* 1733 */ {I_VCMPLT_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+153, IF_AVX|IF_SANDYBRIDGE},
    /* 1734 */ {I_VCMPLT_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+162, IF_AVX|IF_SANDYBRIDGE},
    /* 1735 */ {I_VCMPLT_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+171, IF_AVX|IF_SANDYBRIDGE},
    /* 1736 */ {I_VCMPLTPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+144, IF_AVX|IF_SANDYBRIDGE},
    /* 1737 */ {I_VCMPLTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+153, IF_AVX|IF_SANDYBRIDGE},
    /* 1738 */ {I_VCMPLTPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+162, IF_AVX|IF_SANDYBRIDGE},
    /* 1739 */ {I_VCMPLTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+171, IF_AVX|IF_SANDYBRIDGE},
    /* 1740 */ {I_VCMPLE_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+180, IF_AVX|IF_SANDYBRIDGE},
    /* 1741 */ {I_VCMPLE_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+189, IF_AVX|IF_SANDYBRIDGE},
    /* 1742 */ {I_VCMPLE_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+198, IF_AVX|IF_SANDYBRIDGE},
    /* 1743 */ {I_VCMPLE_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+207, IF_AVX|IF_SANDYBRIDGE},
    /* 1744 */ {I_VCMPLEPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+180, IF_AVX|IF_SANDYBRIDGE},
    /* 1745 */ {I_VCMPLEPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+189, IF_AVX|IF_SANDYBRIDGE},
    /* 1746 */ {I_VCMPLEPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+198, IF_AVX|IF_SANDYBRIDGE},
    /* 1747 */ {I_VCMPLEPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+207, IF_AVX|IF_SANDYBRIDGE},
    /* 1748 */ {I_VCMPUNORD_QPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+216, IF_AVX|IF_SANDYBRIDGE},
    /* 1749 */ {I_VCMPUNORD_QPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+225, IF_AVX|IF_SANDYBRIDGE},
    /* 1750 */ {I_VCMPUNORD_QPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+234, IF_AVX|IF_SANDYBRIDGE},
    /* 1751 */ {I_VCMPUNORD_QPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+243, IF_AVX|IF_SANDYBRIDGE},
    /* 1752 */ {I_VCMPUNORDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+216, IF_AVX|IF_SANDYBRIDGE},
    /* 1753 */ {I_VCMPUNORDPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+225, IF_AVX|IF_SANDYBRIDGE},
    /* 1754 */ {I_VCMPUNORDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+234, IF_AVX|IF_SANDYBRIDGE},
    /* 1755 */ {I_VCMPUNORDPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+243, IF_AVX|IF_SANDYBRIDGE},
    /* 1756 */ {I_VCMPNEQ_UQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+252, IF_AVX|IF_SANDYBRIDGE},
    /* 1757 */ {I_VCMPNEQ_UQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+261, IF_AVX|IF_SANDYBRIDGE},
    /* 1758 */ {I_VCMPNEQ_UQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+270, IF_AVX|IF_SANDYBRIDGE},
    /* 1759 */ {I_VCMPNEQ_UQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+279, IF_AVX|IF_SANDYBRIDGE},
    /* 1760 */ {I_VCMPNEQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+252, IF_AVX|IF_SANDYBRIDGE},
    /* 1761 */ {I_VCMPNEQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+261, IF_AVX|IF_SANDYBRIDGE},
    /* 1762 */ {I_VCMPNEQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+270, IF_AVX|IF_SANDYBRIDGE},
    /* 1763 */ {I_VCMPNEQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+279, IF_AVX|IF_SANDYBRIDGE},
    /* 1764 */ {I_VCMPNLT_USPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+288, IF_AVX|IF_SANDYBRIDGE},
    /* 1765 */ {I_VCMPNLT_USPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+297, IF_AVX|IF_SANDYBRIDGE},
    /* 1766 */ {I_VCMPNLT_USPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+306, IF_AVX|IF_SANDYBRIDGE},
    /* 1767 */ {I_VCMPNLT_USPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+315, IF_AVX|IF_SANDYBRIDGE},
    /* 1768 */ {I_VCMPNLTPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+288, IF_AVX|IF_SANDYBRIDGE},
    /* 1769 */ {I_VCMPNLTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+297, IF_AVX|IF_SANDYBRIDGE},
    /* 1770 */ {I_VCMPNLTPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+306, IF_AVX|IF_SANDYBRIDGE},
    /* 1771 */ {I_VCMPNLTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+315, IF_AVX|IF_SANDYBRIDGE},
    /* 1772 */ {I_VCMPNLE_USPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+324, IF_AVX|IF_SANDYBRIDGE},
    /* 1773 */ {I_VCMPNLE_USPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+333, IF_AVX|IF_SANDYBRIDGE},
    /* 1774 */ {I_VCMPNLE_USPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+342, IF_AVX|IF_SANDYBRIDGE},
    /* 1775 */ {I_VCMPNLE_USPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+351, IF_AVX|IF_SANDYBRIDGE},
    /* 1776 */ {I_VCMPNLEPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+324, IF_AVX|IF_SANDYBRIDGE},
    /* 1777 */ {I_VCMPNLEPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+333, IF_AVX|IF_SANDYBRIDGE},
    /* 1778 */ {I_VCMPNLEPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+342, IF_AVX|IF_SANDYBRIDGE},
    /* 1779 */ {I_VCMPNLEPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+351, IF_AVX|IF_SANDYBRIDGE},
    /* 1780 */ {I_VCMPORD_QPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+360, IF_AVX|IF_SANDYBRIDGE},
    /* 1781 */ {I_VCMPORD_QPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+369, IF_AVX|IF_SANDYBRIDGE},
    /* 1782 */ {I_VCMPORD_QPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+378, IF_AVX|IF_SANDYBRIDGE},
    /* 1783 */ {I_VCMPORD_QPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+387, IF_AVX|IF_SANDYBRIDGE},
    /* 1784 */ {I_VCMPORDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+360, IF_AVX|IF_SANDYBRIDGE},
    /* 1785 */ {I_VCMPORDPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+369, IF_AVX|IF_SANDYBRIDGE},
    /* 1786 */ {I_VCMPORDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+378, IF_AVX|IF_SANDYBRIDGE},
    /* 1787 */ {I_VCMPORDPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+387, IF_AVX|IF_SANDYBRIDGE},
    /* 1788 */ {I_VCMPEQ_UQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+396, IF_AVX|IF_SANDYBRIDGE},
    /* 1789 */ {I_VCMPEQ_UQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+405, IF_AVX|IF_SANDYBRIDGE},
    /* 1790 */ {I_VCMPEQ_UQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+414, IF_AVX|IF_SANDYBRIDGE},
    /* 1791 */ {I_VCMPEQ_UQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+423, IF_AVX|IF_SANDYBRIDGE},
    /* 1792 */ {I_VCMPNGE_USPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+432, IF_AVX|IF_SANDYBRIDGE},
    /* 1793 */ {I_VCMPNGE_USPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+441, IF_AVX|IF_SANDYBRIDGE},
    /* 1794 */ {I_VCMPNGE_USPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+450, IF_AVX|IF_SANDYBRIDGE},
    /* 1795 */ {I_VCMPNGE_USPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+459, IF_AVX|IF_SANDYBRIDGE},
    /* 1796 */ {I_VCMPNGEPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+432, IF_AVX|IF_SANDYBRIDGE},
    /* 1797 */ {I_VCMPNGEPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+441, IF_AVX|IF_SANDYBRIDGE},
    /* 1798 */ {I_VCMPNGEPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+450, IF_AVX|IF_SANDYBRIDGE},
    /* 1799 */ {I_VCMPNGEPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+459, IF_AVX|IF_SANDYBRIDGE},
    /* 1800 */ {I_VCMPNGT_USPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+468, IF_AVX|IF_SANDYBRIDGE},
    /* 1801 */ {I_VCMPNGT_USPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+477, IF_AVX|IF_SANDYBRIDGE},
    /* 1802 */ {I_VCMPNGT_USPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+486, IF_AVX|IF_SANDYBRIDGE},
    /* 1803 */ {I_VCMPNGT_USPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+495, IF_AVX|IF_SANDYBRIDGE},
    /* 1804 */ {I_VCMPNGTPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+468, IF_AVX|IF_SANDYBRIDGE},
    /* 1805 */ {I_VCMPNGTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+477, IF_AVX|IF_SANDYBRIDGE},
    /* 1806 */ {I_VCMPNGTPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+486, IF_AVX|IF_SANDYBRIDGE},
    /* 1807 */ {I_VCMPNGTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+495, IF_AVX|IF_SANDYBRIDGE},
    /* 1808 */ {I_VCMPFALSE_OQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+504, IF_AVX|IF_SANDYBRIDGE},
    /* 1809 */ {I_VCMPFALSE_OQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+513, IF_AVX|IF_SANDYBRIDGE},
    /* 1810 */ {I_VCMPFALSE_OQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+522, IF_AVX|IF_SANDYBRIDGE},
    /* 1811 */ {I_VCMPFALSE_OQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+531, IF_AVX|IF_SANDYBRIDGE},
    /* 1812 */ {I_VCMPFALSEPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+504, IF_AVX|IF_SANDYBRIDGE},
    /* 1813 */ {I_VCMPFALSEPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+513, IF_AVX|IF_SANDYBRIDGE},
    /* 1814 */ {I_VCMPFALSEPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+522, IF_AVX|IF_SANDYBRIDGE},
    /* 1815 */ {I_VCMPFALSEPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+531, IF_AVX|IF_SANDYBRIDGE},
    /* 1816 */ {I_VCMPNEQ_OQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+540, IF_AVX|IF_SANDYBRIDGE},
    /* 1817 */ {I_VCMPNEQ_OQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+549, IF_AVX|IF_SANDYBRIDGE},
    /* 1818 */ {I_VCMPNEQ_OQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+558, IF_AVX|IF_SANDYBRIDGE},
    /* 1819 */ {I_VCMPNEQ_OQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+567, IF_AVX|IF_SANDYBRIDGE},
    /* 1820 */ {I_VCMPGE_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+576, IF_AVX|IF_SANDYBRIDGE},
    /* 1821 */ {I_VCMPGE_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+585, IF_AVX|IF_SANDYBRIDGE},
    /* 1822 */ {I_VCMPGE_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+594, IF_AVX|IF_SANDYBRIDGE},
    /* 1823 */ {I_VCMPGE_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+603, IF_AVX|IF_SANDYBRIDGE},
    /* 1824 */ {I_VCMPGEPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+576, IF_AVX|IF_SANDYBRIDGE},
    /* 1825 */ {I_VCMPGEPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+585, IF_AVX|IF_SANDYBRIDGE},
    /* 1826 */ {I_VCMPGEPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+594, IF_AVX|IF_SANDYBRIDGE},
    /* 1827 */ {I_VCMPGEPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+603, IF_AVX|IF_SANDYBRIDGE},
    /* 1828 */ {I_VCMPGT_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+612, IF_AVX|IF_SANDYBRIDGE},
    /* 1829 */ {I_VCMPGT_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+621, IF_AVX|IF_SANDYBRIDGE},
    /* 1830 */ {I_VCMPGT_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+630, IF_AVX|IF_SANDYBRIDGE},
    /* 1831 */ {I_VCMPGT_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+639, IF_AVX|IF_SANDYBRIDGE},
    /* 1832 */ {I_VCMPGTPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+612, IF_AVX|IF_SANDYBRIDGE},
    /* 1833 */ {I_VCMPGTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+621, IF_AVX|IF_SANDYBRIDGE},
    /* 1834 */ {I_VCMPGTPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+630, IF_AVX|IF_SANDYBRIDGE},
    /* 1835 */ {I_VCMPGTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+639, IF_AVX|IF_SANDYBRIDGE},
    /* 1836 */ {I_VCMPTRUE_UQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+648, IF_AVX|IF_SANDYBRIDGE},
    /* 1837 */ {I_VCMPTRUE_UQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+657, IF_AVX|IF_SANDYBRIDGE},
    /* 1838 */ {I_VCMPTRUE_UQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+666, IF_AVX|IF_SANDYBRIDGE},
    /* 1839 */ {I_VCMPTRUE_UQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+675, IF_AVX|IF_SANDYBRIDGE},
    /* 1840 */ {I_VCMPTRUEPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+648, IF_AVX|IF_SANDYBRIDGE},
    /* 1841 */ {I_VCMPTRUEPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+657, IF_AVX|IF_SANDYBRIDGE},
    /* 1842 */ {I_VCMPTRUEPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+666, IF_AVX|IF_SANDYBRIDGE},
    /* 1843 */ {I_VCMPTRUEPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+675, IF_AVX|IF_SANDYBRIDGE},
    /* 1844 */ {I_VCMPEQ_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+72, IF_AVX|IF_SANDYBRIDGE},
    /* 1845 */ {I_VCMPEQ_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+81, IF_AVX|IF_SANDYBRIDGE},
    /* 1846 */ {I_VCMPEQ_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+90, IF_AVX|IF_SANDYBRIDGE},
    /* 1847 */ {I_VCMPEQ_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+99, IF_AVX|IF_SANDYBRIDGE},
    /* 1848 */ {I_VCMPLT_OQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+684, IF_AVX|IF_SANDYBRIDGE},
    /* 1849 */ {I_VCMPLT_OQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+693, IF_AVX|IF_SANDYBRIDGE},
    /* 1850 */ {I_VCMPLT_OQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+702, IF_AVX|IF_SANDYBRIDGE},
    /* 1851 */ {I_VCMPLT_OQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+711, IF_AVX|IF_SANDYBRIDGE},
    /* 1852 */ {I_VCMPLE_OQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+720, IF_AVX|IF_SANDYBRIDGE},
    /* 1853 */ {I_VCMPLE_OQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+729, IF_AVX|IF_SANDYBRIDGE},
    /* 1854 */ {I_VCMPLE_OQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+738, IF_AVX|IF_SANDYBRIDGE},
    /* 1855 */ {I_VCMPLE_OQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+747, IF_AVX|IF_SANDYBRIDGE},
    /* 1856 */ {I_VCMPUNORD_SPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+756, IF_AVX|IF_SANDYBRIDGE},
    /* 1857 */ {I_VCMPUNORD_SPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+765, IF_AVX|IF_SANDYBRIDGE},
    /* 1858 */ {I_VCMPUNORD_SPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+774, IF_AVX|IF_SANDYBRIDGE},
    /* 1859 */ {I_VCMPUNORD_SPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+783, IF_AVX|IF_SANDYBRIDGE},
    /* 1860 */ {I_VCMPNEQ_USPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+792, IF_AVX|IF_SANDYBRIDGE},
    /* 1861 */ {I_VCMPNEQ_USPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+801, IF_AVX|IF_SANDYBRIDGE},
    /* 1862 */ {I_VCMPNEQ_USPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+810, IF_AVX|IF_SANDYBRIDGE},
    /* 1863 */ {I_VCMPNEQ_USPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+819, IF_AVX|IF_SANDYBRIDGE},
    /* 1864 */ {I_VCMPNLT_UQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+828, IF_AVX|IF_SANDYBRIDGE},
    /* 1865 */ {I_VCMPNLT_UQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+837, IF_AVX|IF_SANDYBRIDGE},
    /* 1866 */ {I_VCMPNLT_UQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+846, IF_AVX|IF_SANDYBRIDGE},
    /* 1867 */ {I_VCMPNLT_UQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+855, IF_AVX|IF_SANDYBRIDGE},
    /* 1868 */ {I_VCMPNLE_UQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+864, IF_AVX|IF_SANDYBRIDGE},
    /* 1869 */ {I_VCMPNLE_UQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+873, IF_AVX|IF_SANDYBRIDGE},
    /* 1870 */ {I_VCMPNLE_UQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+882, IF_AVX|IF_SANDYBRIDGE},
    /* 1871 */ {I_VCMPNLE_UQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+891, IF_AVX|IF_SANDYBRIDGE},
    /* 1872 */ {I_VCMPORD_SPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+900, IF_AVX|IF_SANDYBRIDGE},
    /* 1873 */ {I_VCMPORD_SPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+909, IF_AVX|IF_SANDYBRIDGE},
    /* 1874 */ {I_VCMPORD_SPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+918, IF_AVX|IF_SANDYBRIDGE},
    /* 1875 */ {I_VCMPORD_SPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+927, IF_AVX|IF_SANDYBRIDGE},
    /* 1876 */ {I_VCMPEQ_USPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+936, IF_AVX|IF_SANDYBRIDGE},
    /* 1877 */ {I_VCMPEQ_USPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+945, IF_AVX|IF_SANDYBRIDGE},
    /* 1878 */ {I_VCMPEQ_USPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+954, IF_AVX|IF_SANDYBRIDGE},
    /* 1879 */ {I_VCMPEQ_USPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+963, IF_AVX|IF_SANDYBRIDGE},
    /* 1880 */ {I_VCMPNGE_UQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+972, IF_AVX|IF_SANDYBRIDGE},
    /* 1881 */ {I_VCMPNGE_UQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+981, IF_AVX|IF_SANDYBRIDGE},
    /* 1882 */ {I_VCMPNGE_UQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+990, IF_AVX|IF_SANDYBRIDGE},
    /* 1883 */ {I_VCMPNGE_UQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+999, IF_AVX|IF_SANDYBRIDGE},
    /* 1884 */ {I_VCMPNGT_UQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1008, IF_AVX|IF_SANDYBRIDGE},
    /* 1885 */ {I_VCMPNGT_UQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1017, IF_AVX|IF_SANDYBRIDGE},
    /* 1886 */ {I_VCMPNGT_UQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1026, IF_AVX|IF_SANDYBRIDGE},
    /* 1887 */ {I_VCMPNGT_UQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1035, IF_AVX|IF_SANDYBRIDGE},
    /* 1888 */ {I_VCMPFALSE_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1044, IF_AVX|IF_SANDYBRIDGE},
    /* 1889 */ {I_VCMPFALSE_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1053, IF_AVX|IF_SANDYBRIDGE},
    /* 1890 */ {I_VCMPFALSE_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1062, IF_AVX|IF_SANDYBRIDGE},
    /* 1891 */ {I_VCMPFALSE_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1071, IF_AVX|IF_SANDYBRIDGE},
    /* 1892 */ {I_VCMPNEQ_OSPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1080, IF_AVX|IF_SANDYBRIDGE},
    /* 1893 */ {I_VCMPNEQ_OSPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1089, IF_AVX|IF_SANDYBRIDGE},
    /* 1894 */ {I_VCMPNEQ_OSPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1098, IF_AVX|IF_SANDYBRIDGE},
    /* 1895 */ {I_VCMPNEQ_OSPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1107, IF_AVX|IF_SANDYBRIDGE},
    /* 1896 */ {I_VCMPGE_OQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1116, IF_AVX|IF_SANDYBRIDGE},
    /* 1897 */ {I_VCMPGE_OQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1125, IF_AVX|IF_SANDYBRIDGE},
    /* 1898 */ {I_VCMPGE_OQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1134, IF_AVX|IF_SANDYBRIDGE},
    /* 1899 */ {I_VCMPGE_OQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1143, IF_AVX|IF_SANDYBRIDGE},
    /* 1900 */ {I_VCMPGT_OQPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1152, IF_AVX|IF_SANDYBRIDGE},
    /* 1901 */ {I_VCMPGT_OQPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1161, IF_AVX|IF_SANDYBRIDGE},
    /* 1902 */ {I_VCMPGT_OQPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1170, IF_AVX|IF_SANDYBRIDGE},
    /* 1903 */ {I_VCMPGT_OQPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1179, IF_AVX|IF_SANDYBRIDGE},
    /* 1904 */ {I_VCMPTRUE_USPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1188, IF_AVX|IF_SANDYBRIDGE},
    /* 1905 */ {I_VCMPTRUE_USPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1197, IF_AVX|IF_SANDYBRIDGE},
    /* 1906 */ {I_VCMPTRUE_USPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1206, IF_AVX|IF_SANDYBRIDGE},
    /* 1907 */ {I_VCMPTRUE_USPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1215, IF_AVX|IF_SANDYBRIDGE},
    /* 1908 */ {I_VCMPPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4572, IF_AVX|IF_SANDYBRIDGE},
    /* 1909 */ {I_VCMPPD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4580, IF_AVX|IF_SANDYBRIDGE},
    /* 1910 */ {I_VCMPPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+4588, IF_AVX|IF_SANDYBRIDGE},
    /* 1911 */ {I_VCMPPD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4596, IF_AVX|IF_SANDYBRIDGE},
    /* 1912 */ {I_VCMPEQ_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1224, IF_AVX|IF_SANDYBRIDGE},
    /* 1913 */ {I_VCMPEQ_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1233, IF_AVX|IF_SANDYBRIDGE},
    /* 1914 */ {I_VCMPEQ_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1242, IF_AVX|IF_SANDYBRIDGE},
    /* 1915 */ {I_VCMPEQ_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1251, IF_AVX|IF_SANDYBRIDGE},
    /* 1916 */ {I_VCMPEQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1260, IF_AVX|IF_SANDYBRIDGE},
    /* 1917 */ {I_VCMPEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1269, IF_AVX|IF_SANDYBRIDGE},
    /* 1918 */ {I_VCMPEQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1278, IF_AVX|IF_SANDYBRIDGE},
    /* 1919 */ {I_VCMPEQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1287, IF_AVX|IF_SANDYBRIDGE},
    /* 1920 */ {I_VCMPLT_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1296, IF_AVX|IF_SANDYBRIDGE},
    /* 1921 */ {I_VCMPLT_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1305, IF_AVX|IF_SANDYBRIDGE},
    /* 1922 */ {I_VCMPLT_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1314, IF_AVX|IF_SANDYBRIDGE},
    /* 1923 */ {I_VCMPLT_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1323, IF_AVX|IF_SANDYBRIDGE},
    /* 1924 */ {I_VCMPLTPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1296, IF_AVX|IF_SANDYBRIDGE},
    /* 1925 */ {I_VCMPLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1305, IF_AVX|IF_SANDYBRIDGE},
    /* 1926 */ {I_VCMPLTPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1314, IF_AVX|IF_SANDYBRIDGE},
    /* 1927 */ {I_VCMPLTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1323, IF_AVX|IF_SANDYBRIDGE},
    /* 1928 */ {I_VCMPLE_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1332, IF_AVX|IF_SANDYBRIDGE},
    /* 1929 */ {I_VCMPLE_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1341, IF_AVX|IF_SANDYBRIDGE},
    /* 1930 */ {I_VCMPLE_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1350, IF_AVX|IF_SANDYBRIDGE},
    /* 1931 */ {I_VCMPLE_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1359, IF_AVX|IF_SANDYBRIDGE},
    /* 1932 */ {I_VCMPLEPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1332, IF_AVX|IF_SANDYBRIDGE},
    /* 1933 */ {I_VCMPLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1341, IF_AVX|IF_SANDYBRIDGE},
    /* 1934 */ {I_VCMPLEPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1350, IF_AVX|IF_SANDYBRIDGE},
    /* 1935 */ {I_VCMPLEPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1359, IF_AVX|IF_SANDYBRIDGE},
    /* 1936 */ {I_VCMPUNORD_QPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1368, IF_AVX|IF_SANDYBRIDGE},
    /* 1937 */ {I_VCMPUNORD_QPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1377, IF_AVX|IF_SANDYBRIDGE},
    /* 1938 */ {I_VCMPUNORD_QPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1386, IF_AVX|IF_SANDYBRIDGE},
    /* 1939 */ {I_VCMPUNORD_QPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1395, IF_AVX|IF_SANDYBRIDGE},
    /* 1940 */ {I_VCMPUNORDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1368, IF_AVX|IF_SANDYBRIDGE},
    /* 1941 */ {I_VCMPUNORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1377, IF_AVX|IF_SANDYBRIDGE},
    /* 1942 */ {I_VCMPUNORDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1386, IF_AVX|IF_SANDYBRIDGE},
    /* 1943 */ {I_VCMPUNORDPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1395, IF_AVX|IF_SANDYBRIDGE},
    /* 1944 */ {I_VCMPNEQ_UQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1404, IF_AVX|IF_SANDYBRIDGE},
    /* 1945 */ {I_VCMPNEQ_UQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1413, IF_AVX|IF_SANDYBRIDGE},
    /* 1946 */ {I_VCMPNEQ_UQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1422, IF_AVX|IF_SANDYBRIDGE},
    /* 1947 */ {I_VCMPNEQ_UQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1431, IF_AVX|IF_SANDYBRIDGE},
    /* 1948 */ {I_VCMPNEQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1404, IF_AVX|IF_SANDYBRIDGE},
    /* 1949 */ {I_VCMPNEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1413, IF_AVX|IF_SANDYBRIDGE},
    /* 1950 */ {I_VCMPNEQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1422, IF_AVX|IF_SANDYBRIDGE},
    /* 1951 */ {I_VCMPNEQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1431, IF_AVX|IF_SANDYBRIDGE},
    /* 1952 */ {I_VCMPNLT_USPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1440, IF_AVX|IF_SANDYBRIDGE},
    /* 1953 */ {I_VCMPNLT_USPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1449, IF_AVX|IF_SANDYBRIDGE},
    /* 1954 */ {I_VCMPNLT_USPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1458, IF_AVX|IF_SANDYBRIDGE},
    /* 1955 */ {I_VCMPNLT_USPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1467, IF_AVX|IF_SANDYBRIDGE},
    /* 1956 */ {I_VCMPNLTPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1440, IF_AVX|IF_SANDYBRIDGE},
    /* 1957 */ {I_VCMPNLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1449, IF_AVX|IF_SANDYBRIDGE},
    /* 1958 */ {I_VCMPNLTPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1458, IF_AVX|IF_SANDYBRIDGE},
    /* 1959 */ {I_VCMPNLTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1467, IF_AVX|IF_SANDYBRIDGE},
    /* 1960 */ {I_VCMPNLE_USPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1476, IF_AVX|IF_SANDYBRIDGE},
    /* 1961 */ {I_VCMPNLE_USPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1485, IF_AVX|IF_SANDYBRIDGE},
    /* 1962 */ {I_VCMPNLE_USPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1494, IF_AVX|IF_SANDYBRIDGE},
    /* 1963 */ {I_VCMPNLE_USPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1503, IF_AVX|IF_SANDYBRIDGE},
    /* 1964 */ {I_VCMPNLEPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1476, IF_AVX|IF_SANDYBRIDGE},
    /* 1965 */ {I_VCMPNLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1485, IF_AVX|IF_SANDYBRIDGE},
    /* 1966 */ {I_VCMPNLEPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1494, IF_AVX|IF_SANDYBRIDGE},
    /* 1967 */ {I_VCMPNLEPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1503, IF_AVX|IF_SANDYBRIDGE},
    /* 1968 */ {I_VCMPORD_QPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1512, IF_AVX|IF_SANDYBRIDGE},
    /* 1969 */ {I_VCMPORD_QPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1521, IF_AVX|IF_SANDYBRIDGE},
    /* 1970 */ {I_VCMPORD_QPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1530, IF_AVX|IF_SANDYBRIDGE},
    /* 1971 */ {I_VCMPORD_QPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1539, IF_AVX|IF_SANDYBRIDGE},
    /* 1972 */ {I_VCMPORDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1512, IF_AVX|IF_SANDYBRIDGE},
    /* 1973 */ {I_VCMPORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1521, IF_AVX|IF_SANDYBRIDGE},
    /* 1974 */ {I_VCMPORDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1530, IF_AVX|IF_SANDYBRIDGE},
    /* 1975 */ {I_VCMPORDPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1539, IF_AVX|IF_SANDYBRIDGE},
    /* 1976 */ {I_VCMPEQ_UQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1548, IF_AVX|IF_SANDYBRIDGE},
    /* 1977 */ {I_VCMPEQ_UQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1557, IF_AVX|IF_SANDYBRIDGE},
    /* 1978 */ {I_VCMPEQ_UQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1566, IF_AVX|IF_SANDYBRIDGE},
    /* 1979 */ {I_VCMPEQ_UQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1575, IF_AVX|IF_SANDYBRIDGE},
    /* 1980 */ {I_VCMPNGE_USPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1584, IF_AVX|IF_SANDYBRIDGE},
    /* 1981 */ {I_VCMPNGE_USPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1593, IF_AVX|IF_SANDYBRIDGE},
    /* 1982 */ {I_VCMPNGE_USPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1602, IF_AVX|IF_SANDYBRIDGE},
    /* 1983 */ {I_VCMPNGE_USPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1611, IF_AVX|IF_SANDYBRIDGE},
    /* 1984 */ {I_VCMPNGEPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1584, IF_AVX|IF_SANDYBRIDGE},
    /* 1985 */ {I_VCMPNGEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1593, IF_AVX|IF_SANDYBRIDGE},
    /* 1986 */ {I_VCMPNGEPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1602, IF_AVX|IF_SANDYBRIDGE},
    /* 1987 */ {I_VCMPNGEPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1611, IF_AVX|IF_SANDYBRIDGE},
    /* 1988 */ {I_VCMPNGT_USPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1620, IF_AVX|IF_SANDYBRIDGE},
    /* 1989 */ {I_VCMPNGT_USPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1629, IF_AVX|IF_SANDYBRIDGE},
    /* 1990 */ {I_VCMPNGT_USPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1638, IF_AVX|IF_SANDYBRIDGE},
    /* 1991 */ {I_VCMPNGT_USPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1647, IF_AVX|IF_SANDYBRIDGE},
    /* 1992 */ {I_VCMPNGTPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1620, IF_AVX|IF_SANDYBRIDGE},
    /* 1993 */ {I_VCMPNGTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1629, IF_AVX|IF_SANDYBRIDGE},
    /* 1994 */ {I_VCMPNGTPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1638, IF_AVX|IF_SANDYBRIDGE},
    /* 1995 */ {I_VCMPNGTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1647, IF_AVX|IF_SANDYBRIDGE},
    /* 1996 */ {I_VCMPFALSE_OQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1656, IF_AVX|IF_SANDYBRIDGE},
    /* 1997 */ {I_VCMPFALSE_OQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1665, IF_AVX|IF_SANDYBRIDGE},
    /* 1998 */ {I_VCMPFALSE_OQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1674, IF_AVX|IF_SANDYBRIDGE},
    /* 1999 */ {I_VCMPFALSE_OQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1683, IF_AVX|IF_SANDYBRIDGE},
    /* 2000 */ {I_VCMPFALSEPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1656, IF_AVX|IF_SANDYBRIDGE},
    /* 2001 */ {I_VCMPFALSEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1665, IF_AVX|IF_SANDYBRIDGE},
    /* 2002 */ {I_VCMPFALSEPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1674, IF_AVX|IF_SANDYBRIDGE},
    /* 2003 */ {I_VCMPFALSEPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1683, IF_AVX|IF_SANDYBRIDGE},
    /* 2004 */ {I_VCMPNEQ_OQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1692, IF_AVX|IF_SANDYBRIDGE},
    /* 2005 */ {I_VCMPNEQ_OQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1701, IF_AVX|IF_SANDYBRIDGE},
    /* 2006 */ {I_VCMPNEQ_OQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1710, IF_AVX|IF_SANDYBRIDGE},
    /* 2007 */ {I_VCMPNEQ_OQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1719, IF_AVX|IF_SANDYBRIDGE},
    /* 2008 */ {I_VCMPGE_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1728, IF_AVX|IF_SANDYBRIDGE},
    /* 2009 */ {I_VCMPGE_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1737, IF_AVX|IF_SANDYBRIDGE},
    /* 2010 */ {I_VCMPGE_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1746, IF_AVX|IF_SANDYBRIDGE},
    /* 2011 */ {I_VCMPGE_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1755, IF_AVX|IF_SANDYBRIDGE},
    /* 2012 */ {I_VCMPGEPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1728, IF_AVX|IF_SANDYBRIDGE},
    /* 2013 */ {I_VCMPGEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1737, IF_AVX|IF_SANDYBRIDGE},
    /* 2014 */ {I_VCMPGEPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1746, IF_AVX|IF_SANDYBRIDGE},
    /* 2015 */ {I_VCMPGEPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1755, IF_AVX|IF_SANDYBRIDGE},
    /* 2016 */ {I_VCMPGT_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1764, IF_AVX|IF_SANDYBRIDGE},
    /* 2017 */ {I_VCMPGT_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1773, IF_AVX|IF_SANDYBRIDGE},
    /* 2018 */ {I_VCMPGT_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1782, IF_AVX|IF_SANDYBRIDGE},
    /* 2019 */ {I_VCMPGT_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1791, IF_AVX|IF_SANDYBRIDGE},
    /* 2020 */ {I_VCMPGTPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1764, IF_AVX|IF_SANDYBRIDGE},
    /* 2021 */ {I_VCMPGTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1773, IF_AVX|IF_SANDYBRIDGE},
    /* 2022 */ {I_VCMPGTPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1782, IF_AVX|IF_SANDYBRIDGE},
    /* 2023 */ {I_VCMPGTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1791, IF_AVX|IF_SANDYBRIDGE},
    /* 2024 */ {I_VCMPTRUE_UQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1800, IF_AVX|IF_SANDYBRIDGE},
    /* 2025 */ {I_VCMPTRUE_UQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1809, IF_AVX|IF_SANDYBRIDGE},
    /* 2026 */ {I_VCMPTRUE_UQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1818, IF_AVX|IF_SANDYBRIDGE},
    /* 2027 */ {I_VCMPTRUE_UQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1827, IF_AVX|IF_SANDYBRIDGE},
    /* 2028 */ {I_VCMPTRUEPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1800, IF_AVX|IF_SANDYBRIDGE},
    /* 2029 */ {I_VCMPTRUEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1809, IF_AVX|IF_SANDYBRIDGE},
    /* 2030 */ {I_VCMPTRUEPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1818, IF_AVX|IF_SANDYBRIDGE},
    /* 2031 */ {I_VCMPTRUEPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1827, IF_AVX|IF_SANDYBRIDGE},
    /* 2032 */ {I_VCMPEQ_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1224, IF_AVX|IF_SANDYBRIDGE},
    /* 2033 */ {I_VCMPEQ_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1233, IF_AVX|IF_SANDYBRIDGE},
    /* 2034 */ {I_VCMPEQ_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1242, IF_AVX|IF_SANDYBRIDGE},
    /* 2035 */ {I_VCMPEQ_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1251, IF_AVX|IF_SANDYBRIDGE},
    /* 2036 */ {I_VCMPLT_OQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1836, IF_AVX|IF_SANDYBRIDGE},
    /* 2037 */ {I_VCMPLT_OQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1845, IF_AVX|IF_SANDYBRIDGE},
    /* 2038 */ {I_VCMPLT_OQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1854, IF_AVX|IF_SANDYBRIDGE},
    /* 2039 */ {I_VCMPLT_OQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1863, IF_AVX|IF_SANDYBRIDGE},
    /* 2040 */ {I_VCMPLE_OQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1872, IF_AVX|IF_SANDYBRIDGE},
    /* 2041 */ {I_VCMPLE_OQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1881, IF_AVX|IF_SANDYBRIDGE},
    /* 2042 */ {I_VCMPLE_OQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1890, IF_AVX|IF_SANDYBRIDGE},
    /* 2043 */ {I_VCMPLE_OQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1899, IF_AVX|IF_SANDYBRIDGE},
    /* 2044 */ {I_VCMPUNORD_SPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1908, IF_AVX|IF_SANDYBRIDGE},
    /* 2045 */ {I_VCMPUNORD_SPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1917, IF_AVX|IF_SANDYBRIDGE},
    /* 2046 */ {I_VCMPUNORD_SPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1926, IF_AVX|IF_SANDYBRIDGE},
    /* 2047 */ {I_VCMPUNORD_SPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1935, IF_AVX|IF_SANDYBRIDGE},
    /* 2048 */ {I_VCMPNEQ_USPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1944, IF_AVX|IF_SANDYBRIDGE},
    /* 2049 */ {I_VCMPNEQ_USPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1953, IF_AVX|IF_SANDYBRIDGE},
    /* 2050 */ {I_VCMPNEQ_USPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1962, IF_AVX|IF_SANDYBRIDGE},
    /* 2051 */ {I_VCMPNEQ_USPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+1971, IF_AVX|IF_SANDYBRIDGE},
    /* 2052 */ {I_VCMPNLT_UQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+1980, IF_AVX|IF_SANDYBRIDGE},
    /* 2053 */ {I_VCMPNLT_UQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+1989, IF_AVX|IF_SANDYBRIDGE},
    /* 2054 */ {I_VCMPNLT_UQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+1998, IF_AVX|IF_SANDYBRIDGE},
    /* 2055 */ {I_VCMPNLT_UQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2007, IF_AVX|IF_SANDYBRIDGE},
    /* 2056 */ {I_VCMPNLE_UQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2016, IF_AVX|IF_SANDYBRIDGE},
    /* 2057 */ {I_VCMPNLE_UQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2025, IF_AVX|IF_SANDYBRIDGE},
    /* 2058 */ {I_VCMPNLE_UQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2034, IF_AVX|IF_SANDYBRIDGE},
    /* 2059 */ {I_VCMPNLE_UQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2043, IF_AVX|IF_SANDYBRIDGE},
    /* 2060 */ {I_VCMPORD_SPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2052, IF_AVX|IF_SANDYBRIDGE},
    /* 2061 */ {I_VCMPORD_SPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2061, IF_AVX|IF_SANDYBRIDGE},
    /* 2062 */ {I_VCMPORD_SPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2070, IF_AVX|IF_SANDYBRIDGE},
    /* 2063 */ {I_VCMPORD_SPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2079, IF_AVX|IF_SANDYBRIDGE},
    /* 2064 */ {I_VCMPEQ_USPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2088, IF_AVX|IF_SANDYBRIDGE},
    /* 2065 */ {I_VCMPEQ_USPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2097, IF_AVX|IF_SANDYBRIDGE},
    /* 2066 */ {I_VCMPEQ_USPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2106, IF_AVX|IF_SANDYBRIDGE},
    /* 2067 */ {I_VCMPEQ_USPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2115, IF_AVX|IF_SANDYBRIDGE},
    /* 2068 */ {I_VCMPNGE_UQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2124, IF_AVX|IF_SANDYBRIDGE},
    /* 2069 */ {I_VCMPNGE_UQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2133, IF_AVX|IF_SANDYBRIDGE},
    /* 2070 */ {I_VCMPNGE_UQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2142, IF_AVX|IF_SANDYBRIDGE},
    /* 2071 */ {I_VCMPNGE_UQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2151, IF_AVX|IF_SANDYBRIDGE},
    /* 2072 */ {I_VCMPNGT_UQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2160, IF_AVX|IF_SANDYBRIDGE},
    /* 2073 */ {I_VCMPNGT_UQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2169, IF_AVX|IF_SANDYBRIDGE},
    /* 2074 */ {I_VCMPNGT_UQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2178, IF_AVX|IF_SANDYBRIDGE},
    /* 2075 */ {I_VCMPNGT_UQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2187, IF_AVX|IF_SANDYBRIDGE},
    /* 2076 */ {I_VCMPFALSE_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2196, IF_AVX|IF_SANDYBRIDGE},
    /* 2077 */ {I_VCMPFALSE_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2205, IF_AVX|IF_SANDYBRIDGE},
    /* 2078 */ {I_VCMPFALSE_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2214, IF_AVX|IF_SANDYBRIDGE},
    /* 2079 */ {I_VCMPFALSE_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2223, IF_AVX|IF_SANDYBRIDGE},
    /* 2080 */ {I_VCMPNEQ_OSPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2232, IF_AVX|IF_SANDYBRIDGE},
    /* 2081 */ {I_VCMPNEQ_OSPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2241, IF_AVX|IF_SANDYBRIDGE},
    /* 2082 */ {I_VCMPNEQ_OSPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2250, IF_AVX|IF_SANDYBRIDGE},
    /* 2083 */ {I_VCMPNEQ_OSPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2259, IF_AVX|IF_SANDYBRIDGE},
    /* 2084 */ {I_VCMPGE_OQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2268, IF_AVX|IF_SANDYBRIDGE},
    /* 2085 */ {I_VCMPGE_OQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2277, IF_AVX|IF_SANDYBRIDGE},
    /* 2086 */ {I_VCMPGE_OQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2286, IF_AVX|IF_SANDYBRIDGE},
    /* 2087 */ {I_VCMPGE_OQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2295, IF_AVX|IF_SANDYBRIDGE},
    /* 2088 */ {I_VCMPGT_OQPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2304, IF_AVX|IF_SANDYBRIDGE},
    /* 2089 */ {I_VCMPGT_OQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2313, IF_AVX|IF_SANDYBRIDGE},
    /* 2090 */ {I_VCMPGT_OQPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2322, IF_AVX|IF_SANDYBRIDGE},
    /* 2091 */ {I_VCMPGT_OQPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2331, IF_AVX|IF_SANDYBRIDGE},
    /* 2092 */ {I_VCMPTRUE_USPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+2340, IF_AVX|IF_SANDYBRIDGE},
    /* 2093 */ {I_VCMPTRUE_USPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+2349, IF_AVX|IF_SANDYBRIDGE},
    /* 2094 */ {I_VCMPTRUE_USPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+2358, IF_AVX|IF_SANDYBRIDGE},
    /* 2095 */ {I_VCMPTRUE_USPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+2367, IF_AVX|IF_SANDYBRIDGE},
    /* 2096 */ {I_VCMPPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4604, IF_AVX|IF_SANDYBRIDGE},
    /* 2097 */ {I_VCMPPS, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4612, IF_AVX|IF_SANDYBRIDGE},
    /* 2098 */ {I_VCMPPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+4620, IF_AVX|IF_SANDYBRIDGE},
    /* 2099 */ {I_VCMPPS, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4628, IF_AVX|IF_SANDYBRIDGE},
    /* 2100 */ {I_VCMPEQ_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2376, IF_AVX|IF_SANDYBRIDGE},
    /* 2101 */ {I_VCMPEQ_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2385, IF_AVX|IF_SANDYBRIDGE},
    /* 2102 */ {I_VCMPEQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2394, IF_AVX|IF_SANDYBRIDGE},
    /* 2103 */ {I_VCMPEQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2403, IF_AVX|IF_SANDYBRIDGE},
    /* 2104 */ {I_VCMPLT_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2412, IF_AVX|IF_SANDYBRIDGE},
    /* 2105 */ {I_VCMPLT_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2421, IF_AVX|IF_SANDYBRIDGE},
    /* 2106 */ {I_VCMPLTSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2412, IF_AVX|IF_SANDYBRIDGE},
    /* 2107 */ {I_VCMPLTSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2421, IF_AVX|IF_SANDYBRIDGE},
    /* 2108 */ {I_VCMPLE_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2430, IF_AVX|IF_SANDYBRIDGE},
    /* 2109 */ {I_VCMPLE_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2439, IF_AVX|IF_SANDYBRIDGE},
    /* 2110 */ {I_VCMPLESD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2430, IF_AVX|IF_SANDYBRIDGE},
    /* 2111 */ {I_VCMPLESD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2439, IF_AVX|IF_SANDYBRIDGE},
    /* 2112 */ {I_VCMPUNORD_QSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2448, IF_AVX|IF_SANDYBRIDGE},
    /* 2113 */ {I_VCMPUNORD_QSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2457, IF_AVX|IF_SANDYBRIDGE},
    /* 2114 */ {I_VCMPUNORDSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2448, IF_AVX|IF_SANDYBRIDGE},
    /* 2115 */ {I_VCMPUNORDSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2457, IF_AVX|IF_SANDYBRIDGE},
    /* 2116 */ {I_VCMPNEQ_UQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2466, IF_AVX|IF_SANDYBRIDGE},
    /* 2117 */ {I_VCMPNEQ_UQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2475, IF_AVX|IF_SANDYBRIDGE},
    /* 2118 */ {I_VCMPNEQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2466, IF_AVX|IF_SANDYBRIDGE},
    /* 2119 */ {I_VCMPNEQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2475, IF_AVX|IF_SANDYBRIDGE},
    /* 2120 */ {I_VCMPNLT_USSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2484, IF_AVX|IF_SANDYBRIDGE},
    /* 2121 */ {I_VCMPNLT_USSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2493, IF_AVX|IF_SANDYBRIDGE},
    /* 2122 */ {I_VCMPNLTSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2484, IF_AVX|IF_SANDYBRIDGE},
    /* 2123 */ {I_VCMPNLTSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2493, IF_AVX|IF_SANDYBRIDGE},
    /* 2124 */ {I_VCMPNLE_USSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2502, IF_AVX|IF_SANDYBRIDGE},
    /* 2125 */ {I_VCMPNLE_USSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2511, IF_AVX|IF_SANDYBRIDGE},
    /* 2126 */ {I_VCMPNLESD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2502, IF_AVX|IF_SANDYBRIDGE},
    /* 2127 */ {I_VCMPNLESD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2511, IF_AVX|IF_SANDYBRIDGE},
    /* 2128 */ {I_VCMPORD_QSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2520, IF_AVX|IF_SANDYBRIDGE},
    /* 2129 */ {I_VCMPORD_QSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2529, IF_AVX|IF_SANDYBRIDGE},
    /* 2130 */ {I_VCMPORDSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2520, IF_AVX|IF_SANDYBRIDGE},
    /* 2131 */ {I_VCMPORDSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2529, IF_AVX|IF_SANDYBRIDGE},
    /* 2132 */ {I_VCMPEQ_UQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2538, IF_AVX|IF_SANDYBRIDGE},
    /* 2133 */ {I_VCMPEQ_UQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2547, IF_AVX|IF_SANDYBRIDGE},
    /* 2134 */ {I_VCMPNGE_USSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2556, IF_AVX|IF_SANDYBRIDGE},
    /* 2135 */ {I_VCMPNGE_USSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2565, IF_AVX|IF_SANDYBRIDGE},
    /* 2136 */ {I_VCMPNGESD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2556, IF_AVX|IF_SANDYBRIDGE},
    /* 2137 */ {I_VCMPNGESD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2565, IF_AVX|IF_SANDYBRIDGE},
    /* 2138 */ {I_VCMPNGT_USSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2574, IF_AVX|IF_SANDYBRIDGE},
    /* 2139 */ {I_VCMPNGT_USSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2583, IF_AVX|IF_SANDYBRIDGE},
    /* 2140 */ {I_VCMPNGTSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2574, IF_AVX|IF_SANDYBRIDGE},
    /* 2141 */ {I_VCMPNGTSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2583, IF_AVX|IF_SANDYBRIDGE},
    /* 2142 */ {I_VCMPFALSE_OQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2592, IF_AVX|IF_SANDYBRIDGE},
    /* 2143 */ {I_VCMPFALSE_OQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2601, IF_AVX|IF_SANDYBRIDGE},
    /* 2144 */ {I_VCMPFALSESD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2592, IF_AVX|IF_SANDYBRIDGE},
    /* 2145 */ {I_VCMPFALSESD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2601, IF_AVX|IF_SANDYBRIDGE},
    /* 2146 */ {I_VCMPNEQ_OQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2610, IF_AVX|IF_SANDYBRIDGE},
    /* 2147 */ {I_VCMPNEQ_OQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2619, IF_AVX|IF_SANDYBRIDGE},
    /* 2148 */ {I_VCMPGE_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2628, IF_AVX|IF_SANDYBRIDGE},
    /* 2149 */ {I_VCMPGE_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2637, IF_AVX|IF_SANDYBRIDGE},
    /* 2150 */ {I_VCMPGESD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2628, IF_AVX|IF_SANDYBRIDGE},
    /* 2151 */ {I_VCMPGESD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2637, IF_AVX|IF_SANDYBRIDGE},
    /* 2152 */ {I_VCMPGT_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2646, IF_AVX|IF_SANDYBRIDGE},
    /* 2153 */ {I_VCMPGT_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2655, IF_AVX|IF_SANDYBRIDGE},
    /* 2154 */ {I_VCMPGTSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2646, IF_AVX|IF_SANDYBRIDGE},
    /* 2155 */ {I_VCMPGTSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2655, IF_AVX|IF_SANDYBRIDGE},
    /* 2156 */ {I_VCMPTRUE_UQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2664, IF_AVX|IF_SANDYBRIDGE},
    /* 2157 */ {I_VCMPTRUE_UQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2673, IF_AVX|IF_SANDYBRIDGE},
    /* 2158 */ {I_VCMPTRUESD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2664, IF_AVX|IF_SANDYBRIDGE},
    /* 2159 */ {I_VCMPTRUESD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2673, IF_AVX|IF_SANDYBRIDGE},
    /* 2160 */ {I_VCMPEQ_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2376, IF_AVX|IF_SANDYBRIDGE},
    /* 2161 */ {I_VCMPEQ_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2385, IF_AVX|IF_SANDYBRIDGE},
    /* 2162 */ {I_VCMPLT_OQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2682, IF_AVX|IF_SANDYBRIDGE},
    /* 2163 */ {I_VCMPLT_OQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2691, IF_AVX|IF_SANDYBRIDGE},
    /* 2164 */ {I_VCMPLE_OQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2700, IF_AVX|IF_SANDYBRIDGE},
    /* 2165 */ {I_VCMPLE_OQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2709, IF_AVX|IF_SANDYBRIDGE},
    /* 2166 */ {I_VCMPUNORD_SSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2718, IF_AVX|IF_SANDYBRIDGE},
    /* 2167 */ {I_VCMPUNORD_SSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2727, IF_AVX|IF_SANDYBRIDGE},
    /* 2168 */ {I_VCMPNEQ_USSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2736, IF_AVX|IF_SANDYBRIDGE},
    /* 2169 */ {I_VCMPNEQ_USSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2745, IF_AVX|IF_SANDYBRIDGE},
    /* 2170 */ {I_VCMPNLT_UQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2754, IF_AVX|IF_SANDYBRIDGE},
    /* 2171 */ {I_VCMPNLT_UQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2763, IF_AVX|IF_SANDYBRIDGE},
    /* 2172 */ {I_VCMPNLE_UQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2772, IF_AVX|IF_SANDYBRIDGE},
    /* 2173 */ {I_VCMPNLE_UQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2781, IF_AVX|IF_SANDYBRIDGE},
    /* 2174 */ {I_VCMPORD_SSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2790, IF_AVX|IF_SANDYBRIDGE},
    /* 2175 */ {I_VCMPORD_SSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2799, IF_AVX|IF_SANDYBRIDGE},
    /* 2176 */ {I_VCMPEQ_USSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2808, IF_AVX|IF_SANDYBRIDGE},
    /* 2177 */ {I_VCMPEQ_USSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2817, IF_AVX|IF_SANDYBRIDGE},
    /* 2178 */ {I_VCMPNGE_UQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2826, IF_AVX|IF_SANDYBRIDGE},
    /* 2179 */ {I_VCMPNGE_UQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2835, IF_AVX|IF_SANDYBRIDGE},
    /* 2180 */ {I_VCMPNGT_UQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2844, IF_AVX|IF_SANDYBRIDGE},
    /* 2181 */ {I_VCMPNGT_UQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2853, IF_AVX|IF_SANDYBRIDGE},
    /* 2182 */ {I_VCMPFALSE_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2862, IF_AVX|IF_SANDYBRIDGE},
    /* 2183 */ {I_VCMPFALSE_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2871, IF_AVX|IF_SANDYBRIDGE},
    /* 2184 */ {I_VCMPNEQ_OSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2880, IF_AVX|IF_SANDYBRIDGE},
    /* 2185 */ {I_VCMPNEQ_OSSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2889, IF_AVX|IF_SANDYBRIDGE},
    /* 2186 */ {I_VCMPGE_OQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2898, IF_AVX|IF_SANDYBRIDGE},
    /* 2187 */ {I_VCMPGE_OQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2907, IF_AVX|IF_SANDYBRIDGE},
    /* 2188 */ {I_VCMPGT_OQSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2916, IF_AVX|IF_SANDYBRIDGE},
    /* 2189 */ {I_VCMPGT_OQSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2925, IF_AVX|IF_SANDYBRIDGE},
    /* 2190 */ {I_VCMPTRUE_USSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2934, IF_AVX|IF_SANDYBRIDGE},
    /* 2191 */ {I_VCMPTRUE_USSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2943, IF_AVX|IF_SANDYBRIDGE},
    /* 2192 */ {I_VCMPSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMMEDIATE|BITS8,0}, nasm_bytecodes+4636, IF_AVX|IF_SANDYBRIDGE},
    /* 2193 */ {I_VCMPSD, 3, {XMMREG,RM_XMM|BITS64,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4644, IF_AVX|IF_SANDYBRIDGE},
    /* 2194 */ {I_VCMPEQ_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2952, IF_AVX|IF_SANDYBRIDGE},
    /* 2195 */ {I_VCMPEQ_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2961, IF_AVX|IF_SANDYBRIDGE},
    /* 2196 */ {I_VCMPEQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2970, IF_AVX|IF_SANDYBRIDGE},
    /* 2197 */ {I_VCMPEQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2979, IF_AVX|IF_SANDYBRIDGE},
    /* 2198 */ {I_VCMPLT_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2988, IF_AVX|IF_SANDYBRIDGE},
    /* 2199 */ {I_VCMPLT_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2997, IF_AVX|IF_SANDYBRIDGE},
    /* 2200 */ {I_VCMPLTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2988, IF_AVX|IF_SANDYBRIDGE},
    /* 2201 */ {I_VCMPLTSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2997, IF_AVX|IF_SANDYBRIDGE},
    /* 2202 */ {I_VCMPLE_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3006, IF_AVX|IF_SANDYBRIDGE},
    /* 2203 */ {I_VCMPLE_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3015, IF_AVX|IF_SANDYBRIDGE},
    /* 2204 */ {I_VCMPLESS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3006, IF_AVX|IF_SANDYBRIDGE},
    /* 2205 */ {I_VCMPLESS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3015, IF_AVX|IF_SANDYBRIDGE},
    /* 2206 */ {I_VCMPUNORD_QSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3024, IF_AVX|IF_SANDYBRIDGE},
    /* 2207 */ {I_VCMPUNORD_QSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3033, IF_AVX|IF_SANDYBRIDGE},
    /* 2208 */ {I_VCMPUNORDSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3024, IF_AVX|IF_SANDYBRIDGE},
    /* 2209 */ {I_VCMPUNORDSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3033, IF_AVX|IF_SANDYBRIDGE},
    /* 2210 */ {I_VCMPNEQ_UQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3042, IF_AVX|IF_SANDYBRIDGE},
    /* 2211 */ {I_VCMPNEQ_UQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3051, IF_AVX|IF_SANDYBRIDGE},
    /* 2212 */ {I_VCMPNEQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3042, IF_AVX|IF_SANDYBRIDGE},
    /* 2213 */ {I_VCMPNEQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3051, IF_AVX|IF_SANDYBRIDGE},
    /* 2214 */ {I_VCMPNLT_USSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3060, IF_AVX|IF_SANDYBRIDGE},
    /* 2215 */ {I_VCMPNLT_USSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3069, IF_AVX|IF_SANDYBRIDGE},
    /* 2216 */ {I_VCMPNLTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3060, IF_AVX|IF_SANDYBRIDGE},
    /* 2217 */ {I_VCMPNLTSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3069, IF_AVX|IF_SANDYBRIDGE},
    /* 2218 */ {I_VCMPNLE_USSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3078, IF_AVX|IF_SANDYBRIDGE},
    /* 2219 */ {I_VCMPNLE_USSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3087, IF_AVX|IF_SANDYBRIDGE},
    /* 2220 */ {I_VCMPNLESS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3078, IF_AVX|IF_SANDYBRIDGE},
    /* 2221 */ {I_VCMPNLESS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3087, IF_AVX|IF_SANDYBRIDGE},
    /* 2222 */ {I_VCMPORD_QSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3096, IF_AVX|IF_SANDYBRIDGE},
    /* 2223 */ {I_VCMPORD_QSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3105, IF_AVX|IF_SANDYBRIDGE},
    /* 2224 */ {I_VCMPORDSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3096, IF_AVX|IF_SANDYBRIDGE},
    /* 2225 */ {I_VCMPORDSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3105, IF_AVX|IF_SANDYBRIDGE},
    /* 2226 */ {I_VCMPEQ_UQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3114, IF_AVX|IF_SANDYBRIDGE},
    /* 2227 */ {I_VCMPEQ_UQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3123, IF_AVX|IF_SANDYBRIDGE},
    /* 2228 */ {I_VCMPNGE_USSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3132, IF_AVX|IF_SANDYBRIDGE},
    /* 2229 */ {I_VCMPNGE_USSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3141, IF_AVX|IF_SANDYBRIDGE},
    /* 2230 */ {I_VCMPNGESS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3132, IF_AVX|IF_SANDYBRIDGE},
    /* 2231 */ {I_VCMPNGESS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3141, IF_AVX|IF_SANDYBRIDGE},
    /* 2232 */ {I_VCMPNGT_USSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3150, IF_AVX|IF_SANDYBRIDGE},
    /* 2233 */ {I_VCMPNGT_USSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3159, IF_AVX|IF_SANDYBRIDGE},
    /* 2234 */ {I_VCMPNGTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3150, IF_AVX|IF_SANDYBRIDGE},
    /* 2235 */ {I_VCMPNGTSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3159, IF_AVX|IF_SANDYBRIDGE},
    /* 2236 */ {I_VCMPFALSE_OQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3168, IF_AVX|IF_SANDYBRIDGE},
    /* 2237 */ {I_VCMPFALSE_OQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3177, IF_AVX|IF_SANDYBRIDGE},
    /* 2238 */ {I_VCMPFALSESS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3168, IF_AVX|IF_SANDYBRIDGE},
    /* 2239 */ {I_VCMPFALSESS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3177, IF_AVX|IF_SANDYBRIDGE},
    /* 2240 */ {I_VCMPNEQ_OQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3186, IF_AVX|IF_SANDYBRIDGE},
    /* 2241 */ {I_VCMPNEQ_OQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3195, IF_AVX|IF_SANDYBRIDGE},
    /* 2242 */ {I_VCMPGE_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3204, IF_AVX|IF_SANDYBRIDGE},
    /* 2243 */ {I_VCMPGE_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3213, IF_AVX|IF_SANDYBRIDGE},
    /* 2244 */ {I_VCMPGESS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3204, IF_AVX|IF_SANDYBRIDGE},
    /* 2245 */ {I_VCMPGESS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3213, IF_AVX|IF_SANDYBRIDGE},
    /* 2246 */ {I_VCMPGT_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3222, IF_AVX|IF_SANDYBRIDGE},
    /* 2247 */ {I_VCMPGT_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3231, IF_AVX|IF_SANDYBRIDGE},
    /* 2248 */ {I_VCMPGTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3222, IF_AVX|IF_SANDYBRIDGE},
    /* 2249 */ {I_VCMPGTSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3231, IF_AVX|IF_SANDYBRIDGE},
    /* 2250 */ {I_VCMPTRUE_UQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3240, IF_AVX|IF_SANDYBRIDGE},
    /* 2251 */ {I_VCMPTRUE_UQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3249, IF_AVX|IF_SANDYBRIDGE},
    /* 2252 */ {I_VCMPTRUESS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3240, IF_AVX|IF_SANDYBRIDGE},
    /* 2253 */ {I_VCMPTRUESS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3249, IF_AVX|IF_SANDYBRIDGE},
    /* 2254 */ {I_VCMPEQ_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+2952, IF_AVX|IF_SANDYBRIDGE},
    /* 2255 */ {I_VCMPEQ_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+2961, IF_AVX|IF_SANDYBRIDGE},
    /* 2256 */ {I_VCMPLT_OQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3258, IF_AVX|IF_SANDYBRIDGE},
    /* 2257 */ {I_VCMPLT_OQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3267, IF_AVX|IF_SANDYBRIDGE},
    /* 2258 */ {I_VCMPLE_OQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3276, IF_AVX|IF_SANDYBRIDGE},
    /* 2259 */ {I_VCMPLE_OQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3285, IF_AVX|IF_SANDYBRIDGE},
    /* 2260 */ {I_VCMPUNORD_SSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3294, IF_AVX|IF_SANDYBRIDGE},
    /* 2261 */ {I_VCMPUNORD_SSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3303, IF_AVX|IF_SANDYBRIDGE},
    /* 2262 */ {I_VCMPNEQ_USSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3312, IF_AVX|IF_SANDYBRIDGE},
    /* 2263 */ {I_VCMPNEQ_USSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3321, IF_AVX|IF_SANDYBRIDGE},
    /* 2264 */ {I_VCMPNLT_UQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3330, IF_AVX|IF_SANDYBRIDGE},
    /* 2265 */ {I_VCMPNLT_UQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3339, IF_AVX|IF_SANDYBRIDGE},
    /* 2266 */ {I_VCMPNLE_UQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3348, IF_AVX|IF_SANDYBRIDGE},
    /* 2267 */ {I_VCMPNLE_UQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3357, IF_AVX|IF_SANDYBRIDGE},
    /* 2268 */ {I_VCMPORD_SSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3366, IF_AVX|IF_SANDYBRIDGE},
    /* 2269 */ {I_VCMPORD_SSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3375, IF_AVX|IF_SANDYBRIDGE},
    /* 2270 */ {I_VCMPEQ_USSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3384, IF_AVX|IF_SANDYBRIDGE},
    /* 2271 */ {I_VCMPEQ_USSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3393, IF_AVX|IF_SANDYBRIDGE},
    /* 2272 */ {I_VCMPNGE_UQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3402, IF_AVX|IF_SANDYBRIDGE},
    /* 2273 */ {I_VCMPNGE_UQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3411, IF_AVX|IF_SANDYBRIDGE},
    /* 2274 */ {I_VCMPNGT_UQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3420, IF_AVX|IF_SANDYBRIDGE},
    /* 2275 */ {I_VCMPNGT_UQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3429, IF_AVX|IF_SANDYBRIDGE},
    /* 2276 */ {I_VCMPFALSE_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3438, IF_AVX|IF_SANDYBRIDGE},
    /* 2277 */ {I_VCMPFALSE_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3447, IF_AVX|IF_SANDYBRIDGE},
    /* 2278 */ {I_VCMPNEQ_OSSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3456, IF_AVX|IF_SANDYBRIDGE},
    /* 2279 */ {I_VCMPNEQ_OSSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3465, IF_AVX|IF_SANDYBRIDGE},
    /* 2280 */ {I_VCMPGE_OQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3474, IF_AVX|IF_SANDYBRIDGE},
    /* 2281 */ {I_VCMPGE_OQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3483, IF_AVX|IF_SANDYBRIDGE},
    /* 2282 */ {I_VCMPGT_OQSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3492, IF_AVX|IF_SANDYBRIDGE},
    /* 2283 */ {I_VCMPGT_OQSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3501, IF_AVX|IF_SANDYBRIDGE},
    /* 2284 */ {I_VCMPTRUE_USSS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+3510, IF_AVX|IF_SANDYBRIDGE},
    /* 2285 */ {I_VCMPTRUE_USSS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+3519, IF_AVX|IF_SANDYBRIDGE},
    /* 2286 */ {I_VCMPSS, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMMEDIATE|BITS8,0}, nasm_bytecodes+4652, IF_AVX|IF_SANDYBRIDGE},
    /* 2287 */ {I_VCMPSS, 3, {XMMREG,RM_XMM|BITS64,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4660, IF_AVX|IF_SANDYBRIDGE},
    /* 2288 */ {I_VCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9679, IF_AVX|IF_SANDYBRIDGE},
    /* 2289 */ {I_VCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+9686, IF_AVX|IF_SANDYBRIDGE},
    /* 2290 */ {I_VCVTDQ2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9693, IF_AVX|IF_SANDYBRIDGE},
    /* 2291 */ {I_VCVTDQ2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9700, IF_AVX|IF_SANDYBRIDGE},
    /* 2292 */ {I_VCVTDQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9707, IF_AVX|IF_SANDYBRIDGE},
    /* 2293 */ {I_VCVTDQ2PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9714, IF_AVX|IF_SANDYBRIDGE},
    /* 2294 */ {I_VCVTPD2DQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+9721, IF_AVX|IF_SANDYBRIDGE},
    /* 2295 */ {I_VCVTPD2DQ, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+9721, IF_AVX|IF_SANDYBRIDGE|IF_SO},
    /* 2296 */ {I_VCVTPD2DQ, 2, {XMMREG,YMMREG,0,0,0}, nasm_bytecodes+9728, IF_AVX|IF_SANDYBRIDGE},
    /* 2297 */ {I_VCVTPD2DQ, 2, {XMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+9728, IF_AVX|IF_SANDYBRIDGE|IF_SY},
    /* 2298 */ {I_VCVTPD2PS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+9735, IF_AVX|IF_SANDYBRIDGE},
    /* 2299 */ {I_VCVTPD2PS, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+9735, IF_AVX|IF_SANDYBRIDGE|IF_SO},
    /* 2300 */ {I_VCVTPD2PS, 2, {XMMREG,YMMREG,0,0,0}, nasm_bytecodes+9742, IF_AVX|IF_SANDYBRIDGE},
    /* 2301 */ {I_VCVTPD2PS, 2, {XMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+9742, IF_AVX|IF_SANDYBRIDGE|IF_SY},
    /* 2302 */ {I_VCVTPS2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9749, IF_AVX|IF_SANDYBRIDGE},
    /* 2303 */ {I_VCVTPS2DQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9756, IF_AVX|IF_SANDYBRIDGE},
    /* 2304 */ {I_VCVTPS2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9763, IF_AVX|IF_SANDYBRIDGE},
    /* 2305 */ {I_VCVTPS2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9770, IF_AVX|IF_SANDYBRIDGE},
    /* 2306 */ {I_VCVTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9777, IF_AVX|IF_SANDYBRIDGE},
    /* 2307 */ {I_VCVTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9784, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2308 */ {I_VCVTSD2SS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+9791, IF_AVX|IF_SANDYBRIDGE},
    /* 2309 */ {I_VCVTSD2SS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9798, IF_AVX|IF_SANDYBRIDGE},
    /* 2310 */ {I_VCVTSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, nasm_bytecodes+9805, IF_AVX|IF_SANDYBRIDGE|IF_SD},
    /* 2311 */ {I_VCVTSI2SD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+9812, IF_AVX|IF_SANDYBRIDGE|IF_SD},
    /* 2312 */ {I_VCVTSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, nasm_bytecodes+9819, IF_AVX|IF_SANDYBRIDGE|IF_LONG|IF_SQ},
    /* 2313 */ {I_VCVTSI2SD, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+9826, IF_AVX|IF_SANDYBRIDGE|IF_LONG|IF_SQ},
    /* 2314 */ {I_VCVTSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, nasm_bytecodes+9833, IF_AVX|IF_SANDYBRIDGE|IF_SD},
    /* 2315 */ {I_VCVTSI2SS, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+9840, IF_AVX|IF_SANDYBRIDGE|IF_SD},
    /* 2316 */ {I_VCVTSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, nasm_bytecodes+9847, IF_AVX|IF_SANDYBRIDGE|IF_LONG|IF_SQ},
    /* 2317 */ {I_VCVTSI2SS, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+9854, IF_AVX|IF_SANDYBRIDGE|IF_LONG|IF_SQ},
    /* 2318 */ {I_VCVTSS2SD, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+9861, IF_AVX|IF_SANDYBRIDGE},
    /* 2319 */ {I_VCVTSS2SD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+9868, IF_AVX|IF_SANDYBRIDGE},
    /* 2320 */ {I_VCVTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+9875, IF_AVX|IF_SANDYBRIDGE},
    /* 2321 */ {I_VCVTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+9882, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2322 */ {I_VCVTTPD2DQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+9889, IF_AVX|IF_SANDYBRIDGE},
    /* 2323 */ {I_VCVTTPD2DQ, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+9889, IF_AVX|IF_SANDYBRIDGE|IF_SO},
    /* 2324 */ {I_VCVTTPD2DQ, 2, {XMMREG,YMMREG,0,0,0}, nasm_bytecodes+9896, IF_AVX|IF_SANDYBRIDGE},
    /* 2325 */ {I_VCVTTPD2DQ, 2, {XMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+9896, IF_AVX|IF_SANDYBRIDGE|IF_SY},
    /* 2326 */ {I_VCVTTPS2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9903, IF_AVX|IF_SANDYBRIDGE},
    /* 2327 */ {I_VCVTTPS2DQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9910, IF_AVX|IF_SANDYBRIDGE},
    /* 2328 */ {I_VCVTTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9917, IF_AVX|IF_SANDYBRIDGE},
    /* 2329 */ {I_VCVTTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+9924, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2330 */ {I_VCVTTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+9931, IF_AVX|IF_SANDYBRIDGE},
    /* 2331 */ {I_VCVTTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+9938, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2332 */ {I_VDIVPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9945, IF_AVX|IF_SANDYBRIDGE},
    /* 2333 */ {I_VDIVPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9952, IF_AVX|IF_SANDYBRIDGE},
    /* 2334 */ {I_VDIVPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9959, IF_AVX|IF_SANDYBRIDGE},
    /* 2335 */ {I_VDIVPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9966, IF_AVX|IF_SANDYBRIDGE},
    /* 2336 */ {I_VDIVPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+9973, IF_AVX|IF_SANDYBRIDGE},
    /* 2337 */ {I_VDIVPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+9980, IF_AVX|IF_SANDYBRIDGE},
    /* 2338 */ {I_VDIVPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+9987, IF_AVX|IF_SANDYBRIDGE},
    /* 2339 */ {I_VDIVPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+9994, IF_AVX|IF_SANDYBRIDGE},
    /* 2340 */ {I_VDIVSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+10001, IF_AVX|IF_SANDYBRIDGE},
    /* 2341 */ {I_VDIVSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+10008, IF_AVX|IF_SANDYBRIDGE},
    /* 2342 */ {I_VDIVSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+10015, IF_AVX|IF_SANDYBRIDGE},
    /* 2343 */ {I_VDIVSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+10022, IF_AVX|IF_SANDYBRIDGE},
    /* 2344 */ {I_VDPPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4668, IF_AVX|IF_SANDYBRIDGE},
    /* 2345 */ {I_VDPPD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4676, IF_AVX|IF_SANDYBRIDGE},
    /* 2346 */ {I_VDPPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4684, IF_AVX|IF_SANDYBRIDGE},
    /* 2347 */ {I_VDPPS, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4692, IF_AVX|IF_SANDYBRIDGE},
    /* 2348 */ {I_VDPPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+4700, IF_AVX|IF_SANDYBRIDGE},
    /* 2349 */ {I_VDPPS, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4708, IF_AVX|IF_SANDYBRIDGE},
    /* 2350 */ {I_VEXTRACTF128, 3, {RM_XMM|BITS128,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4716, IF_AVX|IF_SANDYBRIDGE},
    /* 2351 */ {I_VEXTRACTPS, 3, {RM_GPR|BITS32,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4724, IF_AVX|IF_SANDYBRIDGE},
    /* 2352 */ {I_VHADDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10029, IF_AVX|IF_SANDYBRIDGE},
    /* 2353 */ {I_VHADDPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10036, IF_AVX|IF_SANDYBRIDGE},
    /* 2354 */ {I_VHADDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10043, IF_AVX|IF_SANDYBRIDGE},
    /* 2355 */ {I_VHADDPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10050, IF_AVX|IF_SANDYBRIDGE},
    /* 2356 */ {I_VHADDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10057, IF_AVX|IF_SANDYBRIDGE},
    /* 2357 */ {I_VHADDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10064, IF_AVX|IF_SANDYBRIDGE},
    /* 2358 */ {I_VHADDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10071, IF_AVX|IF_SANDYBRIDGE},
    /* 2359 */ {I_VHADDPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10078, IF_AVX|IF_SANDYBRIDGE},
    /* 2360 */ {I_VHSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10085, IF_AVX|IF_SANDYBRIDGE},
    /* 2361 */ {I_VHSUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10092, IF_AVX|IF_SANDYBRIDGE},
    /* 2362 */ {I_VHSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10099, IF_AVX|IF_SANDYBRIDGE},
    /* 2363 */ {I_VHSUBPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10106, IF_AVX|IF_SANDYBRIDGE},
    /* 2364 */ {I_VHSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10113, IF_AVX|IF_SANDYBRIDGE},
    /* 2365 */ {I_VHSUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10120, IF_AVX|IF_SANDYBRIDGE},
    /* 2366 */ {I_VHSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10127, IF_AVX|IF_SANDYBRIDGE},
    /* 2367 */ {I_VHSUBPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10134, IF_AVX|IF_SANDYBRIDGE},
    /* 2368 */ {I_VINSERTF128, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4732, IF_AVX|IF_SANDYBRIDGE},
    /* 2369 */ {I_VINSERTPS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMMEDIATE|BITS8,0}, nasm_bytecodes+4740, IF_AVX|IF_SANDYBRIDGE},
    /* 2370 */ {I_VINSERTPS, 3, {XMMREG,RM_XMM|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4748, IF_AVX|IF_SANDYBRIDGE},
    /* 2371 */ {I_VLDDQU, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+10141, IF_AVX|IF_SANDYBRIDGE},
    /* 2372 */ {I_VLDQQU, 2, {YMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+10148, IF_AVX|IF_SANDYBRIDGE},
    /* 2373 */ {I_VLDDQU, 2, {YMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+10148, IF_AVX|IF_SANDYBRIDGE},
    /* 2374 */ {I_VLDMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+10155, IF_AVX|IF_SANDYBRIDGE},
    /* 2375 */ {I_VMASKMOVDQU, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+10162, IF_AVX|IF_SANDYBRIDGE},
    /* 2376 */ {I_VMASKMOVPS, 3, {XMMREG,XMMREG,MEMORY|BITS128,0,0}, nasm_bytecodes+10169, IF_AVX|IF_SANDYBRIDGE},
    /* 2377 */ {I_VMASKMOVPS, 3, {YMMREG,YMMREG,MEMORY|BITS256,0,0}, nasm_bytecodes+10176, IF_AVX|IF_SANDYBRIDGE},
    /* 2378 */ {I_VMASKMOVPS, 3, {MEMORY|BITS128,XMMREG,XMMREG,0,0}, nasm_bytecodes+10183, IF_AVX|IF_SANDYBRIDGE|IF_SO},
    /* 2379 */ {I_VMASKMOVPS, 3, {MEMORY|BITS256,YMMREG,YMMREG,0,0}, nasm_bytecodes+10190, IF_AVX|IF_SANDYBRIDGE|IF_SY},
    /* 2380 */ {I_VMASKMOVPD, 3, {XMMREG,XMMREG,MEMORY|BITS128,0,0}, nasm_bytecodes+10197, IF_AVX|IF_SANDYBRIDGE},
    /* 2381 */ {I_VMASKMOVPD, 3, {YMMREG,YMMREG,MEMORY|BITS256,0,0}, nasm_bytecodes+10204, IF_AVX|IF_SANDYBRIDGE},
    /* 2382 */ {I_VMASKMOVPD, 3, {MEMORY|BITS128,XMMREG,XMMREG,0,0}, nasm_bytecodes+10211, IF_AVX|IF_SANDYBRIDGE},
    /* 2383 */ {I_VMASKMOVPD, 3, {MEMORY|BITS256,YMMREG,YMMREG,0,0}, nasm_bytecodes+10218, IF_AVX|IF_SANDYBRIDGE},
    /* 2384 */ {I_VMAXPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10225, IF_AVX|IF_SANDYBRIDGE},
    /* 2385 */ {I_VMAXPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10232, IF_AVX|IF_SANDYBRIDGE},
    /* 2386 */ {I_VMAXPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10239, IF_AVX|IF_SANDYBRIDGE},
    /* 2387 */ {I_VMAXPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10246, IF_AVX|IF_SANDYBRIDGE},
    /* 2388 */ {I_VMAXPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10253, IF_AVX|IF_SANDYBRIDGE},
    /* 2389 */ {I_VMAXPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10260, IF_AVX|IF_SANDYBRIDGE},
    /* 2390 */ {I_VMAXPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10267, IF_AVX|IF_SANDYBRIDGE},
    /* 2391 */ {I_VMAXPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10274, IF_AVX|IF_SANDYBRIDGE},
    /* 2392 */ {I_VMAXSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+10281, IF_AVX|IF_SANDYBRIDGE},
    /* 2393 */ {I_VMAXSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+10288, IF_AVX|IF_SANDYBRIDGE},
    /* 2394 */ {I_VMAXSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+10295, IF_AVX|IF_SANDYBRIDGE},
    /* 2395 */ {I_VMAXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+10302, IF_AVX|IF_SANDYBRIDGE},
    /* 2396 */ {I_VMINPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10309, IF_AVX|IF_SANDYBRIDGE},
    /* 2397 */ {I_VMINPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10316, IF_AVX|IF_SANDYBRIDGE},
    /* 2398 */ {I_VMINPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10323, IF_AVX|IF_SANDYBRIDGE},
    /* 2399 */ {I_VMINPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10330, IF_AVX|IF_SANDYBRIDGE},
    /* 2400 */ {I_VMINPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10337, IF_AVX|IF_SANDYBRIDGE},
    /* 2401 */ {I_VMINPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10344, IF_AVX|IF_SANDYBRIDGE},
    /* 2402 */ {I_VMINPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10351, IF_AVX|IF_SANDYBRIDGE},
    /* 2403 */ {I_VMINPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10358, IF_AVX|IF_SANDYBRIDGE},
    /* 2404 */ {I_VMINSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+10365, IF_AVX|IF_SANDYBRIDGE},
    /* 2405 */ {I_VMINSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+10372, IF_AVX|IF_SANDYBRIDGE},
    /* 2406 */ {I_VMINSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+10379, IF_AVX|IF_SANDYBRIDGE},
    /* 2407 */ {I_VMINSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+10386, IF_AVX|IF_SANDYBRIDGE},
    /* 2408 */ {I_VMOVAPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10393, IF_AVX|IF_SANDYBRIDGE},
    /* 2409 */ {I_VMOVAPD, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10400, IF_AVX|IF_SANDYBRIDGE},
    /* 2410 */ {I_VMOVAPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10407, IF_AVX|IF_SANDYBRIDGE},
    /* 2411 */ {I_VMOVAPD, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10414, IF_AVX|IF_SANDYBRIDGE},
    /* 2412 */ {I_VMOVAPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10421, IF_AVX|IF_SANDYBRIDGE},
    /* 2413 */ {I_VMOVAPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10428, IF_AVX|IF_SANDYBRIDGE},
    /* 2414 */ {I_VMOVAPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10435, IF_AVX|IF_SANDYBRIDGE},
    /* 2415 */ {I_VMOVAPS, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10442, IF_AVX|IF_SANDYBRIDGE},
    /* 2416 */ {I_VMOVD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+10449, IF_AVX|IF_SANDYBRIDGE},
    /* 2417 */ {I_VMOVD, 2, {RM_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+10456, IF_AVX|IF_SANDYBRIDGE},
    /* 2418 */ {I_VMOVQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+10463, IF_AVX|IF_SANDYBRIDGE|IF_SQ},
    /* 2419 */ {I_VMOVQ, 2, {RM_XMM|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10470, IF_AVX|IF_SANDYBRIDGE|IF_SQ},
    /* 2420 */ {I_VMOVQ, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+10477, IF_AVX|IF_SANDYBRIDGE|IF_LONG|IF_SQ},
    /* 2421 */ {I_VMOVQ, 2, {RM_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10484, IF_AVX|IF_SANDYBRIDGE|IF_LONG|IF_SQ},
    /* 2422 */ {I_VMOVDDUP, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+10491, IF_AVX|IF_SANDYBRIDGE},
    /* 2423 */ {I_VMOVDDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10498, IF_AVX|IF_SANDYBRIDGE},
    /* 2424 */ {I_VMOVDQA, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10505, IF_AVX|IF_SANDYBRIDGE},
    /* 2425 */ {I_VMOVDQA, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10512, IF_AVX|IF_SANDYBRIDGE},
    /* 2426 */ {I_VMOVQQA, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10519, IF_AVX|IF_SANDYBRIDGE},
    /* 2427 */ {I_VMOVQQA, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10526, IF_AVX|IF_SANDYBRIDGE},
    /* 2428 */ {I_VMOVDQA, 2, {YMMREG,RM_YMM,0,0,0}, nasm_bytecodes+10519, IF_AVX|IF_SANDYBRIDGE},
    /* 2429 */ {I_VMOVDQA, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10526, IF_AVX|IF_SANDYBRIDGE},
    /* 2430 */ {I_VMOVDQU, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10533, IF_AVX|IF_SANDYBRIDGE},
    /* 2431 */ {I_VMOVDQU, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10540, IF_AVX|IF_SANDYBRIDGE},
    /* 2432 */ {I_VMOVQQU, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10547, IF_AVX|IF_SANDYBRIDGE},
    /* 2433 */ {I_VMOVQQU, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10554, IF_AVX|IF_SANDYBRIDGE},
    /* 2434 */ {I_VMOVDQU, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10547, IF_AVX|IF_SANDYBRIDGE},
    /* 2435 */ {I_VMOVDQU, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10554, IF_AVX|IF_SANDYBRIDGE},
    /* 2436 */ {I_VMOVHLPS, 3, {XMMREG,XMMREG,XMMREG,0,0}, nasm_bytecodes+10561, IF_AVX|IF_SANDYBRIDGE},
    /* 2437 */ {I_VMOVHLPS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+10568, IF_AVX|IF_SANDYBRIDGE},
    /* 2438 */ {I_VMOVHPD, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, nasm_bytecodes+10575, IF_AVX|IF_SANDYBRIDGE},
    /* 2439 */ {I_VMOVHPD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+10582, IF_AVX|IF_SANDYBRIDGE},
    /* 2440 */ {I_VMOVHPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10589, IF_AVX|IF_SANDYBRIDGE},
    /* 2441 */ {I_VMOVHPS, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, nasm_bytecodes+10596, IF_AVX|IF_SANDYBRIDGE},
    /* 2442 */ {I_VMOVHPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+10603, IF_AVX|IF_SANDYBRIDGE},
    /* 2443 */ {I_VMOVHPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10610, IF_AVX|IF_SANDYBRIDGE},
    /* 2444 */ {I_VMOVLHPS, 3, {XMMREG,XMMREG,XMMREG,0,0}, nasm_bytecodes+10596, IF_AVX|IF_SANDYBRIDGE},
    /* 2445 */ {I_VMOVLHPS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+10603, IF_AVX|IF_SANDYBRIDGE},
    /* 2446 */ {I_VMOVLPD, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, nasm_bytecodes+10617, IF_AVX|IF_SANDYBRIDGE},
    /* 2447 */ {I_VMOVLPD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+10624, IF_AVX|IF_SANDYBRIDGE},
    /* 2448 */ {I_VMOVLPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10631, IF_AVX|IF_SANDYBRIDGE},
    /* 2449 */ {I_VMOVLPS, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, nasm_bytecodes+10561, IF_AVX|IF_SANDYBRIDGE},
    /* 2450 */ {I_VMOVLPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+10568, IF_AVX|IF_SANDYBRIDGE},
    /* 2451 */ {I_VMOVLPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10638, IF_AVX|IF_SANDYBRIDGE},
    /* 2452 */ {I_VMOVMSKPD, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10645, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2453 */ {I_VMOVMSKPD, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+10645, IF_AVX|IF_SANDYBRIDGE},
    /* 2454 */ {I_VMOVMSKPD, 2, {REG_GPR|BITS64,YMMREG,0,0,0}, nasm_bytecodes+10652, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2455 */ {I_VMOVMSKPD, 2, {REG_GPR|BITS32,YMMREG,0,0,0}, nasm_bytecodes+10652, IF_AVX|IF_SANDYBRIDGE},
    /* 2456 */ {I_VMOVMSKPS, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10659, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2457 */ {I_VMOVMSKPS, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+10659, IF_AVX|IF_SANDYBRIDGE},
    /* 2458 */ {I_VMOVMSKPS, 2, {REG_GPR|BITS64,YMMREG,0,0,0}, nasm_bytecodes+10666, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2459 */ {I_VMOVMSKPS, 2, {REG_GPR|BITS32,YMMREG,0,0,0}, nasm_bytecodes+10666, IF_AVX|IF_SANDYBRIDGE},
    /* 2460 */ {I_VMOVNTDQ, 2, {MEMORY|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10673, IF_AVX|IF_SANDYBRIDGE},
    /* 2461 */ {I_VMOVNTQQ, 2, {MEMORY|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10680, IF_AVX|IF_SANDYBRIDGE},
    /* 2462 */ {I_VMOVNTDQ, 2, {MEMORY|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10680, IF_AVX|IF_SANDYBRIDGE},
    /* 2463 */ {I_VMOVNTDQA, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+10687, IF_AVX|IF_SANDYBRIDGE},
    /* 2464 */ {I_VMOVNTPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10694, IF_AVX|IF_SANDYBRIDGE},
    /* 2465 */ {I_VMOVNTPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10701, IF_AVX|IF_SANDYBRIDGE},
    /* 2466 */ {I_VMOVNTPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10708, IF_AVX|IF_SANDYBRIDGE},
    /* 2467 */ {I_VMOVNTPS, 2, {MEMORY|BITS128,YMMREG,0,0,0}, nasm_bytecodes+10715, IF_AVX|IF_SANDYBRIDGE},
    /* 2468 */ {I_VMOVSD, 3, {XMMREG,XMMREG,XMMREG,0,0}, nasm_bytecodes+10722, IF_AVX|IF_SANDYBRIDGE},
    /* 2469 */ {I_VMOVSD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+10729, IF_AVX|IF_SANDYBRIDGE},
    /* 2470 */ {I_VMOVSD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+10736, IF_AVX|IF_SANDYBRIDGE},
    /* 2471 */ {I_VMOVSD, 3, {XMMREG,XMMREG,XMMREG,0,0}, nasm_bytecodes+10743, IF_AVX|IF_SANDYBRIDGE},
    /* 2472 */ {I_VMOVSD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+10750, IF_AVX|IF_SANDYBRIDGE},
    /* 2473 */ {I_VMOVSD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, nasm_bytecodes+10757, IF_AVX|IF_SANDYBRIDGE},
    /* 2474 */ {I_VMOVSHDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10764, IF_AVX|IF_SANDYBRIDGE},
    /* 2475 */ {I_VMOVSHDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10771, IF_AVX|IF_SANDYBRIDGE},
    /* 2476 */ {I_VMOVSLDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10778, IF_AVX|IF_SANDYBRIDGE},
    /* 2477 */ {I_VMOVSLDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10785, IF_AVX|IF_SANDYBRIDGE},
    /* 2478 */ {I_VMOVSS, 3, {XMMREG,XMMREG,XMMREG,0,0}, nasm_bytecodes+10792, IF_AVX|IF_SANDYBRIDGE},
    /* 2479 */ {I_VMOVSS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+10799, IF_AVX|IF_SANDYBRIDGE},
    /* 2480 */ {I_VMOVSS, 2, {XMMREG,MEMORY|BITS32,0,0,0}, nasm_bytecodes+10806, IF_AVX|IF_SANDYBRIDGE},
    /* 2481 */ {I_VMOVSS, 3, {XMMREG,XMMREG,XMMREG,0,0}, nasm_bytecodes+10813, IF_AVX|IF_SANDYBRIDGE},
    /* 2482 */ {I_VMOVSS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+10820, IF_AVX|IF_SANDYBRIDGE},
    /* 2483 */ {I_VMOVSS, 2, {MEMORY|BITS32,XMMREG,0,0,0}, nasm_bytecodes+10827, IF_AVX|IF_SANDYBRIDGE},
    /* 2484 */ {I_VMOVUPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10834, IF_AVX|IF_SANDYBRIDGE},
    /* 2485 */ {I_VMOVUPD, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10841, IF_AVX|IF_SANDYBRIDGE},
    /* 2486 */ {I_VMOVUPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10848, IF_AVX|IF_SANDYBRIDGE},
    /* 2487 */ {I_VMOVUPD, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10855, IF_AVX|IF_SANDYBRIDGE},
    /* 2488 */ {I_VMOVUPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10862, IF_AVX|IF_SANDYBRIDGE},
    /* 2489 */ {I_VMOVUPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, nasm_bytecodes+10869, IF_AVX|IF_SANDYBRIDGE},
    /* 2490 */ {I_VMOVUPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10876, IF_AVX|IF_SANDYBRIDGE},
    /* 2491 */ {I_VMOVUPS, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, nasm_bytecodes+10883, IF_AVX|IF_SANDYBRIDGE},
    /* 2492 */ {I_VMPSADBW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4756, IF_AVX|IF_SANDYBRIDGE},
    /* 2493 */ {I_VMPSADBW, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4764, IF_AVX|IF_SANDYBRIDGE},
    /* 2494 */ {I_VMULPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10890, IF_AVX|IF_SANDYBRIDGE},
    /* 2495 */ {I_VMULPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10897, IF_AVX|IF_SANDYBRIDGE},
    /* 2496 */ {I_VMULPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10904, IF_AVX|IF_SANDYBRIDGE},
    /* 2497 */ {I_VMULPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10911, IF_AVX|IF_SANDYBRIDGE},
    /* 2498 */ {I_VMULPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10918, IF_AVX|IF_SANDYBRIDGE},
    /* 2499 */ {I_VMULPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10925, IF_AVX|IF_SANDYBRIDGE},
    /* 2500 */ {I_VMULPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10932, IF_AVX|IF_SANDYBRIDGE},
    /* 2501 */ {I_VMULPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10939, IF_AVX|IF_SANDYBRIDGE},
    /* 2502 */ {I_VMULSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+10946, IF_AVX|IF_SANDYBRIDGE},
    /* 2503 */ {I_VMULSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+10953, IF_AVX|IF_SANDYBRIDGE},
    /* 2504 */ {I_VMULSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+10960, IF_AVX|IF_SANDYBRIDGE},
    /* 2505 */ {I_VMULSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+10967, IF_AVX|IF_SANDYBRIDGE},
    /* 2506 */ {I_VORPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+10974, IF_AVX|IF_SANDYBRIDGE},
    /* 2507 */ {I_VORPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+10981, IF_AVX|IF_SANDYBRIDGE},
    /* 2508 */ {I_VORPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+10988, IF_AVX|IF_SANDYBRIDGE},
    /* 2509 */ {I_VORPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+10995, IF_AVX|IF_SANDYBRIDGE},
    /* 2510 */ {I_VORPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11002, IF_AVX|IF_SANDYBRIDGE},
    /* 2511 */ {I_VORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11009, IF_AVX|IF_SANDYBRIDGE},
    /* 2512 */ {I_VORPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+11016, IF_AVX|IF_SANDYBRIDGE},
    /* 2513 */ {I_VORPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+11023, IF_AVX|IF_SANDYBRIDGE},
    /* 2514 */ {I_VPABSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11030, IF_AVX|IF_SANDYBRIDGE},
    /* 2515 */ {I_VPABSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11037, IF_AVX|IF_SANDYBRIDGE},
    /* 2516 */ {I_VPABSD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11044, IF_AVX|IF_SANDYBRIDGE},
    /* 2517 */ {I_VPACKSSWB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11051, IF_AVX|IF_SANDYBRIDGE},
    /* 2518 */ {I_VPACKSSWB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11058, IF_AVX|IF_SANDYBRIDGE},
    /* 2519 */ {I_VPACKSSDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11065, IF_AVX|IF_SANDYBRIDGE},
    /* 2520 */ {I_VPACKSSDW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11072, IF_AVX|IF_SANDYBRIDGE},
    /* 2521 */ {I_VPACKUSWB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11079, IF_AVX|IF_SANDYBRIDGE},
    /* 2522 */ {I_VPACKUSWB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11086, IF_AVX|IF_SANDYBRIDGE},
    /* 2523 */ {I_VPACKUSDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11093, IF_AVX|IF_SANDYBRIDGE},
    /* 2524 */ {I_VPACKUSDW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11100, IF_AVX|IF_SANDYBRIDGE},
    /* 2525 */ {I_VPADDB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11107, IF_AVX|IF_SANDYBRIDGE},
    /* 2526 */ {I_VPADDB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11114, IF_AVX|IF_SANDYBRIDGE},
    /* 2527 */ {I_VPADDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11121, IF_AVX|IF_SANDYBRIDGE},
    /* 2528 */ {I_VPADDW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11128, IF_AVX|IF_SANDYBRIDGE},
    /* 2529 */ {I_VPADDD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11135, IF_AVX|IF_SANDYBRIDGE},
    /* 2530 */ {I_VPADDD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11142, IF_AVX|IF_SANDYBRIDGE},
    /* 2531 */ {I_VPADDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11149, IF_AVX|IF_SANDYBRIDGE},
    /* 2532 */ {I_VPADDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11156, IF_AVX|IF_SANDYBRIDGE},
    /* 2533 */ {I_VPADDSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11163, IF_AVX|IF_SANDYBRIDGE},
    /* 2534 */ {I_VPADDSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11170, IF_AVX|IF_SANDYBRIDGE},
    /* 2535 */ {I_VPADDSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11177, IF_AVX|IF_SANDYBRIDGE},
    /* 2536 */ {I_VPADDSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11184, IF_AVX|IF_SANDYBRIDGE},
    /* 2537 */ {I_VPADDUSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11191, IF_AVX|IF_SANDYBRIDGE},
    /* 2538 */ {I_VPADDUSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11198, IF_AVX|IF_SANDYBRIDGE},
    /* 2539 */ {I_VPADDUSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11205, IF_AVX|IF_SANDYBRIDGE},
    /* 2540 */ {I_VPADDUSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11212, IF_AVX|IF_SANDYBRIDGE},
    /* 2541 */ {I_VPALIGNR, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4772, IF_AVX|IF_SANDYBRIDGE},
    /* 2542 */ {I_VPALIGNR, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4780, IF_AVX|IF_SANDYBRIDGE},
    /* 2543 */ {I_VPAND, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11219, IF_AVX|IF_SANDYBRIDGE},
    /* 2544 */ {I_VPAND, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11226, IF_AVX|IF_SANDYBRIDGE},
    /* 2545 */ {I_VPANDN, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11233, IF_AVX|IF_SANDYBRIDGE},
    /* 2546 */ {I_VPANDN, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11240, IF_AVX|IF_SANDYBRIDGE},
    /* 2547 */ {I_VPAVGB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11247, IF_AVX|IF_SANDYBRIDGE},
    /* 2548 */ {I_VPAVGB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11254, IF_AVX|IF_SANDYBRIDGE},
    /* 2549 */ {I_VPAVGW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11261, IF_AVX|IF_SANDYBRIDGE},
    /* 2550 */ {I_VPAVGW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11268, IF_AVX|IF_SANDYBRIDGE},
    /* 2551 */ {I_VPBLENDVB, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+4788, IF_AVX|IF_SANDYBRIDGE},
    /* 2552 */ {I_VPBLENDVB, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+4796, IF_AVX|IF_SANDYBRIDGE},
    /* 2553 */ {I_VPBLENDW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+4804, IF_AVX|IF_SANDYBRIDGE},
    /* 2554 */ {I_VPBLENDW, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4812, IF_AVX|IF_SANDYBRIDGE},
    /* 2555 */ {I_VPCMPESTRI, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4820, IF_AVX|IF_SANDYBRIDGE},
    /* 2556 */ {I_VPCMPESTRM, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4828, IF_AVX|IF_SANDYBRIDGE},
    /* 2557 */ {I_VPCMPISTRI, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4836, IF_AVX|IF_SANDYBRIDGE},
    /* 2558 */ {I_VPCMPISTRM, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4844, IF_AVX|IF_SANDYBRIDGE},
    /* 2559 */ {I_VPCMPEQB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11275, IF_AVX|IF_SANDYBRIDGE},
    /* 2560 */ {I_VPCMPEQB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11282, IF_AVX|IF_SANDYBRIDGE},
    /* 2561 */ {I_VPCMPEQW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11289, IF_AVX|IF_SANDYBRIDGE},
    /* 2562 */ {I_VPCMPEQW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11296, IF_AVX|IF_SANDYBRIDGE},
    /* 2563 */ {I_VPCMPEQD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11303, IF_AVX|IF_SANDYBRIDGE},
    /* 2564 */ {I_VPCMPEQD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11310, IF_AVX|IF_SANDYBRIDGE},
    /* 2565 */ {I_VPCMPEQQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11317, IF_AVX|IF_SANDYBRIDGE},
    /* 2566 */ {I_VPCMPEQQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11324, IF_AVX|IF_SANDYBRIDGE},
    /* 2567 */ {I_VPCMPGTB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11331, IF_AVX|IF_SANDYBRIDGE},
    /* 2568 */ {I_VPCMPGTB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11338, IF_AVX|IF_SANDYBRIDGE},
    /* 2569 */ {I_VPCMPGTW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11345, IF_AVX|IF_SANDYBRIDGE},
    /* 2570 */ {I_VPCMPGTW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11352, IF_AVX|IF_SANDYBRIDGE},
    /* 2571 */ {I_VPCMPGTD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11359, IF_AVX|IF_SANDYBRIDGE},
    /* 2572 */ {I_VPCMPGTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11366, IF_AVX|IF_SANDYBRIDGE},
    /* 2573 */ {I_VPCMPGTQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11373, IF_AVX|IF_SANDYBRIDGE},
    /* 2574 */ {I_VPCMPGTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11380, IF_AVX|IF_SANDYBRIDGE},
    /* 2575 */ {I_VPERMILPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11387, IF_AVX|IF_SANDYBRIDGE},
    /* 2576 */ {I_VPERMILPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+11394, IF_AVX|IF_SANDYBRIDGE},
    /* 2577 */ {I_VPERMILPD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4852, IF_AVX|IF_SANDYBRIDGE},
    /* 2578 */ {I_VPERMILPD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4860, IF_AVX|IF_SANDYBRIDGE},
    /* 2579 */ {I_VPERMILPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11401, IF_AVX|IF_SANDYBRIDGE},
    /* 2580 */ {I_VPERMILPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+11408, IF_AVX|IF_SANDYBRIDGE},
    /* 2581 */ {I_VPERMILPS, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4868, IF_AVX|IF_SANDYBRIDGE},
    /* 2582 */ {I_VPERMILPS, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4876, IF_AVX|IF_SANDYBRIDGE},
    /* 2583 */ {I_VPERM2F128, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+4884, IF_AVX|IF_SANDYBRIDGE},
    /* 2584 */ {I_VPEXTRB, 3, {REG_GPR|BITS64,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4892, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2585 */ {I_VPEXTRB, 3, {REG_GPR|BITS32,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4892, IF_AVX|IF_SANDYBRIDGE},
    /* 2586 */ {I_VPEXTRB, 3, {MEMORY|BITS8,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4892, IF_AVX|IF_SANDYBRIDGE},
    /* 2587 */ {I_VPEXTRW, 3, {REG_GPR|BITS64,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4900, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2588 */ {I_VPEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4900, IF_AVX|IF_SANDYBRIDGE},
    /* 2589 */ {I_VPEXTRW, 3, {REG_GPR|BITS64,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4908, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2590 */ {I_VPEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4908, IF_AVX|IF_SANDYBRIDGE},
    /* 2591 */ {I_VPEXTRW, 3, {MEMORY|BITS16,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4908, IF_AVX|IF_SANDYBRIDGE},
    /* 2592 */ {I_VPEXTRD, 3, {REG_GPR|BITS64,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4916, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2593 */ {I_VPEXTRD, 3, {RM_GPR|BITS32,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4916, IF_AVX|IF_SANDYBRIDGE},
    /* 2594 */ {I_VPEXTRQ, 3, {RM_GPR|BITS64,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4924, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2595 */ {I_VPHADDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11415, IF_AVX|IF_SANDYBRIDGE},
    /* 2596 */ {I_VPHADDW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11422, IF_AVX|IF_SANDYBRIDGE},
    /* 2597 */ {I_VPHADDD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11429, IF_AVX|IF_SANDYBRIDGE},
    /* 2598 */ {I_VPHADDD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11436, IF_AVX|IF_SANDYBRIDGE},
    /* 2599 */ {I_VPHADDSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11443, IF_AVX|IF_SANDYBRIDGE},
    /* 2600 */ {I_VPHADDSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11450, IF_AVX|IF_SANDYBRIDGE},
    /* 2601 */ {I_VPHMINPOSUW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11457, IF_AVX|IF_SANDYBRIDGE},
    /* 2602 */ {I_VPHSUBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11464, IF_AVX|IF_SANDYBRIDGE},
    /* 2603 */ {I_VPHSUBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11471, IF_AVX|IF_SANDYBRIDGE},
    /* 2604 */ {I_VPHSUBD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11478, IF_AVX|IF_SANDYBRIDGE},
    /* 2605 */ {I_VPHSUBD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11485, IF_AVX|IF_SANDYBRIDGE},
    /* 2606 */ {I_VPHSUBSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11492, IF_AVX|IF_SANDYBRIDGE},
    /* 2607 */ {I_VPHSUBSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11499, IF_AVX|IF_SANDYBRIDGE},
    /* 2608 */ {I_VPINSRB, 4, {XMMREG,XMMREG,MEMORY|BITS8,IMMEDIATE|BITS8,0}, nasm_bytecodes+4932, IF_AVX|IF_SANDYBRIDGE},
    /* 2609 */ {I_VPINSRB, 3, {XMMREG,MEMORY|BITS8,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4940, IF_AVX|IF_SANDYBRIDGE},
    /* 2610 */ {I_VPINSRB, 4, {XMMREG,XMMREG,RM_GPR|BITS8,IMMEDIATE|BITS8,0}, nasm_bytecodes+4932, IF_AVX|IF_SANDYBRIDGE},
    /* 2611 */ {I_VPINSRB, 3, {XMMREG,RM_GPR|BITS8,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4940, IF_AVX|IF_SANDYBRIDGE},
    /* 2612 */ {I_VPINSRB, 4, {XMMREG,XMMREG,REG_GPR|BITS32,IMMEDIATE|BITS8,0}, nasm_bytecodes+4932, IF_AVX|IF_SANDYBRIDGE},
    /* 2613 */ {I_VPINSRB, 3, {XMMREG,REG_GPR|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4940, IF_AVX|IF_SANDYBRIDGE},
    /* 2614 */ {I_VPINSRW, 4, {XMMREG,XMMREG,MEMORY|BITS16,IMMEDIATE|BITS8,0}, nasm_bytecodes+4948, IF_AVX|IF_SANDYBRIDGE},
    /* 2615 */ {I_VPINSRW, 3, {XMMREG,MEMORY|BITS16,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4956, IF_AVX|IF_SANDYBRIDGE},
    /* 2616 */ {I_VPINSRW, 4, {XMMREG,XMMREG,RM_GPR|BITS16,IMMEDIATE|BITS8,0}, nasm_bytecodes+4948, IF_AVX|IF_SANDYBRIDGE},
    /* 2617 */ {I_VPINSRW, 3, {XMMREG,RM_GPR|BITS16,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4956, IF_AVX|IF_SANDYBRIDGE},
    /* 2618 */ {I_VPINSRW, 4, {XMMREG,XMMREG,REG_GPR|BITS32,IMMEDIATE|BITS8,0}, nasm_bytecodes+4948, IF_AVX|IF_SANDYBRIDGE},
    /* 2619 */ {I_VPINSRW, 3, {XMMREG,REG_GPR|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4956, IF_AVX|IF_SANDYBRIDGE},
    /* 2620 */ {I_VPINSRD, 4, {XMMREG,XMMREG,MEMORY|BITS32,IMMEDIATE|BITS8,0}, nasm_bytecodes+4964, IF_AVX|IF_SANDYBRIDGE},
    /* 2621 */ {I_VPINSRD, 3, {XMMREG,MEMORY|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4972, IF_AVX|IF_SANDYBRIDGE},
    /* 2622 */ {I_VPINSRD, 4, {XMMREG,XMMREG,RM_GPR|BITS32,IMMEDIATE|BITS8,0}, nasm_bytecodes+4964, IF_AVX|IF_SANDYBRIDGE},
    /* 2623 */ {I_VPINSRD, 3, {XMMREG,RM_GPR|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4972, IF_AVX|IF_SANDYBRIDGE},
    /* 2624 */ {I_VPINSRQ, 4, {XMMREG,XMMREG,MEMORY|BITS64,IMMEDIATE|BITS8,0}, nasm_bytecodes+4980, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2625 */ {I_VPINSRQ, 3, {XMMREG,MEMORY|BITS64,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4988, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2626 */ {I_VPINSRQ, 4, {XMMREG,XMMREG,RM_GPR|BITS64,IMMEDIATE|BITS8,0}, nasm_bytecodes+4980, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2627 */ {I_VPINSRQ, 3, {XMMREG,RM_GPR|BITS64,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4988, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2628 */ {I_VPMADDWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11506, IF_AVX|IF_SANDYBRIDGE},
    /* 2629 */ {I_VPMADDWD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11513, IF_AVX|IF_SANDYBRIDGE},
    /* 2630 */ {I_VPMADDUBSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11520, IF_AVX|IF_SANDYBRIDGE},
    /* 2631 */ {I_VPMADDUBSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11527, IF_AVX|IF_SANDYBRIDGE},
    /* 2632 */ {I_VPMAXSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11534, IF_AVX|IF_SANDYBRIDGE},
    /* 2633 */ {I_VPMAXSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11541, IF_AVX|IF_SANDYBRIDGE},
    /* 2634 */ {I_VPMAXSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11548, IF_AVX|IF_SANDYBRIDGE},
    /* 2635 */ {I_VPMAXSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11555, IF_AVX|IF_SANDYBRIDGE},
    /* 2636 */ {I_VPMAXSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11562, IF_AVX|IF_SANDYBRIDGE},
    /* 2637 */ {I_VPMAXSD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11569, IF_AVX|IF_SANDYBRIDGE},
    /* 2638 */ {I_VPMAXUB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11576, IF_AVX|IF_SANDYBRIDGE},
    /* 2639 */ {I_VPMAXUB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11583, IF_AVX|IF_SANDYBRIDGE},
    /* 2640 */ {I_VPMAXUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11590, IF_AVX|IF_SANDYBRIDGE},
    /* 2641 */ {I_VPMAXUW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11597, IF_AVX|IF_SANDYBRIDGE},
    /* 2642 */ {I_VPMAXUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11604, IF_AVX|IF_SANDYBRIDGE},
    /* 2643 */ {I_VPMAXUD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11611, IF_AVX|IF_SANDYBRIDGE},
    /* 2644 */ {I_VPMINSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11618, IF_AVX|IF_SANDYBRIDGE},
    /* 2645 */ {I_VPMINSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11625, IF_AVX|IF_SANDYBRIDGE},
    /* 2646 */ {I_VPMINSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11632, IF_AVX|IF_SANDYBRIDGE},
    /* 2647 */ {I_VPMINSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11639, IF_AVX|IF_SANDYBRIDGE},
    /* 2648 */ {I_VPMINSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11646, IF_AVX|IF_SANDYBRIDGE},
    /* 2649 */ {I_VPMINSD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11653, IF_AVX|IF_SANDYBRIDGE},
    /* 2650 */ {I_VPMINUB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11660, IF_AVX|IF_SANDYBRIDGE},
    /* 2651 */ {I_VPMINUB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11667, IF_AVX|IF_SANDYBRIDGE},
    /* 2652 */ {I_VPMINUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11674, IF_AVX|IF_SANDYBRIDGE},
    /* 2653 */ {I_VPMINUW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11681, IF_AVX|IF_SANDYBRIDGE},
    /* 2654 */ {I_VPMINUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11688, IF_AVX|IF_SANDYBRIDGE},
    /* 2655 */ {I_VPMINUD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11695, IF_AVX|IF_SANDYBRIDGE},
    /* 2656 */ {I_VPMOVMSKB, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, nasm_bytecodes+11702, IF_AVX|IF_SANDYBRIDGE|IF_LONG},
    /* 2657 */ {I_VPMOVMSKB, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, nasm_bytecodes+11702, IF_AVX|IF_SANDYBRIDGE},
    /* 2658 */ {I_VPMOVSXBW, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+11709, IF_AVX|IF_SANDYBRIDGE},
    /* 2659 */ {I_VPMOVSXBD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+11716, IF_AVX|IF_SANDYBRIDGE},
    /* 2660 */ {I_VPMOVSXBQ, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, nasm_bytecodes+11723, IF_AVX|IF_SANDYBRIDGE},
    /* 2661 */ {I_VPMOVSXWD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+11730, IF_AVX|IF_SANDYBRIDGE},
    /* 2662 */ {I_VPMOVSXWQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+11737, IF_AVX|IF_SANDYBRIDGE},
    /* 2663 */ {I_VPMOVSXDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+11744, IF_AVX|IF_SANDYBRIDGE},
    /* 2664 */ {I_VPMOVZXBW, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+11751, IF_AVX|IF_SANDYBRIDGE},
    /* 2665 */ {I_VPMOVZXBD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+11758, IF_AVX|IF_SANDYBRIDGE},
    /* 2666 */ {I_VPMOVZXBQ, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, nasm_bytecodes+11765, IF_AVX|IF_SANDYBRIDGE},
    /* 2667 */ {I_VPMOVZXWD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+11772, IF_AVX|IF_SANDYBRIDGE},
    /* 2668 */ {I_VPMOVZXWQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+11779, IF_AVX|IF_SANDYBRIDGE},
    /* 2669 */ {I_VPMOVZXDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+11786, IF_AVX|IF_SANDYBRIDGE},
    /* 2670 */ {I_VPMULHUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11793, IF_AVX|IF_SANDYBRIDGE},
    /* 2671 */ {I_VPMULHUW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11800, IF_AVX|IF_SANDYBRIDGE},
    /* 2672 */ {I_VPMULHRSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11807, IF_AVX|IF_SANDYBRIDGE},
    /* 2673 */ {I_VPMULHRSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11814, IF_AVX|IF_SANDYBRIDGE},
    /* 2674 */ {I_VPMULHW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11821, IF_AVX|IF_SANDYBRIDGE},
    /* 2675 */ {I_VPMULHW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11828, IF_AVX|IF_SANDYBRIDGE},
    /* 2676 */ {I_VPMULLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11835, IF_AVX|IF_SANDYBRIDGE},
    /* 2677 */ {I_VPMULLW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11842, IF_AVX|IF_SANDYBRIDGE},
    /* 2678 */ {I_VPMULLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11849, IF_AVX|IF_SANDYBRIDGE},
    /* 2679 */ {I_VPMULLD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11856, IF_AVX|IF_SANDYBRIDGE},
    /* 2680 */ {I_VPMULUDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11863, IF_AVX|IF_SANDYBRIDGE},
    /* 2681 */ {I_VPMULUDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11870, IF_AVX|IF_SANDYBRIDGE},
    /* 2682 */ {I_VPMULDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11877, IF_AVX|IF_SANDYBRIDGE},
    /* 2683 */ {I_VPMULDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11884, IF_AVX|IF_SANDYBRIDGE},
    /* 2684 */ {I_VPOR, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11891, IF_AVX|IF_SANDYBRIDGE},
    /* 2685 */ {I_VPOR, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11898, IF_AVX|IF_SANDYBRIDGE},
    /* 2686 */ {I_VPSADBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11905, IF_AVX|IF_SANDYBRIDGE},
    /* 2687 */ {I_VPSADBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11912, IF_AVX|IF_SANDYBRIDGE},
    /* 2688 */ {I_VPSHUFB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11919, IF_AVX|IF_SANDYBRIDGE},
    /* 2689 */ {I_VPSHUFB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11926, IF_AVX|IF_SANDYBRIDGE},
    /* 2690 */ {I_VPSHUFD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+4996, IF_AVX|IF_SANDYBRIDGE},
    /* 2691 */ {I_VPSHUFHW, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5004, IF_AVX|IF_SANDYBRIDGE},
    /* 2692 */ {I_VPSHUFLW, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5012, IF_AVX|IF_SANDYBRIDGE},
    /* 2693 */ {I_VPSIGNB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11933, IF_AVX|IF_SANDYBRIDGE},
    /* 2694 */ {I_VPSIGNB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11940, IF_AVX|IF_SANDYBRIDGE},
    /* 2695 */ {I_VPSIGNW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11947, IF_AVX|IF_SANDYBRIDGE},
    /* 2696 */ {I_VPSIGNW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11954, IF_AVX|IF_SANDYBRIDGE},
    /* 2697 */ {I_VPSIGND, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11961, IF_AVX|IF_SANDYBRIDGE},
    /* 2698 */ {I_VPSIGND, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11968, IF_AVX|IF_SANDYBRIDGE},
    /* 2699 */ {I_VPSLLDQ, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5020, IF_AVX|IF_SANDYBRIDGE},
    /* 2700 */ {I_VPSLLDQ, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5028, IF_AVX|IF_SANDYBRIDGE},
    /* 2701 */ {I_VPSRLDQ, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5036, IF_AVX|IF_SANDYBRIDGE},
    /* 2702 */ {I_VPSRLDQ, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5044, IF_AVX|IF_SANDYBRIDGE},
    /* 2703 */ {I_VPSLLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11975, IF_AVX|IF_SANDYBRIDGE},
    /* 2704 */ {I_VPSLLW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11982, IF_AVX|IF_SANDYBRIDGE},
    /* 2705 */ {I_VPSLLW, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5052, IF_AVX|IF_SANDYBRIDGE},
    /* 2706 */ {I_VPSLLW, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5060, IF_AVX|IF_SANDYBRIDGE},
    /* 2707 */ {I_VPSLLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+11989, IF_AVX|IF_SANDYBRIDGE},
    /* 2708 */ {I_VPSLLD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+11996, IF_AVX|IF_SANDYBRIDGE},
    /* 2709 */ {I_VPSLLD, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5068, IF_AVX|IF_SANDYBRIDGE},
    /* 2710 */ {I_VPSLLD, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5076, IF_AVX|IF_SANDYBRIDGE},
    /* 2711 */ {I_VPSLLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12003, IF_AVX|IF_SANDYBRIDGE},
    /* 2712 */ {I_VPSLLQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12010, IF_AVX|IF_SANDYBRIDGE},
    /* 2713 */ {I_VPSLLQ, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5084, IF_AVX|IF_SANDYBRIDGE},
    /* 2714 */ {I_VPSLLQ, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5092, IF_AVX|IF_SANDYBRIDGE},
    /* 2715 */ {I_VPSRAW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12017, IF_AVX|IF_SANDYBRIDGE},
    /* 2716 */ {I_VPSRAW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12024, IF_AVX|IF_SANDYBRIDGE},
    /* 2717 */ {I_VPSRAW, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5100, IF_AVX|IF_SANDYBRIDGE},
    /* 2718 */ {I_VPSRAW, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5108, IF_AVX|IF_SANDYBRIDGE},
    /* 2719 */ {I_VPSRAD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12031, IF_AVX|IF_SANDYBRIDGE},
    /* 2720 */ {I_VPSRAD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12038, IF_AVX|IF_SANDYBRIDGE},
    /* 2721 */ {I_VPSRAD, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5116, IF_AVX|IF_SANDYBRIDGE},
    /* 2722 */ {I_VPSRAD, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5124, IF_AVX|IF_SANDYBRIDGE},
    /* 2723 */ {I_VPSRLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12045, IF_AVX|IF_SANDYBRIDGE},
    /* 2724 */ {I_VPSRLW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12052, IF_AVX|IF_SANDYBRIDGE},
    /* 2725 */ {I_VPSRLW, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5132, IF_AVX|IF_SANDYBRIDGE},
    /* 2726 */ {I_VPSRLW, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5140, IF_AVX|IF_SANDYBRIDGE},
    /* 2727 */ {I_VPSRLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12059, IF_AVX|IF_SANDYBRIDGE},
    /* 2728 */ {I_VPSRLD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12066, IF_AVX|IF_SANDYBRIDGE},
    /* 2729 */ {I_VPSRLD, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5148, IF_AVX|IF_SANDYBRIDGE},
    /* 2730 */ {I_VPSRLD, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5156, IF_AVX|IF_SANDYBRIDGE},
    /* 2731 */ {I_VPSRLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12073, IF_AVX|IF_SANDYBRIDGE},
    /* 2732 */ {I_VPSRLQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12080, IF_AVX|IF_SANDYBRIDGE},
    /* 2733 */ {I_VPSRLQ, 3, {XMMREG,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5164, IF_AVX|IF_SANDYBRIDGE},
    /* 2734 */ {I_VPSRLQ, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+5172, IF_AVX|IF_SANDYBRIDGE},
    /* 2735 */ {I_VPTEST, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12087, IF_AVX|IF_SANDYBRIDGE},
    /* 2736 */ {I_VPTEST, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12094, IF_AVX|IF_SANDYBRIDGE},
    /* 2737 */ {I_VPSUBB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12101, IF_AVX|IF_SANDYBRIDGE},
    /* 2738 */ {I_VPSUBB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12108, IF_AVX|IF_SANDYBRIDGE},
    /* 2739 */ {I_VPSUBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12115, IF_AVX|IF_SANDYBRIDGE},
    /* 2740 */ {I_VPSUBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12122, IF_AVX|IF_SANDYBRIDGE},
    /* 2741 */ {I_VPSUBD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12129, IF_AVX|IF_SANDYBRIDGE},
    /* 2742 */ {I_VPSUBD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12136, IF_AVX|IF_SANDYBRIDGE},
    /* 2743 */ {I_VPSUBQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12143, IF_AVX|IF_SANDYBRIDGE},
    /* 2744 */ {I_VPSUBQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12150, IF_AVX|IF_SANDYBRIDGE},
    /* 2745 */ {I_VPSUBSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12157, IF_AVX|IF_SANDYBRIDGE},
    /* 2746 */ {I_VPSUBSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12164, IF_AVX|IF_SANDYBRIDGE},
    /* 2747 */ {I_VPSUBSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12171, IF_AVX|IF_SANDYBRIDGE},
    /* 2748 */ {I_VPSUBSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12178, IF_AVX|IF_SANDYBRIDGE},
    /* 2749 */ {I_VPSUBUSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12185, IF_AVX|IF_SANDYBRIDGE},
    /* 2750 */ {I_VPSUBUSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12192, IF_AVX|IF_SANDYBRIDGE},
    /* 2751 */ {I_VPSUBUSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12199, IF_AVX|IF_SANDYBRIDGE},
    /* 2752 */ {I_VPSUBUSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12206, IF_AVX|IF_SANDYBRIDGE},
    /* 2753 */ {I_VPUNPCKHBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12213, IF_AVX|IF_SANDYBRIDGE},
    /* 2754 */ {I_VPUNPCKHBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12220, IF_AVX|IF_SANDYBRIDGE},
    /* 2755 */ {I_VPUNPCKHWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12227, IF_AVX|IF_SANDYBRIDGE},
    /* 2756 */ {I_VPUNPCKHWD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12234, IF_AVX|IF_SANDYBRIDGE},
    /* 2757 */ {I_VPUNPCKHDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12241, IF_AVX|IF_SANDYBRIDGE},
    /* 2758 */ {I_VPUNPCKHDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12248, IF_AVX|IF_SANDYBRIDGE},
    /* 2759 */ {I_VPUNPCKHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12255, IF_AVX|IF_SANDYBRIDGE},
    /* 2760 */ {I_VPUNPCKHQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12262, IF_AVX|IF_SANDYBRIDGE},
    /* 2761 */ {I_VPUNPCKLBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12269, IF_AVX|IF_SANDYBRIDGE},
    /* 2762 */ {I_VPUNPCKLBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12276, IF_AVX|IF_SANDYBRIDGE},
    /* 2763 */ {I_VPUNPCKLWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12283, IF_AVX|IF_SANDYBRIDGE},
    /* 2764 */ {I_VPUNPCKLWD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12290, IF_AVX|IF_SANDYBRIDGE},
    /* 2765 */ {I_VPUNPCKLDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12297, IF_AVX|IF_SANDYBRIDGE},
    /* 2766 */ {I_VPUNPCKLDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12304, IF_AVX|IF_SANDYBRIDGE},
    /* 2767 */ {I_VPUNPCKLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12311, IF_AVX|IF_SANDYBRIDGE},
    /* 2768 */ {I_VPUNPCKLQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12318, IF_AVX|IF_SANDYBRIDGE},
    /* 2769 */ {I_VPXOR, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12325, IF_AVX|IF_SANDYBRIDGE},
    /* 2770 */ {I_VPXOR, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12332, IF_AVX|IF_SANDYBRIDGE},
    /* 2771 */ {I_VRCPPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12339, IF_AVX|IF_SANDYBRIDGE},
    /* 2772 */ {I_VRCPPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12346, IF_AVX|IF_SANDYBRIDGE},
    /* 2773 */ {I_VRCPSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+12353, IF_AVX|IF_SANDYBRIDGE},
    /* 2774 */ {I_VRCPSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+12360, IF_AVX|IF_SANDYBRIDGE},
    /* 2775 */ {I_VRSQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12367, IF_AVX|IF_SANDYBRIDGE},
    /* 2776 */ {I_VRSQRTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12374, IF_AVX|IF_SANDYBRIDGE},
    /* 2777 */ {I_VRSQRTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+12381, IF_AVX|IF_SANDYBRIDGE},
    /* 2778 */ {I_VRSQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+12388, IF_AVX|IF_SANDYBRIDGE},
    /* 2779 */ {I_VROUNDPD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5180, IF_AVX|IF_SANDYBRIDGE},
    /* 2780 */ {I_VROUNDPD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5188, IF_AVX|IF_SANDYBRIDGE},
    /* 2781 */ {I_VROUNDPS, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5196, IF_AVX|IF_SANDYBRIDGE},
    /* 2782 */ {I_VROUNDPS, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5204, IF_AVX|IF_SANDYBRIDGE},
    /* 2783 */ {I_VROUNDSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMMEDIATE|BITS8,0}, nasm_bytecodes+5212, IF_AVX|IF_SANDYBRIDGE},
    /* 2784 */ {I_VROUNDSD, 3, {XMMREG,RM_XMM|BITS64,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5220, IF_AVX|IF_SANDYBRIDGE},
    /* 2785 */ {I_VROUNDSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMMEDIATE|BITS8,0}, nasm_bytecodes+5228, IF_AVX|IF_SANDYBRIDGE},
    /* 2786 */ {I_VROUNDSS, 3, {XMMREG,RM_XMM|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5236, IF_AVX|IF_SANDYBRIDGE},
    /* 2787 */ {I_VSHUFPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+5244, IF_AVX|IF_SANDYBRIDGE},
    /* 2788 */ {I_VSHUFPD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5252, IF_AVX|IF_SANDYBRIDGE},
    /* 2789 */ {I_VSHUFPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+5260, IF_AVX|IF_SANDYBRIDGE},
    /* 2790 */ {I_VSHUFPD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5268, IF_AVX|IF_SANDYBRIDGE},
    /* 2791 */ {I_VSHUFPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+5276, IF_AVX|IF_SANDYBRIDGE},
    /* 2792 */ {I_VSHUFPS, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5284, IF_AVX|IF_SANDYBRIDGE},
    /* 2793 */ {I_VSHUFPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+5292, IF_AVX|IF_SANDYBRIDGE},
    /* 2794 */ {I_VSHUFPS, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5300, IF_AVX|IF_SANDYBRIDGE},
    /* 2795 */ {I_VSQRTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12395, IF_AVX|IF_SANDYBRIDGE},
    /* 2796 */ {I_VSQRTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12402, IF_AVX|IF_SANDYBRIDGE},
    /* 2797 */ {I_VSQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12409, IF_AVX|IF_SANDYBRIDGE},
    /* 2798 */ {I_VSQRTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12416, IF_AVX|IF_SANDYBRIDGE},
    /* 2799 */ {I_VSQRTSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+12423, IF_AVX|IF_SANDYBRIDGE},
    /* 2800 */ {I_VSQRTSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+12430, IF_AVX|IF_SANDYBRIDGE},
    /* 2801 */ {I_VSQRTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+12437, IF_AVX|IF_SANDYBRIDGE},
    /* 2802 */ {I_VSQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+12444, IF_AVX|IF_SANDYBRIDGE},
    /* 2803 */ {I_VSTMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, nasm_bytecodes+12451, IF_AVX|IF_SANDYBRIDGE},
    /* 2804 */ {I_VSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12458, IF_AVX|IF_SANDYBRIDGE},
    /* 2805 */ {I_VSUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12465, IF_AVX|IF_SANDYBRIDGE},
    /* 2806 */ {I_VSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12472, IF_AVX|IF_SANDYBRIDGE},
    /* 2807 */ {I_VSUBPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12479, IF_AVX|IF_SANDYBRIDGE},
    /* 2808 */ {I_VSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12486, IF_AVX|IF_SANDYBRIDGE},
    /* 2809 */ {I_VSUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12493, IF_AVX|IF_SANDYBRIDGE},
    /* 2810 */ {I_VSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12500, IF_AVX|IF_SANDYBRIDGE},
    /* 2811 */ {I_VSUBPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12507, IF_AVX|IF_SANDYBRIDGE},
    /* 2812 */ {I_VSUBSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+12514, IF_AVX|IF_SANDYBRIDGE},
    /* 2813 */ {I_VSUBSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+12521, IF_AVX|IF_SANDYBRIDGE},
    /* 2814 */ {I_VSUBSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+12528, IF_AVX|IF_SANDYBRIDGE},
    /* 2815 */ {I_VSUBSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+12535, IF_AVX|IF_SANDYBRIDGE},
    /* 2816 */ {I_VTESTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12542, IF_AVX|IF_SANDYBRIDGE},
    /* 2817 */ {I_VTESTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12549, IF_AVX|IF_SANDYBRIDGE},
    /* 2818 */ {I_VTESTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12556, IF_AVX|IF_SANDYBRIDGE},
    /* 2819 */ {I_VTESTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12563, IF_AVX|IF_SANDYBRIDGE},
    /* 2820 */ {I_VUCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+12570, IF_AVX|IF_SANDYBRIDGE},
    /* 2821 */ {I_VUCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+12577, IF_AVX|IF_SANDYBRIDGE},
    /* 2822 */ {I_VUNPCKHPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12584, IF_AVX|IF_SANDYBRIDGE},
    /* 2823 */ {I_VUNPCKHPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12591, IF_AVX|IF_SANDYBRIDGE},
    /* 2824 */ {I_VUNPCKHPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12598, IF_AVX|IF_SANDYBRIDGE},
    /* 2825 */ {I_VUNPCKHPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12605, IF_AVX|IF_SANDYBRIDGE},
    /* 2826 */ {I_VUNPCKHPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12612, IF_AVX|IF_SANDYBRIDGE},
    /* 2827 */ {I_VUNPCKHPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12619, IF_AVX|IF_SANDYBRIDGE},
    /* 2828 */ {I_VUNPCKHPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12626, IF_AVX|IF_SANDYBRIDGE},
    /* 2829 */ {I_VUNPCKHPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12633, IF_AVX|IF_SANDYBRIDGE},
    /* 2830 */ {I_VUNPCKLPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12640, IF_AVX|IF_SANDYBRIDGE},
    /* 2831 */ {I_VUNPCKLPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12647, IF_AVX|IF_SANDYBRIDGE},
    /* 2832 */ {I_VUNPCKLPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12654, IF_AVX|IF_SANDYBRIDGE},
    /* 2833 */ {I_VUNPCKLPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12661, IF_AVX|IF_SANDYBRIDGE},
    /* 2834 */ {I_VUNPCKLPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12668, IF_AVX|IF_SANDYBRIDGE},
    /* 2835 */ {I_VUNPCKLPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12675, IF_AVX|IF_SANDYBRIDGE},
    /* 2836 */ {I_VUNPCKLPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12682, IF_AVX|IF_SANDYBRIDGE},
    /* 2837 */ {I_VUNPCKLPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12689, IF_AVX|IF_SANDYBRIDGE},
    /* 2838 */ {I_VXORPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12696, IF_AVX|IF_SANDYBRIDGE},
    /* 2839 */ {I_VXORPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12703, IF_AVX|IF_SANDYBRIDGE},
    /* 2840 */ {I_VXORPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12710, IF_AVX|IF_SANDYBRIDGE},
    /* 2841 */ {I_VXORPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12717, IF_AVX|IF_SANDYBRIDGE},
    /* 2842 */ {I_VXORPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12724, IF_AVX|IF_SANDYBRIDGE},
    /* 2843 */ {I_VXORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+12731, IF_AVX|IF_SANDYBRIDGE},
    /* 2844 */ {I_VXORPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12738, IF_AVX|IF_SANDYBRIDGE},
    /* 2845 */ {I_VXORPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+12745, IF_AVX|IF_SANDYBRIDGE},
    /* 2846 */ {I_VZEROALL, 0, {0,0,0,0,0}, nasm_bytecodes+18363, IF_AVX|IF_SANDYBRIDGE},
    /* 2847 */ {I_VZEROUPPER, 0, {0,0,0,0,0}, nasm_bytecodes+18369, IF_AVX|IF_SANDYBRIDGE},
    /* 2848 */ {I_PCLMULLQLQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3528, IF_SSE|IF_WESTMERE},
    /* 2849 */ {I_PCLMULHQLQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3537, IF_SSE|IF_WESTMERE},
    /* 2850 */ {I_PCLMULLQHQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3546, IF_SSE|IF_WESTMERE},
    /* 2851 */ {I_PCLMULHQHQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3555, IF_SSE|IF_WESTMERE},
    /* 2852 */ {I_PCLMULQDQ, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5308, IF_SSE|IF_WESTMERE},
    /* 2853 */ {I_VPCLMULLQLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+3564, IF_AVX|IF_SANDYBRIDGE},
    /* 2854 */ {I_VPCLMULLQLQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3573, IF_AVX|IF_SANDYBRIDGE},
    /* 2855 */ {I_VPCLMULHQLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+3582, IF_AVX|IF_SANDYBRIDGE},
    /* 2856 */ {I_VPCLMULHQLQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3591, IF_AVX|IF_SANDYBRIDGE},
    /* 2857 */ {I_VPCLMULLQHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+3600, IF_AVX|IF_SANDYBRIDGE},
    /* 2858 */ {I_VPCLMULLQHQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3609, IF_AVX|IF_SANDYBRIDGE},
    /* 2859 */ {I_VPCLMULHQHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+3618, IF_AVX|IF_SANDYBRIDGE},
    /* 2860 */ {I_VPCLMULHQHQDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+3627, IF_AVX|IF_SANDYBRIDGE},
    /* 2861 */ {I_VPCLMULQDQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+5316, IF_AVX|IF_SANDYBRIDGE},
    /* 2862 */ {I_VPCLMULQDQ, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5324, IF_AVX|IF_SANDYBRIDGE},
    /* 2863 */ {I_VFMADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12752, IF_FMA|IF_FUTURE},
    /* 2864 */ {I_VFMADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12759, IF_FMA|IF_FUTURE},
    /* 2865 */ {I_VFMADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12766, IF_FMA|IF_FUTURE},
    /* 2866 */ {I_VFMADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12773, IF_FMA|IF_FUTURE},
    /* 2867 */ {I_VFMADD312PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12752, IF_FMA|IF_FUTURE},
    /* 2868 */ {I_VFMADD312PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12759, IF_FMA|IF_FUTURE},
    /* 2869 */ {I_VFMADD312PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12766, IF_FMA|IF_FUTURE},
    /* 2870 */ {I_VFMADD312PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12773, IF_FMA|IF_FUTURE},
    /* 2871 */ {I_VFMADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12780, IF_FMA|IF_FUTURE},
    /* 2872 */ {I_VFMADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12787, IF_FMA|IF_FUTURE},
    /* 2873 */ {I_VFMADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12794, IF_FMA|IF_FUTURE},
    /* 2874 */ {I_VFMADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12801, IF_FMA|IF_FUTURE},
    /* 2875 */ {I_VFMADD123PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12780, IF_FMA|IF_FUTURE},
    /* 2876 */ {I_VFMADD123PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12787, IF_FMA|IF_FUTURE},
    /* 2877 */ {I_VFMADD123PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12794, IF_FMA|IF_FUTURE},
    /* 2878 */ {I_VFMADD123PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12801, IF_FMA|IF_FUTURE},
    /* 2879 */ {I_VFMADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12808, IF_FMA|IF_FUTURE},
    /* 2880 */ {I_VFMADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12815, IF_FMA|IF_FUTURE},
    /* 2881 */ {I_VFMADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12822, IF_FMA|IF_FUTURE},
    /* 2882 */ {I_VFMADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12829, IF_FMA|IF_FUTURE},
    /* 2883 */ {I_VFMADD321PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12808, IF_FMA|IF_FUTURE},
    /* 2884 */ {I_VFMADD321PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12815, IF_FMA|IF_FUTURE},
    /* 2885 */ {I_VFMADD321PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12822, IF_FMA|IF_FUTURE},
    /* 2886 */ {I_VFMADD321PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12829, IF_FMA|IF_FUTURE},
    /* 2887 */ {I_VFMADDSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12836, IF_FMA|IF_FUTURE},
    /* 2888 */ {I_VFMADDSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12843, IF_FMA|IF_FUTURE},
    /* 2889 */ {I_VFMADDSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12850, IF_FMA|IF_FUTURE},
    /* 2890 */ {I_VFMADDSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12857, IF_FMA|IF_FUTURE},
    /* 2891 */ {I_VFMADDSUB312PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12836, IF_FMA|IF_FUTURE},
    /* 2892 */ {I_VFMADDSUB312PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12843, IF_FMA|IF_FUTURE},
    /* 2893 */ {I_VFMADDSUB312PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12850, IF_FMA|IF_FUTURE},
    /* 2894 */ {I_VFMADDSUB312PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12857, IF_FMA|IF_FUTURE},
    /* 2895 */ {I_VFMADDSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12864, IF_FMA|IF_FUTURE},
    /* 2896 */ {I_VFMADDSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12871, IF_FMA|IF_FUTURE},
    /* 2897 */ {I_VFMADDSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12878, IF_FMA|IF_FUTURE},
    /* 2898 */ {I_VFMADDSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12885, IF_FMA|IF_FUTURE},
    /* 2899 */ {I_VFMADDSUB123PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12864, IF_FMA|IF_FUTURE},
    /* 2900 */ {I_VFMADDSUB123PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12871, IF_FMA|IF_FUTURE},
    /* 2901 */ {I_VFMADDSUB123PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12878, IF_FMA|IF_FUTURE},
    /* 2902 */ {I_VFMADDSUB123PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12885, IF_FMA|IF_FUTURE},
    /* 2903 */ {I_VFMADDSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12892, IF_FMA|IF_FUTURE},
    /* 2904 */ {I_VFMADDSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12899, IF_FMA|IF_FUTURE},
    /* 2905 */ {I_VFMADDSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12906, IF_FMA|IF_FUTURE},
    /* 2906 */ {I_VFMADDSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12913, IF_FMA|IF_FUTURE},
    /* 2907 */ {I_VFMADDSUB321PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12892, IF_FMA|IF_FUTURE},
    /* 2908 */ {I_VFMADDSUB321PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12899, IF_FMA|IF_FUTURE},
    /* 2909 */ {I_VFMADDSUB321PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12906, IF_FMA|IF_FUTURE},
    /* 2910 */ {I_VFMADDSUB321PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12913, IF_FMA|IF_FUTURE},
    /* 2911 */ {I_VFMSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12920, IF_FMA|IF_FUTURE},
    /* 2912 */ {I_VFMSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12927, IF_FMA|IF_FUTURE},
    /* 2913 */ {I_VFMSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12934, IF_FMA|IF_FUTURE},
    /* 2914 */ {I_VFMSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12941, IF_FMA|IF_FUTURE},
    /* 2915 */ {I_VFMSUB312PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12920, IF_FMA|IF_FUTURE},
    /* 2916 */ {I_VFMSUB312PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12927, IF_FMA|IF_FUTURE},
    /* 2917 */ {I_VFMSUB312PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12934, IF_FMA|IF_FUTURE},
    /* 2918 */ {I_VFMSUB312PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12941, IF_FMA|IF_FUTURE},
    /* 2919 */ {I_VFMSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12948, IF_FMA|IF_FUTURE},
    /* 2920 */ {I_VFMSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12955, IF_FMA|IF_FUTURE},
    /* 2921 */ {I_VFMSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12962, IF_FMA|IF_FUTURE},
    /* 2922 */ {I_VFMSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12969, IF_FMA|IF_FUTURE},
    /* 2923 */ {I_VFMSUB123PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12948, IF_FMA|IF_FUTURE},
    /* 2924 */ {I_VFMSUB123PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12955, IF_FMA|IF_FUTURE},
    /* 2925 */ {I_VFMSUB123PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12962, IF_FMA|IF_FUTURE},
    /* 2926 */ {I_VFMSUB123PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12969, IF_FMA|IF_FUTURE},
    /* 2927 */ {I_VFMSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12976, IF_FMA|IF_FUTURE},
    /* 2928 */ {I_VFMSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12983, IF_FMA|IF_FUTURE},
    /* 2929 */ {I_VFMSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12990, IF_FMA|IF_FUTURE},
    /* 2930 */ {I_VFMSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12997, IF_FMA|IF_FUTURE},
    /* 2931 */ {I_VFMSUB321PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12976, IF_FMA|IF_FUTURE},
    /* 2932 */ {I_VFMSUB321PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12983, IF_FMA|IF_FUTURE},
    /* 2933 */ {I_VFMSUB321PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+12990, IF_FMA|IF_FUTURE},
    /* 2934 */ {I_VFMSUB321PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+12997, IF_FMA|IF_FUTURE},
    /* 2935 */ {I_VFMSUBADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13004, IF_FMA|IF_FUTURE},
    /* 2936 */ {I_VFMSUBADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13011, IF_FMA|IF_FUTURE},
    /* 2937 */ {I_VFMSUBADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13018, IF_FMA|IF_FUTURE},
    /* 2938 */ {I_VFMSUBADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13025, IF_FMA|IF_FUTURE},
    /* 2939 */ {I_VFMSUBADD312PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13004, IF_FMA|IF_FUTURE},
    /* 2940 */ {I_VFMSUBADD312PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13011, IF_FMA|IF_FUTURE},
    /* 2941 */ {I_VFMSUBADD312PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13018, IF_FMA|IF_FUTURE},
    /* 2942 */ {I_VFMSUBADD312PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13025, IF_FMA|IF_FUTURE},
    /* 2943 */ {I_VFMSUBADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13032, IF_FMA|IF_FUTURE},
    /* 2944 */ {I_VFMSUBADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13039, IF_FMA|IF_FUTURE},
    /* 2945 */ {I_VFMSUBADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13046, IF_FMA|IF_FUTURE},
    /* 2946 */ {I_VFMSUBADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13053, IF_FMA|IF_FUTURE},
    /* 2947 */ {I_VFMSUBADD123PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13032, IF_FMA|IF_FUTURE},
    /* 2948 */ {I_VFMSUBADD123PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13039, IF_FMA|IF_FUTURE},
    /* 2949 */ {I_VFMSUBADD123PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13046, IF_FMA|IF_FUTURE},
    /* 2950 */ {I_VFMSUBADD123PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13053, IF_FMA|IF_FUTURE},
    /* 2951 */ {I_VFMSUBADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13060, IF_FMA|IF_FUTURE},
    /* 2952 */ {I_VFMSUBADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13067, IF_FMA|IF_FUTURE},
    /* 2953 */ {I_VFMSUBADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13074, IF_FMA|IF_FUTURE},
    /* 2954 */ {I_VFMSUBADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13081, IF_FMA|IF_FUTURE},
    /* 2955 */ {I_VFMSUBADD321PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13060, IF_FMA|IF_FUTURE},
    /* 2956 */ {I_VFMSUBADD321PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13067, IF_FMA|IF_FUTURE},
    /* 2957 */ {I_VFMSUBADD321PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13074, IF_FMA|IF_FUTURE},
    /* 2958 */ {I_VFMSUBADD321PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13081, IF_FMA|IF_FUTURE},
    /* 2959 */ {I_VFNMADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13088, IF_FMA|IF_FUTURE},
    /* 2960 */ {I_VFNMADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13095, IF_FMA|IF_FUTURE},
    /* 2961 */ {I_VFNMADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13102, IF_FMA|IF_FUTURE},
    /* 2962 */ {I_VFNMADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13109, IF_FMA|IF_FUTURE},
    /* 2963 */ {I_VFNMADD312PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13088, IF_FMA|IF_FUTURE},
    /* 2964 */ {I_VFNMADD312PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13095, IF_FMA|IF_FUTURE},
    /* 2965 */ {I_VFNMADD312PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13102, IF_FMA|IF_FUTURE},
    /* 2966 */ {I_VFNMADD312PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13109, IF_FMA|IF_FUTURE},
    /* 2967 */ {I_VFNMADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13116, IF_FMA|IF_FUTURE},
    /* 2968 */ {I_VFNMADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13123, IF_FMA|IF_FUTURE},
    /* 2969 */ {I_VFNMADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13130, IF_FMA|IF_FUTURE},
    /* 2970 */ {I_VFNMADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13137, IF_FMA|IF_FUTURE},
    /* 2971 */ {I_VFNMADD123PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13116, IF_FMA|IF_FUTURE},
    /* 2972 */ {I_VFNMADD123PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13123, IF_FMA|IF_FUTURE},
    /* 2973 */ {I_VFNMADD123PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13130, IF_FMA|IF_FUTURE},
    /* 2974 */ {I_VFNMADD123PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13137, IF_FMA|IF_FUTURE},
    /* 2975 */ {I_VFNMADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13144, IF_FMA|IF_FUTURE},
    /* 2976 */ {I_VFNMADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13151, IF_FMA|IF_FUTURE},
    /* 2977 */ {I_VFNMADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13158, IF_FMA|IF_FUTURE},
    /* 2978 */ {I_VFNMADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13165, IF_FMA|IF_FUTURE},
    /* 2979 */ {I_VFNMADD321PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13144, IF_FMA|IF_FUTURE},
    /* 2980 */ {I_VFNMADD321PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13151, IF_FMA|IF_FUTURE},
    /* 2981 */ {I_VFNMADD321PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13158, IF_FMA|IF_FUTURE},
    /* 2982 */ {I_VFNMADD321PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13165, IF_FMA|IF_FUTURE},
    /* 2983 */ {I_VFNMSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13172, IF_FMA|IF_FUTURE},
    /* 2984 */ {I_VFNMSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13179, IF_FMA|IF_FUTURE},
    /* 2985 */ {I_VFNMSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13186, IF_FMA|IF_FUTURE},
    /* 2986 */ {I_VFNMSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13193, IF_FMA|IF_FUTURE},
    /* 2987 */ {I_VFNMSUB312PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13172, IF_FMA|IF_FUTURE},
    /* 2988 */ {I_VFNMSUB312PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13179, IF_FMA|IF_FUTURE},
    /* 2989 */ {I_VFNMSUB312PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13186, IF_FMA|IF_FUTURE},
    /* 2990 */ {I_VFNMSUB312PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13193, IF_FMA|IF_FUTURE},
    /* 2991 */ {I_VFNMSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13200, IF_FMA|IF_FUTURE},
    /* 2992 */ {I_VFNMSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13207, IF_FMA|IF_FUTURE},
    /* 2993 */ {I_VFNMSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13214, IF_FMA|IF_FUTURE},
    /* 2994 */ {I_VFNMSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13221, IF_FMA|IF_FUTURE},
    /* 2995 */ {I_VFNMSUB123PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13200, IF_FMA|IF_FUTURE},
    /* 2996 */ {I_VFNMSUB123PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13207, IF_FMA|IF_FUTURE},
    /* 2997 */ {I_VFNMSUB123PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13214, IF_FMA|IF_FUTURE},
    /* 2998 */ {I_VFNMSUB123PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13221, IF_FMA|IF_FUTURE},
    /* 2999 */ {I_VFNMSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13228, IF_FMA|IF_FUTURE},
    /* 3000 */ {I_VFNMSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13235, IF_FMA|IF_FUTURE},
    /* 3001 */ {I_VFNMSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13242, IF_FMA|IF_FUTURE},
    /* 3002 */ {I_VFNMSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13249, IF_FMA|IF_FUTURE},
    /* 3003 */ {I_VFNMSUB321PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13228, IF_FMA|IF_FUTURE},
    /* 3004 */ {I_VFNMSUB321PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13235, IF_FMA|IF_FUTURE},
    /* 3005 */ {I_VFNMSUB321PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13242, IF_FMA|IF_FUTURE},
    /* 3006 */ {I_VFNMSUB321PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+13249, IF_FMA|IF_FUTURE},
    /* 3007 */ {I_VFMADD132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13256, IF_FMA|IF_FUTURE},
    /* 3008 */ {I_VFMADD132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13263, IF_FMA|IF_FUTURE},
    /* 3009 */ {I_VFMADD312SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13256, IF_FMA|IF_FUTURE},
    /* 3010 */ {I_VFMADD312SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13263, IF_FMA|IF_FUTURE},
    /* 3011 */ {I_VFMADD213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13270, IF_FMA|IF_FUTURE},
    /* 3012 */ {I_VFMADD213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13277, IF_FMA|IF_FUTURE},
    /* 3013 */ {I_VFMADD123SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13270, IF_FMA|IF_FUTURE},
    /* 3014 */ {I_VFMADD123SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13277, IF_FMA|IF_FUTURE},
    /* 3015 */ {I_VFMADD231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13284, IF_FMA|IF_FUTURE},
    /* 3016 */ {I_VFMADD231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13291, IF_FMA|IF_FUTURE},
    /* 3017 */ {I_VFMADD321SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13284, IF_FMA|IF_FUTURE},
    /* 3018 */ {I_VFMADD321SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13291, IF_FMA|IF_FUTURE},
    /* 3019 */ {I_VFMSUB132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13298, IF_FMA|IF_FUTURE},
    /* 3020 */ {I_VFMSUB132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13305, IF_FMA|IF_FUTURE},
    /* 3021 */ {I_VFMSUB312SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13298, IF_FMA|IF_FUTURE},
    /* 3022 */ {I_VFMSUB312SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13305, IF_FMA|IF_FUTURE},
    /* 3023 */ {I_VFMSUB213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13312, IF_FMA|IF_FUTURE},
    /* 3024 */ {I_VFMSUB213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13319, IF_FMA|IF_FUTURE},
    /* 3025 */ {I_VFMSUB123SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13312, IF_FMA|IF_FUTURE},
    /* 3026 */ {I_VFMSUB123SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13319, IF_FMA|IF_FUTURE},
    /* 3027 */ {I_VFMSUB231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13326, IF_FMA|IF_FUTURE},
    /* 3028 */ {I_VFMSUB231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13333, IF_FMA|IF_FUTURE},
    /* 3029 */ {I_VFMSUB321SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13326, IF_FMA|IF_FUTURE},
    /* 3030 */ {I_VFMSUB321SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13333, IF_FMA|IF_FUTURE},
    /* 3031 */ {I_VFNMADD132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13340, IF_FMA|IF_FUTURE},
    /* 3032 */ {I_VFNMADD132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13347, IF_FMA|IF_FUTURE},
    /* 3033 */ {I_VFNMADD312SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13340, IF_FMA|IF_FUTURE},
    /* 3034 */ {I_VFNMADD312SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13347, IF_FMA|IF_FUTURE},
    /* 3035 */ {I_VFNMADD213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13354, IF_FMA|IF_FUTURE},
    /* 3036 */ {I_VFNMADD213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13361, IF_FMA|IF_FUTURE},
    /* 3037 */ {I_VFNMADD123SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13354, IF_FMA|IF_FUTURE},
    /* 3038 */ {I_VFNMADD123SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13361, IF_FMA|IF_FUTURE},
    /* 3039 */ {I_VFNMADD231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13368, IF_FMA|IF_FUTURE},
    /* 3040 */ {I_VFNMADD231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13375, IF_FMA|IF_FUTURE},
    /* 3041 */ {I_VFNMADD321SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13368, IF_FMA|IF_FUTURE},
    /* 3042 */ {I_VFNMADD321SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13375, IF_FMA|IF_FUTURE},
    /* 3043 */ {I_VFNMSUB132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13382, IF_FMA|IF_FUTURE},
    /* 3044 */ {I_VFNMSUB132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13389, IF_FMA|IF_FUTURE},
    /* 3045 */ {I_VFNMSUB312SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13382, IF_FMA|IF_FUTURE},
    /* 3046 */ {I_VFNMSUB312SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13389, IF_FMA|IF_FUTURE},
    /* 3047 */ {I_VFNMSUB213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13396, IF_FMA|IF_FUTURE},
    /* 3048 */ {I_VFNMSUB213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13403, IF_FMA|IF_FUTURE},
    /* 3049 */ {I_VFNMSUB123SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13396, IF_FMA|IF_FUTURE},
    /* 3050 */ {I_VFNMSUB123SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13403, IF_FMA|IF_FUTURE},
    /* 3051 */ {I_VFNMSUB231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13410, IF_FMA|IF_FUTURE},
    /* 3052 */ {I_VFNMSUB231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13417, IF_FMA|IF_FUTURE},
    /* 3053 */ {I_VFNMSUB321SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+13410, IF_FMA|IF_FUTURE},
    /* 3054 */ {I_VFNMSUB321SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+13417, IF_FMA|IF_FUTURE},
    /* 3055 */ {I_RDFSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+13424, IF_LONG|IF_FUTURE},
    /* 3056 */ {I_RDFSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+13431, IF_LONG|IF_FUTURE},
    /* 3057 */ {I_RDGSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+13438, IF_LONG|IF_FUTURE},
    /* 3058 */ {I_RDGSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+13445, IF_LONG|IF_FUTURE},
    /* 3059 */ {I_RDRAND, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18375, IF_FUTURE},
    /* 3060 */ {I_RDRAND, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18381, IF_FUTURE},
    /* 3061 */ {I_RDRAND, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18387, IF_LONG|IF_FUTURE},
    /* 3062 */ {I_WRFSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+13452, IF_LONG|IF_FUTURE},
    /* 3063 */ {I_WRFSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+13459, IF_LONG|IF_FUTURE},
    /* 3064 */ {I_WRGSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+13466, IF_LONG|IF_FUTURE},
    /* 3065 */ {I_WRGSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+13473, IF_LONG|IF_FUTURE},
    /* 3066 */ {I_VCVTPH2PS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13480, IF_AVX|IF_FUTURE},
    /* 3067 */ {I_VCVTPH2PS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+13487, IF_AVX|IF_FUTURE},
    /* 3068 */ {I_VCVTPS2PH, 3, {RM_XMM|BITS128,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5332, IF_AVX|IF_FUTURE},
    /* 3069 */ {I_VCVTPS2PH, 3, {RM_XMM|BITS64,XMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+5340, IF_AVX|IF_FUTURE},
    /* 3070 */ {I_ADCX, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+5348, IF_FUTURE},
    /* 3071 */ {I_ADCX, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+5356, IF_LONG|IF_FUTURE},
    /* 3072 */ {I_ADOX, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+5364, IF_FUTURE},
    /* 3073 */ {I_ADOX, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+5372, IF_LONG|IF_FUTURE},
    /* 3074 */ {I_RDSEED, 1, {REG_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18393, IF_FUTURE},
    /* 3075 */ {I_RDSEED, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18399, IF_FUTURE},
    /* 3076 */ {I_RDSEED, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18405, IF_LONG|IF_FUTURE},
    /* 3077 */ {I_CLAC, 0, {0,0,0,0,0}, nasm_bytecodes+21332, IF_PRIV|IF_FUTURE},
    /* 3078 */ {I_STAC, 0, {0,0,0,0,0}, nasm_bytecodes+21337, IF_PRIV|IF_FUTURE},
    /* 3079 */ {I_XSTORE, 0, {0,0,0,0,0}, nasm_bytecodes+21342, IF_PENT|IF_CYRIX},
    /* 3080 */ {I_XCRYPTECB, 0, {0,0,0,0,0}, nasm_bytecodes+18411, IF_PENT|IF_CYRIX},
    /* 3081 */ {I_XCRYPTCBC, 0, {0,0,0,0,0}, nasm_bytecodes+18417, IF_PENT|IF_CYRIX},
    /* 3082 */ {I_XCRYPTCTR, 0, {0,0,0,0,0}, nasm_bytecodes+18423, IF_PENT|IF_CYRIX},
    /* 3083 */ {I_XCRYPTCFB, 0, {0,0,0,0,0}, nasm_bytecodes+18429, IF_PENT|IF_CYRIX},
    /* 3084 */ {I_XCRYPTOFB, 0, {0,0,0,0,0}, nasm_bytecodes+18435, IF_PENT|IF_CYRIX},
    /* 3085 */ {I_MONTMUL, 0, {0,0,0,0,0}, nasm_bytecodes+18441, IF_PENT|IF_CYRIX},
    /* 3086 */ {I_XSHA1, 0, {0,0,0,0,0}, nasm_bytecodes+18447, IF_PENT|IF_CYRIX},
    /* 3087 */ {I_XSHA256, 0, {0,0,0,0,0}, nasm_bytecodes+18453, IF_PENT|IF_CYRIX},
    /* 3088 */ {I_LLWPCB, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+13494, IF_AMD|IF_386},
    /* 3089 */ {I_LLWPCB, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+13501, IF_AMD|IF_X64},
    /* 3090 */ {I_SLWPCB, 1, {REG_GPR|BITS32,0,0,0,0}, nasm_bytecodes+13508, IF_AMD|IF_386},
    /* 3091 */ {I_SLWPCB, 1, {REG_GPR|BITS64,0,0,0,0}, nasm_bytecodes+13515, IF_AMD|IF_X64},
    /* 3092 */ {I_LWPVAL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+5380, IF_AMD|IF_386},
    /* 3093 */ {I_LWPVAL, 3, {REG_GPR|BITS64,RM_GPR|BITS32,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+5388, IF_AMD|IF_X64},
    /* 3094 */ {I_LWPINS, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+5396, IF_AMD|IF_386},
    /* 3095 */ {I_LWPINS, 3, {REG_GPR|BITS64,RM_GPR|BITS32,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+5404, IF_AMD|IF_X64},
    /* 3096 */ {I_VFMADDPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5412, IF_AMD|IF_SSE5},
    /* 3097 */ {I_VFMADDPD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5420, IF_AMD|IF_SSE5},
    /* 3098 */ {I_VFMADDPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5428, IF_AMD|IF_SSE5},
    /* 3099 */ {I_VFMADDPD, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5436, IF_AMD|IF_SSE5},
    /* 3100 */ {I_VFMADDPD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5444, IF_AMD|IF_SSE5},
    /* 3101 */ {I_VFMADDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5452, IF_AMD|IF_SSE5},
    /* 3102 */ {I_VFMADDPD, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5460, IF_AMD|IF_SSE5},
    /* 3103 */ {I_VFMADDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5468, IF_AMD|IF_SSE5},
    /* 3104 */ {I_VFMADDPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5476, IF_AMD|IF_SSE5},
    /* 3105 */ {I_VFMADDPS, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5484, IF_AMD|IF_SSE5},
    /* 3106 */ {I_VFMADDPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5492, IF_AMD|IF_SSE5},
    /* 3107 */ {I_VFMADDPS, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5500, IF_AMD|IF_SSE5},
    /* 3108 */ {I_VFMADDPS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5508, IF_AMD|IF_SSE5},
    /* 3109 */ {I_VFMADDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5516, IF_AMD|IF_SSE5},
    /* 3110 */ {I_VFMADDPS, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5524, IF_AMD|IF_SSE5},
    /* 3111 */ {I_VFMADDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5532, IF_AMD|IF_SSE5},
    /* 3112 */ {I_VFMADDSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,XMMREG,0}, nasm_bytecodes+5540, IF_AMD|IF_SSE5},
    /* 3113 */ {I_VFMADDSD, 3, {XMMREG,RM_XMM|BITS64,XMMREG,0,0}, nasm_bytecodes+5548, IF_AMD|IF_SSE5},
    /* 3114 */ {I_VFMADDSD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS64,0}, nasm_bytecodes+5556, IF_AMD|IF_SSE5},
    /* 3115 */ {I_VFMADDSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+5564, IF_AMD|IF_SSE5},
    /* 3116 */ {I_VFMADDSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,XMMREG,0}, nasm_bytecodes+5572, IF_AMD|IF_SSE5},
    /* 3117 */ {I_VFMADDSS, 3, {XMMREG,RM_XMM|BITS32,XMMREG,0,0}, nasm_bytecodes+5580, IF_AMD|IF_SSE5},
    /* 3118 */ {I_VFMADDSS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS32,0}, nasm_bytecodes+5588, IF_AMD|IF_SSE5},
    /* 3119 */ {I_VFMADDSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+5596, IF_AMD|IF_SSE5},
    /* 3120 */ {I_VFMADDSUBPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5604, IF_AMD|IF_SSE5},
    /* 3121 */ {I_VFMADDSUBPD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5612, IF_AMD|IF_SSE5},
    /* 3122 */ {I_VFMADDSUBPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5620, IF_AMD|IF_SSE5},
    /* 3123 */ {I_VFMADDSUBPD, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5628, IF_AMD|IF_SSE5},
    /* 3124 */ {I_VFMADDSUBPD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5636, IF_AMD|IF_SSE5},
    /* 3125 */ {I_VFMADDSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5644, IF_AMD|IF_SSE5},
    /* 3126 */ {I_VFMADDSUBPD, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5652, IF_AMD|IF_SSE5},
    /* 3127 */ {I_VFMADDSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5660, IF_AMD|IF_SSE5},
    /* 3128 */ {I_VFMADDSUBPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5668, IF_AMD|IF_SSE5},
    /* 3129 */ {I_VFMADDSUBPS, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5676, IF_AMD|IF_SSE5},
    /* 3130 */ {I_VFMADDSUBPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5684, IF_AMD|IF_SSE5},
    /* 3131 */ {I_VFMADDSUBPS, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5692, IF_AMD|IF_SSE5},
    /* 3132 */ {I_VFMADDSUBPS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5700, IF_AMD|IF_SSE5},
    /* 3133 */ {I_VFMADDSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5708, IF_AMD|IF_SSE5},
    /* 3134 */ {I_VFMADDSUBPS, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5716, IF_AMD|IF_SSE5},
    /* 3135 */ {I_VFMADDSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5724, IF_AMD|IF_SSE5},
    /* 3136 */ {I_VFMSUBADDPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5732, IF_AMD|IF_SSE5},
    /* 3137 */ {I_VFMSUBADDPD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5740, IF_AMD|IF_SSE5},
    /* 3138 */ {I_VFMSUBADDPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5748, IF_AMD|IF_SSE5},
    /* 3139 */ {I_VFMSUBADDPD, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5756, IF_AMD|IF_SSE5},
    /* 3140 */ {I_VFMSUBADDPD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5764, IF_AMD|IF_SSE5},
    /* 3141 */ {I_VFMSUBADDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5772, IF_AMD|IF_SSE5},
    /* 3142 */ {I_VFMSUBADDPD, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5780, IF_AMD|IF_SSE5},
    /* 3143 */ {I_VFMSUBADDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5788, IF_AMD|IF_SSE5},
    /* 3144 */ {I_VFMSUBADDPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5796, IF_AMD|IF_SSE5},
    /* 3145 */ {I_VFMSUBADDPS, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5804, IF_AMD|IF_SSE5},
    /* 3146 */ {I_VFMSUBADDPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5812, IF_AMD|IF_SSE5},
    /* 3147 */ {I_VFMSUBADDPS, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5820, IF_AMD|IF_SSE5},
    /* 3148 */ {I_VFMSUBADDPS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5828, IF_AMD|IF_SSE5},
    /* 3149 */ {I_VFMSUBADDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5836, IF_AMD|IF_SSE5},
    /* 3150 */ {I_VFMSUBADDPS, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5844, IF_AMD|IF_SSE5},
    /* 3151 */ {I_VFMSUBADDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5852, IF_AMD|IF_SSE5},
    /* 3152 */ {I_VFMSUBPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5860, IF_AMD|IF_SSE5},
    /* 3153 */ {I_VFMSUBPD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5868, IF_AMD|IF_SSE5},
    /* 3154 */ {I_VFMSUBPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5876, IF_AMD|IF_SSE5},
    /* 3155 */ {I_VFMSUBPD, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5884, IF_AMD|IF_SSE5},
    /* 3156 */ {I_VFMSUBPD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5892, IF_AMD|IF_SSE5},
    /* 3157 */ {I_VFMSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5900, IF_AMD|IF_SSE5},
    /* 3158 */ {I_VFMSUBPD, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5908, IF_AMD|IF_SSE5},
    /* 3159 */ {I_VFMSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5916, IF_AMD|IF_SSE5},
    /* 3160 */ {I_VFMSUBPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+5924, IF_AMD|IF_SSE5},
    /* 3161 */ {I_VFMSUBPS, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+5932, IF_AMD|IF_SSE5},
    /* 3162 */ {I_VFMSUBPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+5940, IF_AMD|IF_SSE5},
    /* 3163 */ {I_VFMSUBPS, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+5948, IF_AMD|IF_SSE5},
    /* 3164 */ {I_VFMSUBPS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+5956, IF_AMD|IF_SSE5},
    /* 3165 */ {I_VFMSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+5964, IF_AMD|IF_SSE5},
    /* 3166 */ {I_VFMSUBPS, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+5972, IF_AMD|IF_SSE5},
    /* 3167 */ {I_VFMSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+5980, IF_AMD|IF_SSE5},
    /* 3168 */ {I_VFMSUBSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,XMMREG,0}, nasm_bytecodes+5988, IF_AMD|IF_SSE5},
    /* 3169 */ {I_VFMSUBSD, 3, {XMMREG,RM_XMM|BITS64,XMMREG,0,0}, nasm_bytecodes+5996, IF_AMD|IF_SSE5},
    /* 3170 */ {I_VFMSUBSD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS64,0}, nasm_bytecodes+6004, IF_AMD|IF_SSE5},
    /* 3171 */ {I_VFMSUBSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+6012, IF_AMD|IF_SSE5},
    /* 3172 */ {I_VFMSUBSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,XMMREG,0}, nasm_bytecodes+6020, IF_AMD|IF_SSE5},
    /* 3173 */ {I_VFMSUBSS, 3, {XMMREG,RM_XMM|BITS32,XMMREG,0,0}, nasm_bytecodes+6028, IF_AMD|IF_SSE5},
    /* 3174 */ {I_VFMSUBSS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS32,0}, nasm_bytecodes+6036, IF_AMD|IF_SSE5},
    /* 3175 */ {I_VFMSUBSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+6044, IF_AMD|IF_SSE5},
    /* 3176 */ {I_VFNMADDPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6052, IF_AMD|IF_SSE5},
    /* 3177 */ {I_VFNMADDPD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6060, IF_AMD|IF_SSE5},
    /* 3178 */ {I_VFNMADDPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+6068, IF_AMD|IF_SSE5},
    /* 3179 */ {I_VFNMADDPD, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+6076, IF_AMD|IF_SSE5},
    /* 3180 */ {I_VFNMADDPD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+6084, IF_AMD|IF_SSE5},
    /* 3181 */ {I_VFNMADDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+6092, IF_AMD|IF_SSE5},
    /* 3182 */ {I_VFNMADDPD, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+6100, IF_AMD|IF_SSE5},
    /* 3183 */ {I_VFNMADDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+6108, IF_AMD|IF_SSE5},
    /* 3184 */ {I_VFNMADDPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6116, IF_AMD|IF_SSE5},
    /* 3185 */ {I_VFNMADDPS, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6124, IF_AMD|IF_SSE5},
    /* 3186 */ {I_VFNMADDPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+6132, IF_AMD|IF_SSE5},
    /* 3187 */ {I_VFNMADDPS, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+6140, IF_AMD|IF_SSE5},
    /* 3188 */ {I_VFNMADDPS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+6148, IF_AMD|IF_SSE5},
    /* 3189 */ {I_VFNMADDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+6156, IF_AMD|IF_SSE5},
    /* 3190 */ {I_VFNMADDPS, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+6164, IF_AMD|IF_SSE5},
    /* 3191 */ {I_VFNMADDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+6172, IF_AMD|IF_SSE5},
    /* 3192 */ {I_VFNMADDSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,XMMREG,0}, nasm_bytecodes+6180, IF_AMD|IF_SSE5},
    /* 3193 */ {I_VFNMADDSD, 3, {XMMREG,RM_XMM|BITS64,XMMREG,0,0}, nasm_bytecodes+6188, IF_AMD|IF_SSE5},
    /* 3194 */ {I_VFNMADDSD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS64,0}, nasm_bytecodes+6196, IF_AMD|IF_SSE5},
    /* 3195 */ {I_VFNMADDSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+6204, IF_AMD|IF_SSE5},
    /* 3196 */ {I_VFNMADDSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,XMMREG,0}, nasm_bytecodes+6212, IF_AMD|IF_SSE5},
    /* 3197 */ {I_VFNMADDSS, 3, {XMMREG,RM_XMM|BITS32,XMMREG,0,0}, nasm_bytecodes+6220, IF_AMD|IF_SSE5},
    /* 3198 */ {I_VFNMADDSS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS32,0}, nasm_bytecodes+6228, IF_AMD|IF_SSE5},
    /* 3199 */ {I_VFNMADDSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+6236, IF_AMD|IF_SSE5},
    /* 3200 */ {I_VFNMSUBPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6244, IF_AMD|IF_SSE5},
    /* 3201 */ {I_VFNMSUBPD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6252, IF_AMD|IF_SSE5},
    /* 3202 */ {I_VFNMSUBPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+6260, IF_AMD|IF_SSE5},
    /* 3203 */ {I_VFNMSUBPD, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+6268, IF_AMD|IF_SSE5},
    /* 3204 */ {I_VFNMSUBPD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+6276, IF_AMD|IF_SSE5},
    /* 3205 */ {I_VFNMSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+6284, IF_AMD|IF_SSE5},
    /* 3206 */ {I_VFNMSUBPD, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+6292, IF_AMD|IF_SSE5},
    /* 3207 */ {I_VFNMSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+6300, IF_AMD|IF_SSE5},
    /* 3208 */ {I_VFNMSUBPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6308, IF_AMD|IF_SSE5},
    /* 3209 */ {I_VFNMSUBPS, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6316, IF_AMD|IF_SSE5},
    /* 3210 */ {I_VFNMSUBPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+6324, IF_AMD|IF_SSE5},
    /* 3211 */ {I_VFNMSUBPS, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+6332, IF_AMD|IF_SSE5},
    /* 3212 */ {I_VFNMSUBPS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+6340, IF_AMD|IF_SSE5},
    /* 3213 */ {I_VFNMSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+6348, IF_AMD|IF_SSE5},
    /* 3214 */ {I_VFNMSUBPS, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+6356, IF_AMD|IF_SSE5},
    /* 3215 */ {I_VFNMSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+6364, IF_AMD|IF_SSE5},
    /* 3216 */ {I_VFNMSUBSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,XMMREG,0}, nasm_bytecodes+6372, IF_AMD|IF_SSE5},
    /* 3217 */ {I_VFNMSUBSD, 3, {XMMREG,RM_XMM|BITS64,XMMREG,0,0}, nasm_bytecodes+6380, IF_AMD|IF_SSE5},
    /* 3218 */ {I_VFNMSUBSD, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS64,0}, nasm_bytecodes+6388, IF_AMD|IF_SSE5},
    /* 3219 */ {I_VFNMSUBSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, nasm_bytecodes+6396, IF_AMD|IF_SSE5},
    /* 3220 */ {I_VFNMSUBSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,XMMREG,0}, nasm_bytecodes+6404, IF_AMD|IF_SSE5},
    /* 3221 */ {I_VFNMSUBSS, 3, {XMMREG,RM_XMM|BITS32,XMMREG,0,0}, nasm_bytecodes+6412, IF_AMD|IF_SSE5},
    /* 3222 */ {I_VFNMSUBSS, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS32,0}, nasm_bytecodes+6420, IF_AMD|IF_SSE5},
    /* 3223 */ {I_VFNMSUBSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, nasm_bytecodes+6428, IF_AMD|IF_SSE5},
    /* 3224 */ {I_VFRCZPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13522, IF_AMD|IF_SSE5},
    /* 3225 */ {I_VFRCZPD, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13529, IF_AMD|IF_SSE5},
    /* 3226 */ {I_VFRCZPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+13536, IF_AMD|IF_SSE5},
    /* 3227 */ {I_VFRCZPD, 1, {YMMREG,0,0,0,0}, nasm_bytecodes+13543, IF_AMD|IF_SSE5},
    /* 3228 */ {I_VFRCZPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13550, IF_AMD|IF_SSE5},
    /* 3229 */ {I_VFRCZPS, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13557, IF_AMD|IF_SSE5},
    /* 3230 */ {I_VFRCZPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+13564, IF_AMD|IF_SSE5},
    /* 3231 */ {I_VFRCZPS, 1, {YMMREG,0,0,0,0}, nasm_bytecodes+13571, IF_AMD|IF_SSE5},
    /* 3232 */ {I_VFRCZSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, nasm_bytecodes+13578, IF_AMD|IF_SSE5},
    /* 3233 */ {I_VFRCZSD, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13585, IF_AMD|IF_SSE5},
    /* 3234 */ {I_VFRCZSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, nasm_bytecodes+13592, IF_AMD|IF_SSE5},
    /* 3235 */ {I_VFRCZSS, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13599, IF_AMD|IF_SSE5},
    /* 3236 */ {I_VPCMOV, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6436, IF_AMD|IF_SSE5},
    /* 3237 */ {I_VPCMOV, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6444, IF_AMD|IF_SSE5},
    /* 3238 */ {I_VPCMOV, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+6452, IF_AMD|IF_SSE5},
    /* 3239 */ {I_VPCMOV, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+6460, IF_AMD|IF_SSE5},
    /* 3240 */ {I_VPCMOV, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+6468, IF_AMD|IF_SSE5},
    /* 3241 */ {I_VPCMOV, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+6476, IF_AMD|IF_SSE5},
    /* 3242 */ {I_VPCMOV, 4, {YMMREG,YMMREG,YMMREG,RM_YMM|BITS256,0}, nasm_bytecodes+6484, IF_AMD|IF_SSE5},
    /* 3243 */ {I_VPCMOV, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+6492, IF_AMD|IF_SSE5},
    /* 3244 */ {I_VPCOMB, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6500, IF_AMD|IF_SSE5},
    /* 3245 */ {I_VPCOMB, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6508, IF_AMD|IF_SSE5},
    /* 3246 */ {I_VPCOMD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6516, IF_AMD|IF_SSE5},
    /* 3247 */ {I_VPCOMD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6524, IF_AMD|IF_SSE5},
    /* 3248 */ {I_VPCOMQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6532, IF_AMD|IF_SSE5},
    /* 3249 */ {I_VPCOMQ, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6540, IF_AMD|IF_SSE5},
    /* 3250 */ {I_VPCOMUB, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6548, IF_AMD|IF_SSE5},
    /* 3251 */ {I_VPCOMUB, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6556, IF_AMD|IF_SSE5},
    /* 3252 */ {I_VPCOMUD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6564, IF_AMD|IF_SSE5},
    /* 3253 */ {I_VPCOMUD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6572, IF_AMD|IF_SSE5},
    /* 3254 */ {I_VPCOMUQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6580, IF_AMD|IF_SSE5},
    /* 3255 */ {I_VPCOMUQ, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6588, IF_AMD|IF_SSE5},
    /* 3256 */ {I_VPCOMUW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6596, IF_AMD|IF_SSE5},
    /* 3257 */ {I_VPCOMUW, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6604, IF_AMD|IF_SSE5},
    /* 3258 */ {I_VPCOMW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+6612, IF_AMD|IF_SSE5},
    /* 3259 */ {I_VPCOMW, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6620, IF_AMD|IF_SSE5},
    /* 3260 */ {I_VPHADDBD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13606, IF_AMD|IF_SSE5},
    /* 3261 */ {I_VPHADDBD, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13613, IF_AMD|IF_SSE5},
    /* 3262 */ {I_VPHADDBQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13620, IF_AMD|IF_SSE5},
    /* 3263 */ {I_VPHADDBQ, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13627, IF_AMD|IF_SSE5},
    /* 3264 */ {I_VPHADDBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13634, IF_AMD|IF_SSE5},
    /* 3265 */ {I_VPHADDBW, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13641, IF_AMD|IF_SSE5},
    /* 3266 */ {I_VPHADDDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13648, IF_AMD|IF_SSE5},
    /* 3267 */ {I_VPHADDDQ, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13655, IF_AMD|IF_SSE5},
    /* 3268 */ {I_VPHADDUBD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13662, IF_AMD|IF_SSE5},
    /* 3269 */ {I_VPHADDUBD, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13669, IF_AMD|IF_SSE5},
    /* 3270 */ {I_VPHADDUBQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13676, IF_AMD|IF_SSE5},
    /* 3271 */ {I_VPHADDUBQ, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13683, IF_AMD|IF_SSE5},
    /* 3272 */ {I_VPHADDUBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13690, IF_AMD|IF_SSE5},
    /* 3273 */ {I_VPHADDUBW, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13697, IF_AMD|IF_SSE5},
    /* 3274 */ {I_VPHADDUDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13704, IF_AMD|IF_SSE5},
    /* 3275 */ {I_VPHADDUDQ, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13711, IF_AMD|IF_SSE5},
    /* 3276 */ {I_VPHADDUWD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13718, IF_AMD|IF_SSE5},
    /* 3277 */ {I_VPHADDUWD, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13725, IF_AMD|IF_SSE5},
    /* 3278 */ {I_VPHADDUWQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13732, IF_AMD|IF_SSE5},
    /* 3279 */ {I_VPHADDUWQ, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13739, IF_AMD|IF_SSE5},
    /* 3280 */ {I_VPHADDWD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13746, IF_AMD|IF_SSE5},
    /* 3281 */ {I_VPHADDWD, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13753, IF_AMD|IF_SSE5},
    /* 3282 */ {I_VPHADDWQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13760, IF_AMD|IF_SSE5},
    /* 3283 */ {I_VPHADDWQ, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13767, IF_AMD|IF_SSE5},
    /* 3284 */ {I_VPHSUBBW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13774, IF_AMD|IF_SSE5},
    /* 3285 */ {I_VPHSUBBW, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13781, IF_AMD|IF_SSE5},
    /* 3286 */ {I_VPHSUBDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13788, IF_AMD|IF_SSE5},
    /* 3287 */ {I_VPHSUBDQ, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13795, IF_AMD|IF_SSE5},
    /* 3288 */ {I_VPHSUBWD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13802, IF_AMD|IF_SSE5},
    /* 3289 */ {I_VPHSUBWD, 1, {XMMREG,0,0,0,0}, nasm_bytecodes+13809, IF_AMD|IF_SSE5},
    /* 3290 */ {I_VPMACSDD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6628, IF_AMD|IF_SSE5},
    /* 3291 */ {I_VPMACSDD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6636, IF_AMD|IF_SSE5},
    /* 3292 */ {I_VPMACSDQH, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6644, IF_AMD|IF_SSE5},
    /* 3293 */ {I_VPMACSDQH, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6652, IF_AMD|IF_SSE5},
    /* 3294 */ {I_VPMACSDQL, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6660, IF_AMD|IF_SSE5},
    /* 3295 */ {I_VPMACSDQL, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6668, IF_AMD|IF_SSE5},
    /* 3296 */ {I_VPMACSSDD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6676, IF_AMD|IF_SSE5},
    /* 3297 */ {I_VPMACSSDD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6684, IF_AMD|IF_SSE5},
    /* 3298 */ {I_VPMACSSDQH, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6692, IF_AMD|IF_SSE5},
    /* 3299 */ {I_VPMACSSDQH, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6700, IF_AMD|IF_SSE5},
    /* 3300 */ {I_VPMACSSDQL, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6708, IF_AMD|IF_SSE5},
    /* 3301 */ {I_VPMACSSDQL, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6716, IF_AMD|IF_SSE5},
    /* 3302 */ {I_VPMACSSWD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6724, IF_AMD|IF_SSE5},
    /* 3303 */ {I_VPMACSSWD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6732, IF_AMD|IF_SSE5},
    /* 3304 */ {I_VPMACSSWW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6740, IF_AMD|IF_SSE5},
    /* 3305 */ {I_VPMACSSWW, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6748, IF_AMD|IF_SSE5},
    /* 3306 */ {I_VPMACSWD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6756, IF_AMD|IF_SSE5},
    /* 3307 */ {I_VPMACSWD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6764, IF_AMD|IF_SSE5},
    /* 3308 */ {I_VPMACSWW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6772, IF_AMD|IF_SSE5},
    /* 3309 */ {I_VPMACSWW, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6780, IF_AMD|IF_SSE5},
    /* 3310 */ {I_VPMADCSSWD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6788, IF_AMD|IF_SSE5},
    /* 3311 */ {I_VPMADCSSWD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6796, IF_AMD|IF_SSE5},
    /* 3312 */ {I_VPMADCSWD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6804, IF_AMD|IF_SSE5},
    /* 3313 */ {I_VPMADCSWD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6812, IF_AMD|IF_SSE5},
    /* 3314 */ {I_VPPERM, 4, {XMMREG,XMMREG,XMMREG,RM_XMM|BITS128,0}, nasm_bytecodes+6820, IF_AMD|IF_SSE5},
    /* 3315 */ {I_VPPERM, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+6828, IF_AMD|IF_SSE5},
    /* 3316 */ {I_VPPERM, 4, {XMMREG,XMMREG,RM_XMM|BITS128,XMMREG,0}, nasm_bytecodes+6836, IF_AMD|IF_SSE5},
    /* 3317 */ {I_VPPERM, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+6844, IF_AMD|IF_SSE5},
    /* 3318 */ {I_VPROTB, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+13816, IF_AMD|IF_SSE5},
    /* 3319 */ {I_VPROTB, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+13823, IF_AMD|IF_SSE5},
    /* 3320 */ {I_VPROTB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13830, IF_AMD|IF_SSE5},
    /* 3321 */ {I_VPROTB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13837, IF_AMD|IF_SSE5},
    /* 3322 */ {I_VPROTB, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6852, IF_AMD|IF_SSE5},
    /* 3323 */ {I_VPROTB, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+6860, IF_AMD|IF_SSE5},
    /* 3324 */ {I_VPROTD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+13844, IF_AMD|IF_SSE5},
    /* 3325 */ {I_VPROTD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+13851, IF_AMD|IF_SSE5},
    /* 3326 */ {I_VPROTD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13858, IF_AMD|IF_SSE5},
    /* 3327 */ {I_VPROTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13865, IF_AMD|IF_SSE5},
    /* 3328 */ {I_VPROTD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6868, IF_AMD|IF_SSE5},
    /* 3329 */ {I_VPROTD, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+6876, IF_AMD|IF_SSE5},
    /* 3330 */ {I_VPROTQ, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+13872, IF_AMD|IF_SSE5},
    /* 3331 */ {I_VPROTQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+13879, IF_AMD|IF_SSE5},
    /* 3332 */ {I_VPROTQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13886, IF_AMD|IF_SSE5},
    /* 3333 */ {I_VPROTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13893, IF_AMD|IF_SSE5},
    /* 3334 */ {I_VPROTQ, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6884, IF_AMD|IF_SSE5},
    /* 3335 */ {I_VPROTQ, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+6892, IF_AMD|IF_SSE5},
    /* 3336 */ {I_VPROTW, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+13900, IF_AMD|IF_SSE5},
    /* 3337 */ {I_VPROTW, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+13907, IF_AMD|IF_SSE5},
    /* 3338 */ {I_VPROTW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13914, IF_AMD|IF_SSE5},
    /* 3339 */ {I_VPROTW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13921, IF_AMD|IF_SSE5},
    /* 3340 */ {I_VPROTW, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6900, IF_AMD|IF_SSE5},
    /* 3341 */ {I_VPROTW, 2, {XMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+6908, IF_AMD|IF_SSE5},
    /* 3342 */ {I_VPSHAB, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+13928, IF_AMD|IF_SSE5},
    /* 3343 */ {I_VPSHAB, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+13935, IF_AMD|IF_SSE5},
    /* 3344 */ {I_VPSHAB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13942, IF_AMD|IF_SSE5},
    /* 3345 */ {I_VPSHAB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13949, IF_AMD|IF_SSE5},
    /* 3346 */ {I_VPSHAD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+13956, IF_AMD|IF_SSE5},
    /* 3347 */ {I_VPSHAD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+13963, IF_AMD|IF_SSE5},
    /* 3348 */ {I_VPSHAD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13970, IF_AMD|IF_SSE5},
    /* 3349 */ {I_VPSHAD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+13977, IF_AMD|IF_SSE5},
    /* 3350 */ {I_VPSHAQ, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+13984, IF_AMD|IF_SSE5},
    /* 3351 */ {I_VPSHAQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+13991, IF_AMD|IF_SSE5},
    /* 3352 */ {I_VPSHAQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+13998, IF_AMD|IF_SSE5},
    /* 3353 */ {I_VPSHAQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14005, IF_AMD|IF_SSE5},
    /* 3354 */ {I_VPSHAW, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+14012, IF_AMD|IF_SSE5},
    /* 3355 */ {I_VPSHAW, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+14019, IF_AMD|IF_SSE5},
    /* 3356 */ {I_VPSHAW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+14026, IF_AMD|IF_SSE5},
    /* 3357 */ {I_VPSHAW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14033, IF_AMD|IF_SSE5},
    /* 3358 */ {I_VPSHLB, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+14040, IF_AMD|IF_SSE5},
    /* 3359 */ {I_VPSHLB, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+14047, IF_AMD|IF_SSE5},
    /* 3360 */ {I_VPSHLB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+14054, IF_AMD|IF_SSE5},
    /* 3361 */ {I_VPSHLB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14061, IF_AMD|IF_SSE5},
    /* 3362 */ {I_VPSHLD, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+14068, IF_AMD|IF_SSE5},
    /* 3363 */ {I_VPSHLD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+14075, IF_AMD|IF_SSE5},
    /* 3364 */ {I_VPSHLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+14082, IF_AMD|IF_SSE5},
    /* 3365 */ {I_VPSHLD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14089, IF_AMD|IF_SSE5},
    /* 3366 */ {I_VPSHLQ, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+14096, IF_AMD|IF_SSE5},
    /* 3367 */ {I_VPSHLQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+14103, IF_AMD|IF_SSE5},
    /* 3368 */ {I_VPSHLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+14110, IF_AMD|IF_SSE5},
    /* 3369 */ {I_VPSHLQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14117, IF_AMD|IF_SSE5},
    /* 3370 */ {I_VPSHLW, 3, {XMMREG,RM_XMM|BITS128,XMMREG,0,0}, nasm_bytecodes+14124, IF_AMD|IF_SSE5},
    /* 3371 */ {I_VPSHLW, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+14131, IF_AMD|IF_SSE5},
    /* 3372 */ {I_VPSHLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+14138, IF_AMD|IF_SSE5},
    /* 3373 */ {I_VPSHLW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14145, IF_AMD|IF_SSE5},
    /* 3374 */ {I_VMPSADBW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+6916, IF_FUTURE|IF_AVX2},
    /* 3375 */ {I_VMPSADBW, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6924, IF_FUTURE|IF_AVX2},
    /* 3376 */ {I_VPABSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14152, IF_FUTURE|IF_AVX2},
    /* 3377 */ {I_VPABSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14159, IF_FUTURE|IF_AVX2},
    /* 3378 */ {I_VPABSD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14166, IF_FUTURE|IF_AVX2},
    /* 3379 */ {I_VPACKSSWB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14173, IF_FUTURE|IF_AVX2},
    /* 3380 */ {I_VPACKSSWB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14180, IF_FUTURE|IF_AVX2},
    /* 3381 */ {I_VPACKSSDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14187, IF_FUTURE|IF_AVX2},
    /* 3382 */ {I_VPACKSSDW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14194, IF_FUTURE|IF_AVX2},
    /* 3383 */ {I_VPACKUSDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14201, IF_FUTURE|IF_AVX2},
    /* 3384 */ {I_VPACKUSDW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14208, IF_FUTURE|IF_AVX2},
    /* 3385 */ {I_VPACKUSWB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14215, IF_FUTURE|IF_AVX2},
    /* 3386 */ {I_VPACKUSWB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14222, IF_FUTURE|IF_AVX2},
    /* 3387 */ {I_VPADDB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14229, IF_FUTURE|IF_AVX2},
    /* 3388 */ {I_VPADDB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14236, IF_FUTURE|IF_AVX2},
    /* 3389 */ {I_VPADDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14243, IF_FUTURE|IF_AVX2},
    /* 3390 */ {I_VPADDW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14250, IF_FUTURE|IF_AVX2},
    /* 3391 */ {I_VPADDD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14257, IF_FUTURE|IF_AVX2},
    /* 3392 */ {I_VPADDD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14264, IF_FUTURE|IF_AVX2},
    /* 3393 */ {I_VPADDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14271, IF_FUTURE|IF_AVX2},
    /* 3394 */ {I_VPADDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14278, IF_FUTURE|IF_AVX2},
    /* 3395 */ {I_VPADDSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14285, IF_FUTURE|IF_AVX2},
    /* 3396 */ {I_VPADDSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14292, IF_FUTURE|IF_AVX2},
    /* 3397 */ {I_VPADDSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14299, IF_FUTURE|IF_AVX2},
    /* 3398 */ {I_VPADDSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14306, IF_FUTURE|IF_AVX2},
    /* 3399 */ {I_VPADDUSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14313, IF_FUTURE|IF_AVX2},
    /* 3400 */ {I_VPADDUSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14320, IF_FUTURE|IF_AVX2},
    /* 3401 */ {I_VPADDUSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14327, IF_FUTURE|IF_AVX2},
    /* 3402 */ {I_VPADDUSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14334, IF_FUTURE|IF_AVX2},
    /* 3403 */ {I_VPALIGNR, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+6932, IF_FUTURE|IF_AVX2},
    /* 3404 */ {I_VPALIGNR, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6940, IF_FUTURE|IF_AVX2},
    /* 3405 */ {I_VPAND, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14341, IF_FUTURE|IF_AVX2},
    /* 3406 */ {I_VPAND, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14348, IF_FUTURE|IF_AVX2},
    /* 3407 */ {I_VPANDN, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14355, IF_FUTURE|IF_AVX2},
    /* 3408 */ {I_VPANDN, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14362, IF_FUTURE|IF_AVX2},
    /* 3409 */ {I_VPAVGB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14369, IF_FUTURE|IF_AVX2},
    /* 3410 */ {I_VPAVGB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14376, IF_FUTURE|IF_AVX2},
    /* 3411 */ {I_VPAVGW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14383, IF_FUTURE|IF_AVX2},
    /* 3412 */ {I_VPAVGW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14390, IF_FUTURE|IF_AVX2},
    /* 3413 */ {I_VPBLENDVB, 4, {YMMREG,YMMREG,RM_YMM|BITS256,YMMREG,0}, nasm_bytecodes+6948, IF_FUTURE|IF_AVX2},
    /* 3414 */ {I_VPBLENDVB, 3, {YMMREG,RM_YMM|BITS256,YMMREG,0,0}, nasm_bytecodes+6956, IF_FUTURE|IF_AVX2},
    /* 3415 */ {I_VPBLENDW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+6964, IF_FUTURE|IF_AVX2},
    /* 3416 */ {I_VPBLENDW, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6972, IF_FUTURE|IF_AVX2},
    /* 3417 */ {I_VPCMPEQB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14397, IF_FUTURE|IF_AVX2},
    /* 3418 */ {I_VPCMPEQB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14404, IF_FUTURE|IF_AVX2},
    /* 3419 */ {I_VPCMPEQW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14411, IF_FUTURE|IF_AVX2},
    /* 3420 */ {I_VPCMPEQW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14418, IF_FUTURE|IF_AVX2},
    /* 3421 */ {I_VPCMPEQD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14425, IF_FUTURE|IF_AVX2},
    /* 3422 */ {I_VPCMPEQD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14432, IF_FUTURE|IF_AVX2},
    /* 3423 */ {I_VPCMPEQQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14439, IF_FUTURE|IF_AVX2},
    /* 3424 */ {I_VPCMPEQQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14446, IF_FUTURE|IF_AVX2},
    /* 3425 */ {I_VPCMPGTB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14453, IF_FUTURE|IF_AVX2},
    /* 3426 */ {I_VPCMPGTB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14460, IF_FUTURE|IF_AVX2},
    /* 3427 */ {I_VPCMPGTW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14467, IF_FUTURE|IF_AVX2},
    /* 3428 */ {I_VPCMPGTW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14474, IF_FUTURE|IF_AVX2},
    /* 3429 */ {I_VPCMPGTD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14481, IF_FUTURE|IF_AVX2},
    /* 3430 */ {I_VPCMPGTD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14488, IF_FUTURE|IF_AVX2},
    /* 3431 */ {I_VPCMPGTQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14495, IF_FUTURE|IF_AVX2},
    /* 3432 */ {I_VPCMPGTQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14502, IF_FUTURE|IF_AVX2},
    /* 3433 */ {I_VPHADDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14509, IF_FUTURE|IF_AVX2},
    /* 3434 */ {I_VPHADDW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14516, IF_FUTURE|IF_AVX2},
    /* 3435 */ {I_VPHADDD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14523, IF_FUTURE|IF_AVX2},
    /* 3436 */ {I_VPHADDD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14530, IF_FUTURE|IF_AVX2},
    /* 3437 */ {I_VPHADDSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14537, IF_FUTURE|IF_AVX2},
    /* 3438 */ {I_VPHADDSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14544, IF_FUTURE|IF_AVX2},
    /* 3439 */ {I_VPHSUBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14551, IF_FUTURE|IF_AVX2},
    /* 3440 */ {I_VPHSUBW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14558, IF_FUTURE|IF_AVX2},
    /* 3441 */ {I_VPHSUBD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14565, IF_FUTURE|IF_AVX2},
    /* 3442 */ {I_VPHSUBD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14572, IF_FUTURE|IF_AVX2},
    /* 3443 */ {I_VPHSUBSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14579, IF_FUTURE|IF_AVX2},
    /* 3444 */ {I_VPHSUBSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14586, IF_FUTURE|IF_AVX2},
    /* 3445 */ {I_VPMADDUBSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14593, IF_FUTURE|IF_AVX2},
    /* 3446 */ {I_VPMADDUBSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14600, IF_FUTURE|IF_AVX2},
    /* 3447 */ {I_VPMADDWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14607, IF_FUTURE|IF_AVX2},
    /* 3448 */ {I_VPMADDWD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14614, IF_FUTURE|IF_AVX2},
    /* 3449 */ {I_VPMAXSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14621, IF_FUTURE|IF_AVX2},
    /* 3450 */ {I_VPMAXSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14628, IF_FUTURE|IF_AVX2},
    /* 3451 */ {I_VPMAXSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14635, IF_FUTURE|IF_AVX2},
    /* 3452 */ {I_VPMAXSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14642, IF_FUTURE|IF_AVX2},
    /* 3453 */ {I_VPMAXSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14649, IF_FUTURE|IF_AVX2},
    /* 3454 */ {I_VPMAXSD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14656, IF_FUTURE|IF_AVX2},
    /* 3455 */ {I_VPMAXUB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14663, IF_FUTURE|IF_AVX2},
    /* 3456 */ {I_VPMAXUB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14670, IF_FUTURE|IF_AVX2},
    /* 3457 */ {I_VPMAXUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14677, IF_FUTURE|IF_AVX2},
    /* 3458 */ {I_VPMAXUW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14684, IF_FUTURE|IF_AVX2},
    /* 3459 */ {I_VPMAXUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14691, IF_FUTURE|IF_AVX2},
    /* 3460 */ {I_VPMAXUD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14698, IF_FUTURE|IF_AVX2},
    /* 3461 */ {I_VPMINSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14705, IF_FUTURE|IF_AVX2},
    /* 3462 */ {I_VPMINSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14712, IF_FUTURE|IF_AVX2},
    /* 3463 */ {I_VPMINSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14719, IF_FUTURE|IF_AVX2},
    /* 3464 */ {I_VPMINSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14726, IF_FUTURE|IF_AVX2},
    /* 3465 */ {I_VPMINSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14733, IF_FUTURE|IF_AVX2},
    /* 3466 */ {I_VPMINSD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14740, IF_FUTURE|IF_AVX2},
    /* 3467 */ {I_VPMINUB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14747, IF_FUTURE|IF_AVX2},
    /* 3468 */ {I_VPMINUB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14754, IF_FUTURE|IF_AVX2},
    /* 3469 */ {I_VPMINUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14761, IF_FUTURE|IF_AVX2},
    /* 3470 */ {I_VPMINUW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14768, IF_FUTURE|IF_AVX2},
    /* 3471 */ {I_VPMINUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14775, IF_FUTURE|IF_AVX2},
    /* 3472 */ {I_VPMINUD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14782, IF_FUTURE|IF_AVX2},
    /* 3473 */ {I_VPMOVMSKB, 2, {REG_GPR|BITS32,YMMREG,0,0,0}, nasm_bytecodes+14789, IF_FUTURE|IF_AVX2},
    /* 3474 */ {I_VPMOVMSKB, 2, {REG_GPR|BITS64,YMMREG,0,0,0}, nasm_bytecodes+14789, IF_FUTURE|IF_AVX2},
    /* 3475 */ {I_VPMOVSXBW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14796, IF_FUTURE|IF_AVX2},
    /* 3476 */ {I_VPMOVSXBD, 2, {YMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+14803, IF_FUTURE|IF_AVX2},
    /* 3477 */ {I_VPMOVSXBD, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+14803, IF_FUTURE|IF_AVX2},
    /* 3478 */ {I_VPMOVSXBQ, 2, {YMMREG,MEMORY|BITS32,0,0,0}, nasm_bytecodes+14810, IF_FUTURE|IF_AVX2},
    /* 3479 */ {I_VPMOVSXBQ, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+14810, IF_FUTURE|IF_AVX2},
    /* 3480 */ {I_VPMOVSXWD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14817, IF_FUTURE|IF_AVX2},
    /* 3481 */ {I_VPMOVSXWQ, 2, {YMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+14824, IF_FUTURE|IF_AVX2},
    /* 3482 */ {I_VPMOVSXWQ, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+14824, IF_FUTURE|IF_AVX2},
    /* 3483 */ {I_VPMOVSXDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14831, IF_FUTURE|IF_AVX2},
    /* 3484 */ {I_VPMOVZXBW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14838, IF_FUTURE|IF_AVX2},
    /* 3485 */ {I_VPMOVZXBD, 2, {YMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+14845, IF_FUTURE|IF_AVX2},
    /* 3486 */ {I_VPMOVZXBD, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+14845, IF_FUTURE|IF_AVX2},
    /* 3487 */ {I_VPMOVZXBQ, 2, {YMMREG,MEMORY|BITS32,0,0,0}, nasm_bytecodes+14852, IF_FUTURE|IF_AVX2},
    /* 3488 */ {I_VPMOVZXBQ, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+14852, IF_FUTURE|IF_AVX2},
    /* 3489 */ {I_VPMOVZXWD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14859, IF_FUTURE|IF_AVX2},
    /* 3490 */ {I_VPMOVZXWQ, 2, {YMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+14866, IF_FUTURE|IF_AVX2},
    /* 3491 */ {I_VPMOVZXWQ, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+14866, IF_FUTURE|IF_AVX2},
    /* 3492 */ {I_VPMOVZXDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+14873, IF_FUTURE|IF_AVX2},
    /* 3493 */ {I_VPMULDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14880, IF_FUTURE|IF_AVX2},
    /* 3494 */ {I_VPMULDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14887, IF_FUTURE|IF_AVX2},
    /* 3495 */ {I_VPMULHRSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14894, IF_FUTURE|IF_AVX2},
    /* 3496 */ {I_VPMULHRSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14901, IF_FUTURE|IF_AVX2},
    /* 3497 */ {I_VPMULHUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14908, IF_FUTURE|IF_AVX2},
    /* 3498 */ {I_VPMULHUW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14915, IF_FUTURE|IF_AVX2},
    /* 3499 */ {I_VPMULHW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14922, IF_FUTURE|IF_AVX2},
    /* 3500 */ {I_VPMULHW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14929, IF_FUTURE|IF_AVX2},
    /* 3501 */ {I_VPMULLW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14936, IF_FUTURE|IF_AVX2},
    /* 3502 */ {I_VPMULLW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14943, IF_FUTURE|IF_AVX2},
    /* 3503 */ {I_VPMULLD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14950, IF_FUTURE|IF_AVX2},
    /* 3504 */ {I_VPMULLD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14957, IF_FUTURE|IF_AVX2},
    /* 3505 */ {I_VPMULUDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14964, IF_FUTURE|IF_AVX2},
    /* 3506 */ {I_VPMULUDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14971, IF_FUTURE|IF_AVX2},
    /* 3507 */ {I_VPOR, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14978, IF_FUTURE|IF_AVX2},
    /* 3508 */ {I_VPOR, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14985, IF_FUTURE|IF_AVX2},
    /* 3509 */ {I_VPSADBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+14992, IF_FUTURE|IF_AVX2},
    /* 3510 */ {I_VPSADBW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+14999, IF_FUTURE|IF_AVX2},
    /* 3511 */ {I_VPSHUFB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15006, IF_FUTURE|IF_AVX2},
    /* 3512 */ {I_VPSHUFB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15013, IF_FUTURE|IF_AVX2},
    /* 3513 */ {I_VPSHUFD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6980, IF_FUTURE|IF_AVX2},
    /* 3514 */ {I_VPSHUFHW, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6988, IF_FUTURE|IF_AVX2},
    /* 3515 */ {I_VPSHUFLW, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+6996, IF_FUTURE|IF_AVX2},
    /* 3516 */ {I_VPSIGNB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15020, IF_FUTURE|IF_AVX2},
    /* 3517 */ {I_VPSIGNB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15027, IF_FUTURE|IF_AVX2},
    /* 3518 */ {I_VPSIGNW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15034, IF_FUTURE|IF_AVX2},
    /* 3519 */ {I_VPSIGNW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15041, IF_FUTURE|IF_AVX2},
    /* 3520 */ {I_VPSIGND, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15048, IF_FUTURE|IF_AVX2},
    /* 3521 */ {I_VPSIGND, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15055, IF_FUTURE|IF_AVX2},
    /* 3522 */ {I_VPSLLDQ, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7004, IF_FUTURE|IF_AVX2},
    /* 3523 */ {I_VPSLLDQ, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7012, IF_FUTURE|IF_AVX2},
    /* 3524 */ {I_VPSLLW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15062, IF_FUTURE|IF_AVX2},
    /* 3525 */ {I_VPSLLW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15069, IF_FUTURE|IF_AVX2},
    /* 3526 */ {I_VPSLLW, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7020, IF_FUTURE|IF_AVX2},
    /* 3527 */ {I_VPSLLW, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7028, IF_FUTURE|IF_AVX2},
    /* 3528 */ {I_VPSLLD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15076, IF_FUTURE|IF_AVX2},
    /* 3529 */ {I_VPSLLD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15083, IF_FUTURE|IF_AVX2},
    /* 3530 */ {I_VPSLLD, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7036, IF_FUTURE|IF_AVX2},
    /* 3531 */ {I_VPSLLD, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7044, IF_FUTURE|IF_AVX2},
    /* 3532 */ {I_VPSLLQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15090, IF_FUTURE|IF_AVX2},
    /* 3533 */ {I_VPSLLQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15097, IF_FUTURE|IF_AVX2},
    /* 3534 */ {I_VPSLLQ, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7052, IF_FUTURE|IF_AVX2},
    /* 3535 */ {I_VPSLLQ, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7060, IF_FUTURE|IF_AVX2},
    /* 3536 */ {I_VPSRAW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15104, IF_FUTURE|IF_AVX2},
    /* 3537 */ {I_VPSRAW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15111, IF_FUTURE|IF_AVX2},
    /* 3538 */ {I_VPSRAW, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7068, IF_FUTURE|IF_AVX2},
    /* 3539 */ {I_VPSRAW, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7076, IF_FUTURE|IF_AVX2},
    /* 3540 */ {I_VPSRAD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15118, IF_FUTURE|IF_AVX2},
    /* 3541 */ {I_VPSRAD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15125, IF_FUTURE|IF_AVX2},
    /* 3542 */ {I_VPSRAD, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7084, IF_FUTURE|IF_AVX2},
    /* 3543 */ {I_VPSRAD, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7092, IF_FUTURE|IF_AVX2},
    /* 3544 */ {I_VPSRLDQ, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7100, IF_FUTURE|IF_AVX2},
    /* 3545 */ {I_VPSRLDQ, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7108, IF_FUTURE|IF_AVX2},
    /* 3546 */ {I_VPSRLW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15132, IF_FUTURE|IF_AVX2},
    /* 3547 */ {I_VPSRLW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15139, IF_FUTURE|IF_AVX2},
    /* 3548 */ {I_VPSRLW, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7116, IF_FUTURE|IF_AVX2},
    /* 3549 */ {I_VPSRLW, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7124, IF_FUTURE|IF_AVX2},
    /* 3550 */ {I_VPSRLD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15146, IF_FUTURE|IF_AVX2},
    /* 3551 */ {I_VPSRLD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15153, IF_FUTURE|IF_AVX2},
    /* 3552 */ {I_VPSRLD, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7132, IF_FUTURE|IF_AVX2},
    /* 3553 */ {I_VPSRLD, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7140, IF_FUTURE|IF_AVX2},
    /* 3554 */ {I_VPSRLQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15160, IF_FUTURE|IF_AVX2},
    /* 3555 */ {I_VPSRLQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15167, IF_FUTURE|IF_AVX2},
    /* 3556 */ {I_VPSRLQ, 3, {YMMREG,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7148, IF_FUTURE|IF_AVX2},
    /* 3557 */ {I_VPSRLQ, 2, {YMMREG,IMMEDIATE|BITS8,0,0,0}, nasm_bytecodes+7156, IF_FUTURE|IF_AVX2},
    /* 3558 */ {I_VPSUBB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15174, IF_FUTURE|IF_AVX2},
    /* 3559 */ {I_VPSUBB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15181, IF_FUTURE|IF_AVX2},
    /* 3560 */ {I_VPSUBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15188, IF_FUTURE|IF_AVX2},
    /* 3561 */ {I_VPSUBW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15195, IF_FUTURE|IF_AVX2},
    /* 3562 */ {I_VPSUBD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15202, IF_FUTURE|IF_AVX2},
    /* 3563 */ {I_VPSUBD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15209, IF_FUTURE|IF_AVX2},
    /* 3564 */ {I_VPSUBQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15216, IF_FUTURE|IF_AVX2},
    /* 3565 */ {I_VPSUBQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15223, IF_FUTURE|IF_AVX2},
    /* 3566 */ {I_VPSUBSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15230, IF_FUTURE|IF_AVX2},
    /* 3567 */ {I_VPSUBSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15237, IF_FUTURE|IF_AVX2},
    /* 3568 */ {I_VPSUBSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15244, IF_FUTURE|IF_AVX2},
    /* 3569 */ {I_VPSUBSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15251, IF_FUTURE|IF_AVX2},
    /* 3570 */ {I_VPSUBUSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15258, IF_FUTURE|IF_AVX2},
    /* 3571 */ {I_VPSUBUSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15265, IF_FUTURE|IF_AVX2},
    /* 3572 */ {I_VPSUBUSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15272, IF_FUTURE|IF_AVX2},
    /* 3573 */ {I_VPSUBUSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15279, IF_FUTURE|IF_AVX2},
    /* 3574 */ {I_VPUNPCKHBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15286, IF_FUTURE|IF_AVX2},
    /* 3575 */ {I_VPUNPCKHBW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15293, IF_FUTURE|IF_AVX2},
    /* 3576 */ {I_VPUNPCKHWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15300, IF_FUTURE|IF_AVX2},
    /* 3577 */ {I_VPUNPCKHWD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15307, IF_FUTURE|IF_AVX2},
    /* 3578 */ {I_VPUNPCKHDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15314, IF_FUTURE|IF_AVX2},
    /* 3579 */ {I_VPUNPCKHDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15321, IF_FUTURE|IF_AVX2},
    /* 3580 */ {I_VPUNPCKHQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15328, IF_FUTURE|IF_AVX2},
    /* 3581 */ {I_VPUNPCKHQDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15335, IF_FUTURE|IF_AVX2},
    /* 3582 */ {I_VPUNPCKLBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15342, IF_FUTURE|IF_AVX2},
    /* 3583 */ {I_VPUNPCKLBW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15349, IF_FUTURE|IF_AVX2},
    /* 3584 */ {I_VPUNPCKLWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15356, IF_FUTURE|IF_AVX2},
    /* 3585 */ {I_VPUNPCKLWD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15363, IF_FUTURE|IF_AVX2},
    /* 3586 */ {I_VPUNPCKLDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15370, IF_FUTURE|IF_AVX2},
    /* 3587 */ {I_VPUNPCKLDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15377, IF_FUTURE|IF_AVX2},
    /* 3588 */ {I_VPUNPCKLQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15384, IF_FUTURE|IF_AVX2},
    /* 3589 */ {I_VPUNPCKLQDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15391, IF_FUTURE|IF_AVX2},
    /* 3590 */ {I_VPXOR, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15398, IF_FUTURE|IF_AVX2},
    /* 3591 */ {I_VPXOR, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15405, IF_FUTURE|IF_AVX2},
    /* 3592 */ {I_VMOVNTDQA, 2, {YMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+15412, IF_FUTURE|IF_AVX2},
    /* 3593 */ {I_VBROADCASTSS, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+9651, IF_FUTURE|IF_AVX2},
    /* 3594 */ {I_VBROADCASTSS, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+9658, IF_FUTURE|IF_AVX2},
    /* 3595 */ {I_VBROADCASTSD, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+9665, IF_FUTURE|IF_AVX2},
    /* 3596 */ {I_VBROADCASTI128, 2, {YMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+15419, IF_FUTURE|IF_AVX2},
    /* 3597 */ {I_VPBLENDD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+7164, IF_FUTURE|IF_AVX2},
    /* 3598 */ {I_VPBLENDD, 3, {XMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7172, IF_FUTURE|IF_AVX2},
    /* 3599 */ {I_VPBLENDD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+7180, IF_FUTURE|IF_AVX2},
    /* 3600 */ {I_VPBLENDD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7188, IF_FUTURE|IF_AVX2},
    /* 3601 */ {I_VPBROADCASTB, 2, {XMMREG,MEMORY|BITS8,0,0,0}, nasm_bytecodes+15426, IF_FUTURE|IF_AVX2},
    /* 3602 */ {I_VPBROADCASTB, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+15426, IF_FUTURE|IF_AVX2},
    /* 3603 */ {I_VPBROADCASTB, 2, {YMMREG,MEMORY|BITS8,0,0,0}, nasm_bytecodes+15433, IF_FUTURE|IF_AVX2},
    /* 3604 */ {I_VPBROADCASTB, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+15433, IF_FUTURE|IF_AVX2},
    /* 3605 */ {I_VPBROADCASTW, 2, {XMMREG,MEMORY|BITS16,0,0,0}, nasm_bytecodes+15440, IF_FUTURE|IF_AVX2},
    /* 3606 */ {I_VPBROADCASTW, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+15440, IF_FUTURE|IF_AVX2},
    /* 3607 */ {I_VPBROADCASTW, 2, {YMMREG,MEMORY|BITS16,0,0,0}, nasm_bytecodes+15447, IF_FUTURE|IF_AVX2},
    /* 3608 */ {I_VPBROADCASTW, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+15447, IF_FUTURE|IF_AVX2},
    /* 3609 */ {I_VPBROADCASTD, 2, {XMMREG,MEMORY|BITS32,0,0,0}, nasm_bytecodes+15454, IF_FUTURE|IF_AVX2},
    /* 3610 */ {I_VPBROADCASTD, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+15454, IF_FUTURE|IF_AVX2},
    /* 3611 */ {I_VPBROADCASTD, 2, {YMMREG,MEMORY|BITS32,0,0,0}, nasm_bytecodes+15461, IF_FUTURE|IF_AVX2},
    /* 3612 */ {I_VPBROADCASTD, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+15461, IF_FUTURE|IF_AVX2},
    /* 3613 */ {I_VPBROADCASTQ, 2, {XMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+15468, IF_FUTURE|IF_AVX2},
    /* 3614 */ {I_VPBROADCASTQ, 2, {XMMREG,XMMREG,0,0,0}, nasm_bytecodes+15468, IF_FUTURE|IF_AVX2},
    /* 3615 */ {I_VPBROADCASTQ, 2, {YMMREG,MEMORY|BITS64,0,0,0}, nasm_bytecodes+15475, IF_FUTURE|IF_AVX2},
    /* 3616 */ {I_VPBROADCASTQ, 2, {YMMREG,XMMREG,0,0,0}, nasm_bytecodes+15475, IF_FUTURE|IF_AVX2},
    /* 3617 */ {I_VPERMD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15482, IF_FUTURE|IF_AVX2},
    /* 3618 */ {I_VPERMD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15489, IF_FUTURE|IF_AVX2},
    /* 3619 */ {I_VPERMPD, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7196, IF_FUTURE|IF_AVX2},
    /* 3620 */ {I_VPERMPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15496, IF_FUTURE|IF_AVX2},
    /* 3621 */ {I_VPERMPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15503, IF_FUTURE|IF_AVX2},
    /* 3622 */ {I_VPERMQ, 3, {YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7204, IF_FUTURE|IF_AVX2},
    /* 3623 */ {I_VPERM2I128, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMMEDIATE|BITS8,0}, nasm_bytecodes+7212, IF_FUTURE|IF_AVX2},
    /* 3624 */ {I_VEXTRACTI128, 3, {RM_XMM|BITS128,YMMREG,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7220, IF_FUTURE|IF_AVX2},
    /* 3625 */ {I_VINSERTI128, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0}, nasm_bytecodes+7228, IF_FUTURE|IF_AVX2},
    /* 3626 */ {I_VINSERTI128, 3, {YMMREG,RM_XMM|BITS128,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7236, IF_FUTURE|IF_AVX2},
    /* 3627 */ {I_VPMASKMOVD, 3, {XMMREG,XMMREG,MEMORY|BITS128,0,0}, nasm_bytecodes+15510, IF_FUTURE|IF_AVX2},
    /* 3628 */ {I_VPMASKMOVD, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+15517, IF_FUTURE|IF_AVX2},
    /* 3629 */ {I_VPMASKMOVD, 3, {YMMREG,YMMREG,MEMORY|BITS256,0,0}, nasm_bytecodes+15524, IF_FUTURE|IF_AVX2},
    /* 3630 */ {I_VPMASKMOVD, 2, {YMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+15531, IF_FUTURE|IF_AVX2},
    /* 3631 */ {I_VPMASKMOVQ, 3, {XMMREG,XMMREG,MEMORY|BITS128,0,0}, nasm_bytecodes+15538, IF_FUTURE|IF_AVX2},
    /* 3632 */ {I_VPMASKMOVQ, 2, {XMMREG,MEMORY|BITS128,0,0,0}, nasm_bytecodes+15545, IF_FUTURE|IF_AVX2},
    /* 3633 */ {I_VPMASKMOVQ, 3, {YMMREG,YMMREG,MEMORY|BITS256,0,0}, nasm_bytecodes+15552, IF_FUTURE|IF_AVX2},
    /* 3634 */ {I_VPMASKMOVQ, 2, {YMMREG,MEMORY|BITS256,0,0,0}, nasm_bytecodes+15559, IF_FUTURE|IF_AVX2},
    /* 3635 */ {I_VPMASKMOVD, 3, {MEMORY|BITS128,XMMREG,XMMREG,0,0}, nasm_bytecodes+15566, IF_FUTURE|IF_AVX2},
    /* 3636 */ {I_VPMASKMOVD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, nasm_bytecodes+15573, IF_FUTURE|IF_AVX2},
    /* 3637 */ {I_VPMASKMOVD, 3, {MEMORY|BITS256,YMMREG,YMMREG,0,0}, nasm_bytecodes+15580, IF_FUTURE|IF_AVX2},
    /* 3638 */ {I_VPMASKMOVD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, nasm_bytecodes+15587, IF_FUTURE|IF_AVX2},
    /* 3639 */ {I_VPMASKMOVQ, 3, {MEMORY|BITS128,XMMREG,XMMREG,0,0}, nasm_bytecodes+15594, IF_FUTURE|IF_AVX2},
    /* 3640 */ {I_VPMASKMOVQ, 2, {MEMORY|BITS128,XMMREG,0,0,0}, nasm_bytecodes+15601, IF_FUTURE|IF_AVX2},
    /* 3641 */ {I_VPMASKMOVQ, 3, {MEMORY|BITS256,YMMREG,YMMREG,0,0}, nasm_bytecodes+15608, IF_FUTURE|IF_AVX2},
    /* 3642 */ {I_VPMASKMOVQ, 2, {MEMORY|BITS256,YMMREG,0,0,0}, nasm_bytecodes+15615, IF_FUTURE|IF_AVX2},
    /* 3643 */ {I_VPSLLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15622, IF_FUTURE|IF_AVX2},
    /* 3644 */ {I_VPSLLVD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15629, IF_FUTURE|IF_AVX2},
    /* 3645 */ {I_VPSLLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15636, IF_FUTURE|IF_AVX2},
    /* 3646 */ {I_VPSLLVQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15643, IF_FUTURE|IF_AVX2},
    /* 3647 */ {I_VPSLLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15650, IF_FUTURE|IF_AVX2},
    /* 3648 */ {I_VPSLLVD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15657, IF_FUTURE|IF_AVX2},
    /* 3649 */ {I_VPSLLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15664, IF_FUTURE|IF_AVX2},
    /* 3650 */ {I_VPSLLVQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15671, IF_FUTURE|IF_AVX2},
    /* 3651 */ {I_VPSRAVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15678, IF_FUTURE|IF_AVX2},
    /* 3652 */ {I_VPSRAVD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15685, IF_FUTURE|IF_AVX2},
    /* 3653 */ {I_VPSRAVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15692, IF_FUTURE|IF_AVX2},
    /* 3654 */ {I_VPSRAVD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15699, IF_FUTURE|IF_AVX2},
    /* 3655 */ {I_VPSRLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15706, IF_FUTURE|IF_AVX2},
    /* 3656 */ {I_VPSRLVD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15713, IF_FUTURE|IF_AVX2},
    /* 3657 */ {I_VPSRLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, nasm_bytecodes+15720, IF_FUTURE|IF_AVX2},
    /* 3658 */ {I_VPSRLVQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, nasm_bytecodes+15727, IF_FUTURE|IF_AVX2},
    /* 3659 */ {I_VPSRLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15734, IF_FUTURE|IF_AVX2},
    /* 3660 */ {I_VPSRLVD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15741, IF_FUTURE|IF_AVX2},
    /* 3661 */ {I_VPSRLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, nasm_bytecodes+15748, IF_FUTURE|IF_AVX2},
    /* 3662 */ {I_VPSRLVQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, nasm_bytecodes+15755, IF_FUTURE|IF_AVX2},
    /* 3663 */ {I_VGATHERDPD, 3, {XMMREG,XMEM|BITS64,XMMREG,0,0}, nasm_bytecodes+7244, IF_FUTURE|IF_AVX2},
    /* 3664 */ {I_VGATHERQPD, 3, {XMMREG,XMEM|BITS64,XMMREG,0,0}, nasm_bytecodes+7252, IF_FUTURE|IF_AVX2},
    /* 3665 */ {I_VGATHERDPD, 3, {YMMREG,XMEM|BITS64,YMMREG,0,0}, nasm_bytecodes+7260, IF_FUTURE|IF_AVX2},
    /* 3666 */ {I_VGATHERQPD, 3, {YMMREG,YMEM|BITS64,YMMREG,0,0}, nasm_bytecodes+7268, IF_FUTURE|IF_AVX2},
    /* 3667 */ {I_VGATHERDPS, 3, {XMMREG,XMEM|BITS32,XMMREG,0,0}, nasm_bytecodes+7276, IF_FUTURE|IF_AVX2},
    /* 3668 */ {I_VGATHERQPS, 3, {XMMREG,XMEM|BITS32,XMMREG,0,0}, nasm_bytecodes+7284, IF_FUTURE|IF_AVX2},
    /* 3669 */ {I_VGATHERDPS, 3, {YMMREG,YMEM|BITS32,YMMREG,0,0}, nasm_bytecodes+7292, IF_FUTURE|IF_AVX2},
    /* 3670 */ {I_VGATHERQPS, 3, {XMMREG,YMEM|BITS32,XMMREG,0,0}, nasm_bytecodes+7300, IF_FUTURE|IF_AVX2},
    /* 3671 */ {I_VPGATHERDD, 3, {XMMREG,XMEM|BITS32,XMMREG,0,0}, nasm_bytecodes+7308, IF_FUTURE|IF_AVX2},
    /* 3672 */ {I_VPGATHERQD, 3, {XMMREG,XMEM|BITS32,XMMREG,0,0}, nasm_bytecodes+7316, IF_FUTURE|IF_AVX2},
    /* 3673 */ {I_VPGATHERDD, 3, {YMMREG,YMEM|BITS32,YMMREG,0,0}, nasm_bytecodes+7324, IF_FUTURE|IF_AVX2},
    /* 3674 */ {I_VPGATHERQD, 3, {XMMREG,YMEM|BITS32,XMMREG,0,0}, nasm_bytecodes+7332, IF_FUTURE|IF_AVX2},
    /* 3675 */ {I_VPGATHERDQ, 3, {XMMREG,XMEM|BITS64,XMMREG,0,0}, nasm_bytecodes+7340, IF_FUTURE|IF_AVX2},
    /* 3676 */ {I_VPGATHERQQ, 3, {XMMREG,XMEM|BITS64,XMMREG,0,0}, nasm_bytecodes+7348, IF_FUTURE|IF_AVX2},
    /* 3677 */ {I_VPGATHERDQ, 3, {YMMREG,XMEM|BITS64,YMMREG,0,0}, nasm_bytecodes+7356, IF_FUTURE|IF_AVX2},
    /* 3678 */ {I_VPGATHERQQ, 3, {YMMREG,YMEM|BITS64,YMMREG,0,0}, nasm_bytecodes+7364, IF_FUTURE|IF_AVX2},
    /* 3679 */ {I_XABORT, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+21347, IF_FUTURE|IF_RTM},
    /* 3680 */ {I_XABORT, 1, {IMMEDIATE|BITS8,0,0,0,0}, nasm_bytecodes+21347, IF_FUTURE|IF_RTM},
    /* 3681 */ {I_XBEGIN, 1, {IMMEDIATE,0,0,0,0}, nasm_bytecodes+18459, IF_FUTURE|IF_RTM},
    /* 3682 */ {I_XBEGIN, 1, {IMMEDIATE|BITS16,0,0,0,0}, nasm_bytecodes+18465, IF_FUTURE|IF_RTM|IF_NOLONG},
    /* 3683 */ {I_XBEGIN, 1, {IMMEDIATE|BITS32,0,0,0,0}, nasm_bytecodes+18471, IF_FUTURE|IF_RTM|IF_NOLONG},
    /* 3684 */ {I_XBEGIN, 1, {IMMEDIATE|BITS64,0,0,0,0}, nasm_bytecodes+18477, IF_FUTURE|IF_RTM|IF_LONG},
    /* 3685 */ {I_XEND, 0, {0,0,0,0,0}, nasm_bytecodes+21352, IF_FUTURE|IF_RTM},
    /* 3686 */ {I_XTEST, 0, {0,0,0,0,0}, nasm_bytecodes+21357, IF_FUTURE|IF_HLE|IF_RTM},
    /* 3687 */ {I_ANDN, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, nasm_bytecodes+15762, IF_FUTURE|IF_BMI1},
    /* 3688 */ {I_ANDN, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, nasm_bytecodes+15769, IF_LONG|IF_FUTURE|IF_BMI1},
    /* 3689 */ {I_BEXTR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, nasm_bytecodes+15776, IF_FUTURE|IF_BMI1},
    /* 3690 */ {I_BEXTR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, nasm_bytecodes+15783, IF_LONG|IF_FUTURE|IF_BMI1},
    /* 3691 */ {I_BEXTR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+7372, IF_FUTURE|IF_TBM},
    /* 3692 */ {I_BEXTR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMMEDIATE|BITS32,0,0}, nasm_bytecodes+7380, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3693 */ {I_BLCI, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15790, IF_FUTURE|IF_TBM},
    /* 3694 */ {I_BLCI, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15797, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3695 */ {I_BLCIC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15804, IF_FUTURE|IF_TBM},
    /* 3696 */ {I_BLCIC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15811, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3697 */ {I_BLSI, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15818, IF_FUTURE|IF_BMI1},
    /* 3698 */ {I_BLSI, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15825, IF_LONG|IF_FUTURE|IF_BMI1},
    /* 3699 */ {I_BLSIC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15832, IF_FUTURE|IF_TBM},
    /* 3700 */ {I_BLSIC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15839, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3701 */ {I_BLCFILL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15846, IF_FUTURE|IF_TBM},
    /* 3702 */ {I_BLCFILL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15853, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3703 */ {I_BLSFILL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15860, IF_FUTURE|IF_TBM},
    /* 3704 */ {I_BLSFILL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15867, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3705 */ {I_BLCMSK, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15874, IF_FUTURE|IF_TBM},
    /* 3706 */ {I_BLCMSK, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15881, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3707 */ {I_BLSMSK, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15888, IF_FUTURE|IF_BMI1},
    /* 3708 */ {I_BLSMSK, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15895, IF_LONG|IF_FUTURE|IF_BMI1},
    /* 3709 */ {I_BLSR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15902, IF_FUTURE|IF_BMI1},
    /* 3710 */ {I_BLSR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15909, IF_LONG|IF_FUTURE|IF_BMI1},
    /* 3711 */ {I_BLCS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+15916, IF_FUTURE|IF_TBM},
    /* 3712 */ {I_BLCS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+15923, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3713 */ {I_BZHI, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, nasm_bytecodes+15930, IF_FUTURE|IF_BMI2},
    /* 3714 */ {I_BZHI, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, nasm_bytecodes+15937, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3715 */ {I_MULX, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, nasm_bytecodes+15944, IF_FUTURE|IF_BMI2},
    /* 3716 */ {I_MULX, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, nasm_bytecodes+15951, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3717 */ {I_PDEP, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, nasm_bytecodes+15958, IF_FUTURE|IF_BMI2},
    /* 3718 */ {I_PDEP, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, nasm_bytecodes+15965, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3719 */ {I_PEXT, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, nasm_bytecodes+15972, IF_FUTURE|IF_BMI2},
    /* 3720 */ {I_PEXT, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, nasm_bytecodes+15979, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3721 */ {I_RORX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7388, IF_FUTURE|IF_BMI2},
    /* 3722 */ {I_RORX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMMEDIATE|BITS8,0,0}, nasm_bytecodes+7396, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3723 */ {I_SARX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, nasm_bytecodes+15986, IF_FUTURE|IF_BMI2},
    /* 3724 */ {I_SARX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, nasm_bytecodes+15993, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3725 */ {I_SHLX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, nasm_bytecodes+16000, IF_FUTURE|IF_BMI2},
    /* 3726 */ {I_SHLX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, nasm_bytecodes+16007, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3727 */ {I_SHRX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, nasm_bytecodes+16014, IF_FUTURE|IF_BMI2},
    /* 3728 */ {I_SHRX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, nasm_bytecodes+16021, IF_LONG|IF_FUTURE|IF_BMI2},
    /* 3729 */ {I_TZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, nasm_bytecodes+16028, IF_FUTURE|IF_BMI1},
    /* 3730 */ {I_TZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+16035, IF_FUTURE|IF_BMI1},
    /* 3731 */ {I_TZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+16042, IF_LONG|IF_FUTURE|IF_BMI1},
    /* 3732 */ {I_TZMSK, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+16049, IF_FUTURE|IF_TBM},
    /* 3733 */ {I_TZMSK, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+16056, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3734 */ {I_T1MSKC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, nasm_bytecodes+16063, IF_FUTURE|IF_TBM},
    /* 3735 */ {I_T1MSKC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, nasm_bytecodes+16070, IF_LONG|IF_FUTURE|IF_TBM},
    /* 3736 */ {I_HINT_NOP0, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18483, IF_P6|IF_UNDOC},
    /* 3737 */ {I_HINT_NOP0, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18489, IF_P6|IF_UNDOC},
    /* 3738 */ {I_HINT_NOP0, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18495, IF_X64|IF_UNDOC},
    /* 3739 */ {I_HINT_NOP1, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18501, IF_P6|IF_UNDOC},
    /* 3740 */ {I_HINT_NOP1, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18507, IF_P6|IF_UNDOC},
    /* 3741 */ {I_HINT_NOP1, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18513, IF_X64|IF_UNDOC},
    /* 3742 */ {I_HINT_NOP2, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18519, IF_P6|IF_UNDOC},
    /* 3743 */ {I_HINT_NOP2, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18525, IF_P6|IF_UNDOC},
    /* 3744 */ {I_HINT_NOP2, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18531, IF_X64|IF_UNDOC},
    /* 3745 */ {I_HINT_NOP3, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18537, IF_P6|IF_UNDOC},
    /* 3746 */ {I_HINT_NOP3, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18543, IF_P6|IF_UNDOC},
    /* 3747 */ {I_HINT_NOP3, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18549, IF_X64|IF_UNDOC},
    /* 3748 */ {I_HINT_NOP4, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18555, IF_P6|IF_UNDOC},
    /* 3749 */ {I_HINT_NOP4, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18561, IF_P6|IF_UNDOC},
    /* 3750 */ {I_HINT_NOP4, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18567, IF_X64|IF_UNDOC},
    /* 3751 */ {I_HINT_NOP5, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18573, IF_P6|IF_UNDOC},
    /* 3752 */ {I_HINT_NOP5, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18579, IF_P6|IF_UNDOC},
    /* 3753 */ {I_HINT_NOP5, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18585, IF_X64|IF_UNDOC},
    /* 3754 */ {I_HINT_NOP6, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18591, IF_P6|IF_UNDOC},
    /* 3755 */ {I_HINT_NOP6, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18597, IF_P6|IF_UNDOC},
    /* 3756 */ {I_HINT_NOP6, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18603, IF_X64|IF_UNDOC},
    /* 3757 */ {I_HINT_NOP7, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18609, IF_P6|IF_UNDOC},
    /* 3758 */ {I_HINT_NOP7, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18615, IF_P6|IF_UNDOC},
    /* 3759 */ {I_HINT_NOP7, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18621, IF_X64|IF_UNDOC},
    /* 3760 */ {I_HINT_NOP8, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18627, IF_P6|IF_UNDOC},
    /* 3761 */ {I_HINT_NOP8, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18633, IF_P6|IF_UNDOC},
    /* 3762 */ {I_HINT_NOP8, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18639, IF_X64|IF_UNDOC},
    /* 3763 */ {I_HINT_NOP9, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18645, IF_P6|IF_UNDOC},
    /* 3764 */ {I_HINT_NOP9, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18651, IF_P6|IF_UNDOC},
    /* 3765 */ {I_HINT_NOP9, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18657, IF_X64|IF_UNDOC},
    /* 3766 */ {I_HINT_NOP10, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18663, IF_P6|IF_UNDOC},
    /* 3767 */ {I_HINT_NOP10, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18669, IF_P6|IF_UNDOC},
    /* 3768 */ {I_HINT_NOP10, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18675, IF_X64|IF_UNDOC},
    /* 3769 */ {I_HINT_NOP11, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18681, IF_P6|IF_UNDOC},
    /* 3770 */ {I_HINT_NOP11, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18687, IF_P6|IF_UNDOC},
    /* 3771 */ {I_HINT_NOP11, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18693, IF_X64|IF_UNDOC},
    /* 3772 */ {I_HINT_NOP12, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18699, IF_P6|IF_UNDOC},
    /* 3773 */ {I_HINT_NOP12, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18705, IF_P6|IF_UNDOC},
    /* 3774 */ {I_HINT_NOP12, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18711, IF_X64|IF_UNDOC},
    /* 3775 */ {I_HINT_NOP13, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18717, IF_P6|IF_UNDOC},
    /* 3776 */ {I_HINT_NOP13, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18723, IF_P6|IF_UNDOC},
    /* 3777 */ {I_HINT_NOP13, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18729, IF_X64|IF_UNDOC},
    /* 3778 */ {I_HINT_NOP14, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18735, IF_P6|IF_UNDOC},
    /* 3779 */ {I_HINT_NOP14, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18741, IF_P6|IF_UNDOC},
    /* 3780 */ {I_HINT_NOP14, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18747, IF_X64|IF_UNDOC},
    /* 3781 */ {I_HINT_NOP15, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18753, IF_P6|IF_UNDOC},
    /* 3782 */ {I_HINT_NOP15, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18759, IF_P6|IF_UNDOC},
    /* 3783 */ {I_HINT_NOP15, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18765, IF_X64|IF_UNDOC},
    /* 3784 */ {I_HINT_NOP16, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18771, IF_P6|IF_UNDOC},
    /* 3785 */ {I_HINT_NOP16, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18777, IF_P6|IF_UNDOC},
    /* 3786 */ {I_HINT_NOP16, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18783, IF_X64|IF_UNDOC},
    /* 3787 */ {I_HINT_NOP17, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18789, IF_P6|IF_UNDOC},
    /* 3788 */ {I_HINT_NOP17, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18795, IF_P6|IF_UNDOC},
    /* 3789 */ {I_HINT_NOP17, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18801, IF_X64|IF_UNDOC},
    /* 3790 */ {I_HINT_NOP18, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18807, IF_P6|IF_UNDOC},
    /* 3791 */ {I_HINT_NOP18, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18813, IF_P6|IF_UNDOC},
    /* 3792 */ {I_HINT_NOP18, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18819, IF_X64|IF_UNDOC},
    /* 3793 */ {I_HINT_NOP19, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18825, IF_P6|IF_UNDOC},
    /* 3794 */ {I_HINT_NOP19, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18831, IF_P6|IF_UNDOC},
    /* 3795 */ {I_HINT_NOP19, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18837, IF_X64|IF_UNDOC},
    /* 3796 */ {I_HINT_NOP20, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18843, IF_P6|IF_UNDOC},
    /* 3797 */ {I_HINT_NOP20, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18849, IF_P6|IF_UNDOC},
    /* 3798 */ {I_HINT_NOP20, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18855, IF_X64|IF_UNDOC},
    /* 3799 */ {I_HINT_NOP21, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18861, IF_P6|IF_UNDOC},
    /* 3800 */ {I_HINT_NOP21, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18867, IF_P6|IF_UNDOC},
    /* 3801 */ {I_HINT_NOP21, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18873, IF_X64|IF_UNDOC},
    /* 3802 */ {I_HINT_NOP22, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18879, IF_P6|IF_UNDOC},
    /* 3803 */ {I_HINT_NOP22, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18885, IF_P6|IF_UNDOC},
    /* 3804 */ {I_HINT_NOP22, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18891, IF_X64|IF_UNDOC},
    /* 3805 */ {I_HINT_NOP23, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18897, IF_P6|IF_UNDOC},
    /* 3806 */ {I_HINT_NOP23, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18903, IF_P6|IF_UNDOC},
    /* 3807 */ {I_HINT_NOP23, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18909, IF_X64|IF_UNDOC},
    /* 3808 */ {I_HINT_NOP24, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18915, IF_P6|IF_UNDOC},
    /* 3809 */ {I_HINT_NOP24, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18921, IF_P6|IF_UNDOC},
    /* 3810 */ {I_HINT_NOP24, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18927, IF_X64|IF_UNDOC},
    /* 3811 */ {I_HINT_NOP25, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18933, IF_P6|IF_UNDOC},
    /* 3812 */ {I_HINT_NOP25, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18939, IF_P6|IF_UNDOC},
    /* 3813 */ {I_HINT_NOP25, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18945, IF_X64|IF_UNDOC},
    /* 3814 */ {I_HINT_NOP26, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18951, IF_P6|IF_UNDOC},
    /* 3815 */ {I_HINT_NOP26, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18957, IF_P6|IF_UNDOC},
    /* 3816 */ {I_HINT_NOP26, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18963, IF_X64|IF_UNDOC},
    /* 3817 */ {I_HINT_NOP27, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18969, IF_P6|IF_UNDOC},
    /* 3818 */ {I_HINT_NOP27, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18975, IF_P6|IF_UNDOC},
    /* 3819 */ {I_HINT_NOP27, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18981, IF_X64|IF_UNDOC},
    /* 3820 */ {I_HINT_NOP28, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+18987, IF_P6|IF_UNDOC},
    /* 3821 */ {I_HINT_NOP28, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+18993, IF_P6|IF_UNDOC},
    /* 3822 */ {I_HINT_NOP28, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+18999, IF_X64|IF_UNDOC},
    /* 3823 */ {I_HINT_NOP29, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19005, IF_P6|IF_UNDOC},
    /* 3824 */ {I_HINT_NOP29, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19011, IF_P6|IF_UNDOC},
    /* 3825 */ {I_HINT_NOP29, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19017, IF_X64|IF_UNDOC},
    /* 3826 */ {I_HINT_NOP30, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19023, IF_P6|IF_UNDOC},
    /* 3827 */ {I_HINT_NOP30, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19029, IF_P6|IF_UNDOC},
    /* 3828 */ {I_HINT_NOP30, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19035, IF_X64|IF_UNDOC},
    /* 3829 */ {I_HINT_NOP31, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19041, IF_P6|IF_UNDOC},
    /* 3830 */ {I_HINT_NOP31, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19047, IF_P6|IF_UNDOC},
    /* 3831 */ {I_HINT_NOP31, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19053, IF_X64|IF_UNDOC},
    /* 3832 */ {I_HINT_NOP32, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19059, IF_P6|IF_UNDOC},
    /* 3833 */ {I_HINT_NOP32, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19065, IF_P6|IF_UNDOC},
    /* 3834 */ {I_HINT_NOP32, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19071, IF_X64|IF_UNDOC},
    /* 3835 */ {I_HINT_NOP33, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19077, IF_P6|IF_UNDOC},
    /* 3836 */ {I_HINT_NOP33, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19083, IF_P6|IF_UNDOC},
    /* 3837 */ {I_HINT_NOP33, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19089, IF_X64|IF_UNDOC},
    /* 3838 */ {I_HINT_NOP34, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19095, IF_P6|IF_UNDOC},
    /* 3839 */ {I_HINT_NOP34, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19101, IF_P6|IF_UNDOC},
    /* 3840 */ {I_HINT_NOP34, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19107, IF_X64|IF_UNDOC},
    /* 3841 */ {I_HINT_NOP35, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19113, IF_P6|IF_UNDOC},
    /* 3842 */ {I_HINT_NOP35, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19119, IF_P6|IF_UNDOC},
    /* 3843 */ {I_HINT_NOP35, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19125, IF_X64|IF_UNDOC},
    /* 3844 */ {I_HINT_NOP36, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19131, IF_P6|IF_UNDOC},
    /* 3845 */ {I_HINT_NOP36, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19137, IF_P6|IF_UNDOC},
    /* 3846 */ {I_HINT_NOP36, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19143, IF_X64|IF_UNDOC},
    /* 3847 */ {I_HINT_NOP37, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19149, IF_P6|IF_UNDOC},
    /* 3848 */ {I_HINT_NOP37, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19155, IF_P6|IF_UNDOC},
    /* 3849 */ {I_HINT_NOP37, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19161, IF_X64|IF_UNDOC},
    /* 3850 */ {I_HINT_NOP38, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19167, IF_P6|IF_UNDOC},
    /* 3851 */ {I_HINT_NOP38, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19173, IF_P6|IF_UNDOC},
    /* 3852 */ {I_HINT_NOP38, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19179, IF_X64|IF_UNDOC},
    /* 3853 */ {I_HINT_NOP39, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19185, IF_P6|IF_UNDOC},
    /* 3854 */ {I_HINT_NOP39, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19191, IF_P6|IF_UNDOC},
    /* 3855 */ {I_HINT_NOP39, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19197, IF_X64|IF_UNDOC},
    /* 3856 */ {I_HINT_NOP40, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19203, IF_P6|IF_UNDOC},
    /* 3857 */ {I_HINT_NOP40, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19209, IF_P6|IF_UNDOC},
    /* 3858 */ {I_HINT_NOP40, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19215, IF_X64|IF_UNDOC},
    /* 3859 */ {I_HINT_NOP41, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19221, IF_P6|IF_UNDOC},
    /* 3860 */ {I_HINT_NOP41, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19227, IF_P6|IF_UNDOC},
    /* 3861 */ {I_HINT_NOP41, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19233, IF_X64|IF_UNDOC},
    /* 3862 */ {I_HINT_NOP42, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19239, IF_P6|IF_UNDOC},
    /* 3863 */ {I_HINT_NOP42, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19245, IF_P6|IF_UNDOC},
    /* 3864 */ {I_HINT_NOP42, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19251, IF_X64|IF_UNDOC},
    /* 3865 */ {I_HINT_NOP43, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19257, IF_P6|IF_UNDOC},
    /* 3866 */ {I_HINT_NOP43, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19263, IF_P6|IF_UNDOC},
    /* 3867 */ {I_HINT_NOP43, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19269, IF_X64|IF_UNDOC},
    /* 3868 */ {I_HINT_NOP44, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19275, IF_P6|IF_UNDOC},
    /* 3869 */ {I_HINT_NOP44, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19281, IF_P6|IF_UNDOC},
    /* 3870 */ {I_HINT_NOP44, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19287, IF_X64|IF_UNDOC},
    /* 3871 */ {I_HINT_NOP45, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19293, IF_P6|IF_UNDOC},
    /* 3872 */ {I_HINT_NOP45, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19299, IF_P6|IF_UNDOC},
    /* 3873 */ {I_HINT_NOP45, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19305, IF_X64|IF_UNDOC},
    /* 3874 */ {I_HINT_NOP46, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19311, IF_P6|IF_UNDOC},
    /* 3875 */ {I_HINT_NOP46, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19317, IF_P6|IF_UNDOC},
    /* 3876 */ {I_HINT_NOP46, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19323, IF_X64|IF_UNDOC},
    /* 3877 */ {I_HINT_NOP47, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19329, IF_P6|IF_UNDOC},
    /* 3878 */ {I_HINT_NOP47, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19335, IF_P6|IF_UNDOC},
    /* 3879 */ {I_HINT_NOP47, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19341, IF_X64|IF_UNDOC},
    /* 3880 */ {I_HINT_NOP48, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19347, IF_P6|IF_UNDOC},
    /* 3881 */ {I_HINT_NOP48, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19353, IF_P6|IF_UNDOC},
    /* 3882 */ {I_HINT_NOP48, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19359, IF_X64|IF_UNDOC},
    /* 3883 */ {I_HINT_NOP49, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19365, IF_P6|IF_UNDOC},
    /* 3884 */ {I_HINT_NOP49, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19371, IF_P6|IF_UNDOC},
    /* 3885 */ {I_HINT_NOP49, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19377, IF_X64|IF_UNDOC},
    /* 3886 */ {I_HINT_NOP50, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19383, IF_P6|IF_UNDOC},
    /* 3887 */ {I_HINT_NOP50, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19389, IF_P6|IF_UNDOC},
    /* 3888 */ {I_HINT_NOP50, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19395, IF_X64|IF_UNDOC},
    /* 3889 */ {I_HINT_NOP51, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19401, IF_P6|IF_UNDOC},
    /* 3890 */ {I_HINT_NOP51, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19407, IF_P6|IF_UNDOC},
    /* 3891 */ {I_HINT_NOP51, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19413, IF_X64|IF_UNDOC},
    /* 3892 */ {I_HINT_NOP52, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19419, IF_P6|IF_UNDOC},
    /* 3893 */ {I_HINT_NOP52, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19425, IF_P6|IF_UNDOC},
    /* 3894 */ {I_HINT_NOP52, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19431, IF_X64|IF_UNDOC},
    /* 3895 */ {I_HINT_NOP53, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19437, IF_P6|IF_UNDOC},
    /* 3896 */ {I_HINT_NOP53, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19443, IF_P6|IF_UNDOC},
    /* 3897 */ {I_HINT_NOP53, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19449, IF_X64|IF_UNDOC},
    /* 3898 */ {I_HINT_NOP54, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19455, IF_P6|IF_UNDOC},
    /* 3899 */ {I_HINT_NOP54, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19461, IF_P6|IF_UNDOC},
    /* 3900 */ {I_HINT_NOP54, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19467, IF_X64|IF_UNDOC},
    /* 3901 */ {I_HINT_NOP55, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19473, IF_P6|IF_UNDOC},
    /* 3902 */ {I_HINT_NOP55, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19479, IF_P6|IF_UNDOC},
    /* 3903 */ {I_HINT_NOP55, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19485, IF_X64|IF_UNDOC},
    /* 3904 */ {I_HINT_NOP56, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+16767, IF_P6|IF_UNDOC},
    /* 3905 */ {I_HINT_NOP56, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+16773, IF_P6|IF_UNDOC},
    /* 3906 */ {I_HINT_NOP56, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+16779, IF_X64|IF_UNDOC},
    /* 3907 */ {I_HINT_NOP57, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19491, IF_P6|IF_UNDOC},
    /* 3908 */ {I_HINT_NOP57, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19497, IF_P6|IF_UNDOC},
    /* 3909 */ {I_HINT_NOP57, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19503, IF_X64|IF_UNDOC},
    /* 3910 */ {I_HINT_NOP58, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19509, IF_P6|IF_UNDOC},
    /* 3911 */ {I_HINT_NOP58, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19515, IF_P6|IF_UNDOC},
    /* 3912 */ {I_HINT_NOP58, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19521, IF_X64|IF_UNDOC},
    /* 3913 */ {I_HINT_NOP59, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19527, IF_P6|IF_UNDOC},
    /* 3914 */ {I_HINT_NOP59, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19533, IF_P6|IF_UNDOC},
    /* 3915 */ {I_HINT_NOP59, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19539, IF_X64|IF_UNDOC},
    /* 3916 */ {I_HINT_NOP60, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19545, IF_P6|IF_UNDOC},
    /* 3917 */ {I_HINT_NOP60, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19551, IF_P6|IF_UNDOC},
    /* 3918 */ {I_HINT_NOP60, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19557, IF_X64|IF_UNDOC},
    /* 3919 */ {I_HINT_NOP61, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19563, IF_P6|IF_UNDOC},
    /* 3920 */ {I_HINT_NOP61, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19569, IF_P6|IF_UNDOC},
    /* 3921 */ {I_HINT_NOP61, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19575, IF_X64|IF_UNDOC},
    /* 3922 */ {I_HINT_NOP62, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19581, IF_P6|IF_UNDOC},
    /* 3923 */ {I_HINT_NOP62, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19587, IF_P6|IF_UNDOC},
    /* 3924 */ {I_HINT_NOP62, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19593, IF_X64|IF_UNDOC},
    /* 3925 */ {I_HINT_NOP63, 1, {RM_GPR|BITS16,0,0,0,0}, nasm_bytecodes+19599, IF_P6|IF_UNDOC},
    /* 3926 */ {I_HINT_NOP63, 1, {RM_GPR|BITS32,0,0,0,0}, nasm_bytecodes+19605, IF_P6|IF_UNDOC},
    /* 3927 */ {I_HINT_NOP63, 1, {RM_GPR|BITS64,0,0,0,0}, nasm_bytecodes+19611, IF_X64|IF_UNDOC},
};

static const struct itemplate * const itable_00[] = {
    instrux + 36,
    instrux + 37,
};

static const struct itemplate * const itable_01[] = {
    instrux + 38,
    instrux + 39,
    instrux + 40,
    instrux + 41,
    instrux + 42,
    instrux + 43,
};

static const struct itemplate * const itable_02[] = {
    instrux + 44,
    instrux + 45,
};

static const struct itemplate * const itable_03[] = {
    instrux + 46,
    instrux + 47,
    instrux + 48,
    instrux + 49,
    instrux + 50,
    instrux + 51,
};

static const struct itemplate * const itable_04[] = {
    instrux + 55,
};

static const struct itemplate * const itable_05[] = {
    instrux + 56,
    instrux + 57,
    instrux + 58,
};

static const struct itemplate * const itable_06[] = {
    instrux + 838,
};

static const struct itemplate * const itable_07[] = {
    instrux + 787,
};

static const struct itemplate * const itable_08[] = {
    instrux + 690,
    instrux + 691,
};

static const struct itemplate * const itable_09[] = {
    instrux + 692,
    instrux + 693,
    instrux + 694,
    instrux + 695,
    instrux + 696,
    instrux + 697,
};

static const struct itemplate * const itable_0A[] = {
    instrux + 698,
    instrux + 699,
};

static const struct itemplate * const itable_0B[] = {
    instrux + 700,
    instrux + 701,
    instrux + 702,
    instrux + 703,
    instrux + 704,
    instrux + 705,
};

static const struct itemplate * const itable_0C[] = {
    instrux + 709,
};

static const struct itemplate * const itable_0D[] = {
    instrux + 710,
    instrux + 711,
    instrux + 712,
};

static const struct itemplate * const itable_0E[] = {
    instrux + 839,
};

static const struct itemplate * const itable_0F00[] = {
    instrux + 518,
    instrux + 519,
    instrux + 548,
    instrux + 549,
    instrux + 550,
    instrux + 593,
    instrux + 594,
    instrux + 595,
    instrux + 1020,
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    instrux + 1024,
    instrux + 1038,
    instrux + 1039,
    instrux + 1040,
    instrux + 1041,
    instrux + 1042,
    instrux + 1106,
    instrux + 1107,
    instrux + 1108,
    instrux + 1109,
    instrux + 1110,
    instrux + 1111,
};

static const struct itemplate * const itable_0F01[] = {
    instrux + 484,
    instrux + 485,
    instrux + 486,
    instrux + 487,
    instrux + 488,
    instrux + 543,
    instrux + 547,
    instrux + 551,
    instrux + 552,
    instrux + 553,
    instrux + 597,
    instrux + 677,
    instrux + 886,
    instrux + 970,
    instrux + 1019,
    instrux + 1025,
    instrux + 1027,
    instrux + 1028,
    instrux + 1029,
    instrux + 1030,
    instrux + 1075,
    instrux + 1272,
    instrux + 1273,
    instrux + 1508,
    instrux + 1509,
    instrux + 1510,
    instrux + 1512,
    instrux + 1513,
    instrux + 1514,
    instrux + 1515,
    instrux + 1520,
    instrux + 1521,
    instrux + 1522,
    instrux + 1525,
    instrux + 3077,
    instrux + 3078,
    instrux + 3685,
    instrux + 3686,
};

static const struct itemplate * const itable_0F02[] = {
    instrux + 521,
    instrux + 522,
    instrux + 523,
    instrux + 524,
    instrux + 525,
    instrux + 526,
    instrux + 527,
    instrux + 528,
    instrux + 529,
    instrux + 530,
};

static const struct itemplate * const itable_0F03[] = {
    instrux + 580,
    instrux + 581,
    instrux + 582,
    instrux + 583,
    instrux + 584,
    instrux + 585,
    instrux + 586,
    instrux + 587,
    instrux + 588,
    instrux + 589,
};

static const struct itemplate * const itable_0F05[] = {
    instrux + 555,
    instrux + 1076,
};

static const struct itemplate * const itable_0F06[] = {
    instrux + 174,
};

static const struct itemplate * const itable_0F07[] = {
    instrux + 554,
    instrux + 1079,
};

static const struct itemplate * const itable_0F08[] = {
    instrux + 481,
};

static const struct itemplate * const itable_0F09[] = {
    instrux + 1113,
};

static const struct itemplate * const itable_0F0B[] = {
    instrux + 1105,
};

static const struct itemplate * const itable_0F0D[] = {
    instrux + 800,
    instrux + 801,
};

static const struct itemplate * const itable_0F0E[] = {
    instrux + 307,
};

static const struct itemplate * const itable_0F0F[] = {
    instrux + 744,
    instrux + 752,
    instrux + 753,
    instrux + 754,
    instrux + 755,
    instrux + 756,
    instrux + 757,
    instrux + 758,
    instrux + 759,
    instrux + 760,
    instrux + 761,
    instrux + 762,
    instrux + 763,
    instrux + 764,
    instrux + 765,
    instrux + 766,
    instrux + 767,
    instrux + 768,
    instrux + 773,
    instrux + 1301,
    instrux + 1302,
    instrux + 1303,
    instrux + 1304,
    instrux + 1305,
    instrux + 1644,
    instrux + 1645,
};

static const struct itemplate * const itable_0F10[] = {
    instrux + 1246,
    instrux + 1248,
    instrux + 1249,
    instrux + 1477,
    instrux + 1480,
    instrux + 1481,
    instrux + 1484,
};

static const struct itemplate * const itable_0F11[] = {
    instrux + 1247,
    instrux + 1250,
    instrux + 1478,
    instrux + 1479,
    instrux + 1482,
    instrux + 1483,
};

static const struct itemplate * const itable_0F12[] = {
    instrux + 1240,
    instrux + 1242,
    instrux + 1474,
    instrux + 1505,
    instrux + 1507,
};

static const struct itemplate * const itable_0F13[] = {
    instrux + 1241,
    instrux + 1473,
};

static const struct itemplate * const itable_0F14[] = {
    instrux + 1266,
    instrux + 1496,
};

static const struct itemplate * const itable_0F15[] = {
    instrux + 1265,
    instrux + 1495,
};

static const struct itemplate * const itable_0F16[] = {
    instrux + 1237,
    instrux + 1239,
    instrux + 1472,
    instrux + 1506,
};

static const struct itemplate * const itable_0F17[] = {
    instrux + 1238,
    instrux + 1471,
};

static const struct itemplate * const itable_0F18[] = {
    instrux + 1280,
    instrux + 1281,
    instrux + 1282,
    instrux + 1283,
    instrux + 3736,
    instrux + 3737,
    instrux + 3738,
    instrux + 3739,
    instrux + 3740,
    instrux + 3741,
    instrux + 3742,
    instrux + 3743,
    instrux + 3744,
    instrux + 3745,
    instrux + 3746,
    instrux + 3747,
    instrux + 3748,
    instrux + 3749,
    instrux + 3750,
    instrux + 3751,
    instrux + 3752,
    instrux + 3753,
    instrux + 3754,
    instrux + 3755,
    instrux + 3756,
    instrux + 3757,
    instrux + 3758,
    instrux + 3759,
};

static const struct itemplate * const itable_0F19[] = {
    instrux + 3760,
    instrux + 3761,
    instrux + 3762,
    instrux + 3763,
    instrux + 3764,
    instrux + 3765,
    instrux + 3766,
    instrux + 3767,
    instrux + 3768,
    instrux + 3769,
    instrux + 3770,
    instrux + 3771,
    instrux + 3772,
    instrux + 3773,
    instrux + 3774,
    instrux + 3775,
    instrux + 3776,
    instrux + 3777,
    instrux + 3778,
    instrux + 3779,
    instrux + 3780,
    instrux + 3781,
    instrux + 3782,
    instrux + 3783,
};

static const struct itemplate * const itable_0F1A[] = {
    instrux + 3784,
    instrux + 3785,
    instrux + 3786,
    instrux + 3787,
    instrux + 3788,
    instrux + 3789,
    instrux + 3790,
    instrux + 3791,
    instrux + 3792,
    instrux + 3793,
    instrux + 3794,
    instrux + 3795,
    instrux + 3796,
    instrux + 3797,
    instrux + 3798,
    instrux + 3799,
    instrux + 3800,
    instrux + 3801,
    instrux + 3802,
    instrux + 3803,
    instrux + 3804,
    instrux + 3805,
    instrux + 3806,
    instrux + 3807,
};

static const struct itemplate * const itable_0F1B[] = {
    instrux + 3808,
    instrux + 3809,
    instrux + 3810,
    instrux + 3811,
    instrux + 3812,
    instrux + 3813,
    instrux + 3814,
    instrux + 3815,
    instrux + 3816,
    instrux + 3817,
    instrux + 3818,
    instrux + 3819,
    instrux + 3820,
    instrux + 3821,
    instrux + 3822,
    instrux + 3823,
    instrux + 3824,
    instrux + 3825,
    instrux + 3826,
    instrux + 3827,
    instrux + 3828,
    instrux + 3829,
    instrux + 3830,
    instrux + 3831,
};

static const struct itemplate * const itable_0F1C[] = {
    instrux + 3832,
    instrux + 3833,
    instrux + 3834,
    instrux + 3835,
    instrux + 3836,
    instrux + 3837,
    instrux + 3838,
    instrux + 3839,
    instrux + 3840,
    instrux + 3841,
    instrux + 3842,
    instrux + 3843,
    instrux + 3844,
    instrux + 3845,
    instrux + 3846,
    instrux + 3847,
    instrux + 3848,
    instrux + 3849,
    instrux + 3850,
    instrux + 3851,
    instrux + 3852,
    instrux + 3853,
    instrux + 3854,
    instrux + 3855,
};

static const struct itemplate * const itable_0F1D[] = {
    instrux + 3856,
    instrux + 3857,
    instrux + 3858,
    instrux + 3859,
    instrux + 3860,
    instrux + 3861,
    instrux + 3862,
    instrux + 3863,
    instrux + 3864,
    instrux + 3865,
    instrux + 3866,
    instrux + 3867,
    instrux + 3868,
    instrux + 3869,
    instrux + 3870,
    instrux + 3871,
    instrux + 3872,
    instrux + 3873,
    instrux + 3874,
    instrux + 3875,
    instrux + 3876,
    instrux + 3877,
    instrux + 3878,
    instrux + 3879,
};

static const struct itemplate * const itable_0F1E[] = {
    instrux + 3880,
    instrux + 3881,
    instrux + 3882,
    instrux + 3883,
    instrux + 3884,
    instrux + 3885,
    instrux + 3886,
    instrux + 3887,
    instrux + 3888,
    instrux + 3889,
    instrux + 3890,
    instrux + 3891,
    instrux + 3892,
    instrux + 3893,
    instrux + 3894,
    instrux + 3895,
    instrux + 3896,
    instrux + 3897,
    instrux + 3898,
    instrux + 3899,
    instrux + 3900,
    instrux + 3901,
    instrux + 3902,
    instrux + 3903,
};

static const struct itemplate * const itable_0F1F[] = {
    instrux + 683,
    instrux + 684,
    instrux + 685,
    instrux + 3904,
    instrux + 3905,
    instrux + 3906,
    instrux + 3907,
    instrux + 3908,
    instrux + 3909,
    instrux + 3910,
    instrux + 3911,
    instrux + 3912,
    instrux + 3913,
    instrux + 3914,
    instrux + 3915,
    instrux + 3916,
    instrux + 3917,
    instrux + 3918,
    instrux + 3919,
    instrux + 3920,
    instrux + 3921,
    instrux + 3922,
    instrux + 3923,
    instrux + 3924,
    instrux + 3925,
    instrux + 3926,
    instrux + 3927,
};

static const struct itemplate * const itable_0F20[] = {
    instrux + 614,
    instrux + 615,
};

static const struct itemplate * const itable_0F21[] = {
    instrux + 618,
    instrux + 619,
};

static const struct itemplate * const itable_0F22[] = {
    instrux + 616,
    instrux + 617,
};

static const struct itemplate * const itable_0F23[] = {
    instrux + 620,
    instrux + 621,
};

static const struct itemplate * const itable_0F28[] = {
    instrux + 1235,
    instrux + 1467,
    instrux + 1470,
};

static const struct itemplate * const itable_0F29[] = {
    instrux + 1236,
    instrux + 1468,
    instrux + 1469,
};

static const struct itemplate * const itable_0F2A[] = {
    instrux + 1217,
    instrux + 1219,
    instrux + 1220,
    instrux + 1443,
    instrux + 1451,
    instrux + 1452,
};

static const struct itemplate * const itable_0F2B[] = {
    instrux + 1245,
    instrux + 1311,
    instrux + 1567,
    instrux + 1568,
};

static const struct itemplate * const itable_0F2C[] = {
    instrux + 1225,
    instrux + 1226,
    instrux + 1227,
    instrux + 1454,
    instrux + 1457,
    instrux + 1458,
    instrux + 1459,
    instrux + 1460,
};

static const struct itemplate * const itable_0F2D[] = {
    instrux + 1218,
    instrux + 1221,
    instrux + 1222,
    instrux + 1223,
    instrux + 1224,
    instrux + 1441,
    instrux + 1446,
    instrux + 1447,
    instrux + 1448,
    instrux + 1449,
};

static const struct itemplate * const itable_0F2E[] = {
    instrux + 1264,
    instrux + 1494,
};

static const struct itemplate * const itable_0F2F[] = {
    instrux + 1216,
    instrux + 1437,
};

static const struct itemplate * const itable_0F30[] = {
    instrux + 1115,
};

static const struct itemplate * const itable_0F31[] = {
    instrux + 885,
};

static const struct itemplate * const itable_0F32[] = {
    instrux + 883,
};

static const struct itemplate * const itable_0F33[] = {
    instrux + 884,
};

static const struct itemplate * const itable_0F34[] = {
    instrux + 1077,
};

static const struct itemplate * const itable_0F35[] = {
    instrux + 1078,
};

static const struct itemplate * const itable_0F36[] = {
    instrux + 882,
};

static const struct itemplate * const itable_0F37[] = {
    instrux + 1114,
    instrux + 1643,
};

static const struct itemplate * const itable_0F3800[] = {
    instrux + 1555,
    instrux + 1556,
};

static const struct itemplate * const itable_0F3801[] = {
    instrux + 1539,
    instrux + 1540,
};

static const struct itemplate * const itable_0F3802[] = {
    instrux + 1541,
    instrux + 1542,
};

static const struct itemplate * const itable_0F3803[] = {
    instrux + 1543,
    instrux + 1544,
};

static const struct itemplate * const itable_0F3804[] = {
    instrux + 1551,
    instrux + 1552,
};

static const struct itemplate * const itable_0F3805[] = {
    instrux + 1545,
    instrux + 1546,
};

static const struct itemplate * const itable_0F3806[] = {
    instrux + 1547,
    instrux + 1548,
};

static const struct itemplate * const itable_0F3807[] = {
    instrux + 1549,
    instrux + 1550,
};

static const struct itemplate * const itable_0F3808[] = {
    instrux + 1557,
    instrux + 1558,
};

static const struct itemplate * const itable_0F3809[] = {
    instrux + 1559,
    instrux + 1560,
};

static const struct itemplate * const itable_0F380A[] = {
    instrux + 1561,
    instrux + 1562,
};

static const struct itemplate * const itable_0F380B[] = {
    instrux + 1553,
    instrux + 1554,
};

static const struct itemplate * const itable_0F3810[] = {
    instrux + 1584,
};

static const struct itemplate * const itable_0F3814[] = {
    instrux + 1575,
};

static const struct itemplate * const itable_0F3815[] = {
    instrux + 1574,
};

static const struct itemplate * const itable_0F3817[] = {
    instrux + 1625,
};

static const struct itemplate * const itable_0F381C[] = {
    instrux + 1531,
    instrux + 1532,
};

static const struct itemplate * const itable_0F381D[] = {
    instrux + 1533,
    instrux + 1534,
};

static const struct itemplate * const itable_0F381E[] = {
    instrux + 1535,
    instrux + 1536,
};

static const struct itemplate * const itable_0F3820[] = {
    instrux + 1611,
};

static const struct itemplate * const itable_0F3821[] = {
    instrux + 1612,
};

static const struct itemplate * const itable_0F3822[] = {
    instrux + 1613,
};

static const struct itemplate * const itable_0F3823[] = {
    instrux + 1614,
};

static const struct itemplate * const itable_0F3824[] = {
    instrux + 1615,
};

static const struct itemplate * const itable_0F3825[] = {
    instrux + 1616,
};

static const struct itemplate * const itable_0F3828[] = {
    instrux + 1623,
};

static const struct itemplate * const itable_0F3829[] = {
    instrux + 1586,
};

static const struct itemplate * const itable_0F382A[] = {
    instrux + 1581,
};

static const struct itemplate * const itable_0F382B[] = {
    instrux + 1583,
};

static const struct itemplate * const itable_0F3830[] = {
    instrux + 1617,
};

static const struct itemplate * const itable_0F3831[] = {
    instrux + 1618,
};

static const struct itemplate * const itable_0F3832[] = {
    instrux + 1619,
};

static const struct itemplate * const itable_0F3833[] = {
    instrux + 1620,
};

static const struct itemplate * const itable_0F3834[] = {
    instrux + 1621,
};

static const struct itemplate * const itable_0F3835[] = {
    instrux + 1622,
};

static const struct itemplate * const itable_0F3837[] = {
    instrux + 1639,
};

static const struct itemplate * const itable_0F3838[] = {
    instrux + 1607,
};

static const struct itemplate * const itable_0F3839[] = {
    instrux + 1608,
};

static const struct itemplate * const itable_0F383A[] = {
    instrux + 1610,
};

static const struct itemplate * const itable_0F383B[] = {
    instrux + 1609,
};

static const struct itemplate * const itable_0F383C[] = {
    instrux + 1603,
};

static const struct itemplate * const itable_0F383D[] = {
    instrux + 1604,
};

static const struct itemplate * const itable_0F383E[] = {
    instrux + 1606,
};

static const struct itemplate * const itable_0F383F[] = {
    instrux + 1605,
};

static const struct itemplate * const itable_0F3840[] = {
    instrux + 1624,
};

static const struct itemplate * const itable_0F3841[] = {
    instrux + 1595,
};

static const struct itemplate * const itable_0F3880[] = {
    instrux + 1527,
    instrux + 1528,
};

static const struct itemplate * const itable_0F3881[] = {
    instrux + 1529,
    instrux + 1530,
};

static const struct itemplate * const itable_0F3882[] = {
    instrux + 482,
    instrux + 483,
};

static const struct itemplate * const itable_0F38DB[] = {
    instrux + 1656,
};

static const struct itemplate * const itable_0F38DC[] = {
    instrux + 1652,
};

static const struct itemplate * const itable_0F38DD[] = {
    instrux + 1653,
};

static const struct itemplate * const itable_0F38DE[] = {
    instrux + 1654,
};

static const struct itemplate * const itable_0F38DF[] = {
    instrux + 1655,
};

static const struct itemplate * const itable_0F38F0[] = {
    instrux + 1630,
    instrux + 1633,
    instrux + 1646,
    instrux + 1647,
    instrux + 1648,
};

static const struct itemplate * const itable_0F38F1[] = {
    instrux + 1631,
    instrux + 1632,
    instrux + 1634,
    instrux + 1649,
    instrux + 1650,
    instrux + 1651,
};

static const struct itemplate * const itable_0F38F6[] = {
    instrux + 3070,
    instrux + 3071,
    instrux + 3072,
    instrux + 3073,
};

static const struct itemplate * const itable_0F39[] = {
    instrux + 238,
};

static const struct itemplate * const itable_0F3A08[] = {
    instrux + 1627,
};

static const struct itemplate * const itable_0F3A09[] = {
    instrux + 1626,
};

static const struct itemplate * const itable_0F3A0A[] = {
    instrux + 1629,
};

static const struct itemplate * const itable_0F3A0B[] = {
    instrux + 1628,
};

static const struct itemplate * const itable_0F3A0C[] = {
    instrux + 1573,
};

static const struct itemplate * const itable_0F3A0D[] = {
    instrux + 1572,
};

static const struct itemplate * const itable_0F3A0E[] = {
    instrux + 1585,
};

static const struct itemplate * const itable_0F3A0F[] = {
    instrux + 1537,
    instrux + 1538,
};

static const struct itemplate * const itable_0F3A14[] = {
    instrux + 1587,
    instrux + 1588,
    instrux + 1589,
};

static const struct itemplate * const itable_0F3A15[] = {
    instrux + 1592,
    instrux + 1593,
    instrux + 1594,
};

static const struct itemplate * const itable_0F3A16[] = {
    instrux + 1590,
    instrux + 1591,
};

static const struct itemplate * const itable_0F3A17[] = {
    instrux + 1578,
    instrux + 1579,
};

static const struct itemplate * const itable_0F3A20[] = {
    instrux + 1596,
    instrux + 1597,
    instrux + 1598,
};

static const struct itemplate * const itable_0F3A21[] = {
    instrux + 1580,
};

static const struct itemplate * const itable_0F3A22[] = {
    instrux + 1599,
    instrux + 1600,
    instrux + 1601,
    instrux + 1602,
};

static const struct itemplate * const itable_0F3A40[] = {
    instrux + 1577,
};

static const struct itemplate * const itable_0F3A41[] = {
    instrux + 1576,
};

static const struct itemplate * const itable_0F3A42[] = {
    instrux + 1582,
};

static const struct itemplate * const itable_0F3A44[] = {
    instrux + 2848,
    instrux + 2849,
    instrux + 2850,
    instrux + 2851,
    instrux + 2852,
};

static const struct itemplate * const itable_0F3A60[] = {
    instrux + 1636,
};

static const struct itemplate * const itable_0F3A61[] = {
    instrux + 1635,
};

static const struct itemplate * const itable_0F3A62[] = {
    instrux + 1638,
};

static const struct itemplate * const itable_0F3A63[] = {
    instrux + 1637,
};

static const struct itemplate * const itable_0F3ADF[] = {
    instrux + 1657,
};

static const struct itemplate * const itable_0F3C[] = {
    instrux + 222,
};

static const struct itemplate * const itable_0F3D[] = {
    instrux + 221,
};

static const struct itemplate * const itable_0F40[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F41[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F42[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F43[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F44[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F45[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F46[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F47[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F48[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F49[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F4A[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F4B[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F4C[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F4D[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F4E[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F4F[] = {
    instrux + 1179,
    instrux + 1180,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
};

static const struct itemplate * const itable_0F50[] = {
    instrux + 743,
    instrux + 1243,
    instrux + 1244,
    instrux + 1475,
    instrux + 1476,
};

static const struct itemplate * const itable_0F51[] = {
    instrux + 735,
    instrux + 1259,
    instrux + 1260,
    instrux + 1490,
    instrux + 1491,
};

static const struct itemplate * const itable_0F52[] = {
    instrux + 771,
    instrux + 1256,
    instrux + 1257,
};

static const struct itemplate * const itable_0F53[] = {
    instrux + 1254,
    instrux + 1255,
};

static const struct itemplate * const itable_0F54[] = {
    instrux + 751,
    instrux + 1195,
    instrux + 1418,
};

static const struct itemplate * const itable_0F55[] = {
    instrux + 821,
    instrux + 1194,
    instrux + 1417,
};

static const struct itemplate * const itable_0F56[] = {
    instrux + 1253,
    instrux + 1487,
};

static const struct itemplate * const itable_0F57[] = {
    instrux + 1267,
    instrux + 1497,
};

static const struct itemplate * const itable_0F58[] = {
    instrux + 780,
    instrux + 1192,
    instrux + 1193,
    instrux + 1415,
    instrux + 1416,
};

static const struct itemplate * const itable_0F59[] = {
    instrux + 774,
    instrux + 1251,
    instrux + 1252,
    instrux + 1485,
    instrux + 1486,
};

static const struct itemplate * const itable_0F5A[] = {
    instrux + 779,
    instrux + 1442,
    instrux + 1445,
    instrux + 1450,
    instrux + 1453,
};

static const struct itemplate * const itable_0F5B[] = {
    instrux + 778,
    instrux + 1439,
    instrux + 1444,
    instrux + 1456,
};

static const struct itemplate * const itable_0F5C[] = {
    instrux + 777,
    instrux + 1262,
    instrux + 1263,
    instrux + 1492,
    instrux + 1493,
};

static const struct itemplate * const itable_0F5D[] = {
    instrux + 772,
    instrux + 1233,
    instrux + 1234,
    instrux + 1465,
    instrux + 1466,
};

static const struct itemplate * const itable_0F5E[] = {
    instrux + 769,
    instrux + 1228,
    instrux + 1229,
    instrux + 1461,
    instrux + 1462,
};

static const struct itemplate * const itable_0F5F[] = {
    instrux + 1231,
    instrux + 1232,
    instrux + 1463,
    instrux + 1464,
};

static const struct itemplate * const itable_0F60[] = {
    instrux + 829,
    instrux + 1410,
};

static const struct itemplate * const itable_0F61[] = {
    instrux + 831,
    instrux + 1411,
};

static const struct itemplate * const itable_0F62[] = {
    instrux + 830,
    instrux + 1412,
};

static const struct itemplate * const itable_0F63[] = {
    instrux + 730,
    instrux + 1334,
};

static const struct itemplate * const itable_0F64[] = {
    instrux + 748,
    instrux + 1353,
};

static const struct itemplate * const itable_0F65[] = {
    instrux + 750,
    instrux + 1354,
};

static const struct itemplate * const itable_0F66[] = {
    instrux + 749,
    instrux + 1355,
};

static const struct itemplate * const itable_0F67[] = {
    instrux + 731,
    instrux + 1336,
};

static const struct itemplate * const itable_0F68[] = {
    instrux + 826,
    instrux + 1406,
};

static const struct itemplate * const itable_0F69[] = {
    instrux + 828,
    instrux + 1407,
};

static const struct itemplate * const itable_0F6A[] = {
    instrux + 827,
    instrux + 1408,
};

static const struct itemplate * const itable_0F6B[] = {
    instrux + 729,
    instrux + 1335,
};

static const struct itemplate * const itable_0F6C[] = {
    instrux + 1413,
};

static const struct itemplate * const itable_0F6D[] = {
    instrux + 1409,
};

static const struct itemplate * const itable_0F6E[] = {
    instrux + 650,
    instrux + 654,
    instrux + 1315,
    instrux + 1316,
    instrux + 1331,
};

static const struct itemplate * const itable_0F6F[] = {
    instrux + 652,
    instrux + 1318,
    instrux + 1320,
    instrux + 1322,
    instrux + 1324,
};

static const struct itemplate * const itable_0F70[] = {
    instrux + 1300,
    instrux + 1373,
    instrux + 1374,
    instrux + 1375,
    instrux + 1376,
    instrux + 1377,
    instrux + 1378,
};

static const struct itemplate * const itable_0F71[] = {
    instrux + 807,
    instrux + 811,
    instrux + 817,
    instrux + 1381,
    instrux + 1387,
    instrux + 1392,
};

static const struct itemplate * const itable_0F72[] = {
    instrux + 803,
    instrux + 809,
    instrux + 813,
    instrux + 1383,
    instrux + 1389,
    instrux + 1394,
};

static const struct itemplate * const itable_0F73[] = {
    instrux + 805,
    instrux + 815,
    instrux + 1379,
    instrux + 1385,
    instrux + 1390,
    instrux + 1396,
};

static const struct itemplate * const itable_0F74[] = {
    instrux + 745,
    instrux + 1350,
};

static const struct itemplate * const itable_0F75[] = {
    instrux + 747,
    instrux + 1351,
};

static const struct itemplate * const itable_0F76[] = {
    instrux + 746,
    instrux + 1352,
};

static const struct itemplate * const itable_0F77[] = {
    instrux + 239,
};

static const struct itemplate * const itable_0F78[] = {
    instrux + 1073,
    instrux + 1518,
    instrux + 1519,
    instrux + 1563,
    instrux + 1565,
};

static const struct itemplate * const itable_0F79[] = {
    instrux + 917,
    instrux + 1523,
    instrux + 1524,
    instrux + 1564,
    instrux + 1566,
};

static const struct itemplate * const itable_0F7B[] = {
    instrux + 918,
};

static const struct itemplate * const itable_0F7C[] = {
    instrux + 1074,
    instrux + 1500,
    instrux + 1501,
};

static const struct itemplate * const itable_0F7D[] = {
    instrux + 920,
    instrux + 1502,
    instrux + 1503,
};

static const struct itemplate * const itable_0F7E[] = {
    instrux + 651,
    instrux + 655,
    instrux + 1314,
    instrux + 1317,
    instrux + 1327,
    instrux + 1330,
    instrux + 1332,
};

static const struct itemplate * const itable_0F7F[] = {
    instrux + 653,
    instrux + 1319,
    instrux + 1321,
    instrux + 1323,
    instrux + 1325,
};

static const struct itemplate * const itable_0F80[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F81[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F82[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F83[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F84[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F85[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F86[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F87[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F88[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F89[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F8A[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F8B[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F8C[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F8D[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F8E[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F8F[] = {
    instrux + 1185,
    instrux + 1186,
    instrux + 1187,
    instrux + 1188,
};

static const struct itemplate * const itable_0F90[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F91[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F92[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F93[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F94[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F95[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F96[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F97[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F98[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F99[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F9A[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F9B[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F9C[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F9D[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F9E[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0F9F[] = {
    instrux + 1190,
    instrux + 1191,
};

static const struct itemplate * const itable_0FA0[] = {
    instrux + 842,
};

static const struct itemplate * const itable_0FA1[] = {
    instrux + 790,
};

static const struct itemplate * const itable_0FA2[] = {
    instrux + 220,
};

static const struct itemplate * const itable_0FA3[] = {
    instrux + 114,
    instrux + 115,
    instrux + 116,
    instrux + 117,
    instrux + 118,
    instrux + 119,
};

static const struct itemplate * const itable_0FA4[] = {
    instrux + 983,
    instrux + 984,
    instrux + 985,
    instrux + 986,
    instrux + 987,
    instrux + 988,
};

static const struct itemplate * const itable_0FA5[] = {
    instrux + 989,
    instrux + 990,
    instrux + 991,
    instrux + 992,
    instrux + 993,
    instrux + 994,
};

static const struct itemplate * const itable_0FA6C0[] = {
    instrux + 3085,
};

static const struct itemplate * const itable_0FA6C8[] = {
    instrux + 3086,
};

static const struct itemplate * const itable_0FA6D0[] = {
    instrux + 3087,
};

static const struct itemplate * const itable_0FA7C0[] = {
    instrux + 3079,
};

static const struct itemplate * const itable_0FA7C8[] = {
    instrux + 3080,
};

static const struct itemplate * const itable_0FA7D0[] = {
    instrux + 3081,
};

static const struct itemplate * const itable_0FA7D8[] = {
    instrux + 3082,
};

static const struct itemplate * const itable_0FA7E0[] = {
    instrux + 3083,
};

static const struct itemplate * const itable_0FA7E8[] = {
    instrux + 3084,
};

static const struct itemplate * const itable_0FA8[] = {
    instrux + 843,
};

static const struct itemplate * const itable_0FA9[] = {
    instrux + 791,
};

static const struct itemplate * const itable_0FAA[] = {
    instrux + 919,
};

static const struct itemplate * const itable_0FAB[] = {
    instrux + 141,
    instrux + 142,
    instrux + 143,
    instrux + 144,
    instrux + 145,
    instrux + 146,
};

static const struct itemplate * const itable_0FAC[] = {
    instrux + 1007,
    instrux + 1008,
    instrux + 1009,
    instrux + 1010,
    instrux + 1011,
    instrux + 1012,
};

static const struct itemplate * const itable_0FAD[] = {
    instrux + 1013,
    instrux + 1014,
    instrux + 1015,
    instrux + 1016,
    instrux + 1017,
    instrux + 1018,
};

static const struct itemplate * const itable_0FAE[] = {
    instrux + 539,
    instrux + 596,
    instrux + 969,
    instrux + 1230,
    instrux + 1261,
    instrux + 1268,
    instrux + 1269,
    instrux + 1270,
    instrux + 1271,
    instrux + 1274,
    instrux + 1275,
    instrux + 1276,
    instrux + 1277,
    instrux + 1278,
    instrux + 1279,
    instrux + 1284,
    instrux + 1307,
    instrux + 1312,
    instrux + 1313,
    instrux + 3055,
    instrux + 3056,
    instrux + 3057,
    instrux + 3058,
    instrux + 3062,
    instrux + 3063,
    instrux + 3064,
    instrux + 3065,
};

static const struct itemplate * const itable_0FAF[] = {
    instrux + 438,
    instrux + 439,
    instrux + 440,
    instrux + 441,
    instrux + 442,
    instrux + 443,
};

static const struct itemplate * const itable_0FB0[] = {
    instrux + 210,
    instrux + 211,
};

static const struct itemplate * const itable_0FB1[] = {
    instrux + 212,
    instrux + 213,
    instrux + 214,
    instrux + 215,
    instrux + 216,
    instrux + 217,
};

static const struct itemplate * const itable_0FB2[] = {
    instrux + 590,
    instrux + 591,
    instrux + 592,
};

static const struct itemplate * const itable_0FB3[] = {
    instrux + 132,
    instrux + 133,
    instrux + 134,
    instrux + 135,
    instrux + 136,
    instrux + 137,
};

static const struct itemplate * const itable_0FB4[] = {
    instrux + 540,
    instrux + 541,
    instrux + 542,
};

static const struct itemplate * const itable_0FB5[] = {
    instrux + 544,
    instrux + 545,
    instrux + 546,
};

static const struct itemplate * const itable_0FB6[] = {
    instrux + 667,
    instrux + 668,
    instrux + 669,
    instrux + 671,
};

static const struct itemplate * const itable_0FB7[] = {
    instrux + 670,
    instrux + 672,
};

static const struct itemplate * const itable_0FB8[] = {
    instrux + 515,
    instrux + 516,
    instrux + 517,
    instrux + 1640,
    instrux + 1641,
    instrux + 1642,
};

static const struct itemplate * const itable_0FB9[] = {
    instrux + 1104,
};

static const struct itemplate * const itable_0FBA[] = {
    instrux + 120,
    instrux + 121,
    instrux + 122,
    instrux + 129,
    instrux + 130,
    instrux + 131,
    instrux + 138,
    instrux + 139,
    instrux + 140,
    instrux + 147,
    instrux + 148,
    instrux + 149,
};

static const struct itemplate * const itable_0FBB[] = {
    instrux + 123,
    instrux + 124,
    instrux + 125,
    instrux + 126,
    instrux + 127,
    instrux + 128,
};

static const struct itemplate * const itable_0FBC[] = {
    instrux + 100,
    instrux + 101,
    instrux + 102,
    instrux + 103,
    instrux + 104,
    instrux + 105,
    instrux + 3729,
    instrux + 3730,
    instrux + 3731,
};

static const struct itemplate * const itable_0FBD[] = {
    instrux + 106,
    instrux + 107,
    instrux + 108,
    instrux + 109,
    instrux + 110,
    instrux + 111,
    instrux + 1569,
    instrux + 1570,
    instrux + 1571,
};

static const struct itemplate * const itable_0FBE[] = {
    instrux + 660,
    instrux + 661,
    instrux + 662,
    instrux + 664,
};

static const struct itemplate * const itable_0FBF[] = {
    instrux + 663,
    instrux + 665,
};

static const struct itemplate * const itable_0FC0[] = {
    instrux + 1116,
    instrux + 1117,
};

static const struct itemplate * const itable_0FC1[] = {
    instrux + 1118,
    instrux + 1119,
    instrux + 1120,
    instrux + 1121,
    instrux + 1122,
    instrux + 1123,
};

static const struct itemplate * const itable_0FC2[] = {
    instrux + 1196,
    instrux + 1197,
    instrux + 1198,
    instrux + 1199,
    instrux + 1200,
    instrux + 1201,
    instrux + 1202,
    instrux + 1203,
    instrux + 1204,
    instrux + 1205,
    instrux + 1206,
    instrux + 1207,
    instrux + 1208,
    instrux + 1209,
    instrux + 1210,
    instrux + 1211,
    instrux + 1212,
    instrux + 1213,
    instrux + 1214,
    instrux + 1215,
    instrux + 1419,
    instrux + 1420,
    instrux + 1421,
    instrux + 1422,
    instrux + 1423,
    instrux + 1424,
    instrux + 1425,
    instrux + 1426,
    instrux + 1427,
    instrux + 1428,
    instrux + 1429,
    instrux + 1430,
    instrux + 1431,
    instrux + 1432,
    instrux + 1433,
    instrux + 1434,
    instrux + 1435,
    instrux + 1436,
};

static const struct itemplate * const itable_0FC3[] = {
    instrux + 1309,
    instrux + 1310,
};

static const struct itemplate * const itable_0FC4[] = {
    instrux + 1290,
    instrux + 1291,
    instrux + 1292,
    instrux + 1357,
    instrux + 1358,
    instrux + 1359,
};

static const struct itemplate * const itable_0FC5[] = {
    instrux + 1289,
    instrux + 1356,
};

static const struct itemplate * const itable_0FC6[] = {
    instrux + 1258,
    instrux + 1488,
    instrux + 1489,
};

static const struct itemplate * const itable_0FC7[] = {
    instrux + 218,
    instrux + 219,
    instrux + 1511,
    instrux + 1516,
    instrux + 1517,
    instrux + 1526,
    instrux + 3059,
    instrux + 3060,
    instrux + 3061,
    instrux + 3074,
    instrux + 3075,
    instrux + 3076,
};

static const struct itemplate * const itable_0FC8[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FC9[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FCA[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FCB[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FCC[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FCD[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FCE[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FCF[] = {
    instrux + 112,
    instrux + 113,
};

static const struct itemplate * const itable_0FD0[] = {
    instrux + 1498,
    instrux + 1499,
};

static const struct itemplate * const itable_0FD1[] = {
    instrux + 816,
    instrux + 1391,
};

static const struct itemplate * const itable_0FD2[] = {
    instrux + 812,
    instrux + 1393,
};

static const struct itemplate * const itable_0FD3[] = {
    instrux + 814,
    instrux + 1395,
};

static const struct itemplate * const itable_0FD4[] = {
    instrux + 1340,
    instrux + 1341,
};

static const struct itemplate * const itable_0FD5[] = {
    instrux + 776,
    instrux + 1368,
};

static const struct itemplate * const itable_0FD6[] = {
    instrux + 1326,
    instrux + 1328,
    instrux + 1329,
    instrux + 1333,
};

static const struct itemplate * const itable_0FD7[] = {
    instrux + 1297,
    instrux + 1365,
};

static const struct itemplate * const itable_0FD8[] = {
    instrux + 823,
    instrux + 1404,
};

static const struct itemplate * const itable_0FD9[] = {
    instrux + 824,
    instrux + 1405,
};

static const struct itemplate * const itable_0FDA[] = {
    instrux + 1296,
    instrux + 1364,
};

static const struct itemplate * const itable_0FDB[] = {
    instrux + 740,
    instrux + 1346,
};

static const struct itemplate * const itable_0FDC[] = {
    instrux + 737,
    instrux + 1344,
};

static const struct itemplate * const itable_0FDD[] = {
    instrux + 738,
    instrux + 1345,
};

static const struct itemplate * const itable_0FDE[] = {
    instrux + 1294,
    instrux + 1362,
};

static const struct itemplate * const itable_0FDF[] = {
    instrux + 741,
    instrux + 1347,
};

static const struct itemplate * const itable_0FE0[] = {
    instrux + 1287,
    instrux + 1348,
};

static const struct itemplate * const itable_0FE1[] = {
    instrux + 810,
    instrux + 1386,
};

static const struct itemplate * const itable_0FE2[] = {
    instrux + 808,
    instrux + 1388,
};

static const struct itemplate * const itable_0FE3[] = {
    instrux + 1288,
    instrux + 1349,
};

static const struct itemplate * const itable_0FE4[] = {
    instrux + 1298,
    instrux + 1366,
};

static const struct itemplate * const itable_0FE5[] = {
    instrux + 775,
    instrux + 1367,
};

static const struct itemplate * const itable_0FE6[] = {
    instrux + 1438,
    instrux + 1440,
    instrux + 1455,
};

static const struct itemplate * const itable_0FE7[] = {
    instrux + 1286,
    instrux + 1308,
};

static const struct itemplate * const itable_0FE8[] = {
    instrux + 820,
    instrux + 1402,
};

static const struct itemplate * const itable_0FE9[] = {
    instrux + 822,
    instrux + 1403,
};

static const struct itemplate * const itable_0FEA[] = {
    instrux + 1295,
    instrux + 1363,
};

static const struct itemplate * const itable_0FEB[] = {
    instrux + 799,
    instrux + 1371,
};

static const struct itemplate * const itable_0FEC[] = {
    instrux + 734,
    instrux + 1342,
};

static const struct itemplate * const itable_0FED[] = {
    instrux + 736,
    instrux + 1343,
};

static const struct itemplate * const itable_0FEE[] = {
    instrux + 1293,
    instrux + 1361,
};

static const struct itemplate * const itable_0FEF[] = {
    instrux + 857,
    instrux + 1414,
};

static const struct itemplate * const itable_0FF0[] = {
    instrux + 1504,
};

static const struct itemplate * const itable_0FF1[] = {
    instrux + 806,
    instrux + 1380,
};

static const struct itemplate * const itable_0FF2[] = {
    instrux + 802,
    instrux + 1382,
};

static const struct itemplate * const itable_0FF3[] = {
    instrux + 804,
    instrux + 1384,
};

static const struct itemplate * const itable_0FF4[] = {
    instrux + 1369,
    instrux + 1370,
};

static const struct itemplate * const itable_0FF5[] = {
    instrux + 770,
    instrux + 1360,
};

static const struct itemplate * const itable_0FF6[] = {
    instrux + 1299,
    instrux + 1372,
};

static const struct itemplate * const itable_0FF7[] = {
    instrux + 1285,
    instrux + 1306,
};

static const struct itemplate * const itable_0FF8[] = {
    instrux + 818,
    instrux + 1397,
};

static const struct itemplate * const itable_0FF9[] = {
    instrux + 825,
    instrux + 1398,
};

static const struct itemplate * const itable_0FFA[] = {
    instrux + 819,
    instrux + 1399,
};

static const struct itemplate * const itable_0FFB[] = {
    instrux + 1400,
    instrux + 1401,
};

static const struct itemplate * const itable_0FFC[] = {
    instrux + 732,
    instrux + 1337,
};

static const struct itemplate * const itable_0FFD[] = {
    instrux + 739,
    instrux + 1338,
};

static const struct itemplate * const itable_0FFE[] = {
    instrux + 733,
    instrux + 1339,
};

static const struct itemplate * const itable_0FFF[] = {
    instrux + 1103,
};

static const struct itemplate * const itable_10[] = {
    instrux + 7,
    instrux + 8,
};

static const struct itemplate * const itable_11[] = {
    instrux + 9,
    instrux + 10,
    instrux + 11,
    instrux + 12,
    instrux + 13,
    instrux + 14,
};

static const struct itemplate * const itable_12[] = {
    instrux + 15,
    instrux + 16,
};

static const struct itemplate * const itable_13[] = {
    instrux + 17,
    instrux + 18,
    instrux + 19,
    instrux + 20,
    instrux + 21,
    instrux + 22,
};

static const struct itemplate * const itable_14[] = {
    instrux + 26,
};

static const struct itemplate * const itable_15[] = {
    instrux + 27,
    instrux + 28,
    instrux + 29,
};

static const struct itemplate * const itable_16[] = {
    instrux + 840,
};

static const struct itemplate * const itable_17[] = {
    instrux + 788,
};

static const struct itemplate * const itable_18[] = {
    instrux + 935,
    instrux + 936,
};

static const struct itemplate * const itable_19[] = {
    instrux + 937,
    instrux + 938,
    instrux + 939,
    instrux + 940,
    instrux + 941,
    instrux + 942,
};

static const struct itemplate * const itable_1A[] = {
    instrux + 943,
    instrux + 944,
};

static const struct itemplate * const itable_1B[] = {
    instrux + 945,
    instrux + 946,
    instrux + 947,
    instrux + 948,
    instrux + 949,
    instrux + 950,
};

static const struct itemplate * const itable_1C[] = {
    instrux + 954,
};

static const struct itemplate * const itable_1D[] = {
    instrux + 955,
    instrux + 956,
    instrux + 957,
};

static const struct itemplate * const itable_1E[] = {
    instrux + 841,
};

static const struct itemplate * const itable_1F[] = {
    instrux + 789,
};

static const struct itemplate * const itable_20[] = {
    instrux + 66,
    instrux + 67,
};

static const struct itemplate * const itable_21[] = {
    instrux + 68,
    instrux + 69,
    instrux + 70,
    instrux + 71,
    instrux + 72,
    instrux + 73,
};

static const struct itemplate * const itable_22[] = {
    instrux + 74,
    instrux + 75,
};

static const struct itemplate * const itable_23[] = {
    instrux + 76,
    instrux + 77,
    instrux + 78,
    instrux + 79,
    instrux + 80,
    instrux + 81,
};

static const struct itemplate * const itable_24[] = {
    instrux + 85,
};

static const struct itemplate * const itable_25[] = {
    instrux + 86,
    instrux + 87,
    instrux + 88,
};

static const struct itemplate * const itable_27[] = {
    instrux + 226,
};

static const struct itemplate * const itable_28[] = {
    instrux + 1043,
    instrux + 1044,
};

static const struct itemplate * const itable_29[] = {
    instrux + 1045,
    instrux + 1046,
    instrux + 1047,
    instrux + 1048,
    instrux + 1049,
    instrux + 1050,
};

static const struct itemplate * const itable_2A[] = {
    instrux + 1051,
    instrux + 1052,
};

static const struct itemplate * const itable_2B[] = {
    instrux + 1053,
    instrux + 1054,
    instrux + 1055,
    instrux + 1056,
    instrux + 1057,
    instrux + 1058,
};

static const struct itemplate * const itable_2C[] = {
    instrux + 1062,
};

static const struct itemplate * const itable_2D[] = {
    instrux + 1063,
    instrux + 1064,
    instrux + 1065,
};

static const struct itemplate * const itable_2F[] = {
    instrux + 227,
};

static const struct itemplate * const itable_30[] = {
    instrux + 1149,
    instrux + 1150,
};

static const struct itemplate * const itable_31[] = {
    instrux + 1151,
    instrux + 1152,
    instrux + 1153,
    instrux + 1154,
    instrux + 1155,
    instrux + 1156,
};

static const struct itemplate * const itable_32[] = {
    instrux + 1157,
    instrux + 1158,
};

static const struct itemplate * const itable_33[] = {
    instrux + 1159,
    instrux + 1160,
    instrux + 1161,
    instrux + 1162,
    instrux + 1163,
    instrux + 1164,
};

static const struct itemplate * const itable_34[] = {
    instrux + 1168,
};

static const struct itemplate * const itable_35[] = {
    instrux + 1169,
    instrux + 1170,
    instrux + 1171,
};

static const struct itemplate * const itable_37[] = {
    instrux + 1,
};

static const struct itemplate * const itable_38[] = {
    instrux + 176,
    instrux + 177,
};

static const struct itemplate * const itable_39[] = {
    instrux + 178,
    instrux + 179,
    instrux + 180,
    instrux + 181,
    instrux + 182,
    instrux + 183,
};

static const struct itemplate * const itable_3A[] = {
    instrux + 184,
    instrux + 185,
};

static const struct itemplate * const itable_3B[] = {
    instrux + 186,
    instrux + 187,
    instrux + 188,
    instrux + 189,
    instrux + 190,
    instrux + 191,
};

static const struct itemplate * const itable_3C[] = {
    instrux + 195,
};

static const struct itemplate * const itable_3D[] = {
    instrux + 196,
    instrux + 197,
    instrux + 198,
};

static const struct itemplate * const itable_3F[] = {
    instrux + 6,
};

static const struct itemplate * const itable_40[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_41[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_42[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_43[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_44[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_45[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_46[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_47[] = {
    instrux + 468,
    instrux + 469,
};

static const struct itemplate * const itable_48[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_49[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_4A[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_4B[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_4C[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_4D[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_4E[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_4F[] = {
    instrux + 228,
    instrux + 229,
};

static const struct itemplate * const itable_50[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_51[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_52[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_53[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_54[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_55[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_56[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_57[] = {
    instrux + 832,
    instrux + 833,
    instrux + 834,
};

static const struct itemplate * const itable_58[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_59[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_5A[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_5B[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_5C[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_5D[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_5E[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_5F[] = {
    instrux + 781,
    instrux + 782,
    instrux + 783,
};

static const struct itemplate * const itable_60[] = {
    instrux + 850,
    instrux + 851,
    instrux + 852,
};

static const struct itemplate * const itable_61[] = {
    instrux + 792,
    instrux + 793,
    instrux + 794,
};

static const struct itemplate * const itable_62[] = {
    instrux + 98,
    instrux + 99,
};

static const struct itemplate * const itable_63[] = {
    instrux + 96,
    instrux + 97,
    instrux + 666,
};

static const struct itemplate * const itable_68[] = {
    instrux + 845,
    instrux + 846,
    instrux + 847,
    instrux + 848,
    instrux + 849,
};

static const struct itemplate * const itable_69[] = {
    instrux + 445,
    instrux + 447,
    instrux + 449,
    instrux + 451,
    instrux + 453,
    instrux + 455,
    instrux + 457,
    instrux + 459,
    instrux + 461,
};

static const struct itemplate * const itable_6A[] = {
    instrux + 844,
};

static const struct itemplate * const itable_6B[] = {
    instrux + 444,
    instrux + 446,
    instrux + 448,
    instrux + 450,
    instrux + 452,
    instrux + 454,
    instrux + 456,
    instrux + 458,
    instrux + 460,
};

static const struct itemplate * const itable_6C[] = {
    instrux + 474,
};

static const struct itemplate * const itable_6D[] = {
    instrux + 475,
    instrux + 476,
};

static const struct itemplate * const itable_6E[] = {
    instrux + 726,
};

static const struct itemplate * const itable_6F[] = {
    instrux + 727,
    instrux + 728,
};

static const struct itemplate * const itable_70[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_71[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_72[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_73[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_74[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_75[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_76[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_77[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_78[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_79[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_7A[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_7B[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_7C[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_7D[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_7E[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_7F[] = {
    instrux + 1189,
};

static const struct itemplate * const itable_80[] = {
    instrux + 30,
    instrux + 59,
    instrux + 63,
    instrux + 89,
    instrux + 93,
    instrux + 199,
    instrux + 203,
    instrux + 713,
    instrux + 717,
    instrux + 958,
    instrux + 962,
    instrux + 1066,
    instrux + 1070,
    instrux + 1172,
    instrux + 1176,
};

static const struct itemplate * const itable_81[] = {
    instrux + 31,
    instrux + 32,
    instrux + 33,
    instrux + 34,
    instrux + 35,
    instrux + 60,
    instrux + 61,
    instrux + 62,
    instrux + 64,
    instrux + 65,
    instrux + 90,
    instrux + 91,
    instrux + 92,
    instrux + 94,
    instrux + 95,
    instrux + 200,
    instrux + 201,
    instrux + 202,
    instrux + 204,
    instrux + 205,
    instrux + 714,
    instrux + 715,
    instrux + 716,
    instrux + 718,
    instrux + 719,
    instrux + 959,
    instrux + 960,
    instrux + 961,
    instrux + 963,
    instrux + 964,
    instrux + 1067,
    instrux + 1068,
    instrux + 1069,
    instrux + 1071,
    instrux + 1072,
    instrux + 1173,
    instrux + 1174,
    instrux + 1175,
    instrux + 1177,
    instrux + 1178,
};

static const struct itemplate * const itable_83[] = {
    instrux + 23,
    instrux + 24,
    instrux + 25,
    instrux + 52,
    instrux + 53,
    instrux + 54,
    instrux + 82,
    instrux + 83,
    instrux + 84,
    instrux + 192,
    instrux + 193,
    instrux + 194,
    instrux + 706,
    instrux + 707,
    instrux + 708,
    instrux + 951,
    instrux + 952,
    instrux + 953,
    instrux + 1059,
    instrux + 1060,
    instrux + 1061,
    instrux + 1165,
    instrux + 1166,
    instrux + 1167,
};

static const struct itemplate * const itable_84[] = {
    instrux + 1080,
    instrux + 1081,
    instrux + 1088,
};

static const struct itemplate * const itable_85[] = {
    instrux + 1082,
    instrux + 1083,
    instrux + 1084,
    instrux + 1085,
    instrux + 1086,
    instrux + 1087,
    instrux + 1089,
    instrux + 1090,
    instrux + 1091,
};

static const struct itemplate * const itable_86[] = {
    instrux + 1131,
    instrux + 1132,
    instrux + 1139,
    instrux + 1140,
};

static const struct itemplate * const itable_87[] = {
    instrux + 1133,
    instrux + 1134,
    instrux + 1135,
    instrux + 1136,
    instrux + 1137,
    instrux + 1138,
    instrux + 1141,
    instrux + 1142,
    instrux + 1143,
    instrux + 1144,
    instrux + 1145,
    instrux + 1146,
};

static const struct itemplate * const itable_88[] = {
    instrux + 622,
    instrux + 623,
};

static const struct itemplate * const itable_89[] = {
    instrux + 624,
    instrux + 625,
    instrux + 626,
    instrux + 627,
    instrux + 628,
    instrux + 629,
};

static const struct itemplate * const itable_8A[] = {
    instrux + 630,
    instrux + 631,
};

static const struct itemplate * const itable_8B[] = {
    instrux + 632,
    instrux + 633,
    instrux + 634,
    instrux + 635,
    instrux + 636,
    instrux + 637,
};

static const struct itemplate * const itable_8C[] = {
    instrux + 598,
    instrux + 599,
    instrux + 600,
    instrux + 601,
};

static const struct itemplate * const itable_8D[] = {
    instrux + 533,
    instrux + 534,
    instrux + 535,
};

static const struct itemplate * const itable_8E[] = {
    instrux + 602,
    instrux + 603,
    instrux + 604,
    instrux + 605,
};

static const struct itemplate * const itable_8F[] = {
    instrux + 784,
    instrux + 785,
    instrux + 786,
};

static const struct itemplate * const itable_90[] = {
    instrux + 682,
    instrux + 742,
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
    instrux + 1130,
};

static const struct itemplate * const itable_91[] = {
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
};

static const struct itemplate * const itable_92[] = {
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
};

static const struct itemplate * const itable_93[] = {
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
};

static const struct itemplate * const itable_94[] = {
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
};

static const struct itemplate * const itable_95[] = {
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
};

static const struct itemplate * const itable_96[] = {
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
};

static const struct itemplate * const itable_97[] = {
    instrux + 1124,
    instrux + 1125,
    instrux + 1126,
    instrux + 1127,
    instrux + 1128,
    instrux + 1129,
};

static const struct itemplate * const itable_98[] = {
    instrux + 168,
    instrux + 170,
    instrux + 225,
};

static const struct itemplate * const itable_99[] = {
    instrux + 169,
    instrux + 223,
    instrux + 224,
};

static const struct itemplate * const itable_9A[] = {
    instrux + 154,
    instrux + 155,
    instrux + 156,
    instrux + 157,
    instrux + 158,
};

static const struct itemplate * const itable_9C[] = {
    instrux + 853,
    instrux + 854,
    instrux + 855,
    instrux + 856,
};

static const struct itemplate * const itable_9D[] = {
    instrux + 795,
    instrux + 796,
    instrux + 797,
    instrux + 798,
};

static const struct itemplate * const itable_9E[] = {
    instrux + 921,
};

static const struct itemplate * const itable_9F[] = {
    instrux + 520,
};

static const struct itemplate * const itable_A0[] = {
    instrux + 606,
};

static const struct itemplate * const itable_A1[] = {
    instrux + 607,
    instrux + 608,
    instrux + 609,
};

static const struct itemplate * const itable_A2[] = {
    instrux + 610,
};

static const struct itemplate * const itable_A3[] = {
    instrux + 611,
    instrux + 612,
    instrux + 613,
};

static const struct itemplate * const itable_A4[] = {
    instrux + 656,
};

static const struct itemplate * const itable_A5[] = {
    instrux + 657,
    instrux + 658,
    instrux + 659,
};

static const struct itemplate * const itable_A6[] = {
    instrux + 206,
};

static const struct itemplate * const itable_A7[] = {
    instrux + 207,
    instrux + 208,
    instrux + 209,
};

static const struct itemplate * const itable_A8[] = {
    instrux + 1092,
};

static const struct itemplate * const itable_A9[] = {
    instrux + 1093,
    instrux + 1094,
    instrux + 1095,
};

static const struct itemplate * const itable_AA[] = {
    instrux + 1034,
};

static const struct itemplate * const itable_AB[] = {
    instrux + 1035,
    instrux + 1036,
    instrux + 1037,
};

static const struct itemplate * const itable_AC[] = {
    instrux + 556,
};

static const struct itemplate * const itable_AD[] = {
    instrux + 557,
    instrux + 558,
    instrux + 559,
};

static const struct itemplate * const itable_AE[] = {
    instrux + 965,
};

static const struct itemplate * const itable_AF[] = {
    instrux + 966,
    instrux + 967,
    instrux + 968,
};

static const struct itemplate * const itable_B0[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B1[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B2[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B3[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B4[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B5[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B6[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B7[] = {
    instrux + 638,
};

static const struct itemplate * const itable_B8[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_B9[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_BA[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_BB[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_BC[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_BD[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_BE[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_BF[] = {
    instrux + 639,
    instrux + 640,
    instrux + 641,
};

static const struct itemplate * const itable_C0[] = {
    instrux + 860,
    instrux + 872,
    instrux + 895,
    instrux + 907,
    instrux + 925,
    instrux + 973,
    instrux + 997,
};

static const struct itemplate * const itable_C1[] = {
    instrux + 863,
    instrux + 866,
    instrux + 869,
    instrux + 875,
    instrux + 878,
    instrux + 881,
    instrux + 898,
    instrux + 901,
    instrux + 904,
    instrux + 910,
    instrux + 913,
    instrux + 916,
    instrux + 928,
    instrux + 931,
    instrux + 934,
    instrux + 976,
    instrux + 979,
    instrux + 982,
    instrux + 1000,
    instrux + 1003,
    instrux + 1006,
};

static const struct itemplate * const itable_C2[] = {
    instrux + 888,
    instrux + 892,
};

static const struct itemplate * const itable_C3[] = {
    instrux + 887,
    instrux + 891,
};

static const struct itemplate * const itable_C4[] = {
    instrux + 537,
    instrux + 538,
};

static const struct itemplate * const itable_C5[] = {
    instrux + 531,
    instrux + 532,
};

static const struct itemplate * const itable_C6[] = {
    instrux + 642,
    instrux + 647,
    instrux + 3679,
    instrux + 3680,
};

static const struct itemplate * const itable_C7[] = {
    instrux + 643,
    instrux + 644,
    instrux + 645,
    instrux + 646,
    instrux + 648,
    instrux + 649,
    instrux + 3681,
    instrux + 3682,
    instrux + 3683,
    instrux + 3684,
};

static const struct itemplate * const itable_C8[] = {
    instrux + 240,
};

static const struct itemplate * const itable_C9[] = {
    instrux + 536,
};

static const struct itemplate * const itable_CA[] = {
    instrux + 890,
};

static const struct itemplate * const itable_CB[] = {
    instrux + 889,
};

static const struct itemplate * const itable_CC[] = {
    instrux + 479,
};

static const struct itemplate * const itable_CD[] = {
    instrux + 477,
};

static const struct itemplate * const itable_CE[] = {
    instrux + 480,
};

static const struct itemplate * const itable_CF[] = {
    instrux + 489,
    instrux + 490,
    instrux + 491,
    instrux + 492,
};

static const struct itemplate * const itable_D0[] = {
    instrux + 858,
    instrux + 870,
    instrux + 893,
    instrux + 905,
    instrux + 923,
    instrux + 971,
    instrux + 995,
};

static const struct itemplate * const itable_D1[] = {
    instrux + 861,
    instrux + 864,
    instrux + 867,
    instrux + 873,
    instrux + 876,
    instrux + 879,
    instrux + 896,
    instrux + 899,
    instrux + 902,
    instrux + 908,
    instrux + 911,
    instrux + 914,
    instrux + 926,
    instrux + 929,
    instrux + 932,
    instrux + 974,
    instrux + 977,
    instrux + 980,
    instrux + 998,
    instrux + 1001,
    instrux + 1004,
};

static const struct itemplate * const itable_D2[] = {
    instrux + 859,
    instrux + 871,
    instrux + 894,
    instrux + 906,
    instrux + 924,
    instrux + 972,
    instrux + 996,
};

static const struct itemplate * const itable_D3[] = {
    instrux + 862,
    instrux + 865,
    instrux + 868,
    instrux + 874,
    instrux + 877,
    instrux + 880,
    instrux + 897,
    instrux + 900,
    instrux + 903,
    instrux + 909,
    instrux + 912,
    instrux + 915,
    instrux + 927,
    instrux + 930,
    instrux + 933,
    instrux + 975,
    instrux + 978,
    instrux + 981,
    instrux + 999,
    instrux + 1002,
    instrux + 1005,
};

static const struct itemplate * const itable_D4[] = {
    instrux + 4,
    instrux + 5,
};

static const struct itemplate * const itable_D5[] = {
    instrux + 2,
    instrux + 3,
};

static const struct itemplate * const itable_D6[] = {
    instrux + 922,
};

static const struct itemplate * const itable_D7[] = {
    instrux + 1147,
    instrux + 1148,
};

static const struct itemplate * const itable_D8[] = {
    instrux + 245,
    instrux + 248,
    instrux + 250,
    instrux + 275,
    instrux + 277,
    instrux + 278,
    instrux + 283,
    instrux + 285,
    instrux + 286,
    instrux + 291,
    instrux + 294,
    instrux + 296,
    instrux + 299,
    instrux + 303,
    instrux + 304,
    instrux + 355,
    instrux + 359,
    instrux + 360,
    instrux + 396,
    instrux + 400,
    instrux + 401,
    instrux + 404,
    instrux + 408,
    instrux + 409,
};

static const struct itemplate * const itable_D9[] = {
    instrux + 243,
    instrux + 244,
    instrux + 257,
    instrux + 288,
    instrux + 289,
    instrux + 328,
    instrux + 342,
    instrux + 345,
    instrux + 346,
    instrux + 347,
    instrux + 348,
    instrux + 349,
    instrux + 350,
    instrux + 351,
    instrux + 352,
    instrux + 353,
    instrux + 354,
    instrux + 367,
    instrux + 369,
    instrux + 370,
    instrux + 373,
    instrux + 374,
    instrux + 375,
    instrux + 376,
    instrux + 377,
    instrux + 380,
    instrux + 382,
    instrux + 383,
    instrux + 384,
    instrux + 385,
    instrux + 388,
    instrux + 389,
    instrux + 390,
    instrux + 412,
    instrux + 422,
    instrux + 423,
    instrux + 424,
    instrux + 425,
    instrux + 426,
    instrux + 427,
    instrux + 428,
};

static const struct itemplate * const itable_DA[] = {
    instrux + 259,
    instrux + 260,
    instrux + 261,
    instrux + 262,
    instrux + 263,
    instrux + 264,
    instrux + 273,
    instrux + 274,
    instrux + 313,
    instrux + 315,
    instrux + 317,
    instrux + 319,
    instrux + 321,
    instrux + 326,
    instrux + 338,
    instrux + 340,
    instrux + 421,
};

static const struct itemplate * const itable_DB[] = {
    instrux + 258,
    instrux + 265,
    instrux + 266,
    instrux + 267,
    instrux + 268,
    instrux + 269,
    instrux + 270,
    instrux + 271,
    instrux + 272,
    instrux + 279,
    instrux + 280,
    instrux + 290,
    instrux + 308,
    instrux + 323,
    instrux + 329,
    instrux + 330,
    instrux + 332,
    instrux + 336,
    instrux + 344,
    instrux + 363,
    instrux + 364,
    instrux + 365,
    instrux + 366,
    instrux + 381,
    instrux + 392,
    instrux + 415,
    instrux + 416,
};

static const struct itemplate * const itable_DC[] = {
    instrux + 246,
    instrux + 247,
    instrux + 249,
    instrux + 276,
    instrux + 284,
    instrux + 292,
    instrux + 293,
    instrux + 295,
    instrux + 300,
    instrux + 301,
    instrux + 302,
    instrux + 356,
    instrux + 357,
    instrux + 358,
    instrux + 397,
    instrux + 398,
    instrux + 399,
    instrux + 405,
    instrux + 406,
    instrux + 407,
};

static const struct itemplate * const itable_DD[] = {
    instrux + 309,
    instrux + 310,
    instrux + 337,
    instrux + 343,
    instrux + 368,
    instrux + 371,
    instrux + 378,
    instrux + 379,
    instrux + 386,
    instrux + 387,
    instrux + 391,
    instrux + 393,
    instrux + 394,
    instrux + 413,
    instrux + 414,
    instrux + 419,
    instrux + 420,
};

static const struct itemplate * const itable_DE[] = {
    instrux + 251,
    instrux + 252,
    instrux + 287,
    instrux + 297,
    instrux + 298,
    instrux + 305,
    instrux + 306,
    instrux + 314,
    instrux + 316,
    instrux + 318,
    instrux + 320,
    instrux + 322,
    instrux + 327,
    instrux + 339,
    instrux + 341,
    instrux + 361,
    instrux + 362,
    instrux + 402,
    instrux + 403,
    instrux + 410,
    instrux + 411,
};

static const struct itemplate * const itable_DF[] = {
    instrux + 253,
    instrux + 254,
    instrux + 255,
    instrux + 256,
    instrux + 281,
    instrux + 282,
    instrux + 311,
    instrux + 312,
    instrux + 324,
    instrux + 325,
    instrux + 331,
    instrux + 333,
    instrux + 334,
    instrux + 335,
    instrux + 372,
    instrux + 395,
    instrux + 417,
    instrux + 418,
};

static const struct itemplate * const itable_E0[] = {
    instrux + 568,
    instrux + 569,
    instrux + 570,
    instrux + 571,
    instrux + 572,
    instrux + 573,
    instrux + 574,
    instrux + 575,
};

static const struct itemplate * const itable_E1[] = {
    instrux + 564,
    instrux + 565,
    instrux + 566,
    instrux + 567,
    instrux + 576,
    instrux + 577,
    instrux + 578,
    instrux + 579,
};

static const struct itemplate * const itable_E2[] = {
    instrux + 560,
    instrux + 561,
    instrux + 562,
    instrux + 563,
};

static const struct itemplate * const itable_E3[] = {
    instrux + 493,
    instrux + 494,
    instrux + 495,
};

static const struct itemplate * const itable_E4[] = {
    instrux + 462,
};

static const struct itemplate * const itable_E5[] = {
    instrux + 463,
    instrux + 464,
};

static const struct itemplate * const itable_E6[] = {
    instrux + 720,
};

static const struct itemplate * const itable_E7[] = {
    instrux + 721,
    instrux + 722,
};

static const struct itemplate * const itable_E8[] = {
    instrux + 150,
    instrux + 151,
    instrux + 152,
    instrux + 153,
};

static const struct itemplate * const itable_E9[] = {
    instrux + 497,
    instrux + 498,
    instrux + 499,
    instrux + 500,
};

static const struct itemplate * const itable_EA[] = {
    instrux + 501,
    instrux + 502,
    instrux + 503,
    instrux + 504,
    instrux + 505,
};

static const struct itemplate * const itable_EB[] = {
    instrux + 496,
};

static const struct itemplate * const itable_EC[] = {
    instrux + 465,
};

static const struct itemplate * const itable_ED[] = {
    instrux + 466,
    instrux + 467,
};

static const struct itemplate * const itable_EE[] = {
    instrux + 723,
};

static const struct itemplate * const itable_EF[] = {
    instrux + 724,
    instrux + 725,
};

static const struct itemplate * const itable_F1[] = {
    instrux + 478,
    instrux + 1026,
};

static const struct itemplate * const itable_F4[] = {
    instrux + 429,
};

static const struct itemplate * const itable_F5[] = {
    instrux + 175,
};

static const struct itemplate * const itable_F6[] = {
    instrux + 234,
    instrux + 430,
    instrux + 434,
    instrux + 673,
    instrux + 678,
    instrux + 686,
    instrux + 1096,
    instrux + 1100,
};

static const struct itemplate * const itable_F7[] = {
    instrux + 235,
    instrux + 236,
    instrux + 237,
    instrux + 431,
    instrux + 432,
    instrux + 433,
    instrux + 435,
    instrux + 436,
    instrux + 437,
    instrux + 674,
    instrux + 675,
    instrux + 676,
    instrux + 679,
    instrux + 680,
    instrux + 681,
    instrux + 687,
    instrux + 688,
    instrux + 689,
    instrux + 1097,
    instrux + 1098,
    instrux + 1099,
    instrux + 1101,
    instrux + 1102,
};

static const struct itemplate * const itable_F8[] = {
    instrux + 171,
};

static const struct itemplate * const itable_F9[] = {
    instrux + 1031,
};

static const struct itemplate * const itable_FA[] = {
    instrux + 173,
};

static const struct itemplate * const itable_FB[] = {
    instrux + 1033,
};

static const struct itemplate * const itable_FC[] = {
    instrux + 172,
};

static const struct itemplate * const itable_FD[] = {
    instrux + 1032,
};

static const struct itemplate * const itable_FE[] = {
    instrux + 230,
    instrux + 470,
};

static const struct itemplate * const itable_FF[] = {
    instrux + 159,
    instrux + 160,
    instrux + 161,
    instrux + 162,
    instrux + 163,
    instrux + 164,
    instrux + 165,
    instrux + 166,
    instrux + 167,
    instrux + 231,
    instrux + 232,
    instrux + 233,
    instrux + 471,
    instrux + 472,
    instrux + 473,
    instrux + 506,
    instrux + 507,
    instrux + 508,
    instrux + 509,
    instrux + 510,
    instrux + 511,
    instrux + 512,
    instrux + 513,
    instrux + 514,
    instrux + 835,
    instrux + 836,
    instrux + 837,
};

static const struct itemplate * const itable_vex01010[] = {
    instrux + 2488,
    instrux + 2490,
};

static const struct itemplate * const itable_vex01011[] = {
    instrux + 2489,
    instrux + 2491,
};

static const struct itemplate * const itable_vex01012[] = {
    instrux + 2436,
    instrux + 2437,
    instrux + 2449,
    instrux + 2450,
};

static const struct itemplate * const itable_vex01013[] = {
    instrux + 2451,
};

static const struct itemplate * const itable_vex01014[] = {
    instrux + 2834,
    instrux + 2835,
    instrux + 2836,
    instrux + 2837,
};

static const struct itemplate * const itable_vex01015[] = {
    instrux + 2826,
    instrux + 2827,
    instrux + 2828,
    instrux + 2829,
};

static const struct itemplate * const itable_vex01016[] = {
    instrux + 2441,
    instrux + 2442,
    instrux + 2444,
    instrux + 2445,
};

static const struct itemplate * const itable_vex01017[] = {
    instrux + 2443,
};

static const struct itemplate * const itable_vex01028[] = {
    instrux + 2412,
    instrux + 2414,
};

static const struct itemplate * const itable_vex01029[] = {
    instrux + 2413,
    instrux + 2415,
};

static const struct itemplate * const itable_vex0102B[] = {
    instrux + 2466,
    instrux + 2467,
};

static const struct itemplate * const itable_vex0102E[] = {
    instrux + 2821,
};

static const struct itemplate * const itable_vex0102F[] = {
    instrux + 2289,
};

static const struct itemplate * const itable_vex01050[] = {
    instrux + 2456,
    instrux + 2457,
    instrux + 2458,
    instrux + 2459,
};

static const struct itemplate * const itable_vex01051[] = {
    instrux + 2797,
    instrux + 2798,
};

static const struct itemplate * const itable_vex01052[] = {
    instrux + 2775,
    instrux + 2776,
};

static const struct itemplate * const itable_vex01053[] = {
    instrux + 2771,
    instrux + 2772,
};

static const struct itemplate * const itable_vex01054[] = {
    instrux + 1692,
    instrux + 1693,
    instrux + 1694,
    instrux + 1695,
};

static const struct itemplate * const itable_vex01055[] = {
    instrux + 1700,
    instrux + 1701,
    instrux + 1702,
    instrux + 1703,
};

static const struct itemplate * const itable_vex01056[] = {
    instrux + 2510,
    instrux + 2511,
    instrux + 2512,
    instrux + 2513,
};

static const struct itemplate * const itable_vex01057[] = {
    instrux + 2842,
    instrux + 2843,
    instrux + 2844,
    instrux + 2845,
};

static const struct itemplate * const itable_vex01058[] = {
    instrux + 1672,
    instrux + 1673,
    instrux + 1674,
    instrux + 1675,
};

static const struct itemplate * const itable_vex01059[] = {
    instrux + 2498,
    instrux + 2499,
    instrux + 2500,
    instrux + 2501,
};

static const struct itemplate * const itable_vex0105A[] = {
    instrux + 2304,
    instrux + 2305,
};

static const struct itemplate * const itable_vex0105B[] = {
    instrux + 2292,
    instrux + 2293,
};

static const struct itemplate * const itable_vex0105C[] = {
    instrux + 2808,
    instrux + 2809,
    instrux + 2810,
    instrux + 2811,
};

static const struct itemplate * const itable_vex0105D[] = {
    instrux + 2400,
    instrux + 2401,
    instrux + 2402,
    instrux + 2403,
};

static const struct itemplate * const itable_vex0105E[] = {
    instrux + 2336,
    instrux + 2337,
    instrux + 2338,
    instrux + 2339,
};

static const struct itemplate * const itable_vex0105F[] = {
    instrux + 2388,
    instrux + 2389,
    instrux + 2390,
    instrux + 2391,
};

static const struct itemplate * const itable_vex01077[] = {
    instrux + 2846,
    instrux + 2847,
};

static const struct itemplate * const itable_vex010AE[] = {
    instrux + 2374,
    instrux + 2803,
};

static const struct itemplate * const itable_vex010C2[] = {
    instrux + 1912,
    instrux + 1913,
    instrux + 1914,
    instrux + 1915,
    instrux + 1916,
    instrux + 1917,
    instrux + 1918,
    instrux + 1919,
    instrux + 1920,
    instrux + 1921,
    instrux + 1922,
    instrux + 1923,
    instrux + 1924,
    instrux + 1925,
    instrux + 1926,
    instrux + 1927,
    instrux + 1928,
    instrux + 1929,
    instrux + 1930,
    instrux + 1931,
    instrux + 1932,
    instrux + 1933,
    instrux + 1934,
    instrux + 1935,
    instrux + 1936,
    instrux + 1937,
    instrux + 1938,
    instrux + 1939,
    instrux + 1940,
    instrux + 1941,
    instrux + 1942,
    instrux + 1943,
    instrux + 1944,
    instrux + 1945,
    instrux + 1946,
    instrux + 1947,
    instrux + 1948,
    instrux + 1949,
    instrux + 1950,
    instrux + 1951,
    instrux + 1952,
    instrux + 1953,
    instrux + 1954,
    instrux + 1955,
    instrux + 1956,
    instrux + 1957,
    instrux + 1958,
    instrux + 1959,
    instrux + 1960,
    instrux + 1961,
    instrux + 1962,
    instrux + 1963,
    instrux + 1964,
    instrux + 1965,
    instrux + 1966,
    instrux + 1967,
    instrux + 1968,
    instrux + 1969,
    instrux + 1970,
    instrux + 1971,
    instrux + 1972,
    instrux + 1973,
    instrux + 1974,
    instrux + 1975,
    instrux + 1976,
    instrux + 1977,
    instrux + 1978,
    instrux + 1979,
    instrux + 1980,
    instrux + 1981,
    instrux + 1982,
    instrux + 1983,
    instrux + 1984,
    instrux + 1985,
    instrux + 1986,
    instrux + 1987,
    instrux + 1988,
    instrux + 1989,
    instrux + 1990,
    instrux + 1991,
    instrux + 1992,
    instrux + 1993,
    instrux + 1994,
    instrux + 1995,
    instrux + 1996,
    instrux + 1997,
    instrux + 1998,
    instrux + 1999,
    instrux + 2000,
    instrux + 2001,
    instrux + 2002,
    instrux + 2003,
    instrux + 2004,
    instrux + 2005,
    instrux + 2006,
    instrux + 2007,
    instrux + 2008,
    instrux + 2009,
    instrux + 2010,
    instrux + 2011,
    instrux + 2012,
    instrux + 2013,
    instrux + 2014,
    instrux + 2015,
    instrux + 2016,
    instrux + 2017,
    instrux + 2018,
    instrux + 2019,
    instrux + 2020,
    instrux + 2021,
    instrux + 2022,
    instrux + 2023,
    instrux + 2024,
    instrux + 2025,
    instrux + 2026,
    instrux + 2027,
    instrux + 2028,
    instrux + 2029,
    instrux + 2030,
    instrux + 2031,
    instrux + 2032,
    instrux + 2033,
    instrux + 2034,
    instrux + 2035,
    instrux + 2036,
    instrux + 2037,
    instrux + 2038,
    instrux + 2039,
    instrux + 2040,
    instrux + 2041,
    instrux + 2042,
    instrux + 2043,
    instrux + 2044,
    instrux + 2045,
    instrux + 2046,
    instrux + 2047,
    instrux + 2048,
    instrux + 2049,
    instrux + 2050,
    instrux + 2051,
    instrux + 2052,
    instrux + 2053,
    instrux + 2054,
    instrux + 2055,
    instrux + 2056,
    instrux + 2057,
    instrux + 2058,
    instrux + 2059,
    instrux + 2060,
    instrux + 2061,
    instrux + 2062,
    instrux + 2063,
    instrux + 2064,
    instrux + 2065,
    instrux + 2066,
    instrux + 2067,
    instrux + 2068,
    instrux + 2069,
    instrux + 2070,
    instrux + 2071,
    instrux + 2072,
    instrux + 2073,
    instrux + 2074,
    instrux + 2075,
    instrux + 2076,
    instrux + 2077,
    instrux + 2078,
    instrux + 2079,
    instrux + 2080,
    instrux + 2081,
    instrux + 2082,
    instrux + 2083,
    instrux + 2084,
    instrux + 2085,
    instrux + 2086,
    instrux + 2087,
    instrux + 2088,
    instrux + 2089,
    instrux + 2090,
    instrux + 2091,
    instrux + 2092,
    instrux + 2093,
    instrux + 2094,
    instrux + 2095,
    instrux + 2096,
    instrux + 2097,
    instrux + 2098,
    instrux + 2099,
};

static const struct itemplate * const itable_vex010C6[] = {
    instrux + 2791,
    instrux + 2792,
    instrux + 2793,
    instrux + 2794,
};

static const struct itemplate * const itable_vex01110[] = {
    instrux + 2484,
    instrux + 2486,
};

static const struct itemplate * const itable_vex01111[] = {
    instrux + 2485,
    instrux + 2487,
};

static const struct itemplate * const itable_vex01112[] = {
    instrux + 2446,
    instrux + 2447,
};

static const struct itemplate * const itable_vex01113[] = {
    instrux + 2448,
};

static const struct itemplate * const itable_vex01114[] = {
    instrux + 2830,
    instrux + 2831,
    instrux + 2832,
    instrux + 2833,
};

static const struct itemplate * const itable_vex01115[] = {
    instrux + 2822,
    instrux + 2823,
    instrux + 2824,
    instrux + 2825,
};

static const struct itemplate * const itable_vex01116[] = {
    instrux + 2438,
    instrux + 2439,
};

static const struct itemplate * const itable_vex01117[] = {
    instrux + 2440,
};

static const struct itemplate * const itable_vex01128[] = {
    instrux + 2408,
    instrux + 2410,
};

static const struct itemplate * const itable_vex01129[] = {
    instrux + 2409,
    instrux + 2411,
};

static const struct itemplate * const itable_vex0112B[] = {
    instrux + 2464,
    instrux + 2465,
};

static const struct itemplate * const itable_vex0112E[] = {
    instrux + 2820,
};

static const struct itemplate * const itable_vex0112F[] = {
    instrux + 2288,
};

static const struct itemplate * const itable_vex01150[] = {
    instrux + 2452,
    instrux + 2453,
    instrux + 2454,
    instrux + 2455,
};

static const struct itemplate * const itable_vex01151[] = {
    instrux + 2795,
    instrux + 2796,
};

static const struct itemplate * const itable_vex01154[] = {
    instrux + 1688,
    instrux + 1689,
    instrux + 1690,
    instrux + 1691,
};

static const struct itemplate * const itable_vex01155[] = {
    instrux + 1696,
    instrux + 1697,
    instrux + 1698,
    instrux + 1699,
};

static const struct itemplate * const itable_vex01156[] = {
    instrux + 2506,
    instrux + 2507,
    instrux + 2508,
    instrux + 2509,
};

static const struct itemplate * const itable_vex01157[] = {
    instrux + 2838,
    instrux + 2839,
    instrux + 2840,
    instrux + 2841,
};

static const struct itemplate * const itable_vex01158[] = {
    instrux + 1668,
    instrux + 1669,
    instrux + 1670,
    instrux + 1671,
};

static const struct itemplate * const itable_vex01159[] = {
    instrux + 2494,
    instrux + 2495,
    instrux + 2496,
    instrux + 2497,
};

static const struct itemplate * const itable_vex0115A[] = {
    instrux + 2298,
    instrux + 2299,
    instrux + 2300,
    instrux + 2301,
};

static const struct itemplate * const itable_vex0115B[] = {
    instrux + 2302,
    instrux + 2303,
};

static const struct itemplate * const itable_vex0115C[] = {
    instrux + 2804,
    instrux + 2805,
    instrux + 2806,
    instrux + 2807,
};

static const struct itemplate * const itable_vex0115D[] = {
    instrux + 2396,
    instrux + 2397,
    instrux + 2398,
    instrux + 2399,
};

static const struct itemplate * const itable_vex0115E[] = {
    instrux + 2332,
    instrux + 2333,
    instrux + 2334,
    instrux + 2335,
};

static const struct itemplate * const itable_vex0115F[] = {
    instrux + 2384,
    instrux + 2385,
    instrux + 2386,
    instrux + 2387,
};

static const struct itemplate * const itable_vex01160[] = {
    instrux + 2761,
    instrux + 2762,
    instrux + 3582,
    instrux + 3583,
};

static const struct itemplate * const itable_vex01161[] = {
    instrux + 2763,
    instrux + 2764,
    instrux + 3584,
    instrux + 3585,
};

static const struct itemplate * const itable_vex01162[] = {
    instrux + 2765,
    instrux + 2766,
    instrux + 3586,
    instrux + 3587,
};

static const struct itemplate * const itable_vex01163[] = {
    instrux + 2517,
    instrux + 2518,
    instrux + 3379,
    instrux + 3380,
};

static const struct itemplate * const itable_vex01164[] = {
    instrux + 2567,
    instrux + 2568,
    instrux + 3425,
    instrux + 3426,
};

static const struct itemplate * const itable_vex01165[] = {
    instrux + 2569,
    instrux + 2570,
    instrux + 3427,
    instrux + 3428,
};

static const struct itemplate * const itable_vex01166[] = {
    instrux + 2571,
    instrux + 2572,
    instrux + 3429,
    instrux + 3430,
};

static const struct itemplate * const itable_vex01167[] = {
    instrux + 2521,
    instrux + 2522,
    instrux + 3385,
    instrux + 3386,
};

static const struct itemplate * const itable_vex01168[] = {
    instrux + 2753,
    instrux + 2754,
    instrux + 3574,
    instrux + 3575,
};

static const struct itemplate * const itable_vex01169[] = {
    instrux + 2755,
    instrux + 2756,
    instrux + 3576,
    instrux + 3577,
};

static const struct itemplate * const itable_vex0116A[] = {
    instrux + 2757,
    instrux + 2758,
    instrux + 3578,
    instrux + 3579,
};

static const struct itemplate * const itable_vex0116B[] = {
    instrux + 2519,
    instrux + 2520,
    instrux + 3381,
    instrux + 3382,
};

static const struct itemplate * const itable_vex0116C[] = {
    instrux + 2767,
    instrux + 2768,
    instrux + 3588,
    instrux + 3589,
};

static const struct itemplate * const itable_vex0116D[] = {
    instrux + 2759,
    instrux + 2760,
    instrux + 3580,
    instrux + 3581,
};

static const struct itemplate * const itable_vex0116E[] = {
    instrux + 2416,
    instrux + 2420,
};

static const struct itemplate * const itable_vex0116F[] = {
    instrux + 2424,
    instrux + 2426,
    instrux + 2428,
};

static const struct itemplate * const itable_vex01170[] = {
    instrux + 2690,
    instrux + 3513,
};

static const struct itemplate * const itable_vex01171[] = {
    instrux + 2705,
    instrux + 2706,
    instrux + 2717,
    instrux + 2718,
    instrux + 2725,
    instrux + 2726,
    instrux + 3526,
    instrux + 3527,
    instrux + 3538,
    instrux + 3539,
    instrux + 3548,
    instrux + 3549,
};

static const struct itemplate * const itable_vex01172[] = {
    instrux + 2709,
    instrux + 2710,
    instrux + 2721,
    instrux + 2722,
    instrux + 2729,
    instrux + 2730,
    instrux + 3530,
    instrux + 3531,
    instrux + 3542,
    instrux + 3543,
    instrux + 3552,
    instrux + 3553,
};

static const struct itemplate * const itable_vex01173[] = {
    instrux + 2699,
    instrux + 2700,
    instrux + 2701,
    instrux + 2702,
    instrux + 2713,
    instrux + 2714,
    instrux + 2733,
    instrux + 2734,
    instrux + 3522,
    instrux + 3523,
    instrux + 3534,
    instrux + 3535,
    instrux + 3544,
    instrux + 3545,
    instrux + 3556,
    instrux + 3557,
};

static const struct itemplate * const itable_vex01174[] = {
    instrux + 2559,
    instrux + 2560,
    instrux + 3417,
    instrux + 3418,
};

static const struct itemplate * const itable_vex01175[] = {
    instrux + 2561,
    instrux + 2562,
    instrux + 3419,
    instrux + 3420,
};

static const struct itemplate * const itable_vex01176[] = {
    instrux + 2563,
    instrux + 2564,
    instrux + 3421,
    instrux + 3422,
};

static const struct itemplate * const itable_vex0117C[] = {
    instrux + 2352,
    instrux + 2353,
    instrux + 2354,
    instrux + 2355,
};

static const struct itemplate * const itable_vex0117D[] = {
    instrux + 2360,
    instrux + 2361,
    instrux + 2362,
    instrux + 2363,
};

static const struct itemplate * const itable_vex0117E[] = {
    instrux + 2417,
    instrux + 2421,
};

static const struct itemplate * const itable_vex0117F[] = {
    instrux + 2425,
    instrux + 2427,
    instrux + 2429,
};

static const struct itemplate * const itable_vex011C2[] = {
    instrux + 1724,
    instrux + 1725,
    instrux + 1726,
    instrux + 1727,
    instrux + 1728,
    instrux + 1729,
    instrux + 1730,
    instrux + 1731,
    instrux + 1732,
    instrux + 1733,
    instrux + 1734,
    instrux + 1735,
    instrux + 1736,
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1740,
    instrux + 1741,
    instrux + 1742,
    instrux + 1743,
    instrux + 1744,
    instrux + 1745,
    instrux + 1746,
    instrux + 1747,
    instrux + 1748,
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1752,
    instrux + 1753,
    instrux + 1754,
    instrux + 1755,
    instrux + 1756,
    instrux + 1757,
    instrux + 1758,
    instrux + 1759,
    instrux + 1760,
    instrux + 1761,
    instrux + 1762,
    instrux + 1763,
    instrux + 1764,
    instrux + 1765,
    instrux + 1766,
    instrux + 1767,
    instrux + 1768,
    instrux + 1769,
    instrux + 1770,
    instrux + 1771,
    instrux + 1772,
    instrux + 1773,
    instrux + 1774,
    instrux + 1775,
    instrux + 1776,
    instrux + 1777,
    instrux + 1778,
    instrux + 1779,
    instrux + 1780,
    instrux + 1781,
    instrux + 1782,
    instrux + 1783,
    instrux + 1784,
    instrux + 1785,
    instrux + 1786,
    instrux + 1787,
    instrux + 1788,
    instrux + 1789,
    instrux + 1790,
    instrux + 1791,
    instrux + 1792,
    instrux + 1793,
    instrux + 1794,
    instrux + 1795,
    instrux + 1796,
    instrux + 1797,
    instrux + 1798,
    instrux + 1799,
    instrux + 1800,
    instrux + 1801,
    instrux + 1802,
    instrux + 1803,
    instrux + 1804,
    instrux + 1805,
    instrux + 1806,
    instrux + 1807,
    instrux + 1808,
    instrux + 1809,
    instrux + 1810,
    instrux + 1811,
    instrux + 1812,
    instrux + 1813,
    instrux + 1814,
    instrux + 1815,
    instrux + 1816,
    instrux + 1817,
    instrux + 1818,
    instrux + 1819,
    instrux + 1820,
    instrux + 1821,
    instrux + 1822,
    instrux + 1823,
    instrux + 1824,
    instrux + 1825,
    instrux + 1826,
    instrux + 1827,
    instrux + 1828,
    instrux + 1829,
    instrux + 1830,
    instrux + 1831,
    instrux + 1832,
    instrux + 1833,
    instrux + 1834,
    instrux + 1835,
    instrux + 1836,
    instrux + 1837,
    instrux + 1838,
    instrux + 1839,
    instrux + 1840,
    instrux + 1841,
    instrux + 1842,
    instrux + 1843,
    instrux + 1844,
    instrux + 1845,
    instrux + 1846,
    instrux + 1847,
    instrux + 1848,
    instrux + 1849,
    instrux + 1850,
    instrux + 1851,
    instrux + 1852,
    instrux + 1853,
    instrux + 1854,
    instrux + 1855,
    instrux + 1856,
    instrux + 1857,
    instrux + 1858,
    instrux + 1859,
    instrux + 1860,
    instrux + 1861,
    instrux + 1862,
    instrux + 1863,
    instrux + 1864,
    instrux + 1865,
    instrux + 1866,
    instrux + 1867,
    instrux + 1868,
    instrux + 1869,
    instrux + 1870,
    instrux + 1871,
    instrux + 1872,
    instrux + 1873,
    instrux + 1874,
    instrux + 1875,
    instrux + 1876,
    instrux + 1877,
    instrux + 1878,
    instrux + 1879,
    instrux + 1880,
    instrux + 1881,
    instrux + 1882,
    instrux + 1883,
    instrux + 1884,
    instrux + 1885,
    instrux + 1886,
    instrux + 1887,
    instrux + 1888,
    instrux + 1889,
    instrux + 1890,
    instrux + 1891,
    instrux + 1892,
    instrux + 1893,
    instrux + 1894,
    instrux + 1895,
    instrux + 1896,
    instrux + 1897,
    instrux + 1898,
    instrux + 1899,
    instrux + 1900,
    instrux + 1901,
    instrux + 1902,
    instrux + 1903,
    instrux + 1904,
    instrux + 1905,
    instrux + 1906,
    instrux + 1907,
    instrux + 1908,
    instrux + 1909,
    instrux + 1910,
    instrux + 1911,
};

static const struct itemplate * const itable_vex011C4[] = {
    instrux + 2614,
    instrux + 2615,
    instrux + 2616,
    instrux + 2617,
    instrux + 2618,
    instrux + 2619,
};

static const struct itemplate * const itable_vex011C5[] = {
    instrux + 2587,
    instrux + 2588,
};

static const struct itemplate * const itable_vex011C6[] = {
    instrux + 2787,
    instrux + 2788,
    instrux + 2789,
    instrux + 2790,
};

static const struct itemplate * const itable_vex011D0[] = {
    instrux + 1680,
    instrux + 1681,
    instrux + 1682,
    instrux + 1683,
};

static const struct itemplate * const itable_vex011D1[] = {
    instrux + 2723,
    instrux + 2724,
    instrux + 3546,
    instrux + 3547,
};

static const struct itemplate * const itable_vex011D2[] = {
    instrux + 2727,
    instrux + 2728,
    instrux + 3550,
    instrux + 3551,
};

static const struct itemplate * const itable_vex011D3[] = {
    instrux + 2731,
    instrux + 2732,
    instrux + 3554,
    instrux + 3555,
};

static const struct itemplate * const itable_vex011D4[] = {
    instrux + 2531,
    instrux + 2532,
    instrux + 3393,
    instrux + 3394,
};

static const struct itemplate * const itable_vex011D5[] = {
    instrux + 2676,
    instrux + 2677,
    instrux + 3501,
    instrux + 3502,
};

static const struct itemplate * const itable_vex011D6[] = {
    instrux + 2419,
};

static const struct itemplate * const itable_vex011D7[] = {
    instrux + 2656,
    instrux + 2657,
    instrux + 3473,
    instrux + 3474,
};

static const struct itemplate * const itable_vex011D8[] = {
    instrux + 2749,
    instrux + 2750,
    instrux + 3570,
    instrux + 3571,
};

static const struct itemplate * const itable_vex011D9[] = {
    instrux + 2751,
    instrux + 2752,
    instrux + 3572,
    instrux + 3573,
};

static const struct itemplate * const itable_vex011DA[] = {
    instrux + 2650,
    instrux + 2651,
    instrux + 3467,
    instrux + 3468,
};

static const struct itemplate * const itable_vex011DB[] = {
    instrux + 2543,
    instrux + 2544,
    instrux + 3405,
    instrux + 3406,
};

static const struct itemplate * const itable_vex011DC[] = {
    instrux + 2537,
    instrux + 2538,
    instrux + 3399,
    instrux + 3400,
};

static const struct itemplate * const itable_vex011DD[] = {
    instrux + 2539,
    instrux + 2540,
    instrux + 3401,
    instrux + 3402,
};

static const struct itemplate * const itable_vex011DE[] = {
    instrux + 2638,
    instrux + 2639,
    instrux + 3455,
    instrux + 3456,
};

static const struct itemplate * const itable_vex011DF[] = {
    instrux + 2545,
    instrux + 2546,
    instrux + 3407,
    instrux + 3408,
};

static const struct itemplate * const itable_vex011E0[] = {
    instrux + 2547,
    instrux + 2548,
    instrux + 3409,
    instrux + 3410,
};

static const struct itemplate * const itable_vex011E1[] = {
    instrux + 2715,
    instrux + 2716,
    instrux + 3536,
    instrux + 3537,
};

static const struct itemplate * const itable_vex011E2[] = {
    instrux + 2719,
    instrux + 2720,
    instrux + 3540,
    instrux + 3541,
};

static const struct itemplate * const itable_vex011E3[] = {
    instrux + 2549,
    instrux + 2550,
    instrux + 3411,
    instrux + 3412,
};

static const struct itemplate * const itable_vex011E4[] = {
    instrux + 2670,
    instrux + 2671,
    instrux + 3497,
    instrux + 3498,
};

static const struct itemplate * const itable_vex011E5[] = {
    instrux + 2674,
    instrux + 2675,
    instrux + 3499,
    instrux + 3500,
};

static const struct itemplate * const itable_vex011E6[] = {
    instrux + 2322,
    instrux + 2323,
    instrux + 2324,
    instrux + 2325,
};

static const struct itemplate * const itable_vex011E7[] = {
    instrux + 2460,
    instrux + 2461,
    instrux + 2462,
};

static const struct itemplate * const itable_vex011E8[] = {
    instrux + 2745,
    instrux + 2746,
    instrux + 3566,
    instrux + 3567,
};

static const struct itemplate * const itable_vex011E9[] = {
    instrux + 2747,
    instrux + 2748,
    instrux + 3568,
    instrux + 3569,
};

static const struct itemplate * const itable_vex011EA[] = {
    instrux + 2646,
    instrux + 2647,
    instrux + 3463,
    instrux + 3464,
};

static const struct itemplate * const itable_vex011EB[] = {
    instrux + 2684,
    instrux + 2685,
    instrux + 3507,
    instrux + 3508,
};

static const struct itemplate * const itable_vex011EC[] = {
    instrux + 2533,
    instrux + 2534,
    instrux + 3395,
    instrux + 3396,
};

static const struct itemplate * const itable_vex011ED[] = {
    instrux + 2535,
    instrux + 2536,
    instrux + 3397,
    instrux + 3398,
};

static const struct itemplate * const itable_vex011EE[] = {
    instrux + 2634,
    instrux + 2635,
    instrux + 3451,
    instrux + 3452,
};

static const struct itemplate * const itable_vex011EF[] = {
    instrux + 2769,
    instrux + 2770,
    instrux + 3590,
    instrux + 3591,
};

static const struct itemplate * const itable_vex011F1[] = {
    instrux + 2703,
    instrux + 2704,
    instrux + 3524,
    instrux + 3525,
};

static const struct itemplate * const itable_vex011F2[] = {
    instrux + 2707,
    instrux + 2708,
    instrux + 3528,
    instrux + 3529,
};

static const struct itemplate * const itable_vex011F3[] = {
    instrux + 2711,
    instrux + 2712,
    instrux + 3532,
    instrux + 3533,
};

static const struct itemplate * const itable_vex011F4[] = {
    instrux + 2680,
    instrux + 2681,
    instrux + 3505,
    instrux + 3506,
};

static const struct itemplate * const itable_vex011F5[] = {
    instrux + 2628,
    instrux + 2629,
    instrux + 3447,
    instrux + 3448,
};

static const struct itemplate * const itable_vex011F6[] = {
    instrux + 2686,
    instrux + 2687,
    instrux + 3509,
    instrux + 3510,
};

static const struct itemplate * const itable_vex011F7[] = {
    instrux + 2375,
};

static const struct itemplate * const itable_vex011F8[] = {
    instrux + 2737,
    instrux + 2738,
    instrux + 3558,
    instrux + 3559,
};

static const struct itemplate * const itable_vex011F9[] = {
    instrux + 2739,
    instrux + 2740,
    instrux + 3560,
    instrux + 3561,
};

static const struct itemplate * const itable_vex011FA[] = {
    instrux + 2741,
    instrux + 2742,
    instrux + 3562,
    instrux + 3563,
};

static const struct itemplate * const itable_vex011FB[] = {
    instrux + 2743,
    instrux + 2744,
    instrux + 3564,
    instrux + 3565,
};

static const struct itemplate * const itable_vex011FC[] = {
    instrux + 2525,
    instrux + 2526,
    instrux + 3387,
    instrux + 3388,
};

static const struct itemplate * const itable_vex011FD[] = {
    instrux + 2527,
    instrux + 2528,
    instrux + 3389,
    instrux + 3390,
};

static const struct itemplate * const itable_vex011FE[] = {
    instrux + 2529,
    instrux + 2530,
    instrux + 3391,
    instrux + 3392,
};

static const struct itemplate * const itable_vex01210[] = {
    instrux + 2478,
    instrux + 2479,
    instrux + 2480,
};

static const struct itemplate * const itable_vex01211[] = {
    instrux + 2481,
    instrux + 2482,
    instrux + 2483,
};

static const struct itemplate * const itable_vex01212[] = {
    instrux + 2476,
    instrux + 2477,
};

static const struct itemplate * const itable_vex01216[] = {
    instrux + 2474,
    instrux + 2475,
};

static const struct itemplate * const itable_vex0122A[] = {
    instrux + 2314,
    instrux + 2315,
    instrux + 2316,
    instrux + 2317,
};

static const struct itemplate * const itable_vex0122C[] = {
    instrux + 2330,
    instrux + 2331,
};

static const struct itemplate * const itable_vex0122D[] = {
    instrux + 2320,
    instrux + 2321,
};

static const struct itemplate * const itable_vex01251[] = {
    instrux + 2801,
    instrux + 2802,
};

static const struct itemplate * const itable_vex01252[] = {
    instrux + 2777,
    instrux + 2778,
};

static const struct itemplate * const itable_vex01253[] = {
    instrux + 2773,
    instrux + 2774,
};

static const struct itemplate * const itable_vex01258[] = {
    instrux + 1678,
    instrux + 1679,
};

static const struct itemplate * const itable_vex01259[] = {
    instrux + 2504,
    instrux + 2505,
};

static const struct itemplate * const itable_vex0125A[] = {
    instrux + 2318,
    instrux + 2319,
};

static const struct itemplate * const itable_vex0125B[] = {
    instrux + 2326,
    instrux + 2327,
};

static const struct itemplate * const itable_vex0125C[] = {
    instrux + 2814,
    instrux + 2815,
};

static const struct itemplate * const itable_vex0125D[] = {
    instrux + 2406,
    instrux + 2407,
};

static const struct itemplate * const itable_vex0125E[] = {
    instrux + 2342,
    instrux + 2343,
};

static const struct itemplate * const itable_vex0125F[] = {
    instrux + 2394,
    instrux + 2395,
};

static const struct itemplate * const itable_vex0126F[] = {
    instrux + 2430,
    instrux + 2432,
    instrux + 2434,
};

static const struct itemplate * const itable_vex01270[] = {
    instrux + 2691,
    instrux + 3514,
};

static const struct itemplate * const itable_vex0127E[] = {
    instrux + 2418,
};

static const struct itemplate * const itable_vex0127F[] = {
    instrux + 2431,
    instrux + 2433,
    instrux + 2435,
};

static const struct itemplate * const itable_vex012C2[] = {
    instrux + 2194,
    instrux + 2195,
    instrux + 2196,
    instrux + 2197,
    instrux + 2198,
    instrux + 2199,
    instrux + 2200,
    instrux + 2201,
    instrux + 2202,
    instrux + 2203,
    instrux + 2204,
    instrux + 2205,
    instrux + 2206,
    instrux + 2207,
    instrux + 2208,
    instrux + 2209,
    instrux + 2210,
    instrux + 2211,
    instrux + 2212,
    instrux + 2213,
    instrux + 2214,
    instrux + 2215,
    instrux + 2216,
    instrux + 2217,
    instrux + 2218,
    instrux + 2219,
    instrux + 2220,
    instrux + 2221,
    instrux + 2222,
    instrux + 2223,
    instrux + 2224,
    instrux + 2225,
    instrux + 2226,
    instrux + 2227,
    instrux + 2228,
    instrux + 2229,
    instrux + 2230,
    instrux + 2231,
    instrux + 2232,
    instrux + 2233,
    instrux + 2234,
    instrux + 2235,
    instrux + 2236,
    instrux + 2237,
    instrux + 2238,
    instrux + 2239,
    instrux + 2240,
    instrux + 2241,
    instrux + 2242,
    instrux + 2243,
    instrux + 2244,
    instrux + 2245,
    instrux + 2246,
    instrux + 2247,
    instrux + 2248,
    instrux + 2249,
    instrux + 2250,
    instrux + 2251,
    instrux + 2252,
    instrux + 2253,
    instrux + 2254,
    instrux + 2255,
    instrux + 2256,
    instrux + 2257,
    instrux + 2258,
    instrux + 2259,
    instrux + 2260,
    instrux + 2261,
    instrux + 2262,
    instrux + 2263,
    instrux + 2264,
    instrux + 2265,
    instrux + 2266,
    instrux + 2267,
    instrux + 2268,
    instrux + 2269,
    instrux + 2270,
    instrux + 2271,
    instrux + 2272,
    instrux + 2273,
    instrux + 2274,
    instrux + 2275,
    instrux + 2276,
    instrux + 2277,
    instrux + 2278,
    instrux + 2279,
    instrux + 2280,
    instrux + 2281,
    instrux + 2282,
    instrux + 2283,
    instrux + 2284,
    instrux + 2285,
    instrux + 2286,
    instrux + 2287,
};

static const struct itemplate * const itable_vex012E6[] = {
    instrux + 2290,
    instrux + 2291,
};

static const struct itemplate * const itable_vex01310[] = {
    instrux + 2468,
    instrux + 2469,
    instrux + 2470,
};

static const struct itemplate * const itable_vex01311[] = {
    instrux + 2471,
    instrux + 2472,
    instrux + 2473,
};

static const struct itemplate * const itable_vex01312[] = {
    instrux + 2422,
    instrux + 2423,
};

static const struct itemplate * const itable_vex0132A[] = {
    instrux + 2310,
    instrux + 2311,
    instrux + 2312,
    instrux + 2313,
};

static const struct itemplate * const itable_vex0132C[] = {
    instrux + 2328,
    instrux + 2329,
};

static const struct itemplate * const itable_vex0132D[] = {
    instrux + 2306,
    instrux + 2307,
};

static const struct itemplate * const itable_vex01351[] = {
    instrux + 2799,
    instrux + 2800,
};

static const struct itemplate * const itable_vex01358[] = {
    instrux + 1676,
    instrux + 1677,
};

static const struct itemplate * const itable_vex01359[] = {
    instrux + 2502,
    instrux + 2503,
};

static const struct itemplate * const itable_vex0135A[] = {
    instrux + 2308,
    instrux + 2309,
};

static const struct itemplate * const itable_vex0135C[] = {
    instrux + 2812,
    instrux + 2813,
};

static const struct itemplate * const itable_vex0135D[] = {
    instrux + 2404,
    instrux + 2405,
};

static const struct itemplate * const itable_vex0135E[] = {
    instrux + 2340,
    instrux + 2341,
};

static const struct itemplate * const itable_vex0135F[] = {
    instrux + 2392,
    instrux + 2393,
};

static const struct itemplate * const itable_vex01370[] = {
    instrux + 2692,
    instrux + 3515,
};

static const struct itemplate * const itable_vex0137C[] = {
    instrux + 2356,
    instrux + 2357,
    instrux + 2358,
    instrux + 2359,
};

static const struct itemplate * const itable_vex0137D[] = {
    instrux + 2364,
    instrux + 2365,
    instrux + 2366,
    instrux + 2367,
};

static const struct itemplate * const itable_vex013C2[] = {
    instrux + 2100,
    instrux + 2101,
    instrux + 2102,
    instrux + 2103,
    instrux + 2104,
    instrux + 2105,
    instrux + 2106,
    instrux + 2107,
    instrux + 2108,
    instrux + 2109,
    instrux + 2110,
    instrux + 2111,
    instrux + 2112,
    instrux + 2113,
    instrux + 2114,
    instrux + 2115,
    instrux + 2116,
    instrux + 2117,
    instrux + 2118,
    instrux + 2119,
    instrux + 2120,
    instrux + 2121,
    instrux + 2122,
    instrux + 2123,
    instrux + 2124,
    instrux + 2125,
    instrux + 2126,
    instrux + 2127,
    instrux + 2128,
    instrux + 2129,
    instrux + 2130,
    instrux + 2131,
    instrux + 2132,
    instrux + 2133,
    instrux + 2134,
    instrux + 2135,
    instrux + 2136,
    instrux + 2137,
    instrux + 2138,
    instrux + 2139,
    instrux + 2140,
    instrux + 2141,
    instrux + 2142,
    instrux + 2143,
    instrux + 2144,
    instrux + 2145,
    instrux + 2146,
    instrux + 2147,
    instrux + 2148,
    instrux + 2149,
    instrux + 2150,
    instrux + 2151,
    instrux + 2152,
    instrux + 2153,
    instrux + 2154,
    instrux + 2155,
    instrux + 2156,
    instrux + 2157,
    instrux + 2158,
    instrux + 2159,
    instrux + 2160,
    instrux + 2161,
    instrux + 2162,
    instrux + 2163,
    instrux + 2164,
    instrux + 2165,
    instrux + 2166,
    instrux + 2167,
    instrux + 2168,
    instrux + 2169,
    instrux + 2170,
    instrux + 2171,
    instrux + 2172,
    instrux + 2173,
    instrux + 2174,
    instrux + 2175,
    instrux + 2176,
    instrux + 2177,
    instrux + 2178,
    instrux + 2179,
    instrux + 2180,
    instrux + 2181,
    instrux + 2182,
    instrux + 2183,
    instrux + 2184,
    instrux + 2185,
    instrux + 2186,
    instrux + 2187,
    instrux + 2188,
    instrux + 2189,
    instrux + 2190,
    instrux + 2191,
    instrux + 2192,
    instrux + 2193,
};

static const struct itemplate * const itable_vex013D0[] = {
    instrux + 1684,
    instrux + 1685,
    instrux + 1686,
    instrux + 1687,
};

static const struct itemplate * const itable_vex013E6[] = {
    instrux + 2294,
    instrux + 2295,
    instrux + 2296,
    instrux + 2297,
};

static const struct itemplate * const itable_vex013F0[] = {
    instrux + 2371,
    instrux + 2372,
    instrux + 2373,
};

static const struct itemplate * const itable_vex020F2[] = {
    instrux + 3687,
    instrux + 3688,
};

static const struct itemplate * const itable_vex020F3[] = {
    instrux + 3697,
    instrux + 3698,
    instrux + 3707,
    instrux + 3708,
    instrux + 3709,
    instrux + 3710,
};

static const struct itemplate * const itable_vex020F5[] = {
    instrux + 3713,
    instrux + 3714,
};

static const struct itemplate * const itable_vex020F7[] = {
    instrux + 3689,
    instrux + 3690,
};

static const struct itemplate * const itable_vex02100[] = {
    instrux + 2688,
    instrux + 2689,
    instrux + 3511,
    instrux + 3512,
};

static const struct itemplate * const itable_vex02101[] = {
    instrux + 2595,
    instrux + 2596,
    instrux + 3433,
    instrux + 3434,
};

static const struct itemplate * const itable_vex02102[] = {
    instrux + 2597,
    instrux + 2598,
    instrux + 3435,
    instrux + 3436,
};

static const struct itemplate * const itable_vex02103[] = {
    instrux + 2599,
    instrux + 2600,
    instrux + 3437,
    instrux + 3438,
};

static const struct itemplate * const itable_vex02104[] = {
    instrux + 2630,
    instrux + 2631,
    instrux + 3445,
    instrux + 3446,
};

static const struct itemplate * const itable_vex02105[] = {
    instrux + 2602,
    instrux + 2603,
    instrux + 3439,
    instrux + 3440,
};

static const struct itemplate * const itable_vex02106[] = {
    instrux + 2604,
    instrux + 2605,
    instrux + 3441,
    instrux + 3442,
};

static const struct itemplate * const itable_vex02107[] = {
    instrux + 2606,
    instrux + 2607,
    instrux + 3443,
    instrux + 3444,
};

static const struct itemplate * const itable_vex02108[] = {
    instrux + 2693,
    instrux + 2694,
    instrux + 3516,
    instrux + 3517,
};

static const struct itemplate * const itable_vex02109[] = {
    instrux + 2695,
    instrux + 2696,
    instrux + 3518,
    instrux + 3519,
};

static const struct itemplate * const itable_vex0210A[] = {
    instrux + 2697,
    instrux + 2698,
    instrux + 3520,
    instrux + 3521,
};

static const struct itemplate * const itable_vex0210B[] = {
    instrux + 2672,
    instrux + 2673,
    instrux + 3495,
    instrux + 3496,
};

static const struct itemplate * const itable_vex0210C[] = {
    instrux + 2579,
    instrux + 2580,
};

static const struct itemplate * const itable_vex0210D[] = {
    instrux + 2575,
    instrux + 2576,
};

static const struct itemplate * const itable_vex0210E[] = {
    instrux + 2816,
    instrux + 2817,
};

static const struct itemplate * const itable_vex0210F[] = {
    instrux + 2818,
    instrux + 2819,
};

static const struct itemplate * const itable_vex02113[] = {
    instrux + 3066,
    instrux + 3067,
};

static const struct itemplate * const itable_vex02116[] = {
    instrux + 3620,
    instrux + 3621,
};

static const struct itemplate * const itable_vex02117[] = {
    instrux + 2735,
    instrux + 2736,
};

static const struct itemplate * const itable_vex02118[] = {
    instrux + 1720,
    instrux + 1721,
    instrux + 3593,
    instrux + 3594,
};

static const struct itemplate * const itable_vex02119[] = {
    instrux + 1722,
    instrux + 3595,
};

static const struct itemplate * const itable_vex0211A[] = {
    instrux + 1723,
};

static const struct itemplate * const itable_vex0211C[] = {
    instrux + 2514,
    instrux + 3376,
};

static const struct itemplate * const itable_vex0211D[] = {
    instrux + 2515,
    instrux + 3377,
};

static const struct itemplate * const itable_vex0211E[] = {
    instrux + 2516,
    instrux + 3378,
};

static const struct itemplate * const itable_vex02120[] = {
    instrux + 2658,
    instrux + 3475,
};

static const struct itemplate * const itable_vex02121[] = {
    instrux + 2659,
    instrux + 3476,
    instrux + 3477,
};

static const struct itemplate * const itable_vex02122[] = {
    instrux + 2660,
    instrux + 3478,
    instrux + 3479,
};

static const struct itemplate * const itable_vex02123[] = {
    instrux + 2661,
    instrux + 3480,
};

static const struct itemplate * const itable_vex02124[] = {
    instrux + 2662,
    instrux + 3481,
    instrux + 3482,
};

static const struct itemplate * const itable_vex02125[] = {
    instrux + 2663,
    instrux + 3483,
};

static const struct itemplate * const itable_vex02128[] = {
    instrux + 2682,
    instrux + 2683,
    instrux + 3493,
    instrux + 3494,
};

static const struct itemplate * const itable_vex02129[] = {
    instrux + 2565,
    instrux + 2566,
    instrux + 3423,
    instrux + 3424,
};

static const struct itemplate * const itable_vex0212A[] = {
    instrux + 2463,
    instrux + 3592,
};

static const struct itemplate * const itable_vex0212B[] = {
    instrux + 2523,
    instrux + 2524,
    instrux + 3383,
    instrux + 3384,
};

static const struct itemplate * const itable_vex0212C[] = {
    instrux + 2376,
    instrux + 2377,
};

static const struct itemplate * const itable_vex0212D[] = {
    instrux + 2380,
    instrux + 2381,
};

static const struct itemplate * const itable_vex0212E[] = {
    instrux + 2378,
    instrux + 2379,
};

static const struct itemplate * const itable_vex0212F[] = {
    instrux + 2382,
    instrux + 2383,
};

static const struct itemplate * const itable_vex02130[] = {
    instrux + 2664,
    instrux + 3484,
};

static const struct itemplate * const itable_vex02131[] = {
    instrux + 2665,
    instrux + 3485,
    instrux + 3486,
};

static const struct itemplate * const itable_vex02132[] = {
    instrux + 2666,
    instrux + 3487,
    instrux + 3488,
};

static const struct itemplate * const itable_vex02133[] = {
    instrux + 2667,
    instrux + 3489,
};

static const struct itemplate * const itable_vex02134[] = {
    instrux + 2668,
    instrux + 3490,
    instrux + 3491,
};

static const struct itemplate * const itable_vex02135[] = {
    instrux + 2669,
    instrux + 3492,
};

static const struct itemplate * const itable_vex02136[] = {
    instrux + 3617,
    instrux + 3618,
};

static const struct itemplate * const itable_vex02137[] = {
    instrux + 2573,
    instrux + 2574,
    instrux + 3431,
    instrux + 3432,
};

static const struct itemplate * const itable_vex02138[] = {
    instrux + 2644,
    instrux + 2645,
    instrux + 3461,
    instrux + 3462,
};

static const struct itemplate * const itable_vex02139[] = {
    instrux + 2648,
    instrux + 2649,
    instrux + 3465,
    instrux + 3466,
};

static const struct itemplate * const itable_vex0213A[] = {
    instrux + 2652,
    instrux + 2653,
    instrux + 3469,
    instrux + 3470,
};

static const struct itemplate * const itable_vex0213B[] = {
    instrux + 2654,
    instrux + 2655,
    instrux + 3471,
    instrux + 3472,
};

static const struct itemplate * const itable_vex0213C[] = {
    instrux + 2632,
    instrux + 2633,
    instrux + 3449,
    instrux + 3450,
};

static const struct itemplate * const itable_vex0213D[] = {
    instrux + 2636,
    instrux + 2637,
    instrux + 3453,
    instrux + 3454,
};

static const struct itemplate * const itable_vex0213E[] = {
    instrux + 2640,
    instrux + 2641,
    instrux + 3457,
    instrux + 3458,
};

static const struct itemplate * const itable_vex0213F[] = {
    instrux + 2642,
    instrux + 2643,
    instrux + 3459,
    instrux + 3460,
};

static const struct itemplate * const itable_vex02140[] = {
    instrux + 2678,
    instrux + 2679,
    instrux + 3503,
    instrux + 3504,
};

static const struct itemplate * const itable_vex02141[] = {
    instrux + 2601,
};

static const struct itemplate * const itable_vex02145[] = {
    instrux + 3655,
    instrux + 3656,
    instrux + 3657,
    instrux + 3658,
    instrux + 3659,
    instrux + 3660,
    instrux + 3661,
    instrux + 3662,
};

static const struct itemplate * const itable_vex02146[] = {
    instrux + 3651,
    instrux + 3652,
    instrux + 3653,
    instrux + 3654,
};

static const struct itemplate * const itable_vex02147[] = {
    instrux + 3643,
    instrux + 3644,
    instrux + 3645,
    instrux + 3646,
    instrux + 3647,
    instrux + 3648,
    instrux + 3649,
    instrux + 3650,
};

static const struct itemplate * const itable_vex02158[] = {
    instrux + 3609,
    instrux + 3610,
    instrux + 3611,
    instrux + 3612,
};

static const struct itemplate * const itable_vex02159[] = {
    instrux + 3613,
    instrux + 3614,
    instrux + 3615,
    instrux + 3616,
};

static const struct itemplate * const itable_vex0215A[] = {
    instrux + 3596,
};

static const struct itemplate * const itable_vex02178[] = {
    instrux + 3601,
    instrux + 3602,
    instrux + 3603,
    instrux + 3604,
};

static const struct itemplate * const itable_vex02179[] = {
    instrux + 3605,
    instrux + 3606,
    instrux + 3607,
    instrux + 3608,
};

static const struct itemplate * const itable_vex0218C[] = {
    instrux + 3627,
    instrux + 3628,
    instrux + 3629,
    instrux + 3630,
    instrux + 3631,
    instrux + 3632,
    instrux + 3633,
    instrux + 3634,
};

static const struct itemplate * const itable_vex0218E[] = {
    instrux + 3635,
    instrux + 3636,
    instrux + 3637,
    instrux + 3638,
    instrux + 3639,
    instrux + 3640,
    instrux + 3641,
    instrux + 3642,
};

static const struct itemplate * const itable_vex02190[] = {
    instrux + 3671,
    instrux + 3673,
    instrux + 3675,
    instrux + 3677,
};

static const struct itemplate * const itable_vex02191[] = {
    instrux + 3672,
    instrux + 3674,
    instrux + 3676,
    instrux + 3678,
};

static const struct itemplate * const itable_vex02192[] = {
    instrux + 3663,
    instrux + 3665,
    instrux + 3667,
    instrux + 3669,
};

static const struct itemplate * const itable_vex02193[] = {
    instrux + 3664,
    instrux + 3666,
    instrux + 3668,
    instrux + 3670,
};

static const struct itemplate * const itable_vex02196[] = {
    instrux + 2887,
    instrux + 2888,
    instrux + 2889,
    instrux + 2890,
    instrux + 2891,
    instrux + 2892,
    instrux + 2893,
    instrux + 2894,
};

static const struct itemplate * const itable_vex02197[] = {
    instrux + 2935,
    instrux + 2936,
    instrux + 2937,
    instrux + 2938,
    instrux + 2939,
    instrux + 2940,
    instrux + 2941,
    instrux + 2942,
};

static const struct itemplate * const itable_vex02198[] = {
    instrux + 2863,
    instrux + 2864,
    instrux + 2865,
    instrux + 2866,
    instrux + 2867,
    instrux + 2868,
    instrux + 2869,
    instrux + 2870,
};

static const struct itemplate * const itable_vex02199[] = {
    instrux + 3007,
    instrux + 3008,
    instrux + 3009,
    instrux + 3010,
};

static const struct itemplate * const itable_vex0219A[] = {
    instrux + 2911,
    instrux + 2912,
    instrux + 2913,
    instrux + 2914,
    instrux + 2915,
    instrux + 2916,
    instrux + 2917,
    instrux + 2918,
};

static const struct itemplate * const itable_vex0219B[] = {
    instrux + 3019,
    instrux + 3020,
    instrux + 3021,
    instrux + 3022,
};

static const struct itemplate * const itable_vex0219C[] = {
    instrux + 2959,
    instrux + 2960,
    instrux + 2961,
    instrux + 2962,
    instrux + 2963,
    instrux + 2964,
    instrux + 2965,
    instrux + 2966,
};

static const struct itemplate * const itable_vex0219D[] = {
    instrux + 3031,
    instrux + 3032,
    instrux + 3033,
    instrux + 3034,
};

static const struct itemplate * const itable_vex0219E[] = {
    instrux + 2983,
    instrux + 2984,
    instrux + 2985,
    instrux + 2986,
    instrux + 2987,
    instrux + 2988,
    instrux + 2989,
    instrux + 2990,
};

static const struct itemplate * const itable_vex0219F[] = {
    instrux + 3043,
    instrux + 3044,
    instrux + 3045,
    instrux + 3046,
};

static const struct itemplate * const itable_vex021A6[] = {
    instrux + 2895,
    instrux + 2896,
    instrux + 2897,
    instrux + 2898,
    instrux + 2899,
    instrux + 2900,
    instrux + 2901,
    instrux + 2902,
};

static const struct itemplate * const itable_vex021A7[] = {
    instrux + 2943,
    instrux + 2944,
    instrux + 2945,
    instrux + 2946,
    instrux + 2947,
    instrux + 2948,
    instrux + 2949,
    instrux + 2950,
};

static const struct itemplate * const itable_vex021A8[] = {
    instrux + 2871,
    instrux + 2872,
    instrux + 2873,
    instrux + 2874,
    instrux + 2875,
    instrux + 2876,
    instrux + 2877,
    instrux + 2878,
};

static const struct itemplate * const itable_vex021A9[] = {
    instrux + 3011,
    instrux + 3012,
    instrux + 3013,
    instrux + 3014,
};

static const struct itemplate * const itable_vex021AA[] = {
    instrux + 2919,
    instrux + 2920,
    instrux + 2921,
    instrux + 2922,
    instrux + 2923,
    instrux + 2924,
    instrux + 2925,
    instrux + 2926,
};

static const struct itemplate * const itable_vex021AB[] = {
    instrux + 3023,
    instrux + 3024,
    instrux + 3025,
    instrux + 3026,
};

static const struct itemplate * const itable_vex021AC[] = {
    instrux + 2967,
    instrux + 2968,
    instrux + 2969,
    instrux + 2970,
    instrux + 2971,
    instrux + 2972,
    instrux + 2973,
    instrux + 2974,
};

static const struct itemplate * const itable_vex021AD[] = {
    instrux + 3035,
    instrux + 3036,
    instrux + 3037,
    instrux + 3038,
};

static const struct itemplate * const itable_vex021AE[] = {
    instrux + 2991,
    instrux + 2992,
    instrux + 2993,
    instrux + 2994,
    instrux + 2995,
    instrux + 2996,
    instrux + 2997,
    instrux + 2998,
};

static const struct itemplate * const itable_vex021AF[] = {
    instrux + 3047,
    instrux + 3048,
    instrux + 3049,
    instrux + 3050,
};

static const struct itemplate * const itable_vex021B6[] = {
    instrux + 2903,
    instrux + 2904,
    instrux + 2905,
    instrux + 2906,
    instrux + 2907,
    instrux + 2908,
    instrux + 2909,
    instrux + 2910,
};

static const struct itemplate * const itable_vex021B7[] = {
    instrux + 2951,
    instrux + 2952,
    instrux + 2953,
    instrux + 2954,
    instrux + 2955,
    instrux + 2956,
    instrux + 2957,
    instrux + 2958,
};

static const struct itemplate * const itable_vex021B8[] = {
    instrux + 2879,
    instrux + 2880,
    instrux + 2881,
    instrux + 2882,
    instrux + 2883,
    instrux + 2884,
    instrux + 2885,
    instrux + 2886,
};

static const struct itemplate * const itable_vex021B9[] = {
    instrux + 3015,
    instrux + 3016,
    instrux + 3017,
    instrux + 3018,
};

static const struct itemplate * const itable_vex021BA[] = {
    instrux + 2927,
    instrux + 2928,
    instrux + 2929,
    instrux + 2930,
    instrux + 2931,
    instrux + 2932,
    instrux + 2933,
    instrux + 2934,
};

static const struct itemplate * const itable_vex021BB[] = {
    instrux + 3027,
    instrux + 3028,
    instrux + 3029,
    instrux + 3030,
};

static const struct itemplate * const itable_vex021BC[] = {
    instrux + 2975,
    instrux + 2976,
    instrux + 2977,
    instrux + 2978,
    instrux + 2979,
    instrux + 2980,
    instrux + 2981,
    instrux + 2982,
};

static const struct itemplate * const itable_vex021BD[] = {
    instrux + 3039,
    instrux + 3040,
    instrux + 3041,
    instrux + 3042,
};

static const struct itemplate * const itable_vex021BE[] = {
    instrux + 2999,
    instrux + 3000,
    instrux + 3001,
    instrux + 3002,
    instrux + 3003,
    instrux + 3004,
    instrux + 3005,
    instrux + 3006,
};

static const struct itemplate * const itable_vex021BF[] = {
    instrux + 3051,
    instrux + 3052,
    instrux + 3053,
    instrux + 3054,
};

static const struct itemplate * const itable_vex021DB[] = {
    instrux + 1666,
};

static const struct itemplate * const itable_vex021DC[] = {
    instrux + 1658,
    instrux + 1659,
};

static const struct itemplate * const itable_vex021DD[] = {
    instrux + 1660,
    instrux + 1661,
};

static const struct itemplate * const itable_vex021DE[] = {
    instrux + 1662,
    instrux + 1663,
};

static const struct itemplate * const itable_vex021DF[] = {
    instrux + 1664,
    instrux + 1665,
};

static const struct itemplate * const itable_vex021F7[] = {
    instrux + 3725,
    instrux + 3726,
};

static const struct itemplate * const itable_vex022F5[] = {
    instrux + 3719,
    instrux + 3720,
};

static const struct itemplate * const itable_vex022F7[] = {
    instrux + 3723,
    instrux + 3724,
};

static const struct itemplate * const itable_vex023F5[] = {
    instrux + 3717,
    instrux + 3718,
};

static const struct itemplate * const itable_vex023F6[] = {
    instrux + 3715,
    instrux + 3716,
};

static const struct itemplate * const itable_vex023F7[] = {
    instrux + 3727,
    instrux + 3728,
};

static const struct itemplate * const itable_vex03100[] = {
    instrux + 3622,
};

static const struct itemplate * const itable_vex03101[] = {
    instrux + 3619,
};

static const struct itemplate * const itable_vex03102[] = {
    instrux + 3597,
    instrux + 3598,
    instrux + 3599,
    instrux + 3600,
};

static const struct itemplate * const itable_vex03104[] = {
    instrux + 2581,
    instrux + 2582,
};

static const struct itemplate * const itable_vex03105[] = {
    instrux + 2577,
    instrux + 2578,
};

static const struct itemplate * const itable_vex03106[] = {
    instrux + 2583,
};

static const struct itemplate * const itable_vex03108[] = {
    instrux + 2781,
    instrux + 2782,
};

static const struct itemplate * const itable_vex03109[] = {
    instrux + 2779,
    instrux + 2780,
};

static const struct itemplate * const itable_vex0310A[] = {
    instrux + 2785,
    instrux + 2786,
};

static const struct itemplate * const itable_vex0310B[] = {
    instrux + 2783,
    instrux + 2784,
};

static const struct itemplate * const itable_vex0310C[] = {
    instrux + 1708,
    instrux + 1709,
    instrux + 1710,
    instrux + 1711,
};

static const struct itemplate * const itable_vex0310D[] = {
    instrux + 1704,
    instrux + 1705,
    instrux + 1706,
    instrux + 1707,
};

static const struct itemplate * const itable_vex0310E[] = {
    instrux + 2553,
    instrux + 2554,
    instrux + 3415,
    instrux + 3416,
};

static const struct itemplate * const itable_vex0310F[] = {
    instrux + 2541,
    instrux + 2542,
    instrux + 3403,
    instrux + 3404,
};

static const struct itemplate * const itable_vex03114[] = {
    instrux + 2584,
    instrux + 2585,
    instrux + 2586,
};

static const struct itemplate * const itable_vex03115[] = {
    instrux + 2589,
    instrux + 2590,
    instrux + 2591,
};

static const struct itemplate * const itable_vex03116[] = {
    instrux + 2592,
    instrux + 2593,
    instrux + 2594,
};

static const struct itemplate * const itable_vex03117[] = {
    instrux + 2351,
};

static const struct itemplate * const itable_vex03118[] = {
    instrux + 2368,
};

static const struct itemplate * const itable_vex03119[] = {
    instrux + 2350,
};

static const struct itemplate * const itable_vex0311D[] = {
    instrux + 3068,
    instrux + 3069,
};

static const struct itemplate * const itable_vex03120[] = {
    instrux + 2608,
    instrux + 2609,
    instrux + 2610,
    instrux + 2611,
    instrux + 2612,
    instrux + 2613,
};

static const struct itemplate * const itable_vex03121[] = {
    instrux + 2369,
    instrux + 2370,
};

static const struct itemplate * const itable_vex03122[] = {
    instrux + 2620,
    instrux + 2621,
    instrux + 2622,
    instrux + 2623,
    instrux + 2624,
    instrux + 2625,
    instrux + 2626,
    instrux + 2627,
};

static const struct itemplate * const itable_vex03138[] = {
    instrux + 3625,
    instrux + 3626,
};

static const struct itemplate * const itable_vex03139[] = {
    instrux + 3624,
};

static const struct itemplate * const itable_vex03140[] = {
    instrux + 2346,
    instrux + 2347,
    instrux + 2348,
    instrux + 2349,
};

static const struct itemplate * const itable_vex03141[] = {
    instrux + 2344,
    instrux + 2345,
};

static const struct itemplate * const itable_vex03142[] = {
    instrux + 2492,
    instrux + 2493,
    instrux + 3374,
    instrux + 3375,
};

static const struct itemplate * const itable_vex03144[] = {
    instrux + 2853,
    instrux + 2854,
    instrux + 2855,
    instrux + 2856,
    instrux + 2857,
    instrux + 2858,
    instrux + 2859,
    instrux + 2860,
    instrux + 2861,
    instrux + 2862,
};

static const struct itemplate * const itable_vex03146[] = {
    instrux + 3623,
};

static const struct itemplate * const itable_vex0314A[] = {
    instrux + 1716,
    instrux + 1717,
    instrux + 1718,
    instrux + 1719,
};

static const struct itemplate * const itable_vex0314B[] = {
    instrux + 1712,
    instrux + 1713,
    instrux + 1714,
    instrux + 1715,
};

static const struct itemplate * const itable_vex0314C[] = {
    instrux + 2551,
    instrux + 2552,
    instrux + 3413,
    instrux + 3414,
};

static const struct itemplate * const itable_vex0315C[] = {
    instrux + 3128,
    instrux + 3129,
    instrux + 3130,
    instrux + 3131,
    instrux + 3132,
    instrux + 3133,
    instrux + 3134,
    instrux + 3135,
};

static const struct itemplate * const itable_vex0315D[] = {
    instrux + 3120,
    instrux + 3121,
    instrux + 3122,
    instrux + 3123,
    instrux + 3124,
    instrux + 3125,
    instrux + 3126,
    instrux + 3127,
};

static const struct itemplate * const itable_vex0315E[] = {
    instrux + 3144,
    instrux + 3145,
    instrux + 3146,
    instrux + 3147,
    instrux + 3148,
    instrux + 3149,
    instrux + 3150,
    instrux + 3151,
};

static const struct itemplate * const itable_vex0315F[] = {
    instrux + 3136,
    instrux + 3137,
    instrux + 3138,
    instrux + 3139,
    instrux + 3140,
    instrux + 3141,
    instrux + 3142,
    instrux + 3143,
};

static const struct itemplate * const itable_vex03160[] = {
    instrux + 2556,
};

static const struct itemplate * const itable_vex03161[] = {
    instrux + 2555,
};

static const struct itemplate * const itable_vex03162[] = {
    instrux + 2558,
};

static const struct itemplate * const itable_vex03163[] = {
    instrux + 2557,
};

static const struct itemplate * const itable_vex03168[] = {
    instrux + 3104,
    instrux + 3105,
    instrux + 3106,
    instrux + 3107,
    instrux + 3108,
    instrux + 3109,
    instrux + 3110,
    instrux + 3111,
};

static const struct itemplate * const itable_vex03169[] = {
    instrux + 3096,
    instrux + 3097,
    instrux + 3098,
    instrux + 3099,
    instrux + 3100,
    instrux + 3101,
    instrux + 3102,
    instrux + 3103,
};

static const struct itemplate * const itable_vex0316A[] = {
    instrux + 3116,
    instrux + 3117,
    instrux + 3118,
    instrux + 3119,
};

static const struct itemplate * const itable_vex0316B[] = {
    instrux + 3112,
    instrux + 3113,
    instrux + 3114,
    instrux + 3115,
};

static const struct itemplate * const itable_vex0316C[] = {
    instrux + 3160,
    instrux + 3161,
    instrux + 3162,
    instrux + 3163,
    instrux + 3164,
    instrux + 3165,
    instrux + 3166,
    instrux + 3167,
};

static const struct itemplate * const itable_vex0316D[] = {
    instrux + 3152,
    instrux + 3153,
    instrux + 3154,
    instrux + 3155,
    instrux + 3156,
    instrux + 3157,
    instrux + 3158,
    instrux + 3159,
};

static const struct itemplate * const itable_vex0316E[] = {
    instrux + 3172,
    instrux + 3173,
    instrux + 3174,
    instrux + 3175,
};

static const struct itemplate * const itable_vex0316F[] = {
    instrux + 3168,
    instrux + 3169,
    instrux + 3170,
    instrux + 3171,
};

static const struct itemplate * const itable_vex03178[] = {
    instrux + 3184,
    instrux + 3185,
    instrux + 3186,
    instrux + 3187,
    instrux + 3188,
    instrux + 3189,
    instrux + 3190,
    instrux + 3191,
};

static const struct itemplate * const itable_vex03179[] = {
    instrux + 3176,
    instrux + 3177,
    instrux + 3178,
    instrux + 3179,
    instrux + 3180,
    instrux + 3181,
    instrux + 3182,
    instrux + 3183,
};

static const struct itemplate * const itable_vex0317A[] = {
    instrux + 3196,
    instrux + 3197,
    instrux + 3198,
    instrux + 3199,
};

static const struct itemplate * const itable_vex0317B[] = {
    instrux + 3192,
    instrux + 3193,
    instrux + 3194,
    instrux + 3195,
};

static const struct itemplate * const itable_vex0317C[] = {
    instrux + 3208,
    instrux + 3209,
    instrux + 3210,
    instrux + 3211,
    instrux + 3212,
    instrux + 3213,
    instrux + 3214,
    instrux + 3215,
};

static const struct itemplate * const itable_vex0317D[] = {
    instrux + 3200,
    instrux + 3201,
    instrux + 3202,
    instrux + 3203,
    instrux + 3204,
    instrux + 3205,
    instrux + 3206,
    instrux + 3207,
};

static const struct itemplate * const itable_vex0317E[] = {
    instrux + 3220,
    instrux + 3221,
    instrux + 3222,
    instrux + 3223,
};

static const struct itemplate * const itable_vex0317F[] = {
    instrux + 3216,
    instrux + 3217,
    instrux + 3218,
    instrux + 3219,
};

static const struct itemplate * const itable_vex031DF[] = {
    instrux + 1667,
};

static const struct itemplate * const itable_vex033F0[] = {
    instrux + 3721,
    instrux + 3722,
};

static const struct itemplate * const itable_xop08085[] = {
    instrux + 3304,
    instrux + 3305,
};

static const struct itemplate * const itable_xop08086[] = {
    instrux + 3302,
    instrux + 3303,
};

static const struct itemplate * const itable_xop08087[] = {
    instrux + 3300,
    instrux + 3301,
};

static const struct itemplate * const itable_xop0808E[] = {
    instrux + 3296,
    instrux + 3297,
};

static const struct itemplate * const itable_xop0808F[] = {
    instrux + 3298,
    instrux + 3299,
};

static const struct itemplate * const itable_xop08095[] = {
    instrux + 3308,
    instrux + 3309,
};

static const struct itemplate * const itable_xop08096[] = {
    instrux + 3306,
    instrux + 3307,
};

static const struct itemplate * const itable_xop08097[] = {
    instrux + 3294,
    instrux + 3295,
};

static const struct itemplate * const itable_xop0809E[] = {
    instrux + 3290,
    instrux + 3291,
};

static const struct itemplate * const itable_xop0809F[] = {
    instrux + 3292,
    instrux + 3293,
};

static const struct itemplate * const itable_xop080A2[] = {
    instrux + 3236,
    instrux + 3237,
    instrux + 3238,
    instrux + 3239,
    instrux + 3240,
    instrux + 3241,
    instrux + 3242,
    instrux + 3243,
};

static const struct itemplate * const itable_xop080A3[] = {
    instrux + 3314,
    instrux + 3315,
    instrux + 3316,
    instrux + 3317,
};

static const struct itemplate * const itable_xop080A6[] = {
    instrux + 3310,
    instrux + 3311,
};

static const struct itemplate * const itable_xop080B6[] = {
    instrux + 3312,
    instrux + 3313,
};

static const struct itemplate * const itable_xop080C0[] = {
    instrux + 3322,
    instrux + 3323,
};

static const struct itemplate * const itable_xop080C1[] = {
    instrux + 3340,
    instrux + 3341,
};

static const struct itemplate * const itable_xop080C2[] = {
    instrux + 3328,
    instrux + 3329,
};

static const struct itemplate * const itable_xop080C3[] = {
    instrux + 3334,
    instrux + 3335,
};

static const struct itemplate * const itable_xop080CC[] = {
    instrux + 3244,
    instrux + 3245,
};

static const struct itemplate * const itable_xop080CD[] = {
    instrux + 3258,
    instrux + 3259,
};

static const struct itemplate * const itable_xop080CE[] = {
    instrux + 3246,
    instrux + 3247,
};

static const struct itemplate * const itable_xop080CF[] = {
    instrux + 3248,
    instrux + 3249,
};

static const struct itemplate * const itable_xop080EC[] = {
    instrux + 3250,
    instrux + 3251,
};

static const struct itemplate * const itable_xop080ED[] = {
    instrux + 3256,
    instrux + 3257,
};

static const struct itemplate * const itable_xop080EE[] = {
    instrux + 3252,
    instrux + 3253,
};

static const struct itemplate * const itable_xop080EF[] = {
    instrux + 3254,
    instrux + 3255,
};

static const struct itemplate * const itable_xop09001[] = {
    instrux + 3695,
    instrux + 3696,
    instrux + 3699,
    instrux + 3700,
    instrux + 3701,
    instrux + 3702,
    instrux + 3703,
    instrux + 3704,
    instrux + 3711,
    instrux + 3712,
    instrux + 3732,
    instrux + 3733,
    instrux + 3734,
    instrux + 3735,
};

static const struct itemplate * const itable_xop09002[] = {
    instrux + 3693,
    instrux + 3694,
    instrux + 3705,
    instrux + 3706,
};

static const struct itemplate * const itable_xop09012[] = {
    instrux + 3088,
    instrux + 3089,
    instrux + 3090,
    instrux + 3091,
};

static const struct itemplate * const itable_xop09080[] = {
    instrux + 3228,
    instrux + 3229,
    instrux + 3230,
    instrux + 3231,
};

static const struct itemplate * const itable_xop09081[] = {
    instrux + 3224,
    instrux + 3225,
    instrux + 3226,
    instrux + 3227,
};

static const struct itemplate * const itable_xop09082[] = {
    instrux + 3234,
    instrux + 3235,
};

static const struct itemplate * const itable_xop09083[] = {
    instrux + 3232,
    instrux + 3233,
};

static const struct itemplate * const itable_xop09090[] = {
    instrux + 3318,
    instrux + 3319,
    instrux + 3320,
    instrux + 3321,
};

static const struct itemplate * const itable_xop09091[] = {
    instrux + 3336,
    instrux + 3337,
    instrux + 3338,
    instrux + 3339,
};

static const struct itemplate * const itable_xop09092[] = {
    instrux + 3324,
    instrux + 3325,
    instrux + 3326,
    instrux + 3327,
};

static const struct itemplate * const itable_xop09093[] = {
    instrux + 3330,
    instrux + 3331,
    instrux + 3332,
    instrux + 3333,
};

static const struct itemplate * const itable_xop09094[] = {
    instrux + 3358,
    instrux + 3359,
    instrux + 3360,
    instrux + 3361,
};

static const struct itemplate * const itable_xop09095[] = {
    instrux + 3370,
    instrux + 3371,
    instrux + 3372,
    instrux + 3373,
};

static const struct itemplate * const itable_xop09096[] = {
    instrux + 3362,
    instrux + 3363,
    instrux + 3364,
    instrux + 3365,
};

static const struct itemplate * const itable_xop09097[] = {
    instrux + 3366,
    instrux + 3367,
    instrux + 3368,
    instrux + 3369,
};

static const struct itemplate * const itable_xop09098[] = {
    instrux + 3342,
    instrux + 3343,
    instrux + 3344,
    instrux + 3345,
};

static const struct itemplate * const itable_xop09099[] = {
    instrux + 3354,
    instrux + 3355,
    instrux + 3356,
    instrux + 3357,
};

static const struct itemplate * const itable_xop0909A[] = {
    instrux + 3346,
    instrux + 3347,
    instrux + 3348,
    instrux + 3349,
};

static const struct itemplate * const itable_xop0909B[] = {
    instrux + 3350,
    instrux + 3351,
    instrux + 3352,
    instrux + 3353,
};

static const struct itemplate * const itable_xop090C1[] = {
    instrux + 3264,
    instrux + 3265,
};

static const struct itemplate * const itable_xop090C2[] = {
    instrux + 3260,
    instrux + 3261,
};

static const struct itemplate * const itable_xop090C3[] = {
    instrux + 3262,
    instrux + 3263,
};

static const struct itemplate * const itable_xop090C6[] = {
    instrux + 3280,
    instrux + 3281,
};

static const struct itemplate * const itable_xop090C7[] = {
    instrux + 3282,
    instrux + 3283,
};

static const struct itemplate * const itable_xop090CB[] = {
    instrux + 3266,
    instrux + 3267,
};

static const struct itemplate * const itable_xop090D1[] = {
    instrux + 3272,
    instrux + 3273,
};

static const struct itemplate * const itable_xop090D2[] = {
    instrux + 3268,
    instrux + 3269,
};

static const struct itemplate * const itable_xop090D3[] = {
    instrux + 3270,
    instrux + 3271,
};

static const struct itemplate * const itable_xop090D6[] = {
    instrux + 3276,
    instrux + 3277,
};

static const struct itemplate * const itable_xop090D7[] = {
    instrux + 3278,
    instrux + 3279,
};

static const struct itemplate * const itable_xop090DB[] = {
    instrux + 3274,
    instrux + 3275,
};

static const struct itemplate * const itable_xop090E1[] = {
    instrux + 3284,
    instrux + 3285,
};

static const struct itemplate * const itable_xop090E2[] = {
    instrux + 3288,
    instrux + 3289,
};

static const struct itemplate * const itable_xop090E3[] = {
    instrux + 3286,
    instrux + 3287,
};

static const struct itemplate * const itable_xop0A010[] = {
    instrux + 3691,
    instrux + 3692,
};

static const struct itemplate * const itable_xop0A012[] = {
    instrux + 3092,
    instrux + 3093,
    instrux + 3094,
    instrux + 3095,
};

static const struct disasm_index itable_vex010[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { itable_vex01010, 2 },
    /* 0x11 */ { itable_vex01011, 2 },
    /* 0x12 */ { itable_vex01012, 4 },
    /* 0x13 */ { itable_vex01013, 1 },
    /* 0x14 */ { itable_vex01014, 4 },
    /* 0x15 */ { itable_vex01015, 4 },
    /* 0x16 */ { itable_vex01016, 4 },
    /* 0x17 */ { itable_vex01017, 1 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { itable_vex01028, 2 },
    /* 0x29 */ { itable_vex01029, 2 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { itable_vex0102B, 2 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { itable_vex0102E, 1 },
    /* 0x2f */ { itable_vex0102F, 1 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { itable_vex01050, 4 },
    /* 0x51 */ { itable_vex01051, 2 },
    /* 0x52 */ { itable_vex01052, 2 },
    /* 0x53 */ { itable_vex01053, 2 },
    /* 0x54 */ { itable_vex01054, 4 },
    /* 0x55 */ { itable_vex01055, 4 },
    /* 0x56 */ { itable_vex01056, 4 },
    /* 0x57 */ { itable_vex01057, 4 },
    /* 0x58 */ { itable_vex01058, 4 },
    /* 0x59 */ { itable_vex01059, 4 },
    /* 0x5a */ { itable_vex0105A, 2 },
    /* 0x5b */ { itable_vex0105B, 2 },
    /* 0x5c */ { itable_vex0105C, 4 },
    /* 0x5d */ { itable_vex0105D, 4 },
    /* 0x5e */ { itable_vex0105E, 4 },
    /* 0x5f */ { itable_vex0105F, 4 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { itable_vex01077, 2 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { itable_vex010AE, 2 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { itable_vex010C2, 188 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { itable_vex010C6, 4 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex011[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { itable_vex01110, 2 },
    /* 0x11 */ { itable_vex01111, 2 },
    /* 0x12 */ { itable_vex01112, 2 },
    /* 0x13 */ { itable_vex01113, 1 },
    /* 0x14 */ { itable_vex01114, 4 },
    /* 0x15 */ { itable_vex01115, 4 },
    /* 0x16 */ { itable_vex01116, 2 },
    /* 0x17 */ { itable_vex01117, 1 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { itable_vex01128, 2 },
    /* 0x29 */ { itable_vex01129, 2 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { itable_vex0112B, 2 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { itable_vex0112E, 1 },
    /* 0x2f */ { itable_vex0112F, 1 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { itable_vex01150, 4 },
    /* 0x51 */ { itable_vex01151, 2 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { itable_vex01154, 4 },
    /* 0x55 */ { itable_vex01155, 4 },
    /* 0x56 */ { itable_vex01156, 4 },
    /* 0x57 */ { itable_vex01157, 4 },
    /* 0x58 */ { itable_vex01158, 4 },
    /* 0x59 */ { itable_vex01159, 4 },
    /* 0x5a */ { itable_vex0115A, 4 },
    /* 0x5b */ { itable_vex0115B, 2 },
    /* 0x5c */ { itable_vex0115C, 4 },
    /* 0x5d */ { itable_vex0115D, 4 },
    /* 0x5e */ { itable_vex0115E, 4 },
    /* 0x5f */ { itable_vex0115F, 4 },
    /* 0x60 */ { itable_vex01160, 4 },
    /* 0x61 */ { itable_vex01161, 4 },
    /* 0x62 */ { itable_vex01162, 4 },
    /* 0x63 */ { itable_vex01163, 4 },
    /* 0x64 */ { itable_vex01164, 4 },
    /* 0x65 */ { itable_vex01165, 4 },
    /* 0x66 */ { itable_vex01166, 4 },
    /* 0x67 */ { itable_vex01167, 4 },
    /* 0x68 */ { itable_vex01168, 4 },
    /* 0x69 */ { itable_vex01169, 4 },
    /* 0x6a */ { itable_vex0116A, 4 },
    /* 0x6b */ { itable_vex0116B, 4 },
    /* 0x6c */ { itable_vex0116C, 4 },
    /* 0x6d */ { itable_vex0116D, 4 },
    /* 0x6e */ { itable_vex0116E, 2 },
    /* 0x6f */ { itable_vex0116F, 3 },
    /* 0x70 */ { itable_vex01170, 2 },
    /* 0x71 */ { itable_vex01171, 12 },
    /* 0x72 */ { itable_vex01172, 12 },
    /* 0x73 */ { itable_vex01173, 16 },
    /* 0x74 */ { itable_vex01174, 4 },
    /* 0x75 */ { itable_vex01175, 4 },
    /* 0x76 */ { itable_vex01176, 4 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { itable_vex0117C, 4 },
    /* 0x7d */ { itable_vex0117D, 4 },
    /* 0x7e */ { itable_vex0117E, 2 },
    /* 0x7f */ { itable_vex0117F, 3 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { itable_vex011C2, 188 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { itable_vex011C4, 6 },
    /* 0xc5 */ { itable_vex011C5, 2 },
    /* 0xc6 */ { itable_vex011C6, 4 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { itable_vex011D0, 4 },
    /* 0xd1 */ { itable_vex011D1, 4 },
    /* 0xd2 */ { itable_vex011D2, 4 },
    /* 0xd3 */ { itable_vex011D3, 4 },
    /* 0xd4 */ { itable_vex011D4, 4 },
    /* 0xd5 */ { itable_vex011D5, 4 },
    /* 0xd6 */ { itable_vex011D6, 1 },
    /* 0xd7 */ { itable_vex011D7, 4 },
    /* 0xd8 */ { itable_vex011D8, 4 },
    /* 0xd9 */ { itable_vex011D9, 4 },
    /* 0xda */ { itable_vex011DA, 4 },
    /* 0xdb */ { itable_vex011DB, 4 },
    /* 0xdc */ { itable_vex011DC, 4 },
    /* 0xdd */ { itable_vex011DD, 4 },
    /* 0xde */ { itable_vex011DE, 4 },
    /* 0xdf */ { itable_vex011DF, 4 },
    /* 0xe0 */ { itable_vex011E0, 4 },
    /* 0xe1 */ { itable_vex011E1, 4 },
    /* 0xe2 */ { itable_vex011E2, 4 },
    /* 0xe3 */ { itable_vex011E3, 4 },
    /* 0xe4 */ { itable_vex011E4, 4 },
    /* 0xe5 */ { itable_vex011E5, 4 },
    /* 0xe6 */ { itable_vex011E6, 4 },
    /* 0xe7 */ { itable_vex011E7, 3 },
    /* 0xe8 */ { itable_vex011E8, 4 },
    /* 0xe9 */ { itable_vex011E9, 4 },
    /* 0xea */ { itable_vex011EA, 4 },
    /* 0xeb */ { itable_vex011EB, 4 },
    /* 0xec */ { itable_vex011EC, 4 },
    /* 0xed */ { itable_vex011ED, 4 },
    /* 0xee */ { itable_vex011EE, 4 },
    /* 0xef */ { itable_vex011EF, 4 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { itable_vex011F1, 4 },
    /* 0xf2 */ { itable_vex011F2, 4 },
    /* 0xf3 */ { itable_vex011F3, 4 },
    /* 0xf4 */ { itable_vex011F4, 4 },
    /* 0xf5 */ { itable_vex011F5, 4 },
    /* 0xf6 */ { itable_vex011F6, 4 },
    /* 0xf7 */ { itable_vex011F7, 1 },
    /* 0xf8 */ { itable_vex011F8, 4 },
    /* 0xf9 */ { itable_vex011F9, 4 },
    /* 0xfa */ { itable_vex011FA, 4 },
    /* 0xfb */ { itable_vex011FB, 4 },
    /* 0xfc */ { itable_vex011FC, 4 },
    /* 0xfd */ { itable_vex011FD, 4 },
    /* 0xfe */ { itable_vex011FE, 4 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex012[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { itable_vex01210, 3 },
    /* 0x11 */ { itable_vex01211, 3 },
    /* 0x12 */ { itable_vex01212, 2 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { itable_vex01216, 2 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { itable_vex0122A, 4 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { itable_vex0122C, 2 },
    /* 0x2d */ { itable_vex0122D, 2 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { itable_vex01251, 2 },
    /* 0x52 */ { itable_vex01252, 2 },
    /* 0x53 */ { itable_vex01253, 2 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { itable_vex01258, 2 },
    /* 0x59 */ { itable_vex01259, 2 },
    /* 0x5a */ { itable_vex0125A, 2 },
    /* 0x5b */ { itable_vex0125B, 2 },
    /* 0x5c */ { itable_vex0125C, 2 },
    /* 0x5d */ { itable_vex0125D, 2 },
    /* 0x5e */ { itable_vex0125E, 2 },
    /* 0x5f */ { itable_vex0125F, 2 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { itable_vex0126F, 3 },
    /* 0x70 */ { itable_vex01270, 2 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { itable_vex0127E, 1 },
    /* 0x7f */ { itable_vex0127F, 3 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { itable_vex012C2, 94 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { itable_vex012E6, 2 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex013[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { itable_vex01310, 3 },
    /* 0x11 */ { itable_vex01311, 3 },
    /* 0x12 */ { itable_vex01312, 2 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { itable_vex0132A, 4 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { itable_vex0132C, 2 },
    /* 0x2d */ { itable_vex0132D, 2 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { itable_vex01351, 2 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { itable_vex01358, 2 },
    /* 0x59 */ { itable_vex01359, 2 },
    /* 0x5a */ { itable_vex0135A, 2 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { itable_vex0135C, 2 },
    /* 0x5d */ { itable_vex0135D, 2 },
    /* 0x5e */ { itable_vex0135E, 2 },
    /* 0x5f */ { itable_vex0135F, 2 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { itable_vex01370, 2 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { itable_vex0137C, 4 },
    /* 0x7d */ { itable_vex0137D, 4 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { itable_vex013C2, 94 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { itable_vex013D0, 4 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { itable_vex013E6, 4 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { itable_vex013F0, 3 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex020[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { itable_vex020F2, 2 },
    /* 0xf3 */ { itable_vex020F3, 6 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { itable_vex020F5, 2 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { itable_vex020F7, 2 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex021[256] = {
    /* 0x00 */ { itable_vex02100, 4 },
    /* 0x01 */ { itable_vex02101, 4 },
    /* 0x02 */ { itable_vex02102, 4 },
    /* 0x03 */ { itable_vex02103, 4 },
    /* 0x04 */ { itable_vex02104, 4 },
    /* 0x05 */ { itable_vex02105, 4 },
    /* 0x06 */ { itable_vex02106, 4 },
    /* 0x07 */ { itable_vex02107, 4 },
    /* 0x08 */ { itable_vex02108, 4 },
    /* 0x09 */ { itable_vex02109, 4 },
    /* 0x0a */ { itable_vex0210A, 4 },
    /* 0x0b */ { itable_vex0210B, 4 },
    /* 0x0c */ { itable_vex0210C, 2 },
    /* 0x0d */ { itable_vex0210D, 2 },
    /* 0x0e */ { itable_vex0210E, 2 },
    /* 0x0f */ { itable_vex0210F, 2 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { itable_vex02113, 2 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { itable_vex02116, 2 },
    /* 0x17 */ { itable_vex02117, 2 },
    /* 0x18 */ { itable_vex02118, 4 },
    /* 0x19 */ { itable_vex02119, 2 },
    /* 0x1a */ { itable_vex0211A, 1 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { itable_vex0211C, 2 },
    /* 0x1d */ { itable_vex0211D, 2 },
    /* 0x1e */ { itable_vex0211E, 2 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { itable_vex02120, 2 },
    /* 0x21 */ { itable_vex02121, 3 },
    /* 0x22 */ { itable_vex02122, 3 },
    /* 0x23 */ { itable_vex02123, 2 },
    /* 0x24 */ { itable_vex02124, 3 },
    /* 0x25 */ { itable_vex02125, 2 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { itable_vex02128, 4 },
    /* 0x29 */ { itable_vex02129, 4 },
    /* 0x2a */ { itable_vex0212A, 2 },
    /* 0x2b */ { itable_vex0212B, 4 },
    /* 0x2c */ { itable_vex0212C, 2 },
    /* 0x2d */ { itable_vex0212D, 2 },
    /* 0x2e */ { itable_vex0212E, 2 },
    /* 0x2f */ { itable_vex0212F, 2 },
    /* 0x30 */ { itable_vex02130, 2 },
    /* 0x31 */ { itable_vex02131, 3 },
    /* 0x32 */ { itable_vex02132, 3 },
    /* 0x33 */ { itable_vex02133, 2 },
    /* 0x34 */ { itable_vex02134, 3 },
    /* 0x35 */ { itable_vex02135, 2 },
    /* 0x36 */ { itable_vex02136, 2 },
    /* 0x37 */ { itable_vex02137, 4 },
    /* 0x38 */ { itable_vex02138, 4 },
    /* 0x39 */ { itable_vex02139, 4 },
    /* 0x3a */ { itable_vex0213A, 4 },
    /* 0x3b */ { itable_vex0213B, 4 },
    /* 0x3c */ { itable_vex0213C, 4 },
    /* 0x3d */ { itable_vex0213D, 4 },
    /* 0x3e */ { itable_vex0213E, 4 },
    /* 0x3f */ { itable_vex0213F, 4 },
    /* 0x40 */ { itable_vex02140, 4 },
    /* 0x41 */ { itable_vex02141, 1 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { itable_vex02145, 8 },
    /* 0x46 */ { itable_vex02146, 4 },
    /* 0x47 */ { itable_vex02147, 8 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { itable_vex02158, 4 },
    /* 0x59 */ { itable_vex02159, 4 },
    /* 0x5a */ { itable_vex0215A, 1 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { itable_vex02178, 4 },
    /* 0x79 */ { itable_vex02179, 4 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { itable_vex0218C, 8 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { itable_vex0218E, 8 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { itable_vex02190, 4 },
    /* 0x91 */ { itable_vex02191, 4 },
    /* 0x92 */ { itable_vex02192, 4 },
    /* 0x93 */ { itable_vex02193, 4 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { itable_vex02196, 8 },
    /* 0x97 */ { itable_vex02197, 8 },
    /* 0x98 */ { itable_vex02198, 8 },
    /* 0x99 */ { itable_vex02199, 4 },
    /* 0x9a */ { itable_vex0219A, 8 },
    /* 0x9b */ { itable_vex0219B, 4 },
    /* 0x9c */ { itable_vex0219C, 8 },
    /* 0x9d */ { itable_vex0219D, 4 },
    /* 0x9e */ { itable_vex0219E, 8 },
    /* 0x9f */ { itable_vex0219F, 4 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { itable_vex021A6, 8 },
    /* 0xa7 */ { itable_vex021A7, 8 },
    /* 0xa8 */ { itable_vex021A8, 8 },
    /* 0xa9 */ { itable_vex021A9, 4 },
    /* 0xaa */ { itable_vex021AA, 8 },
    /* 0xab */ { itable_vex021AB, 4 },
    /* 0xac */ { itable_vex021AC, 8 },
    /* 0xad */ { itable_vex021AD, 4 },
    /* 0xae */ { itable_vex021AE, 8 },
    /* 0xaf */ { itable_vex021AF, 4 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { itable_vex021B6, 8 },
    /* 0xb7 */ { itable_vex021B7, 8 },
    /* 0xb8 */ { itable_vex021B8, 8 },
    /* 0xb9 */ { itable_vex021B9, 4 },
    /* 0xba */ { itable_vex021BA, 8 },
    /* 0xbb */ { itable_vex021BB, 4 },
    /* 0xbc */ { itable_vex021BC, 8 },
    /* 0xbd */ { itable_vex021BD, 4 },
    /* 0xbe */ { itable_vex021BE, 8 },
    /* 0xbf */ { itable_vex021BF, 4 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { itable_vex021DB, 1 },
    /* 0xdc */ { itable_vex021DC, 2 },
    /* 0xdd */ { itable_vex021DD, 2 },
    /* 0xde */ { itable_vex021DE, 2 },
    /* 0xdf */ { itable_vex021DF, 2 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { itable_vex021F7, 2 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex022[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { itable_vex022F5, 2 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { itable_vex022F7, 2 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex023[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { itable_vex023F5, 2 },
    /* 0xf6 */ { itable_vex023F6, 2 },
    /* 0xf7 */ { itable_vex023F7, 2 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex031[256] = {
    /* 0x00 */ { itable_vex03100, 1 },
    /* 0x01 */ { itable_vex03101, 1 },
    /* 0x02 */ { itable_vex03102, 4 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { itable_vex03104, 2 },
    /* 0x05 */ { itable_vex03105, 2 },
    /* 0x06 */ { itable_vex03106, 1 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { itable_vex03108, 2 },
    /* 0x09 */ { itable_vex03109, 2 },
    /* 0x0a */ { itable_vex0310A, 2 },
    /* 0x0b */ { itable_vex0310B, 2 },
    /* 0x0c */ { itable_vex0310C, 4 },
    /* 0x0d */ { itable_vex0310D, 4 },
    /* 0x0e */ { itable_vex0310E, 4 },
    /* 0x0f */ { itable_vex0310F, 4 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { itable_vex03114, 3 },
    /* 0x15 */ { itable_vex03115, 3 },
    /* 0x16 */ { itable_vex03116, 3 },
    /* 0x17 */ { itable_vex03117, 1 },
    /* 0x18 */ { itable_vex03118, 1 },
    /* 0x19 */ { itable_vex03119, 1 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { itable_vex0311D, 2 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { itable_vex03120, 6 },
    /* 0x21 */ { itable_vex03121, 2 },
    /* 0x22 */ { itable_vex03122, 8 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { itable_vex03138, 2 },
    /* 0x39 */ { itable_vex03139, 1 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { itable_vex03140, 4 },
    /* 0x41 */ { itable_vex03141, 2 },
    /* 0x42 */ { itable_vex03142, 4 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { itable_vex03144, 10 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { itable_vex03146, 1 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { itable_vex0314A, 4 },
    /* 0x4b */ { itable_vex0314B, 4 },
    /* 0x4c */ { itable_vex0314C, 4 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { itable_vex0315C, 8 },
    /* 0x5d */ { itable_vex0315D, 8 },
    /* 0x5e */ { itable_vex0315E, 8 },
    /* 0x5f */ { itable_vex0315F, 8 },
    /* 0x60 */ { itable_vex03160, 1 },
    /* 0x61 */ { itable_vex03161, 1 },
    /* 0x62 */ { itable_vex03162, 1 },
    /* 0x63 */ { itable_vex03163, 1 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { itable_vex03168, 8 },
    /* 0x69 */ { itable_vex03169, 8 },
    /* 0x6a */ { itable_vex0316A, 4 },
    /* 0x6b */ { itable_vex0316B, 4 },
    /* 0x6c */ { itable_vex0316C, 8 },
    /* 0x6d */ { itable_vex0316D, 8 },
    /* 0x6e */ { itable_vex0316E, 4 },
    /* 0x6f */ { itable_vex0316F, 4 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { itable_vex03178, 8 },
    /* 0x79 */ { itable_vex03179, 8 },
    /* 0x7a */ { itable_vex0317A, 4 },
    /* 0x7b */ { itable_vex0317B, 4 },
    /* 0x7c */ { itable_vex0317C, 8 },
    /* 0x7d */ { itable_vex0317D, 8 },
    /* 0x7e */ { itable_vex0317E, 4 },
    /* 0x7f */ { itable_vex0317F, 4 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { itable_vex031DF, 1 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_vex033[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { itable_vex033F0, 2 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_xop080[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { itable_xop08085, 2 },
    /* 0x86 */ { itable_xop08086, 2 },
    /* 0x87 */ { itable_xop08087, 2 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { itable_xop0808E, 2 },
    /* 0x8f */ { itable_xop0808F, 2 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { itable_xop08095, 2 },
    /* 0x96 */ { itable_xop08096, 2 },
    /* 0x97 */ { itable_xop08097, 2 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { itable_xop0809E, 2 },
    /* 0x9f */ { itable_xop0809F, 2 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { itable_xop080A2, 8 },
    /* 0xa3 */ { itable_xop080A3, 4 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { itable_xop080A6, 2 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { itable_xop080B6, 2 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { itable_xop080C0, 2 },
    /* 0xc1 */ { itable_xop080C1, 2 },
    /* 0xc2 */ { itable_xop080C2, 2 },
    /* 0xc3 */ { itable_xop080C3, 2 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { itable_xop080CC, 2 },
    /* 0xcd */ { itable_xop080CD, 2 },
    /* 0xce */ { itable_xop080CE, 2 },
    /* 0xcf */ { itable_xop080CF, 2 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { itable_xop080EC, 2 },
    /* 0xed */ { itable_xop080ED, 2 },
    /* 0xee */ { itable_xop080EE, 2 },
    /* 0xef */ { itable_xop080EF, 2 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_xop090[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { itable_xop09001, 14 },
    /* 0x02 */ { itable_xop09002, 4 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { itable_xop09012, 4 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { itable_xop09080, 4 },
    /* 0x81 */ { itable_xop09081, 4 },
    /* 0x82 */ { itable_xop09082, 2 },
    /* 0x83 */ { itable_xop09083, 2 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { itable_xop09090, 4 },
    /* 0x91 */ { itable_xop09091, 4 },
    /* 0x92 */ { itable_xop09092, 4 },
    /* 0x93 */ { itable_xop09093, 4 },
    /* 0x94 */ { itable_xop09094, 4 },
    /* 0x95 */ { itable_xop09095, 4 },
    /* 0x96 */ { itable_xop09096, 4 },
    /* 0x97 */ { itable_xop09097, 4 },
    /* 0x98 */ { itable_xop09098, 4 },
    /* 0x99 */ { itable_xop09099, 4 },
    /* 0x9a */ { itable_xop0909A, 4 },
    /* 0x9b */ { itable_xop0909B, 4 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { itable_xop090C1, 2 },
    /* 0xc2 */ { itable_xop090C2, 2 },
    /* 0xc3 */ { itable_xop090C3, 2 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { itable_xop090C6, 2 },
    /* 0xc7 */ { itable_xop090C7, 2 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { itable_xop090CB, 2 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { itable_xop090D1, 2 },
    /* 0xd2 */ { itable_xop090D2, 2 },
    /* 0xd3 */ { itable_xop090D3, 2 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { itable_xop090D6, 2 },
    /* 0xd7 */ { itable_xop090D7, 2 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { itable_xop090DB, 2 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { itable_xop090E1, 2 },
    /* 0xe2 */ { itable_xop090E2, 2 },
    /* 0xe3 */ { itable_xop090E3, 2 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_xop0A0[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { itable_xop0A010, 2 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { itable_xop0A012, 4 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_0F38[256] = {
    /* 0x00 */ { itable_0F3800, 2 },
    /* 0x01 */ { itable_0F3801, 2 },
    /* 0x02 */ { itable_0F3802, 2 },
    /* 0x03 */ { itable_0F3803, 2 },
    /* 0x04 */ { itable_0F3804, 2 },
    /* 0x05 */ { itable_0F3805, 2 },
    /* 0x06 */ { itable_0F3806, 2 },
    /* 0x07 */ { itable_0F3807, 2 },
    /* 0x08 */ { itable_0F3808, 2 },
    /* 0x09 */ { itable_0F3809, 2 },
    /* 0x0a */ { itable_0F380A, 2 },
    /* 0x0b */ { itable_0F380B, 2 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { itable_0F3810, 1 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { itable_0F3814, 1 },
    /* 0x15 */ { itable_0F3815, 1 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { itable_0F3817, 1 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { itable_0F381C, 2 },
    /* 0x1d */ { itable_0F381D, 2 },
    /* 0x1e */ { itable_0F381E, 2 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { itable_0F3820, 1 },
    /* 0x21 */ { itable_0F3821, 1 },
    /* 0x22 */ { itable_0F3822, 1 },
    /* 0x23 */ { itable_0F3823, 1 },
    /* 0x24 */ { itable_0F3824, 1 },
    /* 0x25 */ { itable_0F3825, 1 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { itable_0F3828, 1 },
    /* 0x29 */ { itable_0F3829, 1 },
    /* 0x2a */ { itable_0F382A, 1 },
    /* 0x2b */ { itable_0F382B, 1 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { itable_0F3830, 1 },
    /* 0x31 */ { itable_0F3831, 1 },
    /* 0x32 */ { itable_0F3832, 1 },
    /* 0x33 */ { itable_0F3833, 1 },
    /* 0x34 */ { itable_0F3834, 1 },
    /* 0x35 */ { itable_0F3835, 1 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { itable_0F3837, 1 },
    /* 0x38 */ { itable_0F3838, 1 },
    /* 0x39 */ { itable_0F3839, 1 },
    /* 0x3a */ { itable_0F383A, 1 },
    /* 0x3b */ { itable_0F383B, 1 },
    /* 0x3c */ { itable_0F383C, 1 },
    /* 0x3d */ { itable_0F383D, 1 },
    /* 0x3e */ { itable_0F383E, 1 },
    /* 0x3f */ { itable_0F383F, 1 },
    /* 0x40 */ { itable_0F3840, 1 },
    /* 0x41 */ { itable_0F3841, 1 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { itable_0F3880, 2 },
    /* 0x81 */ { itable_0F3881, 2 },
    /* 0x82 */ { itable_0F3882, 2 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { itable_0F38DB, 1 },
    /* 0xdc */ { itable_0F38DC, 1 },
    /* 0xdd */ { itable_0F38DD, 1 },
    /* 0xde */ { itable_0F38DE, 1 },
    /* 0xdf */ { itable_0F38DF, 1 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { itable_0F38F0, 5 },
    /* 0xf1 */ { itable_0F38F1, 6 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { itable_0F38F6, 4 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_0F3A[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { itable_0F3A08, 1 },
    /* 0x09 */ { itable_0F3A09, 1 },
    /* 0x0a */ { itable_0F3A0A, 1 },
    /* 0x0b */ { itable_0F3A0B, 1 },
    /* 0x0c */ { itable_0F3A0C, 1 },
    /* 0x0d */ { itable_0F3A0D, 1 },
    /* 0x0e */ { itable_0F3A0E, 1 },
    /* 0x0f */ { itable_0F3A0F, 2 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { itable_0F3A14, 3 },
    /* 0x15 */ { itable_0F3A15, 3 },
    /* 0x16 */ { itable_0F3A16, 2 },
    /* 0x17 */ { itable_0F3A17, 2 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { itable_0F3A20, 3 },
    /* 0x21 */ { itable_0F3A21, 1 },
    /* 0x22 */ { itable_0F3A22, 4 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { itable_0F3A40, 1 },
    /* 0x41 */ { itable_0F3A41, 1 },
    /* 0x42 */ { itable_0F3A42, 1 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { itable_0F3A44, 5 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { itable_0F3A60, 1 },
    /* 0x61 */ { itable_0F3A61, 1 },
    /* 0x62 */ { itable_0F3A62, 1 },
    /* 0x63 */ { itable_0F3A63, 1 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { NULL, 0 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { NULL, 0 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { NULL, 0 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { itable_0F3ADF, 1 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_0FA6[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { itable_0FA6C0, 1 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { itable_0FA6C8, 1 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { itable_0FA6D0, 1 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { NULL, 0 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { NULL, 0 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { NULL, 0 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_0FA7[256] = {
    /* 0x00 */ { NULL, 0 },
    /* 0x01 */ { NULL, 0 },
    /* 0x02 */ { NULL, 0 },
    /* 0x03 */ { NULL, 0 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { NULL, 0 },
    /* 0x06 */ { NULL, 0 },
    /* 0x07 */ { NULL, 0 },
    /* 0x08 */ { NULL, 0 },
    /* 0x09 */ { NULL, 0 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { NULL, 0 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { NULL, 0 },
    /* 0x0e */ { NULL, 0 },
    /* 0x0f */ { NULL, 0 },
    /* 0x10 */ { NULL, 0 },
    /* 0x11 */ { NULL, 0 },
    /* 0x12 */ { NULL, 0 },
    /* 0x13 */ { NULL, 0 },
    /* 0x14 */ { NULL, 0 },
    /* 0x15 */ { NULL, 0 },
    /* 0x16 */ { NULL, 0 },
    /* 0x17 */ { NULL, 0 },
    /* 0x18 */ { NULL, 0 },
    /* 0x19 */ { NULL, 0 },
    /* 0x1a */ { NULL, 0 },
    /* 0x1b */ { NULL, 0 },
    /* 0x1c */ { NULL, 0 },
    /* 0x1d */ { NULL, 0 },
    /* 0x1e */ { NULL, 0 },
    /* 0x1f */ { NULL, 0 },
    /* 0x20 */ { NULL, 0 },
    /* 0x21 */ { NULL, 0 },
    /* 0x22 */ { NULL, 0 },
    /* 0x23 */ { NULL, 0 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { NULL, 0 },
    /* 0x29 */ { NULL, 0 },
    /* 0x2a */ { NULL, 0 },
    /* 0x2b */ { NULL, 0 },
    /* 0x2c */ { NULL, 0 },
    /* 0x2d */ { NULL, 0 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { NULL, 0 },
    /* 0x30 */ { NULL, 0 },
    /* 0x31 */ { NULL, 0 },
    /* 0x32 */ { NULL, 0 },
    /* 0x33 */ { NULL, 0 },
    /* 0x34 */ { NULL, 0 },
    /* 0x35 */ { NULL, 0 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { NULL, 0 },
    /* 0x38 */ { NULL, 0 },
    /* 0x39 */ { NULL, 0 },
    /* 0x3a */ { NULL, 0 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { NULL, 0 },
    /* 0x3d */ { NULL, 0 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { NULL, 0 },
    /* 0x41 */ { NULL, 0 },
    /* 0x42 */ { NULL, 0 },
    /* 0x43 */ { NULL, 0 },
    /* 0x44 */ { NULL, 0 },
    /* 0x45 */ { NULL, 0 },
    /* 0x46 */ { NULL, 0 },
    /* 0x47 */ { NULL, 0 },
    /* 0x48 */ { NULL, 0 },
    /* 0x49 */ { NULL, 0 },
    /* 0x4a */ { NULL, 0 },
    /* 0x4b */ { NULL, 0 },
    /* 0x4c */ { NULL, 0 },
    /* 0x4d */ { NULL, 0 },
    /* 0x4e */ { NULL, 0 },
    /* 0x4f */ { NULL, 0 },
    /* 0x50 */ { NULL, 0 },
    /* 0x51 */ { NULL, 0 },
    /* 0x52 */ { NULL, 0 },
    /* 0x53 */ { NULL, 0 },
    /* 0x54 */ { NULL, 0 },
    /* 0x55 */ { NULL, 0 },
    /* 0x56 */ { NULL, 0 },
    /* 0x57 */ { NULL, 0 },
    /* 0x58 */ { NULL, 0 },
    /* 0x59 */ { NULL, 0 },
    /* 0x5a */ { NULL, 0 },
    /* 0x5b */ { NULL, 0 },
    /* 0x5c */ { NULL, 0 },
    /* 0x5d */ { NULL, 0 },
    /* 0x5e */ { NULL, 0 },
    /* 0x5f */ { NULL, 0 },
    /* 0x60 */ { NULL, 0 },
    /* 0x61 */ { NULL, 0 },
    /* 0x62 */ { NULL, 0 },
    /* 0x63 */ { NULL, 0 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { NULL, 0 },
    /* 0x69 */ { NULL, 0 },
    /* 0x6a */ { NULL, 0 },
    /* 0x6b */ { NULL, 0 },
    /* 0x6c */ { NULL, 0 },
    /* 0x6d */ { NULL, 0 },
    /* 0x6e */ { NULL, 0 },
    /* 0x6f */ { NULL, 0 },
    /* 0x70 */ { NULL, 0 },
    /* 0x71 */ { NULL, 0 },
    /* 0x72 */ { NULL, 0 },
    /* 0x73 */ { NULL, 0 },
    /* 0x74 */ { NULL, 0 },
    /* 0x75 */ { NULL, 0 },
    /* 0x76 */ { NULL, 0 },
    /* 0x77 */ { NULL, 0 },
    /* 0x78 */ { NULL, 0 },
    /* 0x79 */ { NULL, 0 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { NULL, 0 },
    /* 0x7c */ { NULL, 0 },
    /* 0x7d */ { NULL, 0 },
    /* 0x7e */ { NULL, 0 },
    /* 0x7f */ { NULL, 0 },
    /* 0x80 */ { NULL, 0 },
    /* 0x81 */ { NULL, 0 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { NULL, 0 },
    /* 0x84 */ { NULL, 0 },
    /* 0x85 */ { NULL, 0 },
    /* 0x86 */ { NULL, 0 },
    /* 0x87 */ { NULL, 0 },
    /* 0x88 */ { NULL, 0 },
    /* 0x89 */ { NULL, 0 },
    /* 0x8a */ { NULL, 0 },
    /* 0x8b */ { NULL, 0 },
    /* 0x8c */ { NULL, 0 },
    /* 0x8d */ { NULL, 0 },
    /* 0x8e */ { NULL, 0 },
    /* 0x8f */ { NULL, 0 },
    /* 0x90 */ { NULL, 0 },
    /* 0x91 */ { NULL, 0 },
    /* 0x92 */ { NULL, 0 },
    /* 0x93 */ { NULL, 0 },
    /* 0x94 */ { NULL, 0 },
    /* 0x95 */ { NULL, 0 },
    /* 0x96 */ { NULL, 0 },
    /* 0x97 */ { NULL, 0 },
    /* 0x98 */ { NULL, 0 },
    /* 0x99 */ { NULL, 0 },
    /* 0x9a */ { NULL, 0 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { NULL, 0 },
    /* 0x9d */ { NULL, 0 },
    /* 0x9e */ { NULL, 0 },
    /* 0x9f */ { NULL, 0 },
    /* 0xa0 */ { NULL, 0 },
    /* 0xa1 */ { NULL, 0 },
    /* 0xa2 */ { NULL, 0 },
    /* 0xa3 */ { NULL, 0 },
    /* 0xa4 */ { NULL, 0 },
    /* 0xa5 */ { NULL, 0 },
    /* 0xa6 */ { NULL, 0 },
    /* 0xa7 */ { NULL, 0 },
    /* 0xa8 */ { NULL, 0 },
    /* 0xa9 */ { NULL, 0 },
    /* 0xaa */ { NULL, 0 },
    /* 0xab */ { NULL, 0 },
    /* 0xac */ { NULL, 0 },
    /* 0xad */ { NULL, 0 },
    /* 0xae */ { NULL, 0 },
    /* 0xaf */ { NULL, 0 },
    /* 0xb0 */ { NULL, 0 },
    /* 0xb1 */ { NULL, 0 },
    /* 0xb2 */ { NULL, 0 },
    /* 0xb3 */ { NULL, 0 },
    /* 0xb4 */ { NULL, 0 },
    /* 0xb5 */ { NULL, 0 },
    /* 0xb6 */ { NULL, 0 },
    /* 0xb7 */ { NULL, 0 },
    /* 0xb8 */ { NULL, 0 },
    /* 0xb9 */ { NULL, 0 },
    /* 0xba */ { NULL, 0 },
    /* 0xbb */ { NULL, 0 },
    /* 0xbc */ { NULL, 0 },
    /* 0xbd */ { NULL, 0 },
    /* 0xbe */ { NULL, 0 },
    /* 0xbf */ { NULL, 0 },
    /* 0xc0 */ { itable_0FA7C0, 1 },
    /* 0xc1 */ { NULL, 0 },
    /* 0xc2 */ { NULL, 0 },
    /* 0xc3 */ { NULL, 0 },
    /* 0xc4 */ { NULL, 0 },
    /* 0xc5 */ { NULL, 0 },
    /* 0xc6 */ { NULL, 0 },
    /* 0xc7 */ { NULL, 0 },
    /* 0xc8 */ { itable_0FA7C8, 1 },
    /* 0xc9 */ { NULL, 0 },
    /* 0xca */ { NULL, 0 },
    /* 0xcb */ { NULL, 0 },
    /* 0xcc */ { NULL, 0 },
    /* 0xcd */ { NULL, 0 },
    /* 0xce */ { NULL, 0 },
    /* 0xcf */ { NULL, 0 },
    /* 0xd0 */ { itable_0FA7D0, 1 },
    /* 0xd1 */ { NULL, 0 },
    /* 0xd2 */ { NULL, 0 },
    /* 0xd3 */ { NULL, 0 },
    /* 0xd4 */ { NULL, 0 },
    /* 0xd5 */ { NULL, 0 },
    /* 0xd6 */ { NULL, 0 },
    /* 0xd7 */ { NULL, 0 },
    /* 0xd8 */ { itable_0FA7D8, 1 },
    /* 0xd9 */ { NULL, 0 },
    /* 0xda */ { NULL, 0 },
    /* 0xdb */ { NULL, 0 },
    /* 0xdc */ { NULL, 0 },
    /* 0xdd */ { NULL, 0 },
    /* 0xde */ { NULL, 0 },
    /* 0xdf */ { NULL, 0 },
    /* 0xe0 */ { itable_0FA7E0, 1 },
    /* 0xe1 */ { NULL, 0 },
    /* 0xe2 */ { NULL, 0 },
    /* 0xe3 */ { NULL, 0 },
    /* 0xe4 */ { NULL, 0 },
    /* 0xe5 */ { NULL, 0 },
    /* 0xe6 */ { NULL, 0 },
    /* 0xe7 */ { NULL, 0 },
    /* 0xe8 */ { itable_0FA7E8, 1 },
    /* 0xe9 */ { NULL, 0 },
    /* 0xea */ { NULL, 0 },
    /* 0xeb */ { NULL, 0 },
    /* 0xec */ { NULL, 0 },
    /* 0xed */ { NULL, 0 },
    /* 0xee */ { NULL, 0 },
    /* 0xef */ { NULL, 0 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { NULL, 0 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { NULL, 0 },
    /* 0xf5 */ { NULL, 0 },
    /* 0xf6 */ { NULL, 0 },
    /* 0xf7 */ { NULL, 0 },
    /* 0xf8 */ { NULL, 0 },
    /* 0xf9 */ { NULL, 0 },
    /* 0xfa */ { NULL, 0 },
    /* 0xfb */ { NULL, 0 },
    /* 0xfc */ { NULL, 0 },
    /* 0xfd */ { NULL, 0 },
    /* 0xfe */ { NULL, 0 },
    /* 0xff */ { NULL, 0 },
};

static const struct disasm_index itable_0F[256] = {
    /* 0x00 */ { itable_0F00, 24 },
    /* 0x01 */ { itable_0F01, 38 },
    /* 0x02 */ { itable_0F02, 10 },
    /* 0x03 */ { itable_0F03, 10 },
    /* 0x04 */ { NULL, 0 },
    /* 0x05 */ { itable_0F05, 2 },
    /* 0x06 */ { itable_0F06, 1 },
    /* 0x07 */ { itable_0F07, 2 },
    /* 0x08 */ { itable_0F08, 1 },
    /* 0x09 */ { itable_0F09, 1 },
    /* 0x0a */ { NULL, 0 },
    /* 0x0b */ { itable_0F0B, 1 },
    /* 0x0c */ { NULL, 0 },
    /* 0x0d */ { itable_0F0D, 2 },
    /* 0x0e */ { itable_0F0E, 1 },
    /* 0x0f */ { itable_0F0F, 26 },
    /* 0x10 */ { itable_0F10, 7 },
    /* 0x11 */ { itable_0F11, 6 },
    /* 0x12 */ { itable_0F12, 5 },
    /* 0x13 */ { itable_0F13, 2 },
    /* 0x14 */ { itable_0F14, 2 },
    /* 0x15 */ { itable_0F15, 2 },
    /* 0x16 */ { itable_0F16, 4 },
    /* 0x17 */ { itable_0F17, 2 },
    /* 0x18 */ { itable_0F18, 28 },
    /* 0x19 */ { itable_0F19, 24 },
    /* 0x1a */ { itable_0F1A, 24 },
    /* 0x1b */ { itable_0F1B, 24 },
    /* 0x1c */ { itable_0F1C, 24 },
    /* 0x1d */ { itable_0F1D, 24 },
    /* 0x1e */ { itable_0F1E, 24 },
    /* 0x1f */ { itable_0F1F, 27 },
    /* 0x20 */ { itable_0F20, 2 },
    /* 0x21 */ { itable_0F21, 2 },
    /* 0x22 */ { itable_0F22, 2 },
    /* 0x23 */ { itable_0F23, 2 },
    /* 0x24 */ { NULL, 0 },
    /* 0x25 */ { NULL, 0 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { NULL, 0 },
    /* 0x28 */ { itable_0F28, 3 },
    /* 0x29 */ { itable_0F29, 3 },
    /* 0x2a */ { itable_0F2A, 6 },
    /* 0x2b */ { itable_0F2B, 4 },
    /* 0x2c */ { itable_0F2C, 8 },
    /* 0x2d */ { itable_0F2D, 10 },
    /* 0x2e */ { itable_0F2E, 2 },
    /* 0x2f */ { itable_0F2F, 2 },
    /* 0x30 */ { itable_0F30, 1 },
    /* 0x31 */ { itable_0F31, 1 },
    /* 0x32 */ { itable_0F32, 1 },
    /* 0x33 */ { itable_0F33, 1 },
    /* 0x34 */ { itable_0F34, 1 },
    /* 0x35 */ { itable_0F35, 1 },
    /* 0x36 */ { itable_0F36, 1 },
    /* 0x37 */ { itable_0F37, 2 },
    /* 0x38 */ { itable_0F38, -1 },
    /* 0x39 */ { itable_0F39, 1 },
    /* 0x3a */ { itable_0F3A, -1 },
    /* 0x3b */ { NULL, 0 },
    /* 0x3c */ { itable_0F3C, 1 },
    /* 0x3d */ { itable_0F3D, 1 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { NULL, 0 },
    /* 0x40 */ { itable_0F40, 6 },
    /* 0x41 */ { itable_0F41, 6 },
    /* 0x42 */ { itable_0F42, 6 },
    /* 0x43 */ { itable_0F43, 6 },
    /* 0x44 */ { itable_0F44, 6 },
    /* 0x45 */ { itable_0F45, 6 },
    /* 0x46 */ { itable_0F46, 6 },
    /* 0x47 */ { itable_0F47, 6 },
    /* 0x48 */ { itable_0F48, 6 },
    /* 0x49 */ { itable_0F49, 6 },
    /* 0x4a */ { itable_0F4A, 6 },
    /* 0x4b */ { itable_0F4B, 6 },
    /* 0x4c */ { itable_0F4C, 6 },
    /* 0x4d */ { itable_0F4D, 6 },
    /* 0x4e */ { itable_0F4E, 6 },
    /* 0x4f */ { itable_0F4F, 6 },
    /* 0x50 */ { itable_0F50, 5 },
    /* 0x51 */ { itable_0F51, 5 },
    /* 0x52 */ { itable_0F52, 3 },
    /* 0x53 */ { itable_0F53, 2 },
    /* 0x54 */ { itable_0F54, 3 },
    /* 0x55 */ { itable_0F55, 3 },
    /* 0x56 */ { itable_0F56, 2 },
    /* 0x57 */ { itable_0F57, 2 },
    /* 0x58 */ { itable_0F58, 5 },
    /* 0x59 */ { itable_0F59, 5 },
    /* 0x5a */ { itable_0F5A, 5 },
    /* 0x5b */ { itable_0F5B, 4 },
    /* 0x5c */ { itable_0F5C, 5 },
    /* 0x5d */ { itable_0F5D, 5 },
    /* 0x5e */ { itable_0F5E, 5 },
    /* 0x5f */ { itable_0F5F, 4 },
    /* 0x60 */ { itable_0F60, 2 },
    /* 0x61 */ { itable_0F61, 2 },
    /* 0x62 */ { itable_0F62, 2 },
    /* 0x63 */ { itable_0F63, 2 },
    /* 0x64 */ { itable_0F64, 2 },
    /* 0x65 */ { itable_0F65, 2 },
    /* 0x66 */ { itable_0F66, 2 },
    /* 0x67 */ { itable_0F67, 2 },
    /* 0x68 */ { itable_0F68, 2 },
    /* 0x69 */ { itable_0F69, 2 },
    /* 0x6a */ { itable_0F6A, 2 },
    /* 0x6b */ { itable_0F6B, 2 },
    /* 0x6c */ { itable_0F6C, 1 },
    /* 0x6d */ { itable_0F6D, 1 },
    /* 0x6e */ { itable_0F6E, 5 },
    /* 0x6f */ { itable_0F6F, 5 },
    /* 0x70 */ { itable_0F70, 7 },
    /* 0x71 */ { itable_0F71, 6 },
    /* 0x72 */ { itable_0F72, 6 },
    /* 0x73 */ { itable_0F73, 6 },
    /* 0x74 */ { itable_0F74, 2 },
    /* 0x75 */ { itable_0F75, 2 },
    /* 0x76 */ { itable_0F76, 2 },
    /* 0x77 */ { itable_0F77, 1 },
    /* 0x78 */ { itable_0F78, 5 },
    /* 0x79 */ { itable_0F79, 5 },
    /* 0x7a */ { NULL, 0 },
    /* 0x7b */ { itable_0F7B, 1 },
    /* 0x7c */ { itable_0F7C, 3 },
    /* 0x7d */ { itable_0F7D, 3 },
    /* 0x7e */ { itable_0F7E, 7 },
    /* 0x7f */ { itable_0F7F, 5 },
    /* 0x80 */ { itable_0F80, 4 },
    /* 0x81 */ { itable_0F81, 4 },
    /* 0x82 */ { itable_0F82, 4 },
    /* 0x83 */ { itable_0F83, 4 },
    /* 0x84 */ { itable_0F84, 4 },
    /* 0x85 */ { itable_0F85, 4 },
    /* 0x86 */ { itable_0F86, 4 },
    /* 0x87 */ { itable_0F87, 4 },
    /* 0x88 */ { itable_0F88, 4 },
    /* 0x89 */ { itable_0F89, 4 },
    /* 0x8a */ { itable_0F8A, 4 },
    /* 0x8b */ { itable_0F8B, 4 },
    /* 0x8c */ { itable_0F8C, 4 },
    /* 0x8d */ { itable_0F8D, 4 },
    /* 0x8e */ { itable_0F8E, 4 },
    /* 0x8f */ { itable_0F8F, 4 },
    /* 0x90 */ { itable_0F90, 2 },
    /* 0x91 */ { itable_0F91, 2 },
    /* 0x92 */ { itable_0F92, 2 },
    /* 0x93 */ { itable_0F93, 2 },
    /* 0x94 */ { itable_0F94, 2 },
    /* 0x95 */ { itable_0F95, 2 },
    /* 0x96 */ { itable_0F96, 2 },
    /* 0x97 */ { itable_0F97, 2 },
    /* 0x98 */ { itable_0F98, 2 },
    /* 0x99 */ { itable_0F99, 2 },
    /* 0x9a */ { itable_0F9A, 2 },
    /* 0x9b */ { itable_0F9B, 2 },
    /* 0x9c */ { itable_0F9C, 2 },
    /* 0x9d */ { itable_0F9D, 2 },
    /* 0x9e */ { itable_0F9E, 2 },
    /* 0x9f */ { itable_0F9F, 2 },
    /* 0xa0 */ { itable_0FA0, 1 },
    /* 0xa1 */ { itable_0FA1, 1 },
    /* 0xa2 */ { itable_0FA2, 1 },
    /* 0xa3 */ { itable_0FA3, 6 },
    /* 0xa4 */ { itable_0FA4, 6 },
    /* 0xa5 */ { itable_0FA5, 6 },
    /* 0xa6 */ { itable_0FA6, -1 },
    /* 0xa7 */ { itable_0FA7, -1 },
    /* 0xa8 */ { itable_0FA8, 1 },
    /* 0xa9 */ { itable_0FA9, 1 },
    /* 0xaa */ { itable_0FAA, 1 },
    /* 0xab */ { itable_0FAB, 6 },
    /* 0xac */ { itable_0FAC, 6 },
    /* 0xad */ { itable_0FAD, 6 },
    /* 0xae */ { itable_0FAE, 27 },
    /* 0xaf */ { itable_0FAF, 6 },
    /* 0xb0 */ { itable_0FB0, 2 },
    /* 0xb1 */ { itable_0FB1, 6 },
    /* 0xb2 */ { itable_0FB2, 3 },
    /* 0xb3 */ { itable_0FB3, 6 },
    /* 0xb4 */ { itable_0FB4, 3 },
    /* 0xb5 */ { itable_0FB5, 3 },
    /* 0xb6 */ { itable_0FB6, 4 },
    /* 0xb7 */ { itable_0FB7, 2 },
    /* 0xb8 */ { itable_0FB8, 6 },
    /* 0xb9 */ { itable_0FB9, 1 },
    /* 0xba */ { itable_0FBA, 12 },
    /* 0xbb */ { itable_0FBB, 6 },
    /* 0xbc */ { itable_0FBC, 9 },
    /* 0xbd */ { itable_0FBD, 9 },
    /* 0xbe */ { itable_0FBE, 4 },
    /* 0xbf */ { itable_0FBF, 2 },
    /* 0xc0 */ { itable_0FC0, 2 },
    /* 0xc1 */ { itable_0FC1, 6 },
    /* 0xc2 */ { itable_0FC2, 38 },
    /* 0xc3 */ { itable_0FC3, 2 },
    /* 0xc4 */ { itable_0FC4, 6 },
    /* 0xc5 */ { itable_0FC5, 2 },
    /* 0xc6 */ { itable_0FC6, 3 },
    /* 0xc7 */ { itable_0FC7, 12 },
    /* 0xc8 */ { itable_0FC8, 2 },
    /* 0xc9 */ { itable_0FC9, 2 },
    /* 0xca */ { itable_0FCA, 2 },
    /* 0xcb */ { itable_0FCB, 2 },
    /* 0xcc */ { itable_0FCC, 2 },
    /* 0xcd */ { itable_0FCD, 2 },
    /* 0xce */ { itable_0FCE, 2 },
    /* 0xcf */ { itable_0FCF, 2 },
    /* 0xd0 */ { itable_0FD0, 2 },
    /* 0xd1 */ { itable_0FD1, 2 },
    /* 0xd2 */ { itable_0FD2, 2 },
    /* 0xd3 */ { itable_0FD3, 2 },
    /* 0xd4 */ { itable_0FD4, 2 },
    /* 0xd5 */ { itable_0FD5, 2 },
    /* 0xd6 */ { itable_0FD6, 4 },
    /* 0xd7 */ { itable_0FD7, 2 },
    /* 0xd8 */ { itable_0FD8, 2 },
    /* 0xd9 */ { itable_0FD9, 2 },
    /* 0xda */ { itable_0FDA, 2 },
    /* 0xdb */ { itable_0FDB, 2 },
    /* 0xdc */ { itable_0FDC, 2 },
    /* 0xdd */ { itable_0FDD, 2 },
    /* 0xde */ { itable_0FDE, 2 },
    /* 0xdf */ { itable_0FDF, 2 },
    /* 0xe0 */ { itable_0FE0, 2 },
    /* 0xe1 */ { itable_0FE1, 2 },
    /* 0xe2 */ { itable_0FE2, 2 },
    /* 0xe3 */ { itable_0FE3, 2 },
    /* 0xe4 */ { itable_0FE4, 2 },
    /* 0xe5 */ { itable_0FE5, 2 },
    /* 0xe6 */ { itable_0FE6, 3 },
    /* 0xe7 */ { itable_0FE7, 2 },
    /* 0xe8 */ { itable_0FE8, 2 },
    /* 0xe9 */ { itable_0FE9, 2 },
    /* 0xea */ { itable_0FEA, 2 },
    /* 0xeb */ { itable_0FEB, 2 },
    /* 0xec */ { itable_0FEC, 2 },
    /* 0xed */ { itable_0FED, 2 },
    /* 0xee */ { itable_0FEE, 2 },
    /* 0xef */ { itable_0FEF, 2 },
    /* 0xf0 */ { itable_0FF0, 1 },
    /* 0xf1 */ { itable_0FF1, 2 },
    /* 0xf2 */ { itable_0FF2, 2 },
    /* 0xf3 */ { itable_0FF3, 2 },
    /* 0xf4 */ { itable_0FF4, 2 },
    /* 0xf5 */ { itable_0FF5, 2 },
    /* 0xf6 */ { itable_0FF6, 2 },
    /* 0xf7 */ { itable_0FF7, 2 },
    /* 0xf8 */ { itable_0FF8, 2 },
    /* 0xf9 */ { itable_0FF9, 2 },
    /* 0xfa */ { itable_0FFA, 2 },
    /* 0xfb */ { itable_0FFB, 2 },
    /* 0xfc */ { itable_0FFC, 2 },
    /* 0xfd */ { itable_0FFD, 2 },
    /* 0xfe */ { itable_0FFE, 2 },
    /* 0xff */ { itable_0FFF, 1 },
};

const struct disasm_index itable[256] = {
    /* 0x00 */ { itable_00, 2 },
    /* 0x01 */ { itable_01, 6 },
    /* 0x02 */ { itable_02, 2 },
    /* 0x03 */ { itable_03, 6 },
    /* 0x04 */ { itable_04, 1 },
    /* 0x05 */ { itable_05, 3 },
    /* 0x06 */ { itable_06, 1 },
    /* 0x07 */ { itable_07, 1 },
    /* 0x08 */ { itable_08, 2 },
    /* 0x09 */ { itable_09, 6 },
    /* 0x0a */ { itable_0A, 2 },
    /* 0x0b */ { itable_0B, 6 },
    /* 0x0c */ { itable_0C, 1 },
    /* 0x0d */ { itable_0D, 3 },
    /* 0x0e */ { itable_0E, 1 },
    /* 0x0f */ { itable_0F, -1 },
    /* 0x10 */ { itable_10, 2 },
    /* 0x11 */ { itable_11, 6 },
    /* 0x12 */ { itable_12, 2 },
    /* 0x13 */ { itable_13, 6 },
    /* 0x14 */ { itable_14, 1 },
    /* 0x15 */ { itable_15, 3 },
    /* 0x16 */ { itable_16, 1 },
    /* 0x17 */ { itable_17, 1 },
    /* 0x18 */ { itable_18, 2 },
    /* 0x19 */ { itable_19, 6 },
    /* 0x1a */ { itable_1A, 2 },
    /* 0x1b */ { itable_1B, 6 },
    /* 0x1c */ { itable_1C, 1 },
    /* 0x1d */ { itable_1D, 3 },
    /* 0x1e */ { itable_1E, 1 },
    /* 0x1f */ { itable_1F, 1 },
    /* 0x20 */ { itable_20, 2 },
    /* 0x21 */ { itable_21, 6 },
    /* 0x22 */ { itable_22, 2 },
    /* 0x23 */ { itable_23, 6 },
    /* 0x24 */ { itable_24, 1 },
    /* 0x25 */ { itable_25, 3 },
    /* 0x26 */ { NULL, 0 },
    /* 0x27 */ { itable_27, 1 },
    /* 0x28 */ { itable_28, 2 },
    /* 0x29 */ { itable_29, 6 },
    /* 0x2a */ { itable_2A, 2 },
    /* 0x2b */ { itable_2B, 6 },
    /* 0x2c */ { itable_2C, 1 },
    /* 0x2d */ { itable_2D, 3 },
    /* 0x2e */ { NULL, 0 },
    /* 0x2f */ { itable_2F, 1 },
    /* 0x30 */ { itable_30, 2 },
    /* 0x31 */ { itable_31, 6 },
    /* 0x32 */ { itable_32, 2 },
    /* 0x33 */ { itable_33, 6 },
    /* 0x34 */ { itable_34, 1 },
    /* 0x35 */ { itable_35, 3 },
    /* 0x36 */ { NULL, 0 },
    /* 0x37 */ { itable_37, 1 },
    /* 0x38 */ { itable_38, 2 },
    /* 0x39 */ { itable_39, 6 },
    /* 0x3a */ { itable_3A, 2 },
    /* 0x3b */ { itable_3B, 6 },
    /* 0x3c */ { itable_3C, 1 },
    /* 0x3d */ { itable_3D, 3 },
    /* 0x3e */ { NULL, 0 },
    /* 0x3f */ { itable_3F, 1 },
    /* 0x40 */ { itable_40, 2 },
    /* 0x41 */ { itable_41, 2 },
    /* 0x42 */ { itable_42, 2 },
    /* 0x43 */ { itable_43, 2 },
    /* 0x44 */ { itable_44, 2 },
    /* 0x45 */ { itable_45, 2 },
    /* 0x46 */ { itable_46, 2 },
    /* 0x47 */ { itable_47, 2 },
    /* 0x48 */ { itable_48, 2 },
    /* 0x49 */ { itable_49, 2 },
    /* 0x4a */ { itable_4A, 2 },
    /* 0x4b */ { itable_4B, 2 },
    /* 0x4c */ { itable_4C, 2 },
    /* 0x4d */ { itable_4D, 2 },
    /* 0x4e */ { itable_4E, 2 },
    /* 0x4f */ { itable_4F, 2 },
    /* 0x50 */ { itable_50, 3 },
    /* 0x51 */ { itable_51, 3 },
    /* 0x52 */ { itable_52, 3 },
    /* 0x53 */ { itable_53, 3 },
    /* 0x54 */ { itable_54, 3 },
    /* 0x55 */ { itable_55, 3 },
    /* 0x56 */ { itable_56, 3 },
    /* 0x57 */ { itable_57, 3 },
    /* 0x58 */ { itable_58, 3 },
    /* 0x59 */ { itable_59, 3 },
    /* 0x5a */ { itable_5A, 3 },
    /* 0x5b */ { itable_5B, 3 },
    /* 0x5c */ { itable_5C, 3 },
    /* 0x5d */ { itable_5D, 3 },
    /* 0x5e */ { itable_5E, 3 },
    /* 0x5f */ { itable_5F, 3 },
    /* 0x60 */ { itable_60, 3 },
    /* 0x61 */ { itable_61, 3 },
    /* 0x62 */ { itable_62, 2 },
    /* 0x63 */ { itable_63, 3 },
    /* 0x64 */ { NULL, 0 },
    /* 0x65 */ { NULL, 0 },
    /* 0x66 */ { NULL, 0 },
    /* 0x67 */ { NULL, 0 },
    /* 0x68 */ { itable_68, 5 },
    /* 0x69 */ { itable_69, 9 },
    /* 0x6a */ { itable_6A, 1 },
    /* 0x6b */ { itable_6B, 9 },
    /* 0x6c */ { itable_6C, 1 },
    /* 0x6d */ { itable_6D, 2 },
    /* 0x6e */ { itable_6E, 1 },
    /* 0x6f */ { itable_6F, 2 },
    /* 0x70 */ { itable_70, 1 },
    /* 0x71 */ { itable_71, 1 },
    /* 0x72 */ { itable_72, 1 },
    /* 0x73 */ { itable_73, 1 },
    /* 0x74 */ { itable_74, 1 },
    /* 0x75 */ { itable_75, 1 },
    /* 0x76 */ { itable_76, 1 },
    /* 0x77 */ { itable_77, 1 },
    /* 0x78 */ { itable_78, 1 },
    /* 0x79 */ { itable_79, 1 },
    /* 0x7a */ { itable_7A, 1 },
    /* 0x7b */ { itable_7B, 1 },
    /* 0x7c */ { itable_7C, 1 },
    /* 0x7d */ { itable_7D, 1 },
    /* 0x7e */ { itable_7E, 1 },
    /* 0x7f */ { itable_7F, 1 },
    /* 0x80 */ { itable_80, 15 },
    /* 0x81 */ { itable_81, 40 },
    /* 0x82 */ { NULL, 0 },
    /* 0x83 */ { itable_83, 24 },
    /* 0x84 */ { itable_84, 3 },
    /* 0x85 */ { itable_85, 9 },
    /* 0x86 */ { itable_86, 4 },
    /* 0x87 */ { itable_87, 12 },
    /* 0x88 */ { itable_88, 2 },
    /* 0x89 */ { itable_89, 6 },
    /* 0x8a */ { itable_8A, 2 },
    /* 0x8b */ { itable_8B, 6 },
    /* 0x8c */ { itable_8C, 4 },
    /* 0x8d */ { itable_8D, 3 },
    /* 0x8e */ { itable_8E, 4 },
    /* 0x8f */ { itable_8F, 3 },
    /* 0x90 */ { itable_90, 9 },
    /* 0x91 */ { itable_91, 6 },
    /* 0x92 */ { itable_92, 6 },
    /* 0x93 */ { itable_93, 6 },
    /* 0x94 */ { itable_94, 6 },
    /* 0x95 */ { itable_95, 6 },
    /* 0x96 */ { itable_96, 6 },
    /* 0x97 */ { itable_97, 6 },
    /* 0x98 */ { itable_98, 3 },
    /* 0x99 */ { itable_99, 3 },
    /* 0x9a */ { itable_9A, 5 },
    /* 0x9b */ { NULL, 0 },
    /* 0x9c */ { itable_9C, 4 },
    /* 0x9d */ { itable_9D, 4 },
    /* 0x9e */ { itable_9E, 1 },
    /* 0x9f */ { itable_9F, 1 },
    /* 0xa0 */ { itable_A0, 1 },
    /* 0xa1 */ { itable_A1, 3 },
    /* 0xa2 */ { itable_A2, 1 },
    /* 0xa3 */ { itable_A3, 3 },
    /* 0xa4 */ { itable_A4, 1 },
    /* 0xa5 */ { itable_A5, 3 },
    /* 0xa6 */ { itable_A6, 1 },
    /* 0xa7 */ { itable_A7, 3 },
    /* 0xa8 */ { itable_A8, 1 },
    /* 0xa9 */ { itable_A9, 3 },
    /* 0xaa */ { itable_AA, 1 },
    /* 0xab */ { itable_AB, 3 },
    /* 0xac */ { itable_AC, 1 },
    /* 0xad */ { itable_AD, 3 },
    /* 0xae */ { itable_AE, 1 },
    /* 0xaf */ { itable_AF, 3 },
    /* 0xb0 */ { itable_B0, 1 },
    /* 0xb1 */ { itable_B1, 1 },
    /* 0xb2 */ { itable_B2, 1 },
    /* 0xb3 */ { itable_B3, 1 },
    /* 0xb4 */ { itable_B4, 1 },
    /* 0xb5 */ { itable_B5, 1 },
    /* 0xb6 */ { itable_B6, 1 },
    /* 0xb7 */ { itable_B7, 1 },
    /* 0xb8 */ { itable_B8, 3 },
    /* 0xb9 */ { itable_B9, 3 },
    /* 0xba */ { itable_BA, 3 },
    /* 0xbb */ { itable_BB, 3 },
    /* 0xbc */ { itable_BC, 3 },
    /* 0xbd */ { itable_BD, 3 },
    /* 0xbe */ { itable_BE, 3 },
    /* 0xbf */ { itable_BF, 3 },
    /* 0xc0 */ { itable_C0, 7 },
    /* 0xc1 */ { itable_C1, 21 },
    /* 0xc2 */ { itable_C2, 2 },
    /* 0xc3 */ { itable_C3, 2 },
    /* 0xc4 */ { itable_C4, 2 },
    /* 0xc5 */ { itable_C5, 2 },
    /* 0xc6 */ { itable_C6, 4 },
    /* 0xc7 */ { itable_C7, 10 },
    /* 0xc8 */ { itable_C8, 1 },
    /* 0xc9 */ { itable_C9, 1 },
    /* 0xca */ { itable_CA, 1 },
    /* 0xcb */ { itable_CB, 1 },
    /* 0xcc */ { itable_CC, 1 },
    /* 0xcd */ { itable_CD, 1 },
    /* 0xce */ { itable_CE, 1 },
    /* 0xcf */ { itable_CF, 4 },
    /* 0xd0 */ { itable_D0, 7 },
    /* 0xd1 */ { itable_D1, 21 },
    /* 0xd2 */ { itable_D2, 7 },
    /* 0xd3 */ { itable_D3, 21 },
    /* 0xd4 */ { itable_D4, 2 },
    /* 0xd5 */ { itable_D5, 2 },
    /* 0xd6 */ { itable_D6, 1 },
    /* 0xd7 */ { itable_D7, 2 },
    /* 0xd8 */ { itable_D8, 24 },
    /* 0xd9 */ { itable_D9, 41 },
    /* 0xda */ { itable_DA, 17 },
    /* 0xdb */ { itable_DB, 27 },
    /* 0xdc */ { itable_DC, 20 },
    /* 0xdd */ { itable_DD, 17 },
    /* 0xde */ { itable_DE, 21 },
    /* 0xdf */ { itable_DF, 18 },
    /* 0xe0 */ { itable_E0, 8 },
    /* 0xe1 */ { itable_E1, 8 },
    /* 0xe2 */ { itable_E2, 4 },
    /* 0xe3 */ { itable_E3, 3 },
    /* 0xe4 */ { itable_E4, 1 },
    /* 0xe5 */ { itable_E5, 2 },
    /* 0xe6 */ { itable_E6, 1 },
    /* 0xe7 */ { itable_E7, 2 },
    /* 0xe8 */ { itable_E8, 4 },
    /* 0xe9 */ { itable_E9, 4 },
    /* 0xea */ { itable_EA, 5 },
    /* 0xeb */ { itable_EB, 1 },
    /* 0xec */ { itable_EC, 1 },
    /* 0xed */ { itable_ED, 2 },
    /* 0xee */ { itable_EE, 1 },
    /* 0xef */ { itable_EF, 2 },
    /* 0xf0 */ { NULL, 0 },
    /* 0xf1 */ { itable_F1, 2 },
    /* 0xf2 */ { NULL, 0 },
    /* 0xf3 */ { NULL, 0 },
    /* 0xf4 */ { itable_F4, 1 },
    /* 0xf5 */ { itable_F5, 1 },
    /* 0xf6 */ { itable_F6, 8 },
    /* 0xf7 */ { itable_F7, 23 },
    /* 0xf8 */ { itable_F8, 1 },
    /* 0xf9 */ { itable_F9, 1 },
    /* 0xfa */ { itable_FA, 1 },
    /* 0xfb */ { itable_FB, 1 },
    /* 0xfc */ { itable_FC, 1 },
    /* 0xfd */ { itable_FD, 1 },
    /* 0xfe */ { itable_FE, 2 },
    /* 0xff */ { itable_FF, 27 },
};

const struct disasm_index * const itable_vex[2][32][4] =
{
    {
        { NULL,          NULL,          NULL,          NULL,          },
        { itable_vex010, itable_vex011, itable_vex012, itable_vex013, },
        { itable_vex020, itable_vex021, itable_vex022, itable_vex023, },
        { NULL,          itable_vex031, NULL,          itable_vex033, },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
    },
    {
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { itable_xop080, NULL,          NULL,          NULL,          },
        { itable_xop090, NULL,          NULL,          NULL,          },
        { itable_xop0A0, NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
        { NULL,          NULL,          NULL,          NULL,          },
    },
};
